package drastic.mCmdIF;

import java.net.*;

/**
 * <p>Title: MediaCmdIF</p>
 *
 * <p>Description: Java Interface to MediaCmd structure.  This class makes
 * using the MediaCmd structure easier by supplying simple function calls for
 * basic functions, actually sending and receiving the commands using
 * vvwNetSender and by maintaining often checked values and only updating
 * them as necessary.<br>
 *<br><br>
 * MediaCmdIF - Main Interface API - uses MediaCmd and VVWNetCmdSender<br>
 * MediaCmd - Structure sent and returned from the VVW<br>
 * VVWNETCMDSENDER - used to send and receive the MediaCmds<br>
 * </p>
 *
 * <p>Copyright: Copyright (c) 1998-2007</p>
 *
 * <p>Company: Drastic Technologies Ltd.<br>
 * 523 The Queensway, Suite 102<br>
 * Toronto, ON, M8Y 1J7<br>
 * CANADA<br>
 * 416 255 5636<br>
 * fax 255 8780<br>
 * http://www.drastictech.com<br>
 * engineering@drastictech.com</p>
 *
 * @see VVWNETCMDSENDER
 * @see MediaCmd
 *
 * @author Dean Mitrovich
 * @version 1.0
 */
public class MediaCmdIF extends MEDIACMD
{
    /**
     * Number of good dur rx'd
     */
    protected long dwGoodVbValues = 0;
    /**
     * Current local time in ms
     */
    public long dwCurrentLocalTimeMs = 0;
    /**
     * Current (last) vblank time in ms
     */
    public long dwLastLocalVBlankTimeMs = 0;
    /**
     * Current (last) send time in ms
     */
    public long dwCurrentLocalSentTimeMs = 0;
    /**
     * Current java ms time
     */
    public long dwCurrentStatusTimeMs = 0;
    /**
     * Current java ms for absolute time
     */
    public long dwCurrentAbsoluteTimeMs = 0;
    /**
     * Current absolute position at last dwCurrentAbsoluteTimeMs
     */
    public long dwAbsolutePosition = 0;
    /**
     * Last mode check time
     */
    public long dwLastClipModeCheckMs = 0;
    /**
     * The last known clip mode as of dwLastClipModeCheckMs
     */
    public long m_dwClipMode = 0;
    /**
     * Duration of last frame in me
     * (eg 33.3667, 40.0 etc)
     */
    public double ddVBlankDurMs = 0.0;
    /**
     * Last vblank as a java ms time
     */
    public long dwVblankJava = 0;
    /**
     * Last vblank in using ms
     */
    private long dwVBlankMs = 0;
    /**
     * A general counter
     */
    public long dwCount = 0;
    /**
     * Our actual network sender
     */
    private VVWNETCMDSENDER vvwCmd; // = new VVWNETCMDSENDER();
    /**
     * MediaCmd for getting/holding current state
     */
    private MEDIACMD mCmdState = new MEDIACMD();
    /**
     * MediaCmd for TCSpace edl rx processing
     */
    private MEDIACMD mCmdEDL = new MEDIACMD();
    /**
     * MediaCmd for getting/holding current ms
     */
    private MEDIACMD mCmdClip = new MEDIACMD();
    /**
     * MediaCmd for getting/holding last change ms
     */
    private MEDIACMD mCmdLastChange = new MEDIACMD();
    /**
     * MediaCmd for getting/holding current ms
     */
    private MEDIACMD mCmdCurMs = new MEDIACMD();

    private TCXlat tcxObj = new TCXlat();
    public VidUtil vidUtil = new VidUtil();

    private long m_dwTcType = 0;
    private long m_dwTcTypeLastCheckMs = 0;
    private long m_dwLTCType = 0;
    private long m_dwVITCType = 0;
    private long m_dwTCTCType = 0;

    public boolean fIsVvwCmdNull = false;
    public boolean fIsVvwCmdValid = false;
    public boolean fIsVvwCmdReady = false;
    public boolean fIsVvwCmdConnected= false;

    /**
     * Duration between status updates (milliseconds)
     * (Adjusts based on video standard)
     */
    public int m_lUpdateFreq = 40;

    // Start - Basic play speeds
    public static final int SpeedFwdPlay  = 65520;
    public static final int SpeedPlay = SpeedFwdPlay;
    public static final int SpeedPause = 0;
    public static final int SpeedRevPlay = -65520;
    public static final int SpeedFwdMax = 5896800;
    public static final int SpeedRevMax = -SpeedFwdMax;

    public static final int SPD_FWD_PLAY = 65520;
    public static final int SPD_PAUSE = 0;
    public static final int SPD_REV_PLAY = (-SPD_FWD_PLAY);
    public static final int SPD_FWD_MAX = 5896800;
    public static final int SPD_REV_MAX = (-SPD_FWD_MAX);
    //public static final int SPD_ILLEGAL = 2147483647;
    //public static final int TC_ILLEGAL = 0xFFFFFFFF;
    //public static final int CHAN_ILLEGAL = 0xFFFFFFFF;
    //End - Basic Play speeds


    /**
     * Constructor
     */
    public MediaCmdIF()
    {
        super();
        vvwCmd = new VVWNETCMDSENDER();
        mCmdCurMs.Init();
        mCmdCurMs.Cmd(ctGetValue);
        mCmdCurMs.CmdAlt(gsCurrentMs);
        mCmdCurMs.Flags(-1);
    }

    /**
     * Constructor
     *
     * @param socketManager SocketManager
     */
    public MediaCmdIF(SocketManager socketManager)
    {
        super();
        vvwCmd = new VVWNETCMDSENDER(socketManager);
    }

    /**
     * Is vvwCmd == null
     *
     * @return boolean
     */
    public boolean IsVvwNetSenderNull()
    {
        if (vvwCmd == null)
        {
            fIsVvwCmdNull = true;
        }
        else
        {
            fIsVvwCmdNull = false;
        }
        return fIsVvwCmdNull;
    }

    /**
     * Is vvwCmd Valid
     *
     * @return boolean
     */
    public boolean IsVvwNetSenderValid()
    {
        fIsVvwCmdValid = vvwCmd.IsValid();
        return fIsVvwCmdValid;
    }

    /**
     * Is vvwCmd Ready
     *
     * @return boolean
     */
    public boolean IsVvwNetSenderReady()
    {
        fIsVvwCmdReady = vvwCmd.IsReady();
        return fIsVvwCmdReady;
    }

    /**
     * Is vvwCmd Connected
     *
     * @return boolean
     */
    public boolean IsVvwNetSenderConnected()
    {
        fIsVvwCmdConnected = vvwCmd.IsConnected();
        return fIsVvwCmdConnected;
    }

    /**
     * SetInLtcChannel
     *
     * @param lSetting long
     * @return long
     */
    public long SetInLtcChannel(long lSetting)
    {
        return ValueSet(MEDIACMD.gsAudInLtcChannel, lSetting, 0, 0, 0);
    }

    /**
     * SetOutLtcChannel
     *
     * @param lSetting long
     * @return long
     */
    public long SetOutLtcChannel(long lSetting)
    {
        return ValueSet(MEDIACMD.gsAudOutLtcChannel, lSetting, 0, 0, 0);
    }

    /**
     * SetAudLtcEnable
     *
     * @param lPos long
     * @return long
     */
    public long SetAudLtcEnable(long lPos)
    {
        return ValueSet(MEDIACMD.gsAudLtcEnable, lPos, 0, 0, 0);
    }

    /**
     * GetAudLtcEnable
     *
     * @return long
     */
    public long GetAudLtcEnable()
    {
        return ValueGet(MEDIACMD.gsAudLtcEnable);
    }

    /**
     * GetInputStatus
     *
     * @return String
     */
    public String GetInputStatus()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsVidInSignalFormat);

        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return null;
        }
        String pName = new String(mCmd.arbID());

        if(pName == null)
        {
            return null;
        }
        return pName.trim();
    }

    /**
     * GetRefStatus
     *
     * @return String
     */
    public String GetRefStatus()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsVidOutLockSignalFormat);

        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return null;
        }
        String pName = new String(mCmd.arbID());

        if(mCmd.Position() == -1)
        {
            return "Not Connected";
        }
        else if(pName == null)
        {
            return null;
        }
        return pName.trim();
    }

    // ***
    // Channel Control
    // ***

    /**
     * Get the available channels
     *
     * @return long
     */
    public long GetChannels()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsVVWVersion);
        mCmd.Flags( -1);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            if(mCmd.Channel() < 0)
            {
                return 0;
            }
            else
            {
                return mCmd.Channel();
            }
        }
        else return 0; // -1;
    }

    /**
     * Set the Maximum Channels
     *
     * @param lChannel long
     * @return long
     */
    public long SetMaxChannels( long lChannel)
    {
        if (lChannel < GetChannels())
        {
            SetChannel(lChannel);
            return 0;
        }
        else return -1;
    }

    /**
     * SetChannel
     *
     * @param lChannel long
     * @return long
     */
    public long SetChannel(long lChannel)
    {
        if(lChannel >= GetChannels())
        {
            return -1;
        }
        vvwCmd.setChannel(lChannel);
        GetTCTcTypes();
        GetState();
        GetClipMode(false);
        return 0;
    }

    /**
     * Get the current channel as a long
     *
     * @return long
     */
    public long GetCurChannel()
    {
        if (dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return vvwCmd.getChannel();
    }

    /**
     * Get the last current channel name
     *
     * @return long
     */
    public long GetLastCurChannel()
    {
        return vvwCmd.getChannel();
    }

    /**
     * Get the current channel name
     *
     * @return String
     */
    public String GetCurChannelName()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsVVWChannelName);

        vvwCmd.SendVvwCmd(mCmd);
        return mCmd.getArbName();
    }

    /**
     * Get a specific channel name
     *
     * @param lChannelNum long
     * @return String
     */
    public String GetChannelName(long lChannelNum)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsVVWChannelName);

        vvwCmd.SendVvwCmd(lChannelNum, mCmd);

        return mCmd.getArbName();
    }

    /**
     * Get the current channel type
     * (Currently disabled)
     *
     * @return long
     */
    public long GetCurChannelType()
    {
        //network
        //MEDIACMD mCmd = new MEDIACMD();
        //if(vvwCmd.SendVvwCmd(mCmd))
        //{
        return 0;
        //}
        //else return -1;

    }

    /**
     * Get the path for the current record folder.
     *
     * @return String
     */
    public String GetRecPath()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsRecDrive);
        mCmd.Flags( -1);

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return mCmd.getArbName();
        }
        else
        {
            return "n/a";
        }
    }

    /**
     * Set the path for the current record folder.
     *
     * @param szRecName String
     */
    public void SetRecPath(String szRecName)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsRecDrive);
        mCmd.Flags(MEDIACMD.cfUseClipID | MEDIACMD.cfUseCmdAlt);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szRecName.length() + 1);

        mCmd.setArbName(szRecName, 0, szRecName.length());
        vvwCmd.SendVvwCmd(mCmd);
    }

    /**
     * Send the Hwnd information to MEDIACMD.
     *
     * @param szMessage String
     * @param lX long
     * @param lY long
     * @param lHeight long
     * @param lWidth long
     */
    public void SendHwnd(String szMessage, long lX, long lY, long lHeight, long lWidth)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsHwnds);
        mCmd.Start(lX);
        mCmd.End(lY);
        mCmd.AudioChannels(lHeight);
        mCmd.VideoChannels(lWidth);
        mCmd.Flags(MEDIACMD.cfUseClipID | MEDIACMD.cfUseCmdAlt);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szMessage.length() + 1);

        mCmd.setArbName(szMessage, 0, szMessage.length());

        vvwCmd.SendVvwCmd(mCmd);
    }

    /**
     * Turn the preview window on or off.
     *
     * @param lValue long: 1 == On, 0 == Off.
     * @param szMessage String: Message for the window title.
     * @param lX long
     * @param lY long
     * @param lHeight long
     * @param lWidth long
     */
    public void SetMonitor(long lValue, String szMessage, long lX, long lY, long lHeight, long lWidth)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsMonitor);
        mCmd.Position(lValue);

        mCmd.Start(lX);
        mCmd.End(lY);
        mCmd.AudioChannels(lHeight);
        mCmd.VideoChannels(lWidth);
        mCmd.Flags(MEDIACMD.cfUseClipID | MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szMessage.length() + 1);

        mCmd.setArbName(szMessage, 0, szMessage.length());

        vvwCmd.SendVvwCmd(mCmd);
    }

    // ***
    // Network Specific
    // ***

    /**
     * Verifies whether or not we can connect to the given IP address.
     *
     * @param szServerAddress String
     * @return boolean
     */
    public boolean IsIPValid(String szServerAddress)
    {
        // InetAddress Class represents the IP address as an unsigned 32 bit number used by the IP
        //InetAddress m_inetAddr = null;

        try
        {
            InetAddress.getByName(szServerAddress);
        }
        catch (Exception be)
        {
            // Invalid IP
            return false;
        }

        return true;
    }

    /**
     * Attempt a connection to the server.
     *
     * @param szServerAddress String
     * @param lPort int
     * @return boolean
     */
    public boolean Connect(String szServerAddress, int lPort)
    {
        if(vvwCmd.TryConnect(szServerAddress,lPort))
        {
            // Make update frequency half of MS per Frame
            m_lUpdateFreq = (int)vidUtil.vidMsPerFrame(GetVideoStandard(), 0, 0);

            // Verify selected clip
            IfFirstVerifyClip();

            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Attempt disconnect from server.
     *
     * @param fAllInst boolean
     * @return boolean
     */
    public boolean Disconnect(boolean fAllInst)
    {
        if(vvwCmd.IsValid())
        {
            vvwCmd.Disconnect(fAllInst);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * GetStreamErrors
     *
     * @return int
     */
    public int GetStreamErrors()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }

        return vvwCmd.StreamErrors();
    }

    /**
     * GetLastStreamErrors
     *
     * @return int
     */
    public int GetLastStreamErrors()
    {
        return vvwCmd.StreamErrors();
    }

    /**
     * Returns the current clip name if this is the frist connection.
     */
    public void IfFirstVerifyClip()
    {
        if(vvwCmd.IsFirstConnect())
        {
            // This has the side effect of setting up the connection
            GetClipName();
        }
    }

    // ***
    // Transport Operations
    // ***

    /**
     * Play command
     *
     * @return long
     */
    public long Play()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags(0); // No Loop
        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Looping Play command
     *
     * @return long
     */
    public long LoopPlay()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags(MEDIACMD.cfLoop); // Loop
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Looping Play From To Command
     *
     * @param lFrom long
     * @param lTo long
     * @param fDeferred boolean
     * @return long
     */
    public long LoopPlayFromTo(long lFrom, long lTo, boolean fDeferred)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags( MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd | MEDIACMD.cfLoop);
        mCmd.Start(lFrom);
        mCmd.End(lTo);

        if(fDeferred)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfDeferred);
        }
        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play at a VVW speed specified
     *
     * @param lVVWSpeed long
     * @return long
     */
    public long PlayAtSpeed(long lVVWSpeed)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags(MEDIACMD.cfUseSpeed | MEDIACMD.cfIsShuttle);
        mCmd.Speed(lVVWSpeed);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play from a frame to a frame
     *
     * @param lFrom long
     * @param lTo long
     * @param fDeferred boolean
     * @return long
     */
    public long PlayFromTo(long lFrom, long lTo, boolean fDeferred)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags( MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd );
        mCmd.Start(lFrom);
        mCmd.End(lTo);
        if(fDeferred)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfDeferred);
        }
        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Load a clip
     *
     * @param szClipName String
     * @param lStartFrame long
     * @return long
     */
    public long LoadClip( String szClipName, long lStartFrame)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.setArbName(szClipName,0,8);
        mCmd.Cmd(MEDIACMD.ctPause);
        if(lStartFrame != -1)
        {
            mCmd.Position(lStartFrame);
        }
        else
        {
            mCmd.Position(0);
        }
        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0 ;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play a clip from a frame to a frame
     *
     * @param szClipName String
     * @param lFrom long
     * @param lTo long
     * @param fDeferred boolean
     * @return long
     */
    public long LoadClipFromTo (String szClipName, long lFrom, long lTo, boolean fDeferred)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.Cmd(MEDIACMD.ctPause);
        mCmd.Flags(MEDIACMD.cfUseClipID);
        if(lFrom != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
            mCmd.Start(lFrom);
        }
        else
        {
            mCmd.Start(-1);
        }
        if(lTo != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
            mCmd.End(lTo);
        }
        else
        {
            mCmd.End(-1);
        }
        if(fDeferred)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfDeferred);
        }
        mCmd.setArbName(szClipName,0,8);
        mCmd.arbID(mCmd.arbID());

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Load a Clip Space
     *
     * @param szFileName String
     * @return long
     */
    public long LoadClipSpace(String szFileName)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szFileName.length() + 1);
        mCmd.setArbName(szFileName, 0, szFileName.length());
        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadClipSpace);
        mCmd.Position(0);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0 ;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Load a TC Space
     *
     * @param szFileName String
     * @return long
     */
    public long LoadTcSpace(String szFileName)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szFileName.length() + 1);
        mCmd.setArbName(szFileName, 0, szFileName.length());
        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadTCSpace);
        mCmd.Position(0);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0 ;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Load a Film Space
     *
     * @param szFileName String
     * @return long
     */
    public long LoadFilmSpace(String szFileName)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szFileName.length() + 1);
        mCmd.setArbName(szFileName, 0, szFileName.length());
        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadFilmSpace);
        mCmd.Position(0);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0 ;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Get a Clip Space
     *
     * @return String
     */
    public String GetClipSpace()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadClipSpace);
        mCmd.Position(0);

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            String pName = new String(mCmd.arbID());
            if(pName == null)
            {
                return null;
            }
            else
            {
                return stripString(pName);
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Get a TC Space
     *
     * @return String
     */
    public String GetTcSpace()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadTCSpace);
        mCmd.Position(0);

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            String pName = new String(mCmd.arbID());
            if(pName == null)
            {
                return null;
            }
            else
            {
                return stripString(pName);
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Get a Film Space
     *
     * @return String
     */
    public String GetFilmSpace()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseClipID);
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsLoadFilmSpace);
        mCmd.Position(0);

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            String pName = new String(mCmd.arbID());
            if(pName == null)
            {
                return null;
            }
            else
            {
                return stripString(pName);
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Play a clip
     *
     * @param szClipName String
     * @param fDeferred boolean
     * @return long
     */
    public long PlayClip( String szClipName, boolean fDeferred)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Flags(MEDIACMD.cfUseClipID);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.setArbName(szClipName,0,8);
        mCmd.Cmd(MEDIACMD.ctPlay);
        if(fDeferred)
        {
            mCmd.Flags (mCmd.Flags() | MEDIACMD.cfDeferred);
        }
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play a clip from a frame to a frame
     *
     * @param szClipName String
     * @param lFrom long
     * @param lTo long
     * @param fDeferred boolean
     * @return long
     */
    public long PlayClipFromTo (String szClipName, long lFrom, long lTo, boolean fDeferred)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags(MEDIACMD.cfUseClipID);
        if(lFrom != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
            mCmd.Start(lFrom);
        }
        else
        {
            mCmd.Start(-1);
        }
        if(lTo != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
            mCmd.End(lTo);
        }
        else
        {
            mCmd.End(-1);
        }
        if(fDeferred)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfDeferred);
        }
        mCmd.setArbName(szClipName,0,8);
        mCmd.arbID(mCmd.arbID());

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play Fast Forward
     *
     * @return long
     */
    public long FastForward()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags(MEDIACMD.cfUseSpeed);
        mCmd.Speed( MEDIACMD.SPD_FWD_MAX);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Play Fast Reverse
     *
     * @return long
     */
    public long FastRewind()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPlay);
        mCmd.Flags( MEDIACMD.cfUseSpeed);
        mCmd.Speed( MEDIACMD.SPD_REV_MAX);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Pause displaying current frame
     *
     * @return long
     */
    public long Pause()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Seek to a frame
     *
     * @param lFrame long
     * @return long
     */
    public long Seek(long lFrame)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);
        mCmd.Flags( MEDIACMD.cfUsePosition);
        mCmd.Position(lFrame);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Seek to a frame using the absolute position
     *
     * @param lFrame long
     * @return long
     */
    public long SeekAbs(long lFrame)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);
        mCmd.Flags( MEDIACMD.cfUsePosition | MEDIACMD.cfUseFrameCount);
        mCmd.Position(lFrame);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Seek to an offset from the current position
     *
     * @param lFrameOffset long
     * @return long
     */
    public long SeekRelative( long lFrameOffset)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);

        mCmd.Flags(MEDIACMD.cfUsePositionOffset);

        mCmd.Position(lFrameOffset);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Seek to an offset from the current field
     *
     * @param lFrameOffset long
     * @return long
     */
    public long SeekRelativeField( long lFrameOffset)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);

        mCmd.Flags(MEDIACMD.cfUsePositionOffset | MEDIACMD.cfUseNextField);

        mCmd.Position(lFrameOffset);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Stop - stop play back a show input if supported, else its a pause
     *
     * @return long
     */
    public long Stop()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctStop);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Terminate - Terminate QCRun or VVWSERVICES
     *
     * @return long
     */
    public long Terminate()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctTerminate);
        mCmd.CmdAlt(0xBAADF00D);
        mCmd.Position(0xA0A00A0A);
        mCmd.Start(0xFAD0DAD0);
        mCmd.End(0xB000000F);
        mCmd.Flags(0);
        mCmd.setArbName("CloseVVWNow!", 0, 12);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Crash record
     *
     * @return long
     */
    public long Record()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctRecord);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Record from one frame to another
     *
     * @param lFrom long
     * @param lTo long
     * @return long
     */
    public long RecordFromTo(long lFrom, long lTo)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctRecord);
        mCmd.Flags(0);
        if(lFrom != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
            mCmd.Start(lFrom);
        }
        else
        {
            mCmd.Start(-1);
        }
        if(lTo != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
            mCmd.End(lTo);
        }
        else
        {
            mCmd.End(-1);
        }
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Pause at the following frame
     *
     * @param lPos long
     * @return long
     */
    public long PauseAtNextFrame(long lPos)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);
        mCmd.Flags(MEDIACMD.cfDeferred | MEDIACMD.cfUsePosition);
        mCmd.Position(lPos);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Record stop - prepare a record (clip mode only)
     *
     * @param szClipName String
     * @param lDuration long
     * @return long
     */
    public long RecordStop( String szClipName, long lDuration)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctRecStop);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.Flags(MEDIACMD.cfUseClipID);
        mCmd.setArbName(szClipName, 0,8);
        mCmd.arbID(mCmd.arbID());

        if(lDuration != -1)
        {
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
            mCmd.End(lDuration);
        }
        else
        {
            mCmd.End(-1);
        }
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Set video/audio/info 'channels' to record (preset)
     *
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @return long
     */
    public long SetRecordPresets( long lVidEdit, long lAudEdit, long lInfEdit)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctChanSelect);
        mCmd.Flags( MEDIACMD.cfUsePresets);
        mCmd.VideoChannels(lVidEdit);
        mCmd.AudioChannels(lAudEdit);
        mCmd.InfoChannels(lInfEdit);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Eject the current media
     *
     * @return long
     */
    public long Eject()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctEject);
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    // ***
    // Special Commands (lChannel must be vtr type, lTargetChannel must be internal type)
    // ***

    /**
     * Pre-Transfer media from an external VTR.
     *
     * @param szReelID String
     * @param szComment String
     * @return long
     */
    public long PreTransfer(String szReelID, String szComment) //long lTargetChannel)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();

        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.Flags(MEDIACMD.cfUsePosition | MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUseClipID);
        mCmd.CmdAlt(MEDIACMD.gsMetaData);
        mCmd.Start(0);
        mCmd.End(MEDIACMD.GS_TRUE);
        mCmd.Position(vvwInfoMetaTypes.vvwiReel);

        mCmd.setArbName(szReelID, 0, szReelID.length());
        vvwCmd.SendVvwCmd(mCmd);

        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.Flags(MEDIACMD.cfUsePosition | MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUseClipID);
        mCmd.CmdAlt(MEDIACMD.gsMetaData);
        mCmd.Start(0);
        mCmd.End(MEDIACMD.GS_TRUE);
        mCmd.Position(vvwInfoMetaTypes.vvwiComments);

        mCmd.setArbName(szComment, 0, szComment.length());

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Transfer media to or from an external VTR.
     *
     * @param lTargetChannel long
     * @param lPosition long
     * @param lStart long
     * @param lEnd long
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @param szClipName String
     * @param szFile String
     * @param fToTape boolean
     * @return long
     */
    public long Transfer(long lTargetChannel, long lPosition, long lStart, long lEnd, long lVidEdit, long lAudEdit,
                         long lInfEdit, String szClipName, String szFile, boolean fToTape)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctTransfer);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);

        if(fToTape)
        {
            mCmd.Flags(MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd |  MEDIACMD.cfUsePresets | MEDIACMD.cfInvert | MEDIACMD.cfUseFrameCount);

            mCmd.Channel(vvwCmd.getChannel());
        }
        else
        {
        	mCmd.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd |  MEDIACMD.cfUseClipID | MEDIACMD.cfUsePresets |
        			MEDIACMD.cfUseFrameCount);
        }

        mCmd.CmdAlt(lTargetChannel);
        mCmd.Position(lPosition);
        mCmd.Start(lStart);
        mCmd.End(lEnd);
        mCmd.VideoChannels(lVidEdit);
        mCmd.AudioChannels(lAudEdit);
        mCmd.InfoChannels(lInfEdit);

        mCmd.setArbName(szClipName, 0, 8);

        if(szFile != null)
        {
            mCmd.setArbName(szFile, 9, 9 + szFile.length());
            mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szFile.length() + 10);
        }

        mCmd.arbID(mCmd.arbID());

        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Current Status from the control device
     * If mCmdTemp is shared then this needs to be synchronized so no one else can interfere with it
     * If mCmdTemp is local, then not synchronize is nec, but it will be alloc/dealloc memory at every call
     *
     * @return long
     */
    public /*synchronized */long UpdateStatus()
    {
        long dwVBDiffMs = 0;
        //long dwVBDiffFrames;
        long dwVBCurrent;
        MEDIACMD mCmdTemp = new MEDIACMD();

        try
        {
            if(ddVBlankDurMs == 0.0)
            {
                // Get actual value from device
                // ddVBlankDurMs = 33.36667;
                ddVBlankDurMs = vidUtil.vidMsPerFrame(GetVideoStandard(), 0, 0);
            }
            mCmdTemp.Init();
            mCmdTemp.Cmd(MEDIACMD.ctGetState);
            mCmdTemp.Flags(MEDIACMD.cfUsePosition | MEDIACMD.cfUsePresets | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd |
                MEDIACMD.cfUseClipID);

            mCmdTemp.StructureSize(MEDIACMD.MESSAGE_FULL_SIZE); // new

            if (vvwCmd.SendVvwCmd(mCmdTemp))
            {
                mCmdState = mCmdTemp;
                // Debug out status
                //System.out.println("Status:(" + mCmdState.Cmd() + ", " + mCmdState.Flags() + "flags, " + mCmdState.Position() + "pos, " + mCmdState.Start() + "start, " + mCmdState.End() + "end, " + mCmdState.Speed() + "speed ");
                //
                dwCurrentLocalTimeMs = mCmdState.GetPnext();
                dwCurrentLocalSentTimeMs = mCmdState.GetPprev();
                dwCurrentStatusTimeMs = System.currentTimeMillis();
                dwVBCurrent = mCmdState.CmdAlt();

                if(dwVBCurrent > dwVBlankMs)
                {
                    // dwVBCurrent = current ms returned
                    // dwVBlankMs = last ms returned
                    // dwVBlankDurMs = ms in a frame

                    dwVBDiffMs = (dwVBCurrent - dwVBlankMs) % (long) ddVBlankDurMs;

                    if(dwVBDiffMs > ddVBlankDurMs / 2)
                    {
                        dwVBDiffMs = dwVBDiffMs - (long)ddVBlankDurMs;
                    } // else leave it alone
                }
                // *** Adjustment
                if(dwVBDiffMs < -3 || dwVBDiffMs > 3)
                {
                    // Problem
                    if(dwGoodVbValues > 3)
                    {
                        dwVBCurrent = dwVBlankMs +
                            (long)(((double)(dwVBCurrent - dwVBlankMs + (ddVBlankDurMs / 2))) / ddVBlankDurMs) *
                            (long) ddVBlankDurMs;

                        dwGoodVbValues = 0;
                    }
                    else
                    {
                        //System.out.println("BadVblank,not_enough_good_samples_to_compensate");
                    }
                } else
                {
                    // Good value
                    dwGoodVbValues ++;
                }
                dwVBlankMs = dwVBCurrent;
                dwLastLocalVBlankTimeMs = dwVBlankMs;
                dwVblankJava = dwCurrentLocalTimeMs - (dwCurrentLocalTimeMs - dwLastLocalVBlankTimeMs);

                // This should not be here m_dwTcType = GetTCType();
                return 0;
            }
            else
            {
                // System.out.println("*** UpdateStatus() Failed! Calling mCmdState.InitFailure()***");
                mCmdState.InitFailure();
                return -1;
            }
        }
        catch (Exception e)
        {
           // System.out.println("\n\rIO Exception: " + e.getMessage());
            vvwCmd.Disconnect(true);
            return -1;
        }
    }

    /**
     * Returns MetaData DWORD
     * For Conform (aka VTR) mode only
     * As no clip name is specified, always returns current media
     * So it can also be used for Clip mode by giving lStartPosVTR = -1
     *
     * @param lMetaDataItem long
     * @param lStartPosVTR long
     * @return long
     */
    public synchronized long GetMetaDataDWORD(long lMetaDataItem, long lStartPosVTR)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsMetaData);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart);
        mCmd.Position(lMetaDataItem);
        mCmd.Start(lStartPosVTR); // For VTR Edits only
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            return mCmd.Start();
        }
        else
        {
            return -1;
        }
    }

    /**
     * CheckMetaDataDWORD
     * For Clip (aka Server) mode only
     *
     * @param lMetaDataItem long
     * @param szClipName String
     * @return long
     */
    public synchronized long CheckMetaDataDWORD(long lMetaDataItem, String szClipName)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.CmdAlt(MEDIACMD.gsMetaData);
        mCmd.Flags(MEDIACMD.cfUseClipID | MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart);
        mCmd.Position(lMetaDataItem);
        mCmd.Start(0);
        mCmd.setArbName(szClipName, 0, 8);
        mCmd.arbID(mCmd.arbID());

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return mCmd.Start();
        }
        else
        {
            return -1;
        }
    }

    /**
     * GetMetaDataSTRING
     *
     * @param lMetaDataItem long
     * @param lStartPosVTR long
     * @return String
     */
    public synchronized String GetMetaDataSTRING(long lMetaDataItem, long lStartPosVTR)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsMetaData);
        mCmd.Flags(MEDIACMD.cfUseStart | MEDIACMD.cfUseCmdAlt);
        mCmd.Position(lMetaDataItem);
        mCmd.Start(lStartPosVTR); // For VTR Edits only
        if (vvwCmd.SendVvwCmd(mCmd))
        {
            String pName = new String(mCmd.arbID());
            if(pName == null)
            {
                return null;
            }
            else
            {
                return stripString(pName);
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Strips invalid characters from the end of a string.
     *
     * @param szInput String
     * @return String
     */
    public String stripString(String szInput)
    {
        StringBuffer szOutput = new StringBuffer();
        char c;

        for (int i = 0; i <= szInput.length()-1; i++)
        {
            c = szInput.charAt(i);
            if(c < ' ')
            {
                return szOutput.toString();
            }
            else
            {
                szOutput.append(c);
            }
        }
        return szOutput.toString();
    }

    /**
     * Gets the current state from the control device.
     *
     * @return long
     */
    public long GetState()
    {
        long cmd = mCmdState.Cmd();
        if (cmd == -1 || dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
            cmd = mCmdState.Cmd();
        }
        return cmd;
    }

    /**
     * Returns the last state from the control device
     *
     * @return long
     */
    public long GetLastState()
    {
        return mCmdState.Cmd();
    }

    /**
     * Current Flags
     *
     * @return long
     */
    public long GetFlags()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return mCmdState.Flags();
    }

    /**
     * Last Flags
     *
     * @return long
     */
    public long GetLastFlags()
    {
        return mCmdState.Flags();
    }

    /**
     * Current Speed
     *
     * @return long
     */
    public long GetSpeed()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return mCmdState.Speed();
    }

    /**
     * Last Speed
     *
     * @return long
     */
    public long GetLastSpeed()
   {
       return mCmdState.Speed();
   }

    /**
     * Current Position
     *
     * @return long
     */
    public long GetPosition()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return mCmdState.Position();
    }

    /**
     * Last Position
     *
     * @return long
     */
    public long GetLastPosition()
    {
        return mCmdState.Position();
    }

    /**
     * Returns the current Absolute Position
     *
     * @return long
     */
    public long GetAbsolutePosition()
    {
        long dwPosition;
        if(dwCurrentAbsoluteTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            MEDIACMD mCmd = new MEDIACMD();
            mCmd.Init();
            mCmd.Cmd(MEDIACMD.ctGetState);
            mCmd.Flags(MEDIACMD.cfUseFrameCount | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd | MEDIACMD.cfUsePosition | MEDIACMD.cfUseSpeed);
            if(!(vvwCmd.SendVvwCmd(mCmd)))
            {
                return -1;
            }
            dwPosition = mCmd.Position();
            if(dwPosition != MEDIACMD.GS_NOT_SUPPORTED)
            {
                dwAbsolutePosition = dwPosition;
                dwCurrentAbsoluteTimeMs = System.currentTimeMillis();
            }
        } else
        {
            System.out.print("skip");
        }
        return dwAbsolutePosition;
    }

    /**
     * Get the current vertical blank ms
     *
     * @return long
     */
    public long GetCurVBlankMs()
    {
        UpdateStatus();
        return mCmdState.CmdAlt();
    }

    /**
     * Get the last change in ms
     *
     * @return long
     */
    public long GetLastVBlankMs()
    {
        return dwVBlankMs;//mCmdState.CmdAlt();
    }

    /**
     * Returns the current Start value.
     *
     * @return long
     */
    public long GetStart()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return mCmdState.Start();
    }

    /**
     *  Returns the last Start value.
     *
     * @return long
     */
    public long GetLastStart()
    {
        return mCmdState.Start();
    }

    /**
     * Returns the current End
     *
     * @return long
     */
    public long GetEnd()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return mCmdState.End();
    }

    /**
     * Returns the last End
     *
     * @return long
     */
    public long GetLastEnd()
    {
        return mCmdState.End();
    }

    /**
     * Current Clip Name
     *
     * @return String
     */
    public String GetClipName()
    {
        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return (mCmdState.getArbName(0,8));
    }

    /**
     * Last Clip Name
     *
     * @return String
     */
    public String GetLastClipName()
    {
        return (mCmdState.getArbName(0, 8));
    }

    /**
     * Current File Name
     *
     * @return String
     */
    public String GetFileName( )
    {
        if(dwCurrentLocalTimeMs + m_lUpdateFreq < GetLastCurMs())
        {
            UpdateStatus();
        }

        int index = 0;
        String _PathName = "";
        //String _dir = "";
        String _fileName = "";

        _PathName = mCmdState.getArbName(9,(int)(mCmdState.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));
        index = _PathName.lastIndexOf("\\");

        if(index > 0)
        {
            //_dir = _PathName.substring(0,index+1).trim();
            _fileName = (_PathName.substring(index+1, _PathName.length())).trim();
        }
        else
        {
            //_dir = _PathName.trim();
            _fileName = "<Internal>";
        }

        return (_fileName);
    }

    /**
     * Returns the Primary time code string in the Previously set format
     *
     * @param dwPosition long
     * @return String
     */
    public String FrameToTc(long dwPosition)
    {
        char[] szTcOut = new char[TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";

        if(m_dwTcType == 0 || m_dwTcType == -1 || m_dwTcTypeLastCheckMs + 1500 < System.currentTimeMillis()) {
            m_dwTcTypeLastCheckMs = System.currentTimeMillis();
            m_dwTcType = GetTCType();
        }

        if ( (dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int) m_dwTcType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS, szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut, 0, 11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * Returns the Primary time code string in the Previously set format
     *
     * @return String
     */
    public String GetPrimaryTc()
    {
        long dwPosition = 0;
        char[] szTcOut = new char[TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";

        if(m_dwTcType == 0 || m_dwTcType == -1 || m_dwTcTypeLastCheckMs + 1500 < System.currentTimeMillis())
        {
            m_dwTcTypeLastCheckMs = System.currentTimeMillis();
            m_dwTcType = GetTCType();
        }

        dwPosition = GetCur();
        if((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int) m_dwTcType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS, szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut, 0, 11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * Returns the current time code string in the format "00:00:00:00"
     *
     * @return String
     */
    public String GetCurTC()
    {
        long dwPosition = 0;
        char[] szTcOut = new char [TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";

        if(dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
           if(UpdateStatus() == -1)
           {
               szTimeCodeString = "88:88:88:88";
               return szTimeCodeString;
           }
        }

        if(m_dwTcType == 0 || m_dwTcType == -1 || m_dwTcTypeLastCheckMs + 1500 < System.currentTimeMillis())
        {
            m_dwTcTypeLastCheckMs = System.currentTimeMillis();
            m_dwTcType = GetTCType();
        }
        dwPosition = GetCur();
        if((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED || mCmdState.Cmd() == MEDIACMD.ctError)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int)m_dwTcType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS, szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut,0,11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * Returns the current time code as a numerical value.
     *
     * @return long
     */
    public long GetCur()
    {
        if(dwCurrentStatusTimeMs + (m_lUpdateFreq / 2) < System.currentTimeMillis())
        {
           if(UpdateStatus() == -1)
           {
               return TC_ILLEGAL;
           }
        }
        return mCmdState.Position();
    }

    /**
     * Returns the current time code string in LTC format
     *
     * @return String
     */
    public String GetLtcTc()
    {
        long dwPosition = 0;
        char[] szTcOut = new char[TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";

        // Need call to get LTC/VITC tc type, or return it with valueGet
        if(m_dwLTCType == 0 || m_dwLTCType == -1)
        {
            m_dwLTCType = GetTCType();
        }
        dwPosition = ValueGet(gsLtcTc);
        if((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int) m_dwLTCType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS,
                szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut, 0, 11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * Returns LTC value.
     *
     * @return long
     */
    public long GetLTC()
    {
        return ValueGet(MEDIACMD.gsLtcTc);
    }

    /**
     * Returns LTC type.
     *
     * @return long
     */
    public long GetLTCTcType()
    {
        m_dwLTCType = GetTCType();
        return m_dwLTCType;
    }

    /**
     * Returns the current time code string in VITC format
     *
     * @return String
     */
    public String GetVitcTc()
    {
        long dwPosition = 0;
        char[] szTcOut = new char[TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";

        // Need call to get LTC/VITC tc type, or return it with valueGet
        if(m_dwVITCType == 0 || m_dwVITCType == -1)
        {
            m_dwVITCType = GetTCType();
        }

        dwPosition = ValueGet(MEDIACMD.gsVitcTc);
        if((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int) m_dwTcType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS,
                szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut, 0, 11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * Returns VITC value
     *
     * @return long
     */
    public long GetVITC()
    {
        return ValueGet(MEDIACMD.gsVitcTc);
    }

    /**
     * Returns VITC type.
     *
     * @return long
     */
    public long GetVITCTcType()
    {
        m_dwVITCType = GetTCType();
        return m_dwVITCType;
    }

    /**
     * GetInkCode
     *
     * @return String
     */
    public String GetInkCode()
    {
        StringBuffer szCode = new StringBuffer();
        GetValueMcmd mCmdValues = new GetValueMcmd(MEDIACMD.gsInkCode, 0, 0);
        long i, position;

        position = ValueGet(mCmdValues);

        for(i = 12; i >= 0; i -= 4)
        {
            szCode.append((char) ('0' + ((mCmdValues.plMin >> i) & 0x0000000F)));
        }

        for(i = 20; i >= 0; i -= 4)
        {
            szCode.append((char) ('0' + ((position >> i) & 0x0000000F)));
        }

        return szCode.toString();
    }

    /**
     * GetKeyCode
     *
     * @return String
     */
    public String GetKeyCode()
    {
        StringBuffer szCode = new StringBuffer();
        GetValueMcmd mCmdValues = new GetValueMcmd(MEDIACMD.gsKeyCode, 0, 0);
        long i, position;

        position = ValueGet(mCmdValues);

        for(i = 20; i >= 0; i -= 4)
        {
            szCode.append((char) ('0' + ((mCmdValues.plMin >> i) & 0x0000000F)));
        }

        for(i = 20; i >= 0; i -= 4)
        {
            szCode.append((char) ('0' + ((position >> i) & 0x0000000F)));
        }

        return szCode.toString();
    }

    /**
     * Returns the current time code string in TC format
     *
     * @return String
     */
    public String GetTCTc()
    {
        long dwPosition = 0;
        char[] szTcOut = new char[TCXlat.TC2_MIN_STRING_SIZE];
        String szTimeCodeString = "";
        // Need call to get LTC/VITC tc type, or return it with valueGet
        if(m_dwTCTCType == 0 || m_dwTCTCType == -1)
        {
            m_dwTCTCType = GetTCType();
        }

        dwPosition = ValueGet(MEDIACMD.gsTc);
        if((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szTimeCodeString = "88:88:88:88";
        }
        else
        {
            tcxObj.tc2String(dwPosition, (int)m_dwTCTCType | TCXlat.TC2_STREXT_LEADING | TCXlat.TC2_STREXT_MARKS, szTcOut);
            szTimeCodeString = String.copyValueOf(szTcOut,0,11).trim();
        }
        return szTimeCodeString;
    }

    /**
     * GetTC
     *
     * @return long
     */
    public long GetTC()
    {
        return ValueGet(MEDIACMD.gsTc);
    }

    /**
     * GetTCTcType
     *
     * @return long
     */
    public long GetTCTcType()
    {
        m_dwTCTCType = GetTCType();
        return m_dwTCTCType;
    }

    /**
     * GetTCTcTypes
     *
     * @return long
     */
    public long GetTCTcTypes()
    {
        // FIXME  these should be seperate
        m_dwTcType = GetTCType();
        m_dwTCTCType = m_dwTcType;//GetTCType();
        m_dwLTCType = m_dwTcType;//GetTCType();
        m_dwVITCType = m_dwTcType;//GetTCType();
        return m_dwTcType;
    }

    /**
     * Returns the User Bits value as a string
     *
     * @param lTcSource long
     * @return String
     */
    public String GetUBString(long lTcSource)
    {
        long dwPosition;
        String szUBString = "";

        dwPosition = mCmdState.Position();
        if ((dwPosition == MEDIACMD.TC_ILLEGAL) || dwPosition == MEDIACMD.GS_NOT_SUPPORTED)
        {
            szUBString = "88888888";
        }
        else
        {
            if(lTcSource == MEDIACMD.GS_TCSOURCE_CTL || lTcSource == MEDIACMD.GS_TCSOURCE_TC)
            {
                dwPosition = ValueGet(MEDIACMD.gsUb);
            }
            else if(lTcSource == MEDIACMD.GS_TCSOURCE_LTC)
            {
                dwPosition = ValueGet(MEDIACMD.gsLtcUb);
            }
            else if(lTcSource == MEDIACMD.GS_TCSOURCE_VITC)
            {
                dwPosition = ValueGet(MEDIACMD.gsVitcUb);
            }

            szUBString = ub2String(dwPosition);
        }
        return szUBString;
    }

    /**
     * Convert the numeric userbits in dwUbIn to the string
     *
     * @param dwUbIn long
     * @return String
     */
    public String ub2String(long dwUbIn)
    {
        int i;
        String szUbOut = "00000000";
        char[] charUbOut = szUbOut.toCharArray();

        // Need to handle leading, bcd, invert and marks
        for (i = 0; i < 8; i++)
        {
            if ( ( (dwUbIn >> ( (7 - i) * 4)) & 0x0F) < 10)
            {
                // 0 - 9
                charUbOut[i] = (char) ( (byte) ( (dwUbIn >> ( (7 - i) * 4)) & 0x0F) + '0');
            }
            else
            {
                // A - F
                charUbOut[i] = (char) ( (byte) ( (dwUbIn >> ( (7 - i) * 4)) & 0x0F) + ('A' - 10));
            }
        }
        szUbOut = new String(charUbOut);

        return szUbOut;
    }

    /**
     * Returns the current state as a String
     *
     * @return String
     */
    public String GetCurState()
    {
        if (dwCurrentStatusTimeMs + m_lUpdateFreq < System.currentTimeMillis())
        {
            UpdateStatus();
        }
        return GetLastUpdatedState();
    }

    /**
     * Retuns the state based on the last update
     *
     * @return String
     */
    public String GetLastUpdatedState()
    {
        String sz14ByteState;
        long lSpeed = mCmdState.Speed();

        switch((int)mCmdState.Cmd())
        {
            case 0:				// Stop all action
                sz14ByteState = "Stop";
                break;
            case 1:			// Pause, Seek
                sz14ByteState = "Pause";
                break;
            case 2:				// Play at specified speed (includes pause)
                if((mCmdState.Flags() & MEDIACMD.cfUseSpeed) != MEDIACMD.cfUseSpeed)
                {
                    sz14ByteState = "Play";
                }
                else
                {
                    if (lSpeed == MEDIACMD.SPD_FWD_PLAY)
                    {
                        sz14ByteState = "Play";
                    }
                    else if (lSpeed == MEDIACMD.SPD_REV_PLAY)
                    {
                        sz14ByteState = "Rev Play";
                    }
                    else if (lSpeed == MEDIACMD.SPD_FWD_MAX)
                    {
                        sz14ByteState = "FastForward";
                    }
                    else if (lSpeed == MEDIACMD.SPD_REV_MAX)
                    {
                        sz14ByteState = "FastRewind";
                    }
                    else
                    {
                        sz14ByteState = "Shuttle " + Math.round(lSpeed * 100.0 / 65520.0);
                    }
                }
                break;
            case 3: // Record at specified speed
                sz14ByteState= "Record";
                break;
            case 4:			// Stop ready for recording
                sz14ByteState= "RecordStop";
                break;
            case 5:			// Eject the current media
                sz14ByteState="Eject";
                break;
            case 6:			// Transfer source from one channel to another
                sz14ByteState= "Transfer";
                break;
            case 7:			// Insert a new time code area
                sz14ByteState= "Insert";
                break;
            case 8:			// Erase the specified TC area
                sz14ByteState= "Blank";
                break;
            case 9:			// Slide segment within the specified TC area
                sz14ByteState= "Delete";
                break;
            case 10:				// Trim the segment within the specified TC area
                sz14ByteState= "Trim";
                break;
            case 11:		// Pass though requested channels
                sz14ByteState= "ChanSelect";
                break;
            case 12:			// Returns TC and transport state information
                sz14ByteState= "GetState";
                break;
            case 13:			// Sends a new state per GetState
                sz14ByteState= "SetState";
                break;
            case 14:			// Get value of video, audio of internal variable
                sz14ByteState= "GetValue";
                break;
            case 15:			// Set value of video, audio of internal variable
                sz14ByteState= "SetValue";
                break;
            case 16:	// Returns true is the specified Get/Set value is supported
                sz14ByteState= "ValueSupport";
                break;
            case 17:			// An error has occurred
                sz14ByteState= "Error";
                break;
            case 18:		// Terminate the current command and move to the next in the queue, if any
                sz14ByteState= "Terminate";
                break;
            case 19:				// Abort any queued commands
                sz14ByteState= "Abort";
                break;
            default:
                sz14ByteState= "Unknown";
            break;
        }
        return sz14ByteState;
    }

    // ***
    // Media Operation
    // ***

    // *** Clip Mode ***

    /**
     * Get the next clip in the clip list, send a null string for first clip
     *
     * @param szLastClip String
     * @return String
     */
    public synchronized String GetNextClip( String szLastClip)
    {
        /*
        if(szLastClip.compareTo("") == 0)
        {
            System.out.println("Get first clip, Time = " + System.currentTimeMillis());
        }
        */
        mCmdClip.Init();
        mCmdClip.Cmd(ctGetValue);
        mCmdClip.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmdClip.CmdAlt(MEDIACMD.gsGetNextClip);
        mCmdClip.Flags(-1);

        mCmdClip.setArbName(szLastClip,0,8);

        if(vvwCmd.SendVvwCmd(mCmdClip))
        {
            if(mCmdClip.Position() == MEDIACMD.GS_NOT_SUPPORTED)
            {
                return null;
            }
            else
            {
                return(mCmdClip.getArbName(0, 8));
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Gets the instance variables of the ClipInfo class.
     * Note: a ClipInfo object must be constructed with the clip id in which to get the info
     *
     * @param clipData ClipInfo
     * @return long
     */
    public long GetClipInfo (ClipInfo clipData)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.CmdAlt(MEDIACMD.gsClipInfo);
        mCmd.Flags(-1);
        mCmd.setArbName(clipData.szClipName,0,8);
        mCmd.StructureSize(MEDIACMD.MESSAGE_FULL_SIZE); // new

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            clipData.szClipName = mCmd.getArbName(0,8);
            clipData.lStart = mCmd.Start();
            clipData.lEnd = mCmd.End();
            clipData.lFileOffset = mCmd.Speed();
            clipData.lVidEdit = mCmd.VideoChannels();
            clipData.lAudEdit = mCmd.AudioChannels();
            clipData.lInfEdit = mCmd.InfoChannels();
            clipData.szFileName = mCmd.getArbName(9, (int)(mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Not implemented just yet. Will use the ExClipInfo class to return the values
     *
     * @return long
     */
    public long GetNextClipEx ( )
    {
        //MEDIACMD mCmd = new MEDIACMD();
        //if(vvwCmd.SendVvwCmd (mCmd))
        //	return 0;
        //else
        return -1;
    }

    /**
     * Copy a clip in clip space
     *
     * @param szSourceClip String
     * @param szDestClip String
     * @param lStart long
     * @param lEnd long
     * @param lPos long
     * @return long
     */
    public long CopyClip (String szSourceClip, String szDestClip, long lStart, long lEnd, long lPos)
    {
        if (szSourceClip == null | szDestClip == null)
        {
            return -1;
        }
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctSetValue);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1 + 8 + 1);
        mCmd.Flags( MEDIACMD.cfUseClipID);
        mCmd.CmdAlt(MEDIACMD.gsClipCopy);
        mCmd.setArbName(szSourceClip, 0, 8);
        mCmd.setArbName(szDestClip, 9, 17);
        mCmd.arbID(mCmd.arbID());

        if(lStart != -1)
        {
            mCmd.Start(lStart);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
        }
        if(lEnd != -1)
        {
            mCmd.End(lEnd);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
        }
        if(lPos != -1)
        {
            mCmd.Position(lPos);
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfUsePosition);
        }

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Sets the instance variables of the ClipInfo class.
     * Note: a ClipInfo object must be constructed with the clip id in which to get the info
     *
     * @param szSourceClip String
     * @param szDestClip String
     * @param lStart long
     * @param lEnd long
     * @return long
     */
    public long SetClipInfo(String szSourceClip, String szDestClip, long lStart, long lEnd)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1 + 8 + 1);
        mCmd.CmdAlt(MEDIACMD.gsClipInfo);
        mCmd.Flags(-1);
        mCmd.StructureSize(MEDIACMD.MESSAGE_FULL_SIZE);

        mCmd.Start(lStart);
        mCmd.End(lEnd);
        mCmd.VideoChannels( -1);
        mCmd.AudioChannels( -1);
        mCmd.InfoChannels( -1);
        mCmd.setArbName(szSourceClip, 0, 8);
        mCmd.setArbName(szDestClip, 9, 17);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    // ***
    // VTR Mode Operations
    // ***

    /**
     * Reset the edl in VTR mode to thr first element of the list
     *
     * @return long
     */
    public long EDLResetToStart()
    {
        mCmdEDL = new MEDIACMD();
        mCmdEDL.Init();
        mCmdEDL.Cmd(MEDIACMD.ctGetValue);
        mCmdEDL.Flags(MEDIACMD.cfUsePresets | MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd | MEDIACMD.cfUseClipID);
        mCmdEDL.CmdAlt(MEDIACMD.gsTCSGetTLClipState);
        mCmdEDL.VideoChannels(0);
        mCmdEDL.AudioChannels(0);
        mCmdEDL.InfoChannels(0);
        mCmdEDL.Position(0);
        mCmdEDL.Start(0);
        mCmdEDL.End(-1);
        return 0;
    }

    /**
     * Return the current line in VTR mode
     *
     * @param editInfo VTREditLine
     * @param lPosition long
     * @param firstClip boolean
     * @return long
     */
    public long EDLGetEdit (VTREditLine editInfo, long lPosition, boolean firstClip)
    {
        MEDIACMD mCmd = new MEDIACMD(); // Local copy of MEDIACMD

        if (firstClip) // If requesting the first clip in the list
        {
            mCmd.Init();
            mCmd.Cmd(ctGetValue);
            mCmd.CmdAlt(MEDIACMD.gsTCSGetTLClipInfo);
            mCmd.Flags(-1);
            mCmd.Position(0);
        }
        else
        {
            mCmd.Init();
            mCmd.Cmd(MEDIACMD.ctGetValue);
            mCmd.Flags(MEDIACMD.cfUsePresets | MEDIACMD.cfUsePosition | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd | MEDIACMD.cfUseClipID);
            mCmd.CmdAlt(MEDIACMD.gsTCSGetTLClipState);

            mCmd.Position(lPosition);
            mCmd.Start(0);
            mCmd.End(0);
            mCmd.VideoChannels(0);
            mCmd.AudioChannels(0);
            mCmd.InfoChannels(0);
        }
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 256);

        if(vvwCmd.SendVvwCmd(mCmd)) // Get Clip info to mCmd
        {
            if(mCmd.Position() == MEDIACMD.GS_NOT_SUPPORTED ||
                mCmd.Start() == MEDIACMD.GS_NOT_SUPPORTED ||
                mCmd.End() == MEDIACMD.GS_NOT_SUPPORTED)
            {
                // Good data was not returned
                return -1;
            }

            editInfo.lRecordIn = mCmd.Position();
            editInfo.lPlayIn = mCmd.Start();
            editInfo.lPlayOut = mCmd.End();
            editInfo.lVidEdit = mCmd.VideoChannels();
            editInfo.lAudEdit = mCmd.AudioChannels();
            editInfo.lInfEdit = mCmd.InfoChannels();
            editInfo.szClipName = mCmd.getArbName(0, (int) (mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));
            editInfo.szFileName = mCmd.getArbName(9, (int) (mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));

            int index = editInfo.szFileName.lastIndexOf("\\");

            if(index > 0)
            {
                editInfo.szFileName = (editInfo.szFileName.substring(index + 1, editInfo.szFileName.length())).trim();
            }
            else
            {
                editInfo.szFileName = "<Internal>";
            }

            mCmdEDL.Cmd(MEDIACMD.ctGetValue);
            mCmdEDL.CmdAlt(MEDIACMD.gsTCSGetTLClipState);

            if(vvwCmd.SendVvwCmd(mCmdEDL)) // Get clip state to mCmdEDL
            {
                if(mCmdEDL.Position() == -1 || mCmdEDL.Start() == -1)
                {
                    return -1; // error
                }
                else
                {
                    return 0; // good data
                }
            }
            else // Cannot get clip state to mCmdEDL
            {
                editInfo = null;
                return -1;
            }
        }
        else // Cannot get clip info to mCmd
        {
            editInfo = null;
            return -1;
        }
    }

    /**
     * Seek in the edl in VTR mode to the selected element of the list
     *
     * @param dwSeekToPosition long
     * @return long
     */
    public long EDLSeek(long dwSeekToPosition)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctPause);
        mCmd.Position(dwSeekToPosition);
        mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePosition + MEDIACMD.cfUseFrameCount);

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Return the current line in VTR mode
     *
     * @param editInfo VTREditLine
     * @return long
     */
    public long EDLGetNextEdit(VTREditLine editInfo)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.CopyMCMD(mCmdEDL);
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.CmdAlt(MEDIACMD.gsTCSGetTLClipInfo);

        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 256); // Added by Brett

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            if (mCmd.Position() == MEDIACMD.GS_NOT_SUPPORTED ||
                mCmd.Start() == MEDIACMD.GS_NOT_SUPPORTED ||
                mCmd.End() == MEDIACMD.GS_NOT_SUPPORTED)
            {
                return -1;
            }
            editInfo.lRecordIn = mCmd.Position();
            editInfo.lPlayIn = mCmd.Start();
            editInfo.lPlayOut = mCmd.End();
            editInfo.lVidEdit = mCmd.VideoChannels();
            editInfo.lAudEdit = mCmd.AudioChannels();
            editInfo.lInfEdit = mCmd.InfoChannels();
            editInfo.szClipName = mCmd.getArbName(0, (int) (mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));
            editInfo.szFileName = mCmd.getArbName(9, (int) (mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));
            int index = editInfo.szFileName.lastIndexOf("\\");
            if(index > 0)
            {
                editInfo.szFileName = (editInfo.szFileName.substring(index+1, editInfo.szFileName.length())).trim();
            }
            else
            {
                editInfo.szFileName = "<Internal>";
            }
            mCmdEDL.Cmd(MEDIACMD.ctGetValue);
            mCmdEDL.CmdAlt(MEDIACMD.gsTCSGetTLClipState);
            mCmdEDL.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 256); // Added by Brett
            if(vvwCmd.SendVvwCmd(mCmdEDL))
                if(mCmdEDL.Position() == MEDIACMD.GS_NOT_SUPPORTED ||
                   mCmdEDL.Start() == MEDIACMD.GS_NOT_SUPPORTED)
                {
                    return -1;
                }
                else
                {
                    return 0;
                }
            else
            {
                editInfo = null;
                return -1;
            }
        }
        else
        {
            editInfo = null;
            return -1;
        }
    }


    // ***
    // Shared
    // ***

    /**
     * Gets the last system change VTR or Clip
     *
     * @return long
     */
    public synchronized long GetLastChangeMs()
    {
        mCmdLastChange.Init();
        mCmdLastChange.Cmd(ctGetValue);
        mCmdLastChange.CmdAlt(MEDIACMD.gsLastChangeMs);
        mCmdLastChange.Flags( -1);
        if(vvwCmd.SendVvwCmd(mCmdLastChange))
        {
            return mCmdLastChange.Position();
        }
        else
        {
            return -1;
        }
    }

    /**
     * Not to be used yet
     *
     * @param szClipName String
     * @param szFileName String
     * @param lPosition long
     * @param lStart long
     * @param lEnd long
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @param fRipple boolean
     * @return long
     */
    public long Insert (String szClipName, String szFileName, long lPosition, long lStart, long lEnd, long lVidEdit, long lAudEdit, long lInfEdit, boolean fRipple)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctInsert);
        mCmd.Flags( MEDIACMD.cfUseClipID);
        mCmd.setArbName(szClipName, 0, 8);
        mCmd.setArbName(szFileName, 9, (int)(mCmd.StructureSize() - MEDIACMD.MEDIACMD_BASE_SIZE - 8));

        if(lPosition != -1)
        {
            mCmd.Position(lPosition);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePosition);
        }
        if(lStart != -1)
        {
            mCmd.Start(lStart);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
        }
        if(lEnd != -1)
        {
            mCmd.End( lEnd );
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
        }
        if(lVidEdit != -1)
        {
            mCmd.VideoChannels( lVidEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lAudEdit != -1)
        {
            mCmd.AudioChannels( lAudEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lInfEdit != -1)
        {
            mCmd.InfoChannels( lInfEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(fRipple)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfRipple);
        }

        System.out.println("Inserting: szFileName = " + szFileName + ", lPosition = " + lPosition + ", lStart = " + lStart +
            ", lEnd = " + lEnd + ", Flags() = " + mCmd.Flags());

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Blanks a portion of TC space
     *
     * @param szClipName String
     * @param lStart long
     * @param lEnd long
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @param fRipple boolean
     * @return long
     */
    public long Blank (String szClipName, long lStart, long lEnd, long lVidEdit, long lAudEdit, long lInfEdit, boolean fRipple)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctBlank);
        mCmd.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.Flags( MEDIACMD.cfUseClipID);
        mCmd.setArbName(szClipName, 0,8);
        mCmd.arbID(mCmd.arbID());

        if(lStart != -1)
        {
            mCmd.Start(lStart);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
        }
        if(lEnd != -1)
        {
            mCmd.End( lEnd);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
        }
        if(lVidEdit != -1)
        {
            mCmd.VideoChannels( lVidEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lAudEdit != -1)
        {
            mCmd.AudioChannels( lAudEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lInfEdit != -1)
        {
            mCmd.InfoChannels( lInfEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(fRipple)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfRipple);
        }
        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Deletes the specified clip.
     *
     * @param szClipName String
     * @param lStart long
     * @param lEnd long
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @param fRipple boolean
     * @return long
     */
    public long Delete (String szClipName, long lStart, long lEnd, long lVidEdit, long lAudEdit, long lInfEdit, boolean fRipple)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctDelete);
        mCmd.StructureSize(MediaCmdIF.MEDIACMD_BASE_SIZE + 8 + 1);
        mCmd.Flags( MEDIACMD.cfUseClipID);
        mCmd.setArbName(szClipName, 0,8);
        mCmd.arbID(mCmd.arbID());

        if(lStart != -1)
        {
            mCmd.Start(lStart);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStart);
        }
        if(lEnd != -1)
        {
            mCmd.End( lEnd);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEnd);
        }
        if(lVidEdit != -1)
        {
            mCmd.VideoChannels( lVidEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lAudEdit != -1)
        {
            mCmd.AudioChannels( lAudEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lInfEdit != -1)
        {
            mCmd.InfoChannels( lInfEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(fRipple)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfRipple);
        }
        if (vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Trim
     *
     * @param lPosition long
     * @param lStartOffset long
     * @param lEndOffset long
     * @param lVidEdit long
     * @param lAudEdit long
     * @param lInfEdit long
     * @param fRipple boolean
     * @return long
     */
    public long Trim (long lPosition, long lStartOffset, long lEndOffset, long lVidEdit, long lAudEdit, long lInfEdit, boolean fRipple)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctInsert);
        mCmd.Flags( 0);
        if(lPosition != -1)
        {
            mCmd.Position( lPosition);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePosition);
        }
        if(lStartOffset != -1)
        {
            mCmd.Start(lStartOffset);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseStartOffset);
        }
        if(lEndOffset != -1)
        {
            mCmd.End( lEndOffset);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUseEndOffset);
        }
        if(lVidEdit != -1)
        {
            mCmd.VideoChannels( lVidEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lAudEdit != -1)
        {
            mCmd.AudioChannels( lAudEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(lInfEdit != -1)
        {
            mCmd.InfoChannels( lInfEdit);
            mCmd.Flags( mCmd.Flags() | MEDIACMD.cfUsePresets);
        }
        if(fRipple)
        {
            mCmd.Flags(mCmd.Flags() | MEDIACMD.cfRipple);
        }
        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    // ***
    // Settings
    // ***

    /**
     * Returns the supported attributes of a get/set value
     *
     * @param lValueType long
     * @return long
     */
    public long ValueSupported(long lValueType)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctValueSupported);
        mCmd.Flags( MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(lValueType);
        if(!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        if( mCmd.Position() != 0 && mCmd.Position() != MEDIACMD.GS_NOT_SUPPORTED)
        {
            return 0;
        }
        return -1;
    }

    /**
     * Returns the current setting for a get/set value
     *
     * @param lValueType long
     * @return long
     */
    public long ValueGet(long lValueType)
    {
        long lRtn;
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(lValueType);
        mCmd.Flags(-1);

        if (!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        lRtn = mCmd.Position();
        if( lRtn == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return -1;
        }
        return lRtn;
    }

    /**
     * Gets the specified value
     *
     * @param lValueType long
     * @param vidChannels long
     * @param audChannels long
     * @param infChannels long
     * @return long
     */
    public long ValueGet(long lValueType, long vidChannels, long audChannels, long infChannels)
    {
        long lRtn;
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(lValueType);

        mCmd.Flags(MEDIACMD.cfUsePosition |MEDIACMD.cfUsePresets);
        mCmd.VideoChannels(vidChannels);
        mCmd.AudioChannels(audChannels);
        mCmd.InfoChannels(infChannels);
        if(!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        lRtn = mCmd.Position();

        if(lRtn == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return -1;
        }
        return lRtn;
    }

    /**
     * Gets the specified value.
     *
     * @param lValueType long
     * @param mCmdValues GetValueMcmd
     * @param vidChannels long
     * @param audChannels long
     * @param infChannels long
     * @return long
     */
    public long ValueGet(long lValueType, GetValueMcmd mCmdValues, long vidChannels, long audChannels, long infChannels)
    {
        long lRtn;
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(lValueType);
        mCmd.Flags(MEDIACMD.cfUsePosition |MEDIACMD.cfUsePresets |MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUseStart | MEDIACMD.cfUseEnd);
        mCmd.VideoChannels(vidChannels);
        mCmd.AudioChannels(audChannels);
        mCmd.InfoChannels(infChannels);
        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return MEDIACMD.GS_NOT_SUPPORTED;
        }
        lRtn = mCmd.Position();

        if( lRtn == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return MEDIACMD.GS_NOT_SUPPORTED;
        }

        mCmdValues.plMin = mCmd.Start();
        mCmdValues.plMax = mCmd.End();

        return lRtn;
    }

    /**
     * Get the specified value
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long ValueGet(GetValueMcmd mCmdValues)
    {
        long lRtn;
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(mCmdValues.lValueType);
        mCmd.Flags( -1);
        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return MEDIACMD.GS_NOT_SUPPORTED;
        }
        lRtn = mCmd.Position();

        if( lRtn == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return MEDIACMD.GS_NOT_SUPPORTED;
        }
        // Used as an indictor of interest?  Not sure this is a good idea
        if(mCmdValues.plMin > 0)
        {
            mCmdValues.plMin = mCmd.Start();
        }
        if(mCmdValues.plMax > 0)
        {
            mCmdValues.plMax = mCmd.End();
        }
        return lRtn;
    }

    /**
     * Set the get/set value to setting
     *
     * @param lValueType long
     * @param lSetting long
     * @return long
     */
    public long ValueSet(long lValueType, long lSetting)
    {
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.CmdAlt(lValueType);
        mCmd.Position(lSetting);
        mCmd.Flags(MEDIACMD.cfUsePosition);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Set the get/set value to setting
     *
     * @param lValueType long
     * @param lSetting long
     * @param vidChannels long
     * @param audChannels long
     * @param infChannels long
     * @return long
     */
    public long ValueSet(long lValueType, long lSetting, long vidChannels, long audChannels, long infChannels)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctSetValue);
        mCmd.CmdAlt(lValueType);
        mCmd.Position(lSetting);
        mCmd.Flags(MEDIACMD.cfUsePosition | MEDIACMD.cfUseCmdAlt);
        mCmd.AudioChannels(audChannels);
        mCmd.VideoChannels(vidChannels);
        mCmd.InfoChannels(infChannels);

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            return 0;
        }
        else
        {
            return -1;
        }
    }

    /**
     * Calls ValueXXX with gsClipMode.
     * If equal to 0 then the channel is in clip mode, if 1 the Channel is in VTR mode
     *
     * @param fForceCheck boolean
     * @return long
     */
    public long GetClipMode(boolean fForceCheck)
    {
        if(fForceCheck || dwLastClipModeCheckMs + 2000 < System.currentTimeMillis())
        {
            dwLastClipModeCheckMs = System.currentTimeMillis();
            m_dwClipMode = ValueGet(MEDIACMD.gsClipMode);
        }

        return m_dwClipMode;
    }

    /**
     * Sets the new clip mode
     *
     * @param lSetting long
     * @return long
     */
    public long SetClipMode( long lSetting)
    {
        return ValueSet(MEDIACMD.gsClipMode, lSetting);
    }

    /**
     * Calls ValueXXX with gsTcType
     *
     * @return long
     */
    public long GetTCType()
    {
        return ValueGet(MEDIACMD.gsTcType);
    }

    /**
     * Returns the next TC type that is valid for this DDR to which we are connected.
     *
     * @return long value representing a valid TC type.
     */
    public long GetNextValidTCType()
    {
        long dwTcType;
        long dwPosition, dwStart;

        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(gsTcType);
        mCmd.Flags(-1);
        mCmd.Flags(MEDIACMD.cfUsePosition);

        if (!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }

        long dwTemp = 0;

        dwPosition = mCmd.Position();
        dwStart = mCmd.Start();
        if (dwStart != 0 && dwPosition != 0)
        {
            // If some tc is supported
            while (dwTemp != 0xFF)
            {
                if ((dwStart & (dwPosition << 1)) != 0)
                {
                    // This Is Supported
                    dwTcType = dwPosition << 1;
                    return dwTcType;
                    // so we are done
                }
                dwPosition = dwPosition << 1;
                if (dwPosition > 0x80)
                {
                    dwPosition = 1;
                    // Make sure to check 24 (above check will shift to 2)
                    if ((dwStart & dwPosition) != 0)
                    {
                        // This Is Supported
                        dwTcType = dwPosition;
                        return dwTcType;
                        // so we are done
                    }
                }
            }
        }

        return -1; // If all else fails -- Dean
    }

    /**
     * Sets the TC type
     *
     * @param lSetting long
     * @return long
     */
    public long SetTCType( long lSetting)
    {
        return ValueSet(MEDIACMD.gsTcType, lSetting);
    }

    /**
     * Calls ValueXXX with gsTcSource
     *
     * @return long
     */
    public long GetTCSource()
    {
        return ValueGet(MEDIACMD.gsTcSource);
    }

    /**
     * Sets the TC Source
     *
     * @param lSetting long
     * @return long
     */
    public long SetTCSource(long lSetting)
    {
        return ValueSet(MEDIACMD.gsTcSource, lSetting);
    }

    /**
     * Calls ValueXXX with gsAutoMode. Required for play lists, deferred commands and auto edit commands on VTR.
     *
     * @return long
     */
    public long GetAutoMode()
    {
        return ValueGet(MEDIACMD.gsAutoMode);
    }

    /**
     * SetAutoMode
     *
     * @param lSetting long
     * @return long
     */
    public long SetAutoMode( long lSetting)
    {
        return ValueSet(MEDIACMD.gsAutoMode, lSetting);
    }

    /**
     * Returns the supported audio, video, and info prests for a channel
     *
     * @param presets AvailablePresets
     * @return long
     */
    public long GetAvailablePresets(AvailablePresets presets)
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags( MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsAudChan);

        if(vvwCmd.SendVvwCmd (mCmd))
        {
            if(presets.plVidEdit>0)
            {
                presets.plVidEdit = mCmd.VideoChannels();
            }
            if(presets.plAudEdit>0)
            {
                presets.plAudEdit = mCmd.AudioChannels();
            }
            if(presets.plInfEdit>0)
            {
                presets.plInfEdit = mCmd.InfoChannels();
            }
            return 0;
        }
        else return -1;
    }

    /**
     * Returns the Maximum Buffer Level
     *
     * @return long
     */
    public long GetMaxBufferLevel()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsSysBufferLevel);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return mCmd.InfoChannels();
        }
        else
        {
            return 0;
        }
    }

    /**
     * Returns the Buffer Level
     *
     * @return long
     */
    public long GetBufferLevel()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsSysBufferLevel);

        if(vvwCmd.SendVvwCmd(mCmd))
        {
            return mCmd.Position() + mCmd.Start() + mCmd.End();
        }
        else
        {
            return 0;
        }
    }

    /**
     * Get/Set the AddPreRoll
     */
    /*
    public long GetAddPreRoll()
    {
        return ValueGet(MEDIACMD.gsAddPreroll);
    }

    public long SetAddPreRoll(long lSetting)
    {
        return ValueSet(MEDIACMD.gsAddPreroll, lSetting);
    }
    */

    /**
     * Get the current audio input
     *
     * @param mCmdValues GetValueMcmd
     * @param audChannels long
     * @return long
     */
    public long GetAudioInput(GetValueMcmd mCmdValues,long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudInSelect, mCmdValues, 0, audChannels, 0);
    }

    /**
     * Get the current audio input
     *
     * @param audChannels long
     * @return long
     */
    public long GetAudioInput(long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudInSelect, 0, audChannels, 0);
    }

    /**
     * Set the current audio input
     *
     * @param lSetting long
     * @param audChannels long
     * @return long
     */
    public long SetAudioInput( long lSetting, long audChannels)
    {
        return ValueSet(MEDIACMD.gsAudInSelect, lSetting, 0, audChannels, 0);
    }

    /**
     * Get the current audio input level
     *
     * @param mCmdValues GetValueMcmd
     * @param audChannels long
     * @return long
     */
    public long GetAudioInputLevel(GetValueMcmd mCmdValues, long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudInputLevel, mCmdValues, 0, audChannels, 0);
    }

    /**
     * Get the current audio input level
     *
     * @param audChannels long
     * @return long
     */
    public long GetAudioInputLevel(long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudInputLevel, 0, audChannels, 0);
    }

    /**
     * Set the current audio input level
     *
     * @param lSetting long
     * @param audChannels long
     * @return long
     */
    public long SetAudioInputLevel( long lSetting,long audChannels)
    {
        return ValueSet( MEDIACMD.gsAudInputLevel, lSetting, 0, audChannels, 0);
    }

    /**
     * Get the current audio output
     *
     * @param mCmdValues GetValueMcmd
     * @param audChannels long
     * @return long
     */
    public long GetAudioOutput(GetValueMcmd mCmdValues, long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudOutSelect, mCmdValues, 0, audChannels, 0);
    }

    /**
     * Get the current audio output
     *
     * @param audChannels long
     * @return long
     */
    public long GetAudioOutput(long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudOutSelect, 0, audChannels, 0);
    }

    /**
     * Set the current audio output
     *
     * @param lSetting long
     * @param audChannels long
     * @return long
     */
    public long SetAudioOutput( long lSetting,long audChannels)
    {
        return ValueSet(MEDIACMD.gsAudOutSelect, lSetting, 0, audChannels, 0);
    }

    /**
     * Get the current audio output level
     *
     * @param mCmdValues GetValueMcmd
     * @param audChannels long
     * @return long
     */
    public long GetAudioOutputLevel(GetValueMcmd mCmdValues, long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudOutputLevel, mCmdValues, 0, audChannels, 0);
    }

    /**
     * Get the current audio output level
     *
     * @param audChannels long
     * @return long
     */
    public long GetAudioOutputLevel(long audChannels)
    {
        return ValueGet(MEDIACMD.gsAudOutputLevel, 0, audChannels, 0);
    }

    /**
     * Set the current audio output level
     *
     * @param lSetting long
     * @param audChannels long
     * @return long
     */
    public long SetAudioOutputLevel( long lSetting, long audChannels)
    {
        return ValueSet(MEDIACMD.gsAudOutputLevel, lSetting, 0, audChannels, 0);
    }

    /**
     * Returns the last known RMS peak levels of the input
     *
     * @return long
     */
    public long GetAudioPeakRMS()
    {
        return ValueGet(MEDIACMD.gsAudWavePeakRMS);
    }

    /**
     * Get the current video input
     *
     * @return long
     */
    public long GetVideoInput()
    {
        return ValueGet(MEDIACMD.gsVidInSelect);
    }

    /**
     * Set the current video input
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInput(long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInSelect, lSetting);
    }

    /**
     * Get the current video output
     *
     * @return long
     */
    public long GetVideoOutput()
    {
        return ValueGet(MEDIACMD.gsVidOutSelect);
    }

    /**
     * Set the current video output
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoOutput( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidOutSelect, lSetting);
    }

    /**
     * Get the current video input's 'Setup' TBC setting
     *
     * @return long
     */
    public long GetVideoInputSetup()
    {
        return ValueGet(MEDIACMD.gsVidInSetup);
    }

    /**
     * Get the current video input's 'Setup' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputSetup(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInSetup;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current video input's 'Setup' TBC setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputSetup(long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInSetup, lSetting);
    }

    /**
     * Get the current video input's 'Video' TBC setting
     *
     * @return long
     */
    public long GetVideoInputVideo()
    {
        return ValueGet(MEDIACMD.gsVidInVideo);
    }

    /**
     * Get the current video input's 'Video' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputVideo(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInVideo;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current video input's 'Video' TBC setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputVideo(long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInVideo, lSetting);
    }

    /**
     * Get the current video input's 'Hue' TBC setting
     *
     * @return long
     */
    public long GetVideoInputHue()
    {
        return ValueGet(MEDIACMD.gsVidInHue);
    }

    /**
     * Get the current video input's 'Hue' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputHue(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInHue;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current video input's 'Hue' TBC setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputHue( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInHue, lSetting);
    }

    /**
     * Get the current video input's 'Chroma' TBC setting
     *
     * @return long
     */
    public long GetVideoInputChroma()
    {
        return ValueGet(MEDIACMD.gsVidInChroma);
    }

    /**
     * Get the current video input's 'Chroma' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputChroma(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInChroma;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Get the current video input's 'V Chroma' TBC setting
     *
     * @return long
     */
    public long GetVideoInputVChroma()
    {
        return ValueGet(MEDIACMD.gsVidInVChroma);
    }

    /**
     * Get the current video input's 'V Chroma' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputVChroma(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInVChroma;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Get the current video input's 'U Chroma' TBC setting
     *
     * @return long
     */
    public long GetVideoInputUChroma()
    {
        return ValueGet(MEDIACMD.gsVidInUChroma);
    }

    /**
     * Get the current video input's 'U Chroma' TBC setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoInputUChroma(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidInUChroma;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * SetVideoInputChroma
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputChroma( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInChroma, lSetting);
    }

    /**
     * SetVideoInputVChroma
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputVChroma( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInVChroma, lSetting);
    }

    /**
     * SetVideoInputUChroma
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoInputUChroma( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidInUChroma, lSetting);
    }

    /**
     * Get the current global TBC's 'Setup' setting
     *
     * @return long
     */
    public long GetVideoTBCSetup()
    {
        return ValueGet(MEDIACMD.gsVidSetup);
    }

    /**
     * Get the current global TBC's 'Setup' setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoTBCSetup(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidSetup;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current global TBC's 'Setup' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCSetup( long lSetting)
    {
        return ValueSet( MEDIACMD.gsVidSetup, lSetting);
    }

    /**
     * Get the current global TBC's 'Video' setting
     *
     * @return long
     */
    public long GetVideoTBCVideo()
    {
        return ValueGet(MEDIACMD.gsVidVideo);
    }

    /**
     * Get the current global TBC's 'Video' setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoTBCVideo(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidVideo;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current global TBC's 'Video' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCVideo( long lSetting)
    {
        return ValueSet( MEDIACMD.gsVidVideo, lSetting);
    }

    /**
     * Get the current global TBC's 'Hue' setting
     *
     * @return long
     */
    public long GetVideoTBCHue()
    {
        return ValueGet(MEDIACMD.gsVidHue);
    }

    /**
     * Get the current global TBC's 'Hue' setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoTBCHue(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidHue;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Set the current global TBC's 'Hue' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCHue( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidHue, lSetting);
    }

    /**
     * Get the current global TBC's 'Chroma' setting
     *
     * @return long
     */
    public long GetVideoTBCChroma()
    {
        return ValueGet(MEDIACMD.gsVidChroma);
    }

    /**
     * Get/Set the current global TBC's 'Chroma' setting
     *
     * @param mCmdValues GetValueMcmd
     * @return long
     */
    public long GetVideoTBCChroma(GetValueMcmd mCmdValues)
    {
        mCmdValues.lValueType = MEDIACMD.gsVidChroma;
        mCmdValues.plMax = 1;
        mCmdValues.plMin = 1;
        return ValueGet(mCmdValues);
    }

    /**
     * Get the current global TBC's 'U Chroma' setting
     *
     * @return long
     */
    public long GetVideoTBCUChroma()
    {
        return ValueGet(MEDIACMD.gsVidUChroma);
    }

    /**
     * Set the current global TBC's 'Chroma' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCChroma(long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidChroma, lSetting);
    }

    /**
     * Set the current global TBC's 'V Chroma' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCVChroma(long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidVChroma, lSetting);
    }

    /**
     * Set the current global TBC's 'U Chroma' setting
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoTBCUChroma( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidUChroma, lSetting);
    }

    /**
     * Returns the house/reference lock on or off
     *
     * @return long
     */
    public long GetVideoGenlock()
    {
        return ValueGet(MEDIACMD.gsVidOutGenlock);
    }

    /**
     * Turns the house/reference lock on or off
     *
     * @param lSetting long
     * @return long
     */
    public long SetVideoGenlock( long lSetting)
    {
        return ValueSet(MEDIACMD.gsVidOutGenlock, lSetting);
    }

    /**
     * Get compression rate
     *
     * @return long
     */
    public long GetCompressionRate()
    {
        return ValueGet(MEDIACMD.gsCompRateSize);
    }

    /**
     * Set compression rate
     *
     * @param lSetting long
     * @return long
     */
    public long SetCompressionRate(long lSetting)
    {
        return ValueSet(MEDIACMD.gsCompRateSize, lSetting);
    }

    /**
     * Returns the total number of frames of storage available at current compression rate if the storage space was empty
     *
     * @return long
     */
    public long GetTotalTime()
    {
        return ValueGet(MEDIACMD.gsTotalTimeAvail);
    }

    /**
     * Returns the remaining number of frames of storage available at current compression rate
     *
     * @return long
     */
    public long GetFreeTime()
    {
        return ValueGet(MEDIACMD.gsTotalTimeFree);
    }

    /**
     * Returns the total storage connected in megabytes
     *
     * @return long
     */
    public long GetTotalStorage()
    {
        return ValueGet(MEDIACMD.gsTotalStorageAvail);
    }

    /**
     * Returns the total amount of available storage for recording in megabytes
     *
     * @return long
     */
    public long GetFreeStorage()
    {
        return ValueGet(MEDIACMD.gsTotalStorageFree);
    }

    /**
     * Get current millisecond time
     *
     * @return long
     */
    public long GetLastCurMs()
    {
        if (IsVvwNetSenderConnected())
        {
            // Return pNext
            return dwCurrentLocalTimeMs;
        }
        else
        {
            // Might only have resolution of 10ms depending on platform
            return (System.currentTimeMillis() & 0xFFFFFFFF);
        }
    }

    /**
     * Get current millisecond time
     *
     * @return long
     */
    public long GetLastCurSentMs()
    {
        if (IsVvwNetSenderConnected())
        {
            // Return pNext
            return dwCurrentLocalSentTimeMs;
        }
        else
        {
            // Might only have resolution of 10ms depending on platform
            return (System.currentTimeMillis() & 0xFFFFFFFF);
        }
    }

    /**
     * Get current millisecond Value from MEDIACMD.
     * - Use only to obtain current ms value for first element in an array of MEDIACMD's
     * - Used as in lieu of System.currentTimeMillis() due to inaccuracies
     *
     * @return long
     */
    public synchronized long GetCurMs()
    {
        mCmdCurMs.Init();
        mCmdCurMs.Cmd(ctGetValue);
        mCmdCurMs.CmdAlt(gsCurrentMs);
        mCmdCurMs.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdCurMs.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE);

        if(vvwCmd.SendVvwCmd(mCmdCurMs) == true)
        {
            return mCmdCurMs.Position();
        }
        // Might only have resolution of 10ms depending on platform
        return (System.currentTimeMillis() & 0xFFFFFFFF);
    }

    /**
     * Returns preroll length
     *
     * @return long
     */
    public long GetPrerollLen()
    {
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(gsClipModePreroll);
        mCmd.Flags(-1);

        if (!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        if( mCmd.Position() == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return -1;
        }
        return mCmd.Start();
    }

    /**
     * License check
     *
     * @return long
     */
    public long CheckLicense()
    {
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(gsVVWLicense);
        mCmd.Flags(-1);

        if (!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        if( mCmd.Position() == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return -1;
        }
        return mCmd.Position();
    }

    /**
     * Trial restriction check
     *
     * @return long
     */
    public long CheckTrialRestriction()
    {
        MEDIACMD mCmd = new MEDIACMD();

        mCmd.Init();
        mCmd.Cmd(ctGetValue);
        mCmd.CmdAlt(gsVVWLicense);
        mCmd.Flags(-1);

        if (!(vvwCmd.SendVvwCmd (mCmd)))
        {
            return -1;
        }
        if( mCmd.Position() == MEDIACMD.GS_NOT_SUPPORTED)
        {
            return -1;
        }
        return mCmd.Start();
    }

    /**
     * Get the Video Standard for a channel
     *
     * @return long
     */
    public long GetVideoStandard()
   {
       return ValueGet(MEDIACMD.gsSignalFormat);
   }

    /**
     * Get the available commands for a channel
     *
     * @return long
     */
    public long GetChannelCapabilities()
    {
        return ValueGet(MEDIACMD.gsChanCapabilities);
    }

    /**
     * Returns the version string of the VVW subsystem
     *
     * @return String
     */
    public String GetVVWVersion()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags( MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsVVWVersion);

        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return null;
        }
        String pName = new String(mCmd.arbID());

        if(pName == null)
        {
            return null;
        }
        return pName.trim();
    }

    /**
     * Returns the version string of the MediaReactor subsytem
     *
     * @return String
     */
    public String GetMRVersion()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags( MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsMEVersion);

        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return null;
        }
        String pName = new String(mCmd.arbID());

        if(pName == null)
        {
            return null;
        }
        pName = mCmd.getArbName();
        return pName.trim();
    }

    /**
     * Returns the type string of the VVW channel
     *
     * @return String
     */
    public String GetVVWType()
    {
        MEDIACMD mCmd = new MEDIACMD();
        mCmd.Init();
        mCmd.Cmd(MEDIACMD.ctGetValue);
        mCmd.Flags(MEDIACMD.cfUseCmdAlt);
        mCmd.CmdAlt(MEDIACMD.gsVVWType);

        if(!(vvwCmd.SendVvwCmd(mCmd)))
        {
            return null;
        }
        String pName = new String(mCmd.arbID());

        if(pName == null)
        {
            return null;
        }
        return pName.trim();
    }

    /**
     * Returns the type string of the VVW channel
     *
     * @return String
     */
    public String GetVVWChanType()
   {
       MEDIACMD mCmd = new MEDIACMD();
       mCmd.Init();
       mCmd.Cmd(MEDIACMD.ctGetValue);
       mCmd.Flags(MEDIACMD.cfUseCmdAlt);
       mCmd.CmdAlt(MEDIACMD.gsVVWChannelType);

       if(!(vvwCmd.SendVvwCmd(mCmd)))
       {
           return null;
       }
       String pName = new String(mCmd.arbID());

       if(pName == null)
       {
           return null;
       }
       return pName.trim();
   }

    /**
     * Returns the Picon for the specified Clip Name
     *
     * @param szClipName String
     * @return byte[]
     */
    public byte[] GetPicon(String szClipName)
    {
        // Send Pos = -1 & the File Name (in arbID)
        MEDIACMD mCmdPicon = new MEDIACMD();
        mCmdPicon.Init();
        mCmdPicon.Cmd(MEDIACMD.ctGetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdPicon.CmdAlt(MEDIACMD.gsPiconFrame);
        mCmdPicon.Position(-1);
        mCmdPicon.StructureSize(MEDIACMD.MEDIACMD_BASE_SIZE + szClipName.length() + 1);

        mCmdPicon.setArbName(szClipName, 0, szClipName.length());

        // Send First request
        if(!(vvwCmd.SendVvwCmd (mCmdPicon)))
        {
            return null;
        }

        // mCmd returns the required Size in mCmd.Position()
        long dwBuffLen = mCmdPicon.Position(); // - mCmdPicon.arbID().length;

        // Error check
        if(dwBuffLen <= 0)
        {
            return null;
        }
        // Allocate a char buffer of that length
        byte[] byteBuff = new byte[(int)dwBuffLen];

        // Set the arbID to the Buffer
        mCmdPicon.setArbFromBytes(byteBuff, 0, (int)dwBuffLen);  // .setArbName(szBuffer, 0, (int)dwBuffLen);
        // Put the clip name at the beginnig of arbID
        mCmdPicon.setArbName(szClipName, 0, szClipName.length());

        mCmdPicon.Cmd(MEDIACMD.ctGetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdPicon.CmdAlt(MEDIACMD.gsPiconFrame);

        // Put the buffer length in mCmd.Position
        mCmdPicon.Position(dwBuffLen);

        mCmdPicon.CmdID(MEDIACMD.MEDIACMD_CURRENT);

        // Send Second request
        if (! (vvwCmd.SendVvwCmd(mCmdPicon)))
        {
            return null;
        }
        byte[] pName = mCmdPicon.arbID();

        if(pName == null)
        {
            return null;
        }
        return pName;
    }

    /**
     * Sets the picture icon for the given clip
     *
     * @param szClipName String
     * @param nFrame int
     */
    public void SetPicon(String szClipName, int nFrame)
    {
        MEDIACMD mCmdPicon = new MEDIACMD();
        mCmdPicon.Init();
        mCmdPicon.Cmd(MEDIACMD.ctSetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseClipID);
        mCmdPicon.CmdAlt(MEDIACMD.gsPiconFrame);
        mCmdPicon.Position(nFrame);
        mCmdPicon.setArbName(szClipName, 0, szClipName.length());

        vvwCmd.SendVvwCmd(mCmdPicon);
    }

    /**
     * Returns the preview length for the given clip.
     *
     * @param szClipName String
     * @return long
     */
    public long PreviewLen(String szClipName)
    {
        // Send Pos = -1 & the Clip Name (in arbID)
        MEDIACMD mCmdPicon = new MEDIACMD();
        mCmdPicon.Init();
        mCmdPicon.Cmd(MEDIACMD.ctGetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdPicon.CmdAlt(MEDIACMD.gsJpegFrame);
        mCmdPicon.Position( -1);

        szClipName = szClipName.trim();
        mCmdPicon.setArbName(szClipName, 0, szClipName.length());

        // Send First request
        if(!(vvwCmd.SendVvwCmd(mCmdPicon)))
        {
            return -1;
        }

        // mCmd returns the required Size in mCmd.Position()
        return mCmdPicon.Position(); // Buffer Length
    }

    /**
     * Returns the Preview for the current selection
     *
     * @return byte[]
     */
    public byte[] GetPreview()
    {
        // Send Pos = -1 & the File Name (in arbID)
        MEDIACMD mCmdPicon = new MEDIACMD();
        mCmdPicon.Init();
        mCmdPicon.Cmd(MEDIACMD.ctGetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdPicon.CmdAlt(MEDIACMD.gsJpegFrame);
        mCmdPicon.Position( -1);

        // Send First request
        if(!(vvwCmd.SendVvwCmd(mCmdPicon)))
        {
            return null;
        }

        // mCmd returns the required Size in mCmd.Position()
        long dwBuffLen = mCmdPicon.Position(); // - mCmdPicon.arbID().length;
        // Error check
        if(dwBuffLen <= 0)
        {
            return null;
        }

        // Allocate a char buffer of that length
        byte[] byteBuff = new byte[(int) dwBuffLen];

        // Set the arbID to the Buffer

        mCmdPicon.setArbFromBytes(byteBuff, 0, (int) dwBuffLen);
        // Put the file name at the beginnig of arbID
        //mCmdPicon.setArbName(szFileName, 0, szFileName.length());

        mCmdPicon.Cmd(MEDIACMD.ctGetValue);
        mCmdPicon.Flags(MEDIACMD.cfUseCmdAlt | MEDIACMD.cfUsePosition);
        mCmdPicon.CmdAlt(MEDIACMD.gsJpegFrame);

        // Put the buffer length in mCmd.Position
        mCmdPicon.Position(dwBuffLen);

        mCmdPicon.CmdID(MEDIACMD.MEDIACMD_CURRENT);

        // Send Second request
        if(!(vvwCmd.SendVvwCmd(mCmdPicon)))
        {
            return null;
        }

        byte[] pName = mCmdPicon.arbID();

        if(pName == null)
        {
            return null;
        }
        return pName;
    }

    // ***
    // Utility
    // ***

    /**
     * Returns the maximum possible frame value for a time code type
     *
     * @param lFlags long
     * @return long
     */
    public long TCMaxFrame(long lFlags)
    {
        return (tcxObj.tc2Maximum((int)lFlags));
    }

    /**
     * Convert a time code string to a frame count based on the flags
     *
     * @param szTC String
     * @param lFlags long
     * @return long
     */
    public long TCToFrame(String szTC, long lFlags)
    {
        long intTextIn = 1;
        char chararrIn[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};
        szTC = szTC.trim();

        if (szTC.length() >= 11)
        {
            szTC.getChars(0,11,chararrIn,0);
        }
        else
        {
            szTC.getChars(0,szTC.length(),chararrIn,0);
            for(int i= szTC.length(); i<12;i++)
            {
                chararrIn[i] = 0;
            }
        }
        return(tcxObj.tc2Frame(chararrIn,(int) lFlags, intTextIn));
    }

    /**
     * Convert a frame count to a time codestring based on the flags
     *
     * @param ITC long
     * @param lFlags long
     * @return String
     */
    public String TCToString(long ITC, long lFlags)
    {
        String tcString;
        char[] szTcOut = new char [TCXlat.TC2_MIN_STRING_SIZE];

        szTcOut = tcxObj.tc2String(ITC,(int)lFlags, szTcOut);
        
        /*
         * edited in - stian
         * Fixing bugs...
         */
        tcString = "";
        for(char c : szTcOut) {
        	tcString += c;
        }
        /*
         * end of edit
         * stian
         */
        
//        tcString = szTcOut.toString();		//What is this? :S
//        tcString = tcString.substring(0,11);
        
        return (tcString);
    }

    /**
     * Convert a VVWSpeed (65520 based) to a percentage based speed (100.0)
     *
     * @param lVVWSpeed long
     * @return double
     */
    public double VVWSpeedToPercentage(long lVVWSpeed)
    {
        return(Math.round((double)lVVWSpeed)/655.2);
    }

    /**
     * Convert a percentage based speed (100.0) to a VVWSpeed (65520 based)
     *
     * @param ddPercentageSpeed double
     * @return long
     */
    public long PercentageToVVWSpeed(double ddPercentageSpeed)
    {
        return ((long)(ddPercentageSpeed * 655.2));
    }

    /**
     * <p>Title: GetValueMcmd</p>
     *
     * <p>Description: Stores MediaCmd values</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    public static class GetValueMcmd
    {
        public long lValueType, plMin, plMax;
        public GetValueMcmd(long setValueType, long pMin, long pMax)
        {
            lValueType = setValueType;
            plMin = pMin;
            plMax = pMax;
        }
    }

    /**
     * <p>Title: ClipInfo</p>
     *
     * <p>Description: Stores clip information</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    public static class ClipInfo
    {
        public String szClipName;
        public long  lStart, lEnd, lFileOffset, lVidEdit,lAudEdit, lInfEdit;
        public String  szFileName;
        public ClipInfo(String clipName)
        {
            szClipName = clipName;
        }
    }

    /**
     * <p>Title: ExClipInfo</p>
     *
     * <p>Description: not available</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     *
     * @author Dean Mitrovich
     * @version 1.0
     */
    public static class ExClipInfo
    {
        public long lCreation, lLastModification, lFileSize, lDiskFragments;
    }

    /**
     * <p>Title: VTREditLine</p>
     *
     * <p>Description: Stores VTR edit information (for Conform mode)</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    public static class VTREditLine
    {
        public long lRecordIn, lPlayIn, lPlayOut, lVidEdit, lAudEdit, lInfEdit;
        public String szClipName, szFileName;
    }

    /**
     * <p>Title: AvailablePresets</p>
     *
     * <p>Description: Stores available presets</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    public static class AvailablePresets
    {
        public long plVidEdit, plAudEdit, plInfEdit;
    }

    /**
     * Sends the MediaCmd instance to the server.
     *
     * @param lChannel long
     * @param mCmd MEDIACMD
     * @return boolean
     */
    public boolean SendVvwCmd(long lChannel, MEDIACMD mCmd)
    {
        return(vvwCmd.SendVvwCmd(lChannel, mCmd));
    }

    /**
     * Sets the Timeout value.
     *
     * @param nTime int
     */
    public void setTimeout(int nTime)
    {
       vvwCmd.setTimeout(nTime);
    }

}

