package drastic.mCmdIF;

/**
 * <p>Title: TCXlat</p>
 * 
 * <p>Description: Convert between time code strings and frames</p>
 * 
 * <p>Copyright: Copyright  1998-2007</p>
 * 
 * <p>Company: Drastic Technologies Ltd.<br>
 * 523 The Queensway, Suite 102<br>
 * Toronto, ON, M8Y 1J7<br>
 * CANADA<br>
 * 416 255 5636<br>
 * fax 255 8780<br>
 * http://www.drastictech.com<br>
 * engineering@drastictech.com</p>
 * 
 * @author not attributable
 * @version 1.0
 */
public class TCXlat
{
    /**
     * This string is a global string to solve the problem with older Java MS VM's. It is set on each call of the
     * tc2frame function and if the string value passed to the tc2frame function is null it will then set the string to
     * this global value.
     */
    public String szTc;

    /**
     * Required size of a timecode string
     */
    public static final int TC2_MIN_STRING_SIZE = 14;

    // The Following values are used in dwFlags TimeCode Types
    public static final int TC2_TCTYPE_MASK = 0x000000FF;
    /**
     * Film 24 FPS time code
     */
    public static final int TC2_TCTYPE_FILM = 0x00000001; // 24 fps
    public static final char TC2_MARK_FILM = '-';
    public static final char TC2_CHAR_FILM = 'F';
    /**
     * Non Drop Frame 30 FPS time code
     */
    public static final int TC2_TCTYPE_NDF = 0x00000002; // NTSC Non Drop Frame
    public static final char TC2_MARK_NDF = ':';
    public static final char TC2_CHAR_NDF = 'N';
    /**
     * Drop Frame 29.97 FPS time code
     */
    public static final int TC2_TCTYPE_DF = 0x00000004; // NTSC Drop Frame
    public static final char TC2_MARK_DF = ';';
    public static final char TC2_CHAR_DF = 'D';
    /**
     * PAL 25 FPS time code
     */
    public static final int TC2_TCTYPE_PAL = 0x00000008; // PAL
    public static final char TC2_MARK_PAL = '.';
    public static final char TC2_CHAR_PAL = 'P';
    /**
     * Double PAL 50 FPS
     */
    public static final int TC2_TCTYPE_50 = 0x00000010; // PAL 720p (double rate)
    public static final char TC2_MARK_50 = 0x27;	// its the '
    public static final char TC2_CHAR_50 = 'p';
    /**
     * 720p DROP 59.94 FPS
     */
    public static final int TC2_TCTYPE_5994 = 0x00000020; // NTSC 59.94fps 720p (NTSC DF double)
    public static final char TC2_MARK_5994 = ',';
    public static final char TC2_CHAR_5994 = 'd';
    /**
     * 720p 60 FPS
     */
    public static final int TC2_TCTYPE_60 = 0x00000040; // NTSC 60fps 720p (NTSC NDF double)
    public static final char TC2_MARK_60 = '=';
    public static final char TC2_CHAR_60 = 'n';
    /**
     * 23.98 FILM for NTSC 23.98 FPS  (This is actually 24)
     */
    public static final int TC2_TCTYPE_NTSCFILM = 0x00000080; // NTSC FILM 23.98
    public static final char TC2_MARK_NTSCFILM	= '_';
    public static final char TC2_CHAR_NTSCFILM = 'f';
    /**
     * 23.98 TRUE (actual 23.98 drop per Avid)
     */
    public static final int TC2_TCTYPE_2398 = 0x00000084; // TRUE 23.98
    public static final char TC2_MARK_2398 = '+';
    public static final char TC2_CHAR_2398 = 't';
    /**
     * Hundredths of a second HH:MM:SS:/100  100 FPS effective
     */
    public static final int TC2_TCTYPE_100 = 0x00000044; // Hours:Minutes:Seconds:Hundreds public static final char TC2_MARK_100		'^'
    public static final char TC2_MARK_100 =  '^';
    public static final char TC2_CHAR_100 = '0';

    /**
     * Frame conversion to/from time code is in fields
     */
    public static final int TC2_FTYPE_FIELD = 0x10000000; // Field based (else frame)

    // Basic sting tc representation types
    public static final int TC2_STRTYPE_MASK = 0x00000F00;
    /**
     * Std ascii string
     */
    public static final int TC2_STRTYPE_ASCII = 0x00000100;
    /**
     * RS-422 BCD or Packed int
     */
    public static final int TC2_STRTYPE_BCD = 0x00000200;
    /**
     * Hex packed int
     */
    public static final int TC2_STRTYPE_HEX = 0x00000400;
    /**
     * MPEG Gop TC
     */
    public static final int TC2_STRTYPE_GOP = 0x00000800;
    /**
     * Frames first
     */
    public static final int TC2_STRTYPE_INVERT = 0x00001000;
    /**
     * Allow time codes larger then 24 hours - 1 frame
     */
    public static final int TC2_STRTYPE_OVER_24_HOUR = 0x00002000;

    // Extended string handling
    public static final int TC2_STREXT_MASK = 0xFFFF0000;
    /**
     *  Add : marks in string
     */
    public static final int TC2_STREXT_MARKS = 0x00010000;
    /**
     * Include leading 0s
     */
    public static final int TC2_STREXT_LEADING = 0x00020000;
    /**
     * Add ' N', ' D', ' P' or ' F' at end;
     */
    public static final int TC2_STREXT_TYPE = 0x00040000;
    /**
     * No ";" just ":" in the output string
     */
    public static final int TC2_STREXT_ALLCOLON = 0x00080000;
    /**
     * Add DF Flag in BCD
     */
    public static final int TC2_STREXT_FLAG = 0x00100000;
    /**
     * Add CF Flag in BCD
     */
    public static final int TC2_STREXT_CF = 0x00200000;
    /**
     * Max 29 frames for output (24 in PAL)
     */
    public static final int TC2_STREXT_MAX30 = 0x00400000;
    /**
     * Strings are BSTR (vb mostly)
     */
    public static final int TC2_STREXT_ISBSTR = 0x01000000;
    /**
     * GOP Tc is shifted in DWORD
     */
    public static final int TC2_STREXT_SHIFT7 = 0x40000000;
    /**
     * GOP Save unused bits
     */
    public static final int TC2_STREXT_SAVEBITS = 0x80000000;

    static final int TC2_ILLEGAL = 0xFFFFFFFF; //

    static final int TC2_POS_HOURS = 0;

    static final int TC2_POS_MINUTES = 1;

    static final int TC2_POS_SECONDS = 2;

    static final int TC2_POS_FRAMES = 3;

    static final int TC2_TCFLAG_DF = 0x40;

    static final int TC2_TCFLAG_CF = 0x80;
    /**
     * frames per hour in non-drop mode
     */
    static final int TC2_HOUR_NDF = 108000;
    /**
     * frames per hour in drop frame mode
     */
    static final int TC2_HOUR_DROP = 107892;
    /**
     * frames per hour in pal mode
     */
    static final int TC2_HOUR_PAL = 90000;
    /**
     * frames per hour in film
     */
    static final int TC2_HOUR_FILM = 86400;
    /**
     * frames per hour for 720p PAL
     */
    static final int TC2_HOUR_50 = 180000;
    /**
     * frames per hour for 720p NTSC DF
     */
    static final int TC2_HOUR_5994 = 215784;
    /**
     * frames per hour for 720p NTSC
     */
    static final int TC2_HOUR_60 = 216000;

    // Do Not Use -#define TC2_HOUR_NTSCFILM 86327L  /* frames per hour for 720p NTSCFILM */
    /**
     * hundreds of second per hour
     */
    static final int TC2_HOUR_100 = 360000;

    /**
     * frames per minute in non-drop mode
     */
    static final int TC2_MINUTE_NDF = 1800;
    /**
     * frames per ten minutes in drop frame
     */
    static final int TC2_TENMIN_DROP = 17982;
    /**
     * frames per minute in drop frame mode
     */
    static final int TC2_MINUTE_DROP = 1798;
    /**
     * frames per minute in pal mode
     */
    static final int TC2_MINUTE_PAL = 1500;
    /**
     * frames per minute in film
     */
    static final int TC2_MINUTE_FILM = 1440;
    /**
     * frames per minute in 720p PAL
     */
    static final int TC2_MINUTE_50 = 3000;
    /**
     * frames per five minutes in 720p DF
     */
    static final int TC2_FIVEMIN_5994 = 17982;
    /**
     * frames per minute in 720p DF
     */
    static final int TC2_MINUTE_5994 = 3596;
    /**
     * frames per minute in 720p NDF
     */
    static final int TC2_MINUTE_60 = 3600;

    // Do Not Use -#define TC2_MINUTE_NTSCFILM	1438L    /* frames per minute in ntsc film 23.978 */
    // Do Not Use -#define TC2_FIVE_MINUTE_NTSCFILM 7193L    /* frames per five minute in ntsc film 23.978 */

    /**
     * hundreds of second per hour
     */
    static final int TC2_MINUTE_100 =  6000;

    /**
     * frames per second for non drop frame ntsc
     */
    static final int TC2_SECOND_NDF = 30;

	/**
     * frames per second for drop frame ntsc
     */
    static final int TC2_SECOND_DROP = 30;
    /**
     * frames per second for pal
     */
    static final int TC2_SECOND_PAL = 25;
    /**
     * frames per second for film
     */
    static final int TC2_SECOND_FILM = 24;
    /**
     * frames per second for 720p PAL
     */
    static final int TC2_SECOND_50 = 50;
    /**
     * frames per second for 720p NTSC DF
     */
    static final int TC2_SECOND_5994 = 60;
    /**
     * frames per second for 720p NTSC
     */
    static final int TC2_SECOND_60 = 60;

    // Do Not Use -#define TC2_SECOND_NTSCFILM	 24L  /* frames per second for 720p NTSCFILM */

    /**
     * hundreds of seconds per second
     */
    static final int TC2_SECOND_100 = 100;
    /**
     * 50 second drop
     */
    static final int TC2_NTSCFILM_50SECDROP = 1199;

    /**
     * Returns the maximum number of frames possible in a given time code
     * type as frames or fields.
     *
     * @param dwFlags int Time code type flags (TC2_TCTYPE_xxx) and optional
     * TC2_FTYPE_FIELD flag.
     * @return int the last possible frame in the time code type
     */
    public int tc2Maximum(int dwFlags)
    {
        int dwMax = 0;

        // Find max TC in fields 23:59:59:2x.1
        switch (dwFlags & TC2_TCTYPE_MASK)
        {
            case TC2_TCTYPE_FILM:
            case TC2_TCTYPE_NTSCFILM://	24sf?
                dwMax = 4147199; // 23:59:59:23.1
                break;
            case TC2_TCTYPE_NDF:
                dwMax = 5183999; // 23:59:59:29.1
                break;
            default:
            case TC2_TCTYPE_DF:
                dwMax = 5178815; // 23:59:59:29.1
                break;
            case TC2_TCTYPE_PAL:
                dwMax = 4319999; // 23:59:59:24.1
                break;
            case TC2_TCTYPE_50:	// 720p
                // No fields
                return 4319999 * 2; // 23:59:59:49.1
            case TC2_TCTYPE_5994: // 720p
                // No fields
                return 5178815 * 2; // 23:59:59:59.1
            case TC2_TCTYPE_60: // 720p
                // No fields
                return 5183999 * 2; // 23:59:59:59.1
            case TC2_TCTYPE_2398: //	24sf?
                // No fields
                // This is wrong, need calc
                return 2071871 * 2; // 23:59:59:53.1
            case TC2_TCTYPE_100: // 23:59:59:99
                //
                dwMax = 8640000 - 1;
                break;
        }

        // Return fields or frames as requested
        if((dwFlags & TC2_FTYPE_FIELD) == TC2_FTYPE_FIELD)
        {
            return dwMax;
        }
        return (dwMax >> 1);
    }

    /**
     * Convert the frame count dwTcIn to the string pszTcOut as specified by dwFlags
     *
     * @param dwTcIn long number of frames, zero based
     * @param dwFlags int TC2_TCTYPE, TC2_STREXT and other TC2 formatring flags
     * @param pszTcOut char[] String legacy representation of the frame time code
     * @return char[] String returned
     */
    public char[] tc2String(long dwTcIn, int dwFlags, char[] pszTcOut)
    {
        byte[] bTc = new byte[TC2_MIN_STRING_SIZE];
        byte bTcType = (byte) '!';
        long dwRest = 0; // The remaining part of dwTcIn
        byte bMinutes = 0; // Drop frame calculation

        if (((dwFlags & TC2_STRTYPE_OVER_24_HOUR) != TC2_STRTYPE_OVER_24_HOUR) &&
            dwTcIn > tc2Maximum(dwFlags) + 1)
        {
            dwTcIn = tc2Maximum(dwFlags) + 1;
        }

        if ( (dwFlags & TC2_FTYPE_FIELD) == TC2_FTYPE_FIELD)
        {
            dwTcIn >>= 1;
        }

        switch (dwFlags & TC2_TCTYPE_MASK)
        {
            /* Mode NTSC NON DROP FRAMES */
            default:
            case TC2_TCTYPE_NDF:
                bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_NDF);
                dwRest = dwTcIn % TC2_HOUR_NDF;

                bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_NDF);
                dwRest = dwRest % TC2_MINUTE_NDF;

                bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_NDF);

                bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_NDF);

                bTcType = TC2_MARK_NDF;
                break;

                /* Mode NTSC DROP FRAMES */
            case TC2_TCTYPE_DF:
                bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_DROP);
                dwRest = dwTcIn % TC2_HOUR_DROP;

                bTc[TC2_POS_MINUTES] = (byte) (10 * (dwRest / TC2_TENMIN_DROP));
                dwRest = dwRest % TC2_TENMIN_DROP;

                bMinutes = (byte) ( (dwRest / TC2_MINUTE_DROP));

                if (bMinutes == 0)
                {
                    dwRest %= TC2_MINUTE_NDF;
                }
                else
                {
                    if ( (dwRest %= TC2_MINUTE_DROP) >= 2)
                    {
                        bTc[TC2_POS_MINUTES] += bMinutes;
                    }
                    else
                    {
                        bTc[TC2_POS_MINUTES] += (bMinutes - 1);
                        dwRest += TC2_MINUTE_DROP;
                    }
                }
                bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_DROP);
                bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_DROP);

                bTcType = TC2_MARK_DF;
                break;
                /* Mode PAL */
            case TC2_TCTYPE_PAL:
                bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_PAL);
                dwRest = dwTcIn % TC2_HOUR_PAL;

                bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_PAL);
                dwRest = dwRest % TC2_MINUTE_PAL;

                bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_PAL);

                bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_PAL);

                bTcType = TC2_MARK_PAL;
                break;

                /* Mode FILM NON-DROP FRAMES */
            case TC2_TCTYPE_FILM:
            case TC2_TCTYPE_NTSCFILM:	//  0x00000080	// NTSC FILM 23.97
                //
                // 23.98 calculated as 24 for vtrs
                //
                bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_FILM);
                dwRest = dwTcIn % TC2_HOUR_FILM;

                bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_FILM);
                dwRest = dwRest % TC2_MINUTE_FILM;

                bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_FILM);

                bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_FILM);

                if((dwFlags & TC2_TCTYPE_MASK) == TC2_TCTYPE_FILM)
                {
                    bTcType = TC2_MARK_FILM;
                }
                else
                {
                    bTcType = TC2_MARK_NTSCFILM;
                }
                break;
            case TC2_TCTYPE_50:		//  0x00000010	// PAL 720p (double rate)
                    bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_50);
                    dwRest = dwTcIn % TC2_HOUR_50;

                    bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_50);
                    dwRest = dwRest % TC2_MINUTE_50;

                    bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_50);

                    bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_50);

                    // bTcType = TC2_MARK_50;	// its the '
                    bTcType = TC2_MARK_NDF; // Same as NDF ':'
                    if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                    {
                            bTc[TC2_POS_FRAMES] >>= 1;
                    }
                    break;

            case TC2_TCTYPE_5994:	//  0x00000020	// NTSC 59.94fps 720p (NTSC DF double)
                    bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_5994);
                    dwRest = dwTcIn % TC2_HOUR_5994;

                    bTc[TC2_POS_MINUTES] = (byte) (10L * (dwRest / TC2_FIVEMIN_5994));
                    dwRest = dwRest % TC2_FIVEMIN_5994;

                    bMinutes = (byte) ((dwRest / TC2_MINUTE_5994));

                    if (bMinutes == 0)
                    {
                        dwRest %= TC2_MINUTE_NDF;
                    }
                    else
                    {
                        if ((dwRest %= TC2_MINUTE_5994) >= 2)
                        {
                            bTc[TC2_POS_MINUTES] += bMinutes;
                        }
                        else
                        {
                            bTc[TC2_POS_MINUTES] += (bMinutes - 1);
                            dwRest += TC2_MINUTE_5994;
                        }
                    }
                    bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_5994);
                    bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_5994);

                    bTcType = TC2_MARK_5994;
                    if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                    {
                        bTc[TC2_POS_FRAMES] >>= 1;
                    }
                    break;
            case TC2_TCTYPE_60:		// 0x00000030	// NTSC 60fps 720p (NTSC NDF double)
                    bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_60);
                    dwRest = dwTcIn % TC2_HOUR_60;

                    bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_60);
                    dwRest = dwRest % TC2_MINUTE_60;

                    bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_60);

                    bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_60);

                    bTcType = TC2_MARK_60;

                    if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                    {
                        bTc[TC2_POS_FRAMES] >>= 1;
                    }
                    break;

            case TC2_TCTYPE_2398:
                    // Drop a frame every fifty seconds.....
                    //  all mutliples of 1199
                    // Once additional frames are added, it is a
                    // straight 24 calculation

                    dwTcIn += (long)((dwTcIn) / TC2_NTSCFILM_50SECDROP);

                    bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_FILM);
                    dwRest = dwTcIn % TC2_HOUR_FILM;
                    bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_FILM);
                    dwRest = dwRest % TC2_MINUTE_FILM;
                    bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_FILM);
                    bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_FILM);
                    bTcType = TC2_MARK_2398;
                    break;

            case TC2_TCTYPE_100:	// 0x00000044	// 100ths of second
                    bTc[TC2_POS_HOURS] = (byte) (dwTcIn / TC2_HOUR_100);
                    dwRest = dwTcIn % TC2_HOUR_100;
                    bTc[TC2_POS_MINUTES] = (byte) (dwRest / TC2_MINUTE_100);
                    dwRest = dwRest % TC2_MINUTE_100;
                    bTc[TC2_POS_SECONDS] = (byte) (dwRest / TC2_SECOND_100);
                    bTc[TC2_POS_FRAMES] = (byte) (dwRest % TC2_SECOND_100);
                    bTcType = TC2_MARK_100;
                    break;
        }

        // Do we need to invert the TC order in the bytes
        if ( (dwFlags & TC2_STRTYPE_INVERT) == TC2_STRTYPE_INVERT)
        {
            byte bXfer = bTc[0];
            bTc[0] = bTc[3];
            bTc[3] = bXfer;
            bXfer = bTc[1];
            bTc[1] = bTc[2];
            bTc[2] = bXfer;
        }

        // Do we need to covert the type of string
        switch (dwFlags & TC2_STRTYPE_MASK)
        {
            default:
            case TC2_STRTYPE_ASCII:
            {
                int n = 0, p = 0;
                if ( (dwFlags & TC2_STREXT_LEADING) != TC2_STREXT_LEADING)
                {
                    // Find the leading digit if stripping 0s
                    while (n < 4 && bTc[n] == 0)
                    {
                        n++;
                    }
                    // Handle the first, possible half digit
                    if (bTc[n] >= 10)
                    {
                        // we have a two digit start
                        pszTcOut[p] = (char) ( (bTc[n] / 10) + '0');
                        p++;
                    }
                    pszTcOut[p] = (char) ( (bTc[n] % 10) + '0');
                    p++;
                    if ( (dwFlags & TC2_STREXT_MARKS) == TC2_STREXT_MARKS)
                    {
                        if ( (n == 2) && ( (dwFlags & TC2_STREXT_ALLCOLON) != TC2_STREXT_ALLCOLON))
                        { // Last TC Mark
                            pszTcOut[p] = (char) bTcType;
                        }
                        else if (n < 2 || ( (dwFlags & TC2_STREXT_ALLCOLON) == TC2_STREXT_ALLCOLON))
                        {
                            pszTcOut[p] = ':';
                        }
                        p++;
                    }
                    n++;
                }

                for (; n < 4; n++)
                {
                    pszTcOut[p] = (char) ( (bTc[n] / 10) + '0');
                    p++;
                    pszTcOut[p] = (char) ( (bTc[n] % 10) + '0');
                    p++;
                    if ( (dwFlags & TC2_STREXT_MARKS) == TC2_STREXT_MARKS)
                    {
                        if (n == 2)
                        { // Last TC Mark
                            if((dwFlags & TC2_STREXT_ALLCOLON) != 0)
                            {
                                pszTcOut[p] = ':';
                            } else
                            {
                                pszTcOut[p] = (char) bTcType;
                            }
                            p++;
                        }
                        else if (n < 2)
                        {
                            pszTcOut[p] = ':';
                            p++;
                        }
                    }
                }
                if ( (dwFlags & TC2_STREXT_TYPE) == TC2_STREXT_TYPE)
                {
                    pszTcOut[p] = ' ';
                    p++;
                    switch (dwFlags & TC2_TCTYPE_MASK)
                    {
                        /* Mode NTSC NON DROP FRAMES */
                        default:
                            pszTcOut[p] = ' ';
                            break;
                        case TC2_TCTYPE_NDF:
                            pszTcOut[p] = TC2_CHAR_NDF;
                            break;
                            /* Mode NTSC DROP FRAMES */
                        case TC2_TCTYPE_DF:
                            pszTcOut[p] = TC2_CHAR_DF;
                            break;
                            /* Mode PAL */
                        case TC2_TCTYPE_PAL:
                            pszTcOut[p] = TC2_CHAR_PAL;
                            break;
                            /* Mode FILM NON-DROP FRAMES */
                        case TC2_TCTYPE_FILM:
                            pszTcOut[p] = TC2_CHAR_FILM;
                            break;
                        case TC2_TCTYPE_50:		//	0x00000010	// PAL 720p (double rate)
                            pszTcOut[p] = TC2_CHAR_50;
                            break;
                        case TC2_TCTYPE_5994:	//		0x00000020	// NTSC 59.94fps 720p (NTSC DF double)
                            pszTcOut[p] = TC2_CHAR_5994;
                            break;
                        case TC2_TCTYPE_60:		//	0x00000030	// NTSC 60fps 720p (NTSC NDF double)
                            pszTcOut[p] = TC2_CHAR_60;
                            break;
                        case TC2_TCTYPE_NTSCFILM:	//	0x00000080	// NTSC FILM 23.97
                            pszTcOut[p] = TC2_CHAR_NTSCFILM;
                            break;
                        case TC2_TCTYPE_2398:
                            pszTcOut[p] = TC2_CHAR_2398;
                            break;
                        case TC2_TCTYPE_100:		//      0x00000044	// 100ths
                            pszTcOut[p] = TC2_CHAR_100;		                // Make into 1000ths
                            break;
                    }
                    p++;
                }
                // Do not zero more than nec
                pszTcOut[p] = 0;	// Term string
                p++;
            }
            break;
            case TC2_STRTYPE_BCD:
            {
                int n;
                for (n = 0; n < 4; n++)
                {
                    pszTcOut[n] = (char) ( ( (bTc[n] / 10) << 4) | bTc[n] % 10);
                }
                pszTcOut[4] = 0;
                if ( ( (dwFlags & TC2_STREXT_FLAG) == TC2_STREXT_FLAG)
                    && ( (dwFlags & TC2_TCTYPE_DF) == TC2_TCTYPE_DF))
                {
                    // Add the colour frame flag
                    pszTcOut[0] |= TC2_TCFLAG_DF;
                }
                if ( (dwFlags & TC2_STREXT_CF) == TC2_STREXT_CF)
                {
                    // Tell caller its drop frame
                    pszTcOut[0] |= TC2_TCFLAG_CF;
                }
            }
            break;
            case TC2_STRTYPE_HEX:

                // Do nothing, its already hex
                pszTcOut[0] = (char) bTc[0];
                pszTcOut[1] = (char) bTc[1];
                pszTcOut[2] = (char) bTc[2];
                pszTcOut[3] = (char) bTc[3];
                break;
                //	default:
                //		return TC2_ILLEGAL;
            case TC2_STRTYPE_GOP:
                // Gop TC
                // Bit Size		Meaning					Position	Pos Shift 7
                // 1			Drop Flag				0			31
                // 5			Hour					1			26
                // 6			Minute					6			20
                // 1			Marker (Always Set)		12			19
                // 6			Second					13			13
                // 6			Frame					19			7
                // 1			Closed Gop				25			6
                // 1			Broken Line				26			5
                // 5			(Save Not TC)			27-32 inc	0-4 inc
                //lint -save -e826

            break;
        }
        /*
         * edit in - stian
         * fixing the bugs... :S
         * VERY quick fix, not to be trusted for every option
         */
        pszTcOut = new char[11];
        for(int i=0;i<4;i++) { //Timecode only four ints long really
        	String intString = Integer.toString((int)(bTc[i]));
        	if(intString.length()>1) {
        		pszTcOut[i*3] = intString.charAt(0);
        		pszTcOut[i*3+1] = intString.charAt(1);
        	} else {
        		pszTcOut[i*3] = '0';
        		pszTcOut[i*3+1] = intString.charAt(0);
        	}
        	if(i!=3) pszTcOut[i*3+2] = ':';
        }
        /*
         * end of edit
         */
        return pszTcOut;
    }

    /**
     * Convert a time code string (like "10:02:22:12") to a frame count
     *
     * @param strTcIn String the time code string
     * @param dwFlags int TC2_TCTYPE_xxx time code type (DF, NDF, PAL, etc)
     * @param dwTcOut long legacy frame count out
     * @return long frame count
     */
    public long tc2Frame(String strTcIn, int dwFlags, long dwTcOut)
    {
        char pcharar[] = new char[TC2_MIN_STRING_SIZE];
        strTcIn.getChars(0, strTcIn.length(), pcharar, 0);
        return tc2Frame(pcharar, dwFlags, dwTcOut);
    }

    /**
     * Convert the character array pszTcIn (like "10:02:22:12") to a frame count
     * as specified by dwFlags
     *
     * @param pszTcIn char[] String the time code string
     * @param dwFlags int TC2_TCTYPE_xxx time code type (DF, NDF, PAL, etc)
     * @param dwTcOut long legacy frame count out
     * @return long frame count
     */
    public long tc2Frame(char[] pszTcIn, int dwFlags, long dwTcOut)
    {
        long dwTc = 0; // Temp TC to fix leading 0 problem
        byte[] bTc = new byte [TC2_MIN_STRING_SIZE];
        int n = 0;
        boolean fTcDone = false;
        int dwFoundTcType = TC2_TCTYPE_NDF;

        if (pszTcIn == null)
        {
            return 0;
        }

        int TcInLength = pszTcIn.length;

        // Find the end of the string
        while ( (n < TcInLength) && (!fTcDone))
        {
            switch (dwFlags & TC2_STRTYPE_MASK)
            {
                default:
                case TC2_STRTYPE_ASCII:
                    if (pszTcIn[n] >= '0' && pszTcIn[n] <= '9')
                    {
                        dwTc = (dwTc << 4) + (pszTcIn[n] - '0');
                    }
                    else if (pszTcIn[n] == TC2_MARK_DF || pszTcIn[n] == TC2_CHAR_DF || pszTcIn[n] == TC2_CHAR_5994)
                    {
                        dwFoundTcType = TC2_TCTYPE_DF;
                    }
                    else if (pszTcIn[n] == TC2_CHAR_NDF || pszTcIn[n] == TC2_CHAR_60)
                    {
                        dwFoundTcType = TC2_TCTYPE_NDF;
                    }
                    else if (pszTcIn[n] == TC2_MARK_PAL || pszTcIn[n] == TC2_CHAR_PAL || pszTcIn[n] == TC2_CHAR_50)
                    {
                        dwFoundTcType = TC2_TCTYPE_PAL;
                    }
                    else if (pszTcIn[n] == TC2_MARK_FILM || pszTcIn[n] == TC2_CHAR_FILM)
                    {
                        dwFoundTcType = TC2_TCTYPE_FILM;
                    }
                    else if (pszTcIn[n] == TC2_MARK_NTSCFILM || pszTcIn[n] == TC2_CHAR_NTSCFILM)
                    {
                        dwFoundTcType = TC2_TCTYPE_NTSCFILM;
                    }
                    else if(pszTcIn[n] == TC2_MARK_2398 || pszTcIn[n] == TC2_CHAR_2398)
                    {
                        dwFoundTcType = TC2_TCTYPE_2398;
                    }
                    else if (pszTcIn[n] < ' ')
                    {
                        fTcDone = true;
                    }
                    break;
                case TC2_STRTYPE_BCD:
                    // Do not check - DF and CF flag can screw this up!!!
                    if (n < 4)
                    {
                        dwTc = (dwTc << 8) + (pszTcIn[n] & 0x7F);
                    }
                    else
                    {
                        fTcDone = true;
                    }
                    break;
                case TC2_STRTYPE_HEX:
                    if (n < 4 && pszTcIn[n] >= 0 && pszTcIn[n] <= 60)
                    {
                        dwTc = (dwTc << 8) + ( ( (pszTcIn[n] / 10) << 4) + (pszTcIn[n] % 10));
                    }
                    else
                    {
                        fTcDone = true;
                    }
                    break;
            }
            n++;
        }

        if ( (dwFlags & TC2_STRTYPE_MASK) != TC2_STRTYPE_GOP)
        {
            // Do we need to invert the TC order in the bytes
            if ( (dwFlags & TC2_STRTYPE_INVERT) == TC2_STRTYPE_INVERT)
            {
                // Convert 8 characters in int (nibble array, bcd) to byte array of values
                bTc[3] = (byte) ( ( ( (dwTc >> 28) & 0xF) * 10) + ( (dwTc >> 24) & 0xF));
                bTc[2] = (byte) ( ( ( (dwTc >> 20) & 0xF) * 10) + ( (dwTc >> 16) & 0xF));
                bTc[1] = (byte) ( ( ( (dwTc >> 12) & 0xF) * 10) + ( (dwTc >> 8) & 0xF));
                bTc[0] = (byte) ( ( ( (dwTc >> 4) & 0xF) * 10) + ( (dwTc) & 0xF));
            }
            else
            {
                bTc[0] = (byte) ( ( ( (dwTc >> 28) & 0xF) * 10) + ( (dwTc >> 24) & 0xF));
                bTc[1] = (byte) ( ( ( (dwTc >> 20) & 0xF) * 10) + ( (dwTc >> 16) & 0xF));
                bTc[2] = (byte) ( ( ( (dwTc >> 12) & 0xF) * 10) + ( (dwTc >> 8) & 0xF));
                bTc[3] = (byte) ( ( ( (dwTc >> 4) & 0xF) * 10) + ( (dwTc) & 0xF));
            }

            if ((dwFlags & TC2_TCTYPE_MASK) == 0)
            {
                // *****************
                // *****************
                if (dwFoundTcType == 0)
                {
                    // False
                    dwFoundTcType = TC2_TCTYPE_DF;
                }
                dwFlags = (dwFlags & ~TC2_TCTYPE_MASK) | dwFoundTcType;
                // *****************
                // *****************
            }
        }

        switch (dwFlags & TC2_TCTYPE_MASK)
        {
            /* Mode NTSC NON-DROP FRAMES */
            case (TC2_TCTYPE_NDF | TC2_TCTYPE_DF):
            case TC2_TCTYPE_NDF:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_NDF +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_NDF +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_NDF +
                    (long) bTc[TC2_POS_FRAMES];
                break;

                /* Mode NTSC DROP FRAMES */
            case TC2_TCTYPE_DF:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_DROP +
                    ( (long) bTc[TC2_POS_MINUTES] / 10L) * TC2_TENMIN_DROP +
                    ( (long) bTc[TC2_POS_MINUTES] % 10L) * TC2_MINUTE_DROP +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_DROP +
                    (long) bTc[TC2_POS_FRAMES];
                break;

                /* Mode PAL NON-DROP FRAMES */
            case TC2_TCTYPE_PAL:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_PAL +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_PAL +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_PAL +
                    (long) bTc[TC2_POS_FRAMES];
                break;

                /* Mode FILM NON-DROP FRAMES */
            case TC2_TCTYPE_FILM:
            case TC2_TCTYPE_NTSCFILM:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_FILM +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_FILM +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_FILM +
                    (long) bTc[TC2_POS_FRAMES];
                break;

                /* Mode 50 Hz 720p */
            case TC2_TCTYPE_50:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_50 +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_50 +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_50 +
                    (long) bTc[TC2_POS_FRAMES];

                if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                {
                    // 30/25 based, double frames
                    dwTcOut += (long) bTc[TC2_POS_FRAMES];
                }
                break;

                /* Mode 59.94 Hz 720p */
            case TC2_TCTYPE_5994:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_5994 +
                    ((long) bTc[TC2_POS_MINUTES] / 5L) * TC2_FIVEMIN_5994 +
                    ((long) bTc[TC2_POS_MINUTES] % 5L) * TC2_MINUTE_5994 +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_5994 +
                    (long) bTc[TC2_POS_FRAMES];
                if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                {
                    // 30/25 based, double frames
                    dwTcOut += (long) bTc[TC2_POS_FRAMES];
                }
                break;

                /* Mode 60 Hz 720p */
            case TC2_TCTYPE_60:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_60 +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_60 +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_60 +
                    (long) bTc[TC2_POS_FRAMES];
                if((dwFlags & TC2_STREXT_MAX30) == TC2_STREXT_MAX30)
                {
                    // 30/25 based, double frames
                    dwTcOut += (long) bTc[TC2_POS_FRAMES];
                }
                break;

                /* Mode 23.97 Hz TRUE to time */
            case TC2_TCTYPE_2398:
                dwTcOut = (long) bTc[TC2_POS_HOURS] * TC2_HOUR_FILM +
                    (long) bTc[TC2_POS_MINUTES] * TC2_MINUTE_FILM +
                    (long) bTc[TC2_POS_SECONDS] * TC2_SECOND_FILM +
                    (long) bTc[TC2_POS_FRAMES];
                dwTc = (((dwTcOut)) / (TC2_NTSCFILM_50SECDROP+1));
                dwTcOut -= dwTc;
                break;

            default:
                return TC2_ILLEGAL;
        }

        if ( (dwFlags & TC2_FTYPE_FIELD) == TC2_FTYPE_FIELD)
        {
            dwTcOut = (int) (dwTcOut) << 1;
        }

        return dwTcOut;
    }

    /**
     * Convert the numeric userbits in dwUbIn to the byte array pszUbOut as specified by dwFlags
     *
     * @param dwUbIn int user bits numberic value
     * @param dwFlags int TC2_flags, not currently used
     * @param pszUbOut byte[] string character output
     * @return int 0 if successful
     */
    public int ub2String(int dwUbIn, int dwFlags, byte[] pszUbOut)
    {
        // Need to handle leading, bcd, invert and marks
        for (int i = 0; i < 8; i++)
        {
            byte byteTmp = (byte) ( (dwUbIn >> ( (7 - i) * 4)) & 0x0F);

            if (byteTmp < 10)
            { // 0 - 9
                pszUbOut[i] = (byte) (byteTmp + (byte) '0');
            }
            else
            { // A - F
                pszUbOut[i] = (byte) (byteTmp + ('A' - 10));
            }
        }
        pszUbOut[8] = 0x00;
        return 0;
    }

    /**
     * Convert the user bit hex string in pszUbIn to the numberic value in dwUbOut
     *
     * @param pszUbIn byte[] Hex string (like "A034C8D2")
     * @param dwFlags int TC2_flags, not currently used
     * @param dwUbOut int legacy numeric value
     * @return int hex string numeric value
     */
    public int ub2Value(byte[] pszUbIn, int dwFlags, int dwUbOut)
    {
        int i;

        // Need to handle leading, bcd and remove marks
        dwUbOut = 0;
        for (i = 0; i < 8; i++)
        {
            if (pszUbIn[i] >= '0' && pszUbIn[i] <= '9')
            {
                dwUbOut = dwUbOut << 4;
                dwUbOut += (pszUbIn[i] - '0');
            }
            else if ( (pszUbIn[i] & ~0x20) >= 'A' && (pszUbIn[i] & ~0x20) <= 'F')
            {
                dwUbOut = dwUbOut << 4;
                dwUbOut += ( (pszUbIn[i] & ~0x20) - 'A') + 10;
            }
            else if (pszUbIn[i] == 0)
            {
                // End of string
                break;
            }
            else
            {
                // Doing nothing will leave a 0
            }
        }
        return (int) dwUbOut;
    }

    /**
     * Return a short string representation of a time code type
     *
     * @param dwTcType long TC2_TCTYPE_xxx flag
     * @return String short name of time code type
     */
    public String getTCType(long dwTcType)
    {
        switch((int) dwTcType)
        {
            case TC2_TCTYPE_FILM:
                return ("24");
            case TC2_TCTYPE_NDF:
                return ("NDF");
            case TC2_TCTYPE_DF:
                return ("DF");
            case TC2_TCTYPE_PAL:
                return ("PAL");
            case TC2_TCTYPE_50:
                return ("50");
            case TC2_TCTYPE_5994:
                return ("5994");
            case TC2_TCTYPE_60:
                return ("60");
            case TC2_TCTYPE_NTSCFILM:
                return ("23.98");
            default:
                return ("ERR");
        }
    }
    
    public static int getTC2_SECOND_NDF() {
		return TC2_SECOND_NDF;
	}

	public static int getTC2_SECOND_DROP() {
		return TC2_SECOND_DROP;
	}

	public static int getTC2_SECOND_PAL() {
		return TC2_SECOND_PAL;
	}

	public static int getTC2_SECOND_FILM() {
		return TC2_SECOND_FILM;
	}

	public static int getTC2_SECOND_50() {
		return TC2_SECOND_50;
	}

	public static int getTC2_SECOND_5994() {
		return TC2_SECOND_5994;
	}

	public static int getTC2_SECOND_60() {
		return TC2_SECOND_60;
	}

	public static int getTC2_SECOND_100() {
		return TC2_SECOND_100;
	}
}
