package drastic.mCmdIF;

/**
 * <p>Title: VidUtil</p>
 *
 * <p>Description: QuickClipXO video audio utility structures and functions</p>
 *
 * <p>Copyright: Copyright  2003</p>
 *
 * <p>Company: Drastic Technologies Ltd.<br>
 * 523 The Queensway, Suite 102<br>
 * Toronto, ON, M8Y 1J7<br>
 * CANADA<br>
 * 416 255 5636<br>
 * fax 255 8780<br>
 * http://www.drastictech.com<br>
 * engineering@drastictech.com</p>
 *
 * @author not attributable
 * @version 1.0
 */
public class VidUtil
{
    public TCXlat tCXlat = new TCXlat();
    protected MEDIACMD Mcmd = new MEDIACMD();
    RateScale rateScale = new RateScale();

    /**
     * Rate and Scale values for supported video standards
     *
     * @param	dwGsVidStandard Requested video standard like GS_SIGFORM_NTSC,
     *		GS_SIGFORM_NTSC_CCIR, GS_SIGFORM_PAL, GS_SIGFORM_1035i_30X_260M
     * @param	pdwRate VVWVIDEO dwRate - 30000, 25, 2997
     * @param	pdwScale VVWVIDEO dwScale- 1001,  1,  100
     *
     * @return	0 if rate/scale set, else -1
     */
    RateScale vidRateScale(long dwGsVidStandard, long pdwRate, long pdwScale)
    {
        if(dwGsVidStandard == MEDIACMD.GS_SIGFORM_CUSTOM)
        {
            return null;
        }

        switch ((int)dwGsVidStandard)
        {
            case MEDIACMD.GS_SIGFORM_NTSC:
            case MEDIACMD.GS_SIGFORM_CCIR_NTSC:
                pdwScale = 1001; //100
                pdwRate = 30000; //2997;
                break;
            case MEDIACMD.GS_SIGFORM_PAL:
            case MEDIACMD.GS_SIGFORM_CCIR_PAL:
                pdwScale = 1;
                pdwRate = 25;
                break;
                /* # of Standard Definition standards */

                /* 2200x1125 raster, 1920x1035 production aperture (1888x1017 clean): */
            case MEDIACMD.GS_SIGFORM_1035i_30_260M: /*	(260M)				*/
                pdwScale = 1;
                pdwRate = 30;
                break;
            case MEDIACMD.GS_SIGFORM_1035i_30X_260M: /*	(260M)				*/
                pdwScale = 1001;
                pdwRate = 30000;
                break;

                /* 2200x1125 raster, 1920x1080 production aperture (1888x1062 clean): */
            case MEDIACMD.GS_SIGFORM_1080i_30: /*	(274M-1997 Table1 System 4)	*/
                pdwScale = 1;
                pdwRate = 30;
                break;
            case MEDIACMD.GS_SIGFORM_1080i_30X: /*	(274M-1997 Table1 System 5)	*/
                pdwScale = 1001;
                pdwRate = 30000;
                break;
            case MEDIACMD.GS_SIGFORM_1080i_25: /*	(274M-1997 Table1 System 6)	*/
                pdwScale = 1;
                pdwRate = 25;
                break;
            case MEDIACMD.GS_SIGFORM_1080i_24: /*	(274M-1997 Table1 System 6)	*/
                pdwScale = 1;
                pdwRate = 24;
                break;
            case MEDIACMD.GS_SIGFORM_1080i_24X: /*	(274M-1997 Table1 System 6)	*/
                pdwScale = 1001;
                pdwRate = 24000;
                break;
            case MEDIACMD.GS_SIGFORM_1080_30: /*	(274M-1997 Table1 System 7)	*/
                pdwScale = 1;
                pdwRate = 30;
                break;
            case MEDIACMD.GS_SIGFORM_1080_30X: /*	(274M-1997 Table1 System 8)	*/
                pdwScale = 1001;
                pdwRate = 30000;
                break;
            case MEDIACMD.GS_SIGFORM_1080_25: /*	(274M-1997 Table1 System 9)	*/
                pdwScale = 1;
                pdwRate = 25;
                break;
            case MEDIACMD.GS_SIGFORM_1080_24: /*	(274M-1997 Table1 System 10)	*/
                pdwScale = 1;
                pdwRate = 24;
                break;
            case MEDIACMD.GS_SIGFORM_1080_24X: /*	(274M-1997 Table1 System 11)	*/
                pdwScale = 1001;
                pdwRate = 24000;
                break;
                /* 1650x750 raster, 1280x720 production aperture (1248x702 clean): */
            case MEDIACMD.GS_SIGFORM_720_60: /*	(296M-1996 Table1 System 1)	*/
                pdwScale = 1;
                pdwRate = 60;
                break;
            case MEDIACMD.GS_SIGFORM_720_60X: /*	(296M-1996 Table1 System 2)	*/
                pdwScale = 1001;
                pdwRate = 60000;
                break;
            default:

                // Extact Rate
                pdwScale = 1;
                pdwRate = (dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_FRAMERATE);
                switch ((int)pdwRate)
                {
                    default:

                        // Leave it
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_14_98: //	       14
                        pdwScale = 1001;
                        pdwRate = 15000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_15: //		15
                        pdwScale = 1;
                        pdwRate = 15;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_23_98: //	        23
                        pdwScale = 1001;
                        pdwRate = 24000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_24: //		24
                        pdwScale = 1;
                        pdwRate = 24;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_25: //		25
                        pdwScale = 1;
                        pdwRate = 25;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_29_97: //	        29
                        pdwScale = 1001;
                        pdwRate = 30000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_30: //		30
                        pdwScale = 1;
                        pdwRate = 30;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_47_95: //	        47
                        pdwScale = 1001;
                        pdwRate = 48000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_48: //		48
                        pdwScale = 1;
                        pdwRate = 48;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_50: //		50
                        pdwScale = 1;
                        pdwRate = 50;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_59_94: //	        59
                        pdwScale = 1001;
                        pdwRate = 60000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_60: //		60
                        pdwScale = 1;
                        pdwRate = 60;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_71_93: //	        71
                        pdwScale = 1001;
                        pdwRate = 72000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_72: //		72
                        pdwScale = 1;
                        pdwRate = 72;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_100: //		100		// 0x64
                        pdwScale = 1;
                        pdwRate = 100;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_119_88: //	119		// 0x77
                        pdwScale = 1001;
                        pdwRate = 120000;
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_CUSTOM: //	0x100
                        return null;
                        // break;
                }
                break;
            }
            rateScale.dwRate = pdwRate;
            rateScale.dwScale = pdwScale;

            return rateScale;
        }

    /**
     * Number of frames per second for a video standard
     *
     * @param	dwGsVidStandard Requested video standard like GS_SIGFORM_NTSC,
     *		GS_SIGFORM_NTSC_CCIR, GS_SIGFORM_PAL, GS_SIGFORM_1035i_30X_260M
     * @param	dwRate Rate/Scale will override vid standard, set to 0 to ignore
     * @param	dwScale Rate/Scale will override vid standard, set to 0 to ignore
     *
     * @return	a floating point frames per second like 29.97xxxxxxx
     */
    public double vidFramesPerSecD(long dwGsVidStandard, long dwRate, long dwScale)
    {
        RateScale rs;
        if ((dwRate & dwScale) != 0)
        {
            return (double) dwRate / (double) dwScale;
        }

        rs = vidRateScale(dwGsVidStandard, dwRate, dwScale);
        if (rs != null)
        {
            return (double) rs.dwRate / (double) rs.dwScale;
        }
        return 30000.0 / 1001.0;
    }

    /**
     * Number of milliseconds per frame for a vid std
     *
     * @param	dwGsVidStandard Requested video standard like GS_SIGFORM_NTSC,
     *		GS_SIGFORM_NTSC_CCIR, GS_SIGFORM_PAL, GS_SIGFORM_1035i_30X_260M
     * @param	dwRate Rate/Scale will override vid standard, set to 0 to ignore
     * @param	dwScale Rate/Scale will override vid standard, set to 0 to ignore
     *
     * @return	floating point ms per frame like 33.3367xxxxx
     */
    public double vidMsPerFrameD(long dwGsVidStandard, long dwRate, long dwScale)
    {
        return 1000.0 / vidFramesPerSecD(dwGsVidStandard, dwRate, dwScale);
    }

    /**
     * Truncated milliseconds per frame for a vid std
     *
     * @param	dwGsVidStandard Requested video standard like GS_SIGFORM_NTSC,
     *		GS_SIGFORM_NTSC_CCIR, GS_SIGFORM_PAL, GS_SIGFORM_1035i_30X_260M
     * @param	dwRate Rate/Scale will override vid standard, set to 0 to ignore
     * @param	dwScale Rate/Scale will override vid standard, set to 0 to ignore
     *
     * @return	DWORD ms per frame like 33 or 40
     */
    public long vidMsPerFrame(long dwGsVidStandard, long dwRate, long dwScale)
    {
        double ddMsPerFrame = vidMsPerFrameD(dwGsVidStandard, dwRate, dwScale);
        return (long) (ddMsPerFrame + 0.5);
    }

    /**
     * Number of frames per second for a time code type
     *
     * @param	dwTCType Requested time code standard like TC2_TCTYPE_FILM,
     **		TC2_TCTYPE_NDF, TC2_TCTYPE_DF, TC2_TCTYPE_PAL, TC2_TCTYPE_50,
     *		TC2_TCTYPE_5994, TC2_TCTYPE_60, TC2_TCTYPE_NTSCFILM
     *
     * @return	a floating point frames per second like 29.97xxxxxxx
     */
    public double vidTCFramesPerSecD(long dwTCType)
    {
        switch ((int)dwTCType & TCXlat.TC2_TCTYPE_MASK)
        {
            case TCXlat.TC2_TCTYPE_FILM: // 0x00000001	// 24 fps
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_1080_24, 0, 0);
            case TCXlat.TC2_TCTYPE_NDF: // 0x00000002	// NTSC Non Drop Frame
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_1080i_30, 0, 0);
            default:
            case TCXlat.TC2_TCTYPE_DF: // 0x00000004	// NTSC Drop Frame
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_1080i_30X, 0, 0);
            case TCXlat.TC2_TCTYPE_PAL: // 0x00000008	// PAL
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_CCIR_PAL, 0, 0);
            case TCXlat.TC2_TCTYPE_50: // 0x00000010	// PAL 720p (double rate)
                return 50; // vidFramesPerSecD(GS_SIGFORM_720_50);
            case TCXlat.TC2_TCTYPE_5994: // 0x00000020	// NTSC 59.94fps 720p (NTSC DF double)
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_720_60X, 0, 0);
            case TCXlat.TC2_TCTYPE_60: // 0x00000040	// NTSC 60fps 720p (NTSC NDF double)
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_720_60, 0, 0);
            case TCXlat.TC2_TCTYPE_2398: // 0x00000084	// NTSC FILM 23.97
            case TCXlat.TC2_TCTYPE_NTSCFILM: // 0x00000080	// NTSC FILM 23.97
                return vidFramesPerSecD(MEDIACMD.GS_SIGFORM_1080_24X, 0, 0);

        }
    }

    /**
     * Return a time code type for a particular video standard.  Returns best
     * match, cannot determine DF/NDF or 24/23.98.
     *
     * @param dwGsVidStandard long a GS_SIGFORMFRAMERATE_xxx
     * @return long a TC2_TCTYPE_xxx
     */
    public long vidGetTCTypeSignalFormat(long dwGsVidStandard)
    {
        switch (((int)dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_FRAMERATE))
        {
            case MEDIACMD.GS_SIGFORMFRAMERATE_14_98: //	14
                return TCXlat.TC2_TCTYPE_DF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_15: //	15
                return TCXlat.TC2_TCTYPE_NDF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_23_98: //	23
                return TCXlat.TC2_TCTYPE_NTSCFILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_24: //	24
                return TCXlat.TC2_TCTYPE_FILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_25: //	25
                return TCXlat.TC2_TCTYPE_PAL;
            case MEDIACMD.GS_SIGFORMFRAMERATE_29_97: //	29
                return TCXlat.TC2_TCTYPE_DF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_30: //	30
                return TCXlat.TC2_TCTYPE_NDF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_47_95: //	47
                return TCXlat.TC2_TCTYPE_NTSCFILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_48: //	48
                return TCXlat.TC2_TCTYPE_FILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_50: //	50
                return TCXlat.TC2_TCTYPE_PAL;
            case MEDIACMD.GS_SIGFORMFRAMERATE_59_94: //	59
                return TCXlat.TC2_TCTYPE_DF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_60: //	60
                return TCXlat.TC2_TCTYPE_NDF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_71_93: //	71
                return TCXlat.TC2_TCTYPE_NTSCFILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_72: //	72
                return TCXlat.TC2_TCTYPE_FILM;
            case MEDIACMD.GS_SIGFORMFRAMERATE_100: //	100		// 0x64
                return TCXlat.TC2_TCTYPE_PAL;
            case MEDIACMD.GS_SIGFORMFRAMERATE_119_88: //119		// 0x77
                return TCXlat.TC2_TCTYPE_DF;
            case MEDIACMD.GS_SIGFORMFRAMERATE_CUSTOM: //0x100
                return TCXlat.TC2_TCTYPE_NDF;
        }
        return TCXlat.TC2_TCTYPE_NDF;
    }

    /**
     * Milliseconds per frame for a time code type
     *
     * @param	dwTCType Requested time code standard like TC2_TCTYPE_FILM,
     *		TC2_TCTYPE_NDF, TC2_TCTYPE_DF, TC2_TCTYPE_PAL, TC2_TCTYPE_50,
     *		TC2_TCTYPE_5994, TC2_TCTYPE_60, TC2_TCTYPE_NTSCFILM
     *
     * @return	floating point ms per frame like 33.3367xxxxx
     */
    public double vidTCMsPerFrameD(long dwTCType)
    {
        switch ((int)dwTCType & TCXlat.TC2_TCTYPE_MASK)
        {
            case TCXlat.TC2_TCTYPE_FILM: //0x00000001	// 24 fps
                return 41.666666666666666666666666666667;
            default:
            case TCXlat.TC2_TCTYPE_NDF: //0x00000002	// NTSC Non Drop Frame
            case TCXlat.TC2_TCTYPE_DF: //0x00000004	// NTSC Drop Frame
                return 33.366666666666666666666666666667;
            case TCXlat.TC2_TCTYPE_PAL: //0x00000008	// PAL
                return 40;
            case TCXlat.TC2_TCTYPE_50: //0x00000010	// PAL 720p (double rate)
                return 20;
            case TCXlat.TC2_TCTYPE_5994: //0x00000020	// NTSC 59.94fps 720p (NTSC DF double)
            case TCXlat.TC2_TCTYPE_60: //0x00000040	// NTSC 60fps 720p (NTSC NDF double)
                return 14.985014985014985014985014985015;
            case TCXlat.TC2_TCTYPE_2398: //0x00000084	// NTSC FILM 23.98
            case TCXlat.TC2_TCTYPE_NTSCFILM: //0x00000080	// NTSC FILM 23.98
                return 41.708333333333333333333333333333;
        }
    }

    /**
     * Truncated milliseconds per frame for a time code type
     *
     * @param	dwTCType Requested time code standard like TC2_TCTYPE_FILM,
     *		TC2_TCTYPE_NDF, TC2_TCTYPE_DF, TC2_TCTYPE_PAL, TC2_TCTYPE_50,
     *		TC2_TCTYPE_5994, TC2_TCTYPE_60, TC2_TCTYPE_NTSCFILM
     *
     * @return	DWORD ms per frame like 33 or 40
     */
    public long vidTCMsPerFrame(long dwTCType)
    {
        double ddMsPerFrame = vidTCMsPerFrameD(dwTCType);
        return (long) (ddMsPerFrame + 0.5);
    }

    /**
     * Get a standard size as decimal value (pre shift and mask)
     *
     * @param dwSize long the GS_SIGFORMSIZE_xxx
     * @return long the numeric value for that flag
     */
    public long vidGetSizeFromToken(long dwSize)
    {
        switch ((int)dwSize)
        {
            case MEDIACMD.GS_SIGFORMSIZE_240: //			0x01
                return 240;
            case MEDIACMD.GS_SIGFORMSIZE_243: //			0x02
                return 243;
            case MEDIACMD.GS_SIGFORMSIZE_288: //			0x03	// Gap 4
                return 288;
            case MEDIACMD.GS_SIGFORMSIZE_320: //			0x08
                return 320;
            case MEDIACMD.GS_SIGFORMSIZE_352: //			0x09
                return 352;
            case MEDIACMD.GS_SIGFORMSIZE_360: //			0x0a	// Gap 5
                return 360;
            case MEDIACMD.GS_SIGFORMSIZE_480: //			0x10
                return 480;
            case MEDIACMD.GS_SIGFORMSIZE_483: //			0x11
                return 483;
            case MEDIACMD.GS_SIGFORMSIZE_486: //			0x12	// Gap 1
                return 486;
            case MEDIACMD.GS_SIGFORMSIZE_496: //			0x14	// Gap 2
                return 496;
            case MEDIACMD.GS_SIGFORMSIZE_504: //			0x16
                return 504;
            case MEDIACMD.GS_SIGFORMSIZE_512: //			0x17	// Gap 2
                return 512;
            case MEDIACMD.GS_SIGFORMSIZE_576: //			0x1a
                return 576;
            case MEDIACMD.GS_SIGFORMSIZE_600: //			0x1b
                return 600;
            case MEDIACMD.GS_SIGFORMSIZE_608: //			0x1c	// Gap 3
                return 608;
            case MEDIACMD.GS_SIGFORMSIZE_640: //			0x20
                return 640;
            case MEDIACMD.GS_SIGFORMSIZE_720: //			0x21
                return 720;
            case MEDIACMD.GS_SIGFORMSIZE_768: //			0x22
                return 768;
            case MEDIACMD.GS_SIGFORMSIZE_800: //			0x23
                return 800;
            case MEDIACMD.GS_SIGFORMSIZE_864: //			0x24	// Gap 4
                return 864;
            case MEDIACMD.GS_SIGFORMSIZE_960: //			0x28
                return 960;
            case MEDIACMD.GS_SIGFORMSIZE_968: //			0x29	// Gap 9
                return 968;
            case MEDIACMD.GS_SIGFORMSIZE_1024: //			0x30
                return 1024;
            case MEDIACMD.GS_SIGFORMSIZE_1035: //			0x31
                return 1035;
            case MEDIACMD.GS_SIGFORMSIZE_1044: //			0x32
                return 1044;
            case MEDIACMD.GS_SIGFORMSIZE_1052: //			0x33	// Gap 5
                return 1052;
            case MEDIACMD.GS_SIGFORMSIZE_1080: //			0x38
                return 1080;
            case MEDIACMD.GS_SIGFORMSIZE_1088: //			0x39
                return 1088;
            case MEDIACMD.GS_SIGFORMSIZE_1096: //			0x3a	// Gap 5
                return 1096;
            case MEDIACMD.GS_SIGFORMSIZE_1152: //			0x40
                return 1152;
            case MEDIACMD.GS_SIGFORMSIZE_1200: //			0x41	// Gap 8
                return 1200;
            case MEDIACMD.GS_SIGFORMSIZE_1280: //			0x48	// Gap 8
                return 1280;
            case MEDIACMD.GS_SIGFORMSIZE_1536: //			0x50
                return 1536;
            case MEDIACMD.GS_SIGFORMSIZE_1556: //			0x51	// Gap 8
                return 1556;
            case MEDIACMD.GS_SIGFORMSIZE_1600: //			0x58
                return 1600;
            case MEDIACMD.GS_SIGFORMSIZE_1920: //			0x59	// Gap 7
                return 1920;
            case MEDIACMD.GS_SIGFORMSIZE_2048: //			0x60	// Gap 8
                return 2048;
            case MEDIACMD.GS_SIGFORMSIZE_2650: //			0x68	// Gap 8
                return 2650;
            case MEDIACMD.GS_SIGFORMSIZE_4096: //			0x80
                return 4096;
        }
        return -1;
    }

    /**
     * Get a standard size token from a decimal value (no shift returned)
     *
     * @param dwSize long The target size
     * @return long the GS_SIGFORMSIZE_xxx for that size or -1 if error
     */
    public long vidGetTokenFromSize(long dwSize)
    {
        if (dwSize < 240)
        {
            return -1;
        }
        if (dwSize == 240)
        {
            return MEDIACMD.GS_SIGFORMSIZE_240; //			0x01
        }
        else if (dwSize <= 243)
        {
            return MEDIACMD.GS_SIGFORMSIZE_243; //			0x02
        }
        else if (dwSize <= 288)
        {
            return MEDIACMD.GS_SIGFORMSIZE_288; //			0x03	// Gap 4
        }
        else if (dwSize <= 320)
        {
            return MEDIACMD.GS_SIGFORMSIZE_320; //			0x08
        }
        else if (dwSize <= 352)
        {
            return MEDIACMD.GS_SIGFORMSIZE_352; //			0x09
        }
        else if (dwSize <= 360)
        {
            return MEDIACMD.GS_SIGFORMSIZE_360; //			0x0a	// Gap 5
        }
        else if (dwSize <= 480)
        {
            return MEDIACMD.GS_SIGFORMSIZE_480; //			0x10
        }
        else if (dwSize <= 483)
        {
            return MEDIACMD.GS_SIGFORMSIZE_483; //			0x11
        }
        else if (dwSize <= 486)
        {
            return MEDIACMD.GS_SIGFORMSIZE_486; //			0x12	// Gap 1
        }
        else if (dwSize <= 496)
        {
            return MEDIACMD.GS_SIGFORMSIZE_496; //			0x14	// Gap 2
        }
        else if (dwSize <= 504)
        {
            return MEDIACMD.GS_SIGFORMSIZE_504; //			0x16
        }
        else if (dwSize <= 512)
        {
            return MEDIACMD.GS_SIGFORMSIZE_512; //			0x17	// Gap 2
        }
        else if (dwSize <= 576)
        {
            return MEDIACMD.GS_SIGFORMSIZE_576; //			0x1a
        }
        else if (dwSize <= 600)
        {
            return MEDIACMD.GS_SIGFORMSIZE_600; //			0x1b
        }
        else if (dwSize <= 608)
        {
            return MEDIACMD.GS_SIGFORMSIZE_608; //			0x1c	// Gap 3
        }
        else if (dwSize <= 640)
        {
            return MEDIACMD.GS_SIGFORMSIZE_640; //			0x20
        }
        else if (dwSize <= 720)
        {
            return MEDIACMD.GS_SIGFORMSIZE_720; //			0x21
        }
        else if (dwSize <= 768)
        {
            return MEDIACMD.GS_SIGFORMSIZE_768; //			0x22
        }
        else if (dwSize <= 800)
        {
            return MEDIACMD.GS_SIGFORMSIZE_800; //			0x23
        }
        else if (dwSize <= 864)
        {
            return MEDIACMD.GS_SIGFORMSIZE_864; //			0x24	// Gap 4
        }
        else if (dwSize <= 960)
        {
            return MEDIACMD.GS_SIGFORMSIZE_960; //			0x28
        }
        else if (dwSize <= 968)
        {
            return MEDIACMD.GS_SIGFORMSIZE_968; //			0x29	// Gap 9
        }
        else if (dwSize <= 1024)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1024; //			0x30
        }
        else if (dwSize <= 1036)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1035; //			0x31
        }
        else if (dwSize <= 1044)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1044; //			0x32
        }
        else if (dwSize <= 1052)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1052; //			0x33	// Gap 5
        }
        else if (dwSize <= 1080)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1080; //			0x38
        }
        else if (dwSize <= 1088)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1088; //			0x39
        }
        else if (dwSize <= 1096)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1096; //			0x3a	// Gap 5
        }
        else if (dwSize <= 1152)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1152; //			0x40
        }
        else if (dwSize <= 1200)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1200; //			0x41	// Gap 8
        }
        else if (dwSize <= 1280)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1280; //			0x48	// Gap 8
        }
        else if (dwSize <= 1536)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1536; //			0x50
        }
        else if (dwSize <= 1556)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1556; //			0x51	// Gap 8
        }
        else if (dwSize == 1600)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1600; //			0x58
        }
        else if (dwSize == 1920)
        {
            return MEDIACMD.GS_SIGFORMSIZE_1920; //			0x59	// Gap 7
        }
        else if (dwSize == 2048)
        {
            return MEDIACMD.GS_SIGFORMSIZE_2048; //			0x60	// Gap 8
        }
        else if (dwSize == 2650)
        {
            return MEDIACMD.GS_SIGFORMSIZE_2650; //			0x68	// Gap 8
        }
        else if (dwSize == 4096)
        {
            return MEDIACMD.GS_SIGFORMSIZE_4096; //			0x80
        }
        else
        {
            return -1;
        }
    }

    /**
     * Standard image sizes for a particular video
     *	standard.  These are official sizes and do not take into
     *	account viewgraphics hd size bug in 1035
     *
     * @param	dwGsVidStandard Requested video standard like GS_SIGFORM_NTSC,
     *		GS_SIGFORM_NTSC_CCIR, GS_SIGFORM_PAL, GS_SIGFORM_1035i_30X_260M
     * @param	pdwSizeX The width of the standard
     * @param	pdwSizeY The height of the standard
     * @param	fFieldSize Return height of field instead of frame (half for
     *		interlaced standards, same for progressive)
     *
     * @return	0 if info returned, else -1
     */
    public long vidImageSize(long dwGsVidStandard, long pdwSizeX[], long pdwSizeY[], boolean fFieldSize)
    {
        switch ((int)dwGsVidStandard)
        {
            case MEDIACMD.GS_SIGFORM_NTSC:
                pdwSizeX[0] = 640;
                pdwSizeY[0] = 480;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_CCIR_NTSC:
                pdwSizeX[0] = 720;
                pdwSizeY[0] = 486;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_PAL:
                pdwSizeX[0] = 720;
                pdwSizeY[0] = 576;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_CCIR_PAL:
                pdwSizeX[0] = 720;
                pdwSizeY[0] = 576;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_1035i_30_260M: /*	(260M)				*/
            case MEDIACMD.GS_SIGFORM_1035i_30X_260M: /*	(260M)				*/
                pdwSizeX[0] = 1920;
                pdwSizeY[0] = 1035;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_1080i_30: /*	(274M-1997 Table1 System 4)	*/
            case MEDIACMD.GS_SIGFORM_1080i_30X: /*	(274M-1997 Table1 System 5)	*/
            case MEDIACMD.GS_SIGFORM_1080i_25: /*	(274M-1997 Table1 System 6)	*/
            case MEDIACMD.GS_SIGFORM_1080i_24: /*	(274M-1997 Table1 System 6)	*/
            case MEDIACMD.GS_SIGFORM_1080i_24X: /*	(274M-1997 Table1 System 6)	*/
                pdwSizeX[0] = 1920;
                pdwSizeY[0] = 1080;
                if (fFieldSize)
                {
                    pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                }
                break;
            case MEDIACMD.GS_SIGFORM_1080_30: /*	(274M-1997 Table1 System 7)	*/
            case MEDIACMD.GS_SIGFORM_1080_30X: /*	(274M-1997 Table1 System 8)	*/
            case MEDIACMD.GS_SIGFORM_1080_25: /*	(274M-1997 Table1 System 9)	*/
            case MEDIACMD.GS_SIGFORM_1080_24: /*	(274M-1997 Table1 System 10)	*/
            case MEDIACMD.GS_SIGFORM_1080_24X: /*	(274M-1997 Table1 System 11)	*/
                pdwSizeX[0] = 1920;
                pdwSizeY[0] = 1080; // progressive
                break;
            case MEDIACMD.GS_SIGFORM_720_60: /*	(296M-1996 Table1 System 1)	*/
            case MEDIACMD.GS_SIGFORM_720_30:
            case MEDIACMD.GS_SIGFORM_720_60X: /*	(296M-1996 Table1 System 2)	*/
            case MEDIACMD.GS_SIGFORM_720_30X:
                pdwSizeX[0] = 1280;
                pdwSizeY[0] = 720; // progressive
                break;
            case MEDIACMD.GS_SIGFORM_DCIN_2048sf_24X: //	(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_23_98)
            case MEDIACMD.GS_SIGFORM_DCIN_2048sf_24: //	(GS_SIGFORMSIZE_2048x108 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
            case MEDIACMD.GS_SIGFORM_DCIN_2048_24X: //	(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
            case MEDIACMD.GS_SIGFORM_DCIN_2048_24: //		(GS_SIGFORMSIZE_2048x108 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
                pdwSizeX[0] = 2048;
                pdwSizeY[0] = 1080;
                break;

            case MEDIACMD.GS_SIGFORM_FILM_2048_15X: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_14_98)
                // deprecated case GS_SIGFORM_FILM_2048_14://		GS_SIGFORM_FILM_2048_15X	// Deprecated, do not use
            case MEDIACMD.GS_SIGFORM_FILM_2048_15: //		(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_15)
            case MEDIACMD.GS_SIGFORM_FILM_2048_24X: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
            case MEDIACMD.GS_SIGFORM_FILM_2048_24: //		(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
            case MEDIACMD.GS_SIGFORM_FILM_2048sf_24: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
                pdwSizeX[0] = 2048;
                pdwSizeY[0] = 1536;
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_15X: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_14_98)
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_15: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_15)
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_24X: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_24: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536sf_24: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
                pdwSizeX[0] = 2048;
                pdwSizeY[0] = 1556;
                break;

            case MEDIACMD.GS_SIGFORM_FILM_4096_3112sf_5: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_5)
                pdwSizeX[0] = 4096;
                pdwSizeY[0] = 3112;
                break;
            default:
                pdwSizeX[0] = vidGetSizeFromToken( (dwGsVidStandard &
                    MEDIACMD.GS_SIGFORMMASK_HORIZONTAL) >> MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL);
                pdwSizeY[0] = vidGetSizeFromToken( (dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_VERTICAL) >> MEDIACMD.GS_SIGFORMSHIFT_VERTICAL);
                if (pdwSizeX[0] == -1 || pdwSizeY[0] == -1)
                {
                    return -1;
                }
                if (fFieldSize)
                {
                    if ( (dwGsVidStandard & MEDIACMD.GS_SIGFORMTYPE_INTERLACED) == MEDIACMD.GS_SIGFORMTYPE_INTERLACED)
                    {
                        pdwSizeY[0] = (pdwSizeY[0]) >> 1;
                    }
                }
                break;
        }
        return 0;
    }

    /**
     * Returns the nearest signal format for a particular Rate/Scale, size and p/i flag.
     *
     * @param dwRate long fps == dwRate / dwScale (eg 25/1 or 30000/1001)
     * @param dwScale long fps == dwRate / dwScale (eg 25/1 or 30000/1001)
     * @param dwWidth long width of the video frame
     * @param dwHeight long height of the videof rame
     * @param fProgressive boolean is is progressive (or segmented frame)
     * @return long the nearest signal format (GS_SIGFORM_xxx)
     */
    public long vidGetSignalFormat(long dwRate, long dwScale, long dwWidth, long dwHeight, boolean fProgressive)
    {
        long dwAbsHeight = Math.abs( (long) dwHeight);
        double ddFrameRate = (double) dwRate / (double) dwScale;
        // Film
        if (ddFrameRate >= 14.0 && ddFrameRate < 15.0)
        {
            if (dwAbsHeight == 1536)
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_1536_15X;
            }
            else
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_15X;
            }
        }
        if (ddFrameRate >= 15.0 && ddFrameRate < 16.0)
        {
            if (dwAbsHeight == 1536)
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_1536_15;
            }
            else
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_15;
            }
        }
        // PAL
        if (ddFrameRate > 24.9 && ddFrameRate < 25.1)
        {
            if (dwWidth == 640)
            {
                return MEDIACMD.GS_SIGFORM_PAL;
            }
            if (dwAbsHeight >= 1035 && dwAbsHeight <= 1036)
            {
                // Pal production hi def not supported yet
                // @note Need define for 1035 pal rate
                return MEDIACMD.GS_SIGFORM_1080i_25;
            }
            if (dwAbsHeight == 1080)
            {
                if (fProgressive)
                {
                    return MEDIACMD.GS_SIGFORM_1080_25;
                }
                else
                {
                    return MEDIACMD.GS_SIGFORM_1080i_25;
                }
            }
            return MEDIACMD.GS_SIGFORM_CCIR_PAL;
        }
        // FILM
        if (ddFrameRate >= 24.0 && ddFrameRate < 25.0)
        {
            if (dwAbsHeight == 1080)
            {
                if (dwWidth == 2048)
                {
                    if (fProgressive)
                    {
                        return MEDIACMD.GS_SIGFORM_DCIN_2048_24;
                    }
                    else
                    {
                        return MEDIACMD.GS_SIGFORM_DCIN_2048sf_24;
                    }
                }
                else
                {
                    if (fProgressive)
                    {
                        return MEDIACMD.GS_SIGFORM_1080_24;
                    }
                    else
                    {
                        return MEDIACMD.GS_SIGFORM_1080i_24;
                    }
                }
            }
            else if (dwAbsHeight == 1536)
            {
                if (fProgressive)
                {
                    return MEDIACMD.GS_SIGFORM_FILM_2048_1536_24;
                }
                else
                {
                    return MEDIACMD.GS_SIGFORM_FILM_2048_1536sf_24;
                }
            }
            else if (dwAbsHeight == 1556)
            {
                if (fProgressive)
                {
                    return MEDIACMD.GS_SIGFORM_FILM_2048_24;
                }
                else
                {
                    return MEDIACMD.GS_SIGFORM_FILM_2048sf_24;
                }
            }
        }

        // NTSCFILM
        if (ddFrameRate >= 23.0 && ddFrameRate < 24.0)
        {
            if (dwAbsHeight == 1080)
            {
                if (dwWidth == 2048)
                {
                    if (fProgressive)
                    {
                        return MEDIACMD.GS_SIGFORM_DCIN_2048_24;
                    }
                    else
                    {
                        return MEDIACMD.GS_SIGFORM_DCIN_2048sf_24;
                    }
                }
                else
                {
                    if (fProgressive)
                    {
                        return MEDIACMD.GS_SIGFORM_1080_24;
                    }
                    else
                    {
                        return MEDIACMD.GS_SIGFORM_1080i_24;
                    }
                }
            }
            else if (dwAbsHeight == 1536)
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_1536_24X;
            }
            else if (dwAbsHeight == 1556)
            {
                return MEDIACMD.GS_SIGFORM_FILM_2048_24X;
            }
        }

        // 720 HD
        if (ddFrameRate >= 60.0)
        {
            if (dwAbsHeight == 720)
            {
                return MEDIACMD.GS_SIGFORM_720_60;
            }
            return MEDIACMD.GS_SIGFORM_CUSTOM;
        }
        else if (ddFrameRate >= 59.0)
        {
            if (dwAbsHeight == 720)
            {
                return MEDIACMD.GS_SIGFORM_720_60X;
            }
            return MEDIACMD.GS_SIGFORM_CUSTOM;
        }

        // NTSC
        if (ddFrameRate > 29.0 && ddFrameRate < 30.0)
        {
            if (dwAbsHeight >= 1035 && dwAbsHeight <= 1036)
            {
                return MEDIACMD.GS_SIGFORM_1035i_30X_260M;
            }
            if (dwAbsHeight > 1036)
            {
                return MEDIACMD.GS_SIGFORM_1080i_30X;
            }
            if (dwAbsHeight == 720)
            {
                return MEDIACMD.GS_SIGFORM_720_30X;
            }
            if (dwWidth == 640 || dwWidth == 320)
            {
                return MEDIACMD.GS_SIGFORM_NTSC;
            }
            return MEDIACMD.GS_SIGFORM_CCIR_NTSC;
        }
        if (ddFrameRate >= 30.0 && ddFrameRate < 30.9)
        {
            if (dwAbsHeight >= 1035 && dwAbsHeight <= 1036)
            {
                return MEDIACMD.GS_SIGFORM_1035i_30_260M;
            }
            if (dwAbsHeight > 1036)
            {
                return MEDIACMD.GS_SIGFORM_1080i_30;
            }
            if (dwAbsHeight == 720)
            {
                return MEDIACMD.GS_SIGFORM_720_30;
            }
            if (dwWidth == 640 || dwWidth == 320)
            {
                return MEDIACMD.GS_SIGFORM_NTSC;
            }
            return MEDIACMD.GS_SIGFORM_CCIR_NTSC;
        }

        long dwWidthToken = vidGetTokenFromSize(dwWidth);
        long dwHeightToken = vidGetTokenFromSize(dwAbsHeight);
        if (dwWidthToken == -1 || dwHeightToken == -1)
        {
            return MEDIACMD.GS_SIGFORM_CUSTOM;
        }
        // Still not sure about this calc
        long dwFrameRate = (long)ddFrameRate;
        if (Math.abs(ddFrameRate - (double)dwFrameRate) > 0.8)
        {
            // Less the .94
            ddFrameRate -= 0.8; // Get closer to 0
        }
        dwFrameRate = (long) ddFrameRate;

        if (dwAbsHeight == 480 || dwAbsHeight == 486 || dwAbsHeight == 512 || dwAbsHeight == 576 || dwAbsHeight == 608)
        {
            // Assume interlaced
            return (dwWidthToken << MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL) | (dwHeightToken << MEDIACMD.GS_SIGFORMSHIFT_VERTICAL) |
                (dwFrameRate & MEDIACMD.GS_SIGFORMMASK_FRAMERATE) | MEDIACMD.GS_SIGFORMTYPE_INTERLACED;
        }
        if (dwAbsHeight == 1036 || dwAbsHeight == 1035 || dwAbsHeight == 720)
        {
            // Assume progressive
            return (dwWidthToken << MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL) | (dwHeightToken << MEDIACMD.GS_SIGFORMSHIFT_VERTICAL) |
                (dwFrameRate & MEDIACMD.GS_SIGFORMMASK_FRAMERATE) | MEDIACMD.GS_SIGFORMTYPE_PROGRESSIVE;
        }
        if (dwFrameRate == 23 || dwFrameRate == 24 || dwFrameRate == 47 || dwFrameRate == 48)
        {
            // Assume segmented frame
            return (dwWidthToken << MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL) | (dwHeightToken << MEDIACMD.GS_SIGFORMSHIFT_VERTICAL) |
                (dwFrameRate & MEDIACMD.GS_SIGFORMMASK_FRAMERATE) | MEDIACMD.GS_SIGFORMTYPE_SEGMENTEDFRAME;
        }
        if (dwFrameRate > 60 && dwFrameRate < 100)
        {
            // Assume progressive for 71/72 etc
            return (dwWidthToken << MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL) | (dwHeightToken << MEDIACMD.GS_SIGFORMSHIFT_VERTICAL) |
                (dwFrameRate & MEDIACMD.GS_SIGFORMMASK_FRAMERATE) | MEDIACMD.GS_SIGFORMTYPE_PROGRESSIVE;
        }
        // Prog/Seg/Inter unknown
        return (dwWidthToken << MEDIACMD.GS_SIGFORMSHIFT_HORIZONTAL) | (dwHeightToken << MEDIACMD.GS_SIGFORMSHIFT_VERTICAL) |
            (dwFrameRate & MEDIACMD.GS_SIGFORMMASK_FRAMERATE);
    }

    /**
     * Returns the nearest TC Type for a particular frame rate
     *
     * @param ddFrameRate double standard frames per second like 23.98, 29.97, etc
     * @return long closes TC2_TCTYPE_
     */
    public long vidGetTCType(double ddFrameRate)
    {
        if (ddFrameRate >= 14.0 && ddFrameRate <= 15.5)
        {
            ddFrameRate *= 2.0; // 29.97/30
        }
        if (ddFrameRate >= 47 && ddFrameRate <= 48.5)
        {
            ddFrameRate /= 2.0; // 24/23
        }
        if (ddFrameRate >= 71 && ddFrameRate <= 72.5)
        {
            ddFrameRate /= 3.0; // 24/23
        }
        if (ddFrameRate >= 100 && ddFrameRate <= 120.0)
        {
            ddFrameRate /= 4.0; // 25/29.97
        }
        if (ddFrameRate < 23.97 || ddFrameRate > 60.0)
        {
            // Has to be custom so default to NDF
            return TCXlat.TC2_TCTYPE_NDF;
        }
        // PAL
        if (ddFrameRate > 24.9 && ddFrameRate < 25.1)
        {
            return TCXlat.TC2_TCTYPE_PAL;
        }
        // NTSCFILM
        if (ddFrameRate >= 23.97 && ddFrameRate < 24.0)
        {
            return TCXlat.TC2_TCTYPE_NTSCFILM;
        }
        // FILM
        if (ddFrameRate >= 24.0 && ddFrameRate < 25.0)
        {
            return TCXlat.TC2_TCTYPE_FILM;
        }

        // 720 HD
        if (ddFrameRate >= 60.0)
        {
            return TCXlat.TC2_TCTYPE_60;
        }
        else if (ddFrameRate >= 59.0)
        {
            return TCXlat.TC2_TCTYPE_5994;
        }

        // NTSC
        if (ddFrameRate > 29.0 && ddFrameRate < 30.0)
        {
            return TCXlat.TC2_TCTYPE_DF;
        }
        if (ddFrameRate >= 30.0 && ddFrameRate < 30.9)
        {
            return TCXlat.TC2_TCTYPE_NDF;
        }

        return TCXlat.TC2_TCTYPE_NDF;
    }

    /**
     * PORT FROM C++:  For Reference Only - Do Not Use
     * Return a string description of a GS_SIGFORM_xx video standard.
     *
     * @param dwGsVidStandard long the GS_SIGFORM_xx value
     * @param szDesc String the string name
     * @return long 0 or not supported
     */
    public long vidSignalFormatToStr20(long dwGsVidStandard, String szDesc)
    {
	if(dwGsVidStandard == MEDIACMD.GS_SIGFORM_CUSTOM)
        {
            return MEDIACMD.GS_NOT_SUPPORTED;
        }

        switch ((int)dwGsVidStandard)
        {
            case MEDIACMD.GS_SIGFORM_NTSC:
                // strcpy(szDesc, "NTSC");
                szDesc = "NTSC";
                break;
            case MEDIACMD.GS_SIGFORM_CCIR_NTSC:
                // strcpy(szDesc, "NTSC-CCIR");
                szDesc = "NTSC-CCIR";
                break;
            case MEDIACMD.GS_SIGFORM_PAL:
                // strcpy(szDesc, "PAL");
                szDesc = "PAL";
                break;
            case MEDIACMD.GS_SIGFORM_CCIR_PAL:
                // strcpy(szDesc, "PAL-CCIR");
                szDesc = "PAL-CCIR";
                break;
            case MEDIACMD.GS_SIGFORM_1035i_30_260M: /*	(260M)				*/
                // strcpy(szDesc, "1035x1920i@30");
                szDesc = "1035x1920i@30";
                break;
            case MEDIACMD.GS_SIGFORM_1035i_30X_260M: /*	(260M)				*/
                // strcpy(szDesc, "1035x1920i@29.97");
                szDesc = "1035x1920i@29.97";
                break;
            case MEDIACMD.GS_SIGFORM_1080i_30: /*	(274M-1997 Table1 System 4)	*/
                // strcpy(szDesc, "1080x1920i@30");
                szDesc = "1080x1920i@30";
                break;
            case MEDIACMD.GS_SIGFORM_1080i_30X: /*	(274M-1997 Table1 System 5)	*/
                // strcpy(szDesc, "1080x1920i@29.97");
                szDesc = "1080x1920i@29.97";
                break;
            case MEDIACMD.GS_SIGFORM_1080i_25: /*	(274M-1997 Table1 System 6)	*/
                // strcpy(szDesc, "1080x1920i@25");
                szDesc = "1080x1920i@25";
                break;
            case MEDIACMD.GS_SIGFORM_1080i_24: /*	(274M-1997 Table1 System 6)	*/
                // strcpy(szDesc, "1080x1920i@24");
                szDesc = "1080x1920i@24";
                break;
            case MEDIACMD.GS_SIGFORM_1080i_24X: /*	(274M-1997 Table1 System 6)	*/
                // strcpy(szDesc, "1080x1920i@23.98");
                szDesc = "1080x1920i@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_1080_30: /*	(274M-1997 Table1 System 7)	*/
                // strcpy(szDesc, "1080x1920P@30");
                szDesc = "1080x1920P@30";
                break;
            case MEDIACMD.GS_SIGFORM_1080_30X: /*	(274M-1997 Table1 System 8)	*/
                // strcpy(szDesc, "1080x1920P@29.97");
                szDesc = "1080x1920P@29.97";
                break;
            case MEDIACMD.GS_SIGFORM_1080_25: /*	(274M-1997 Table1 System 9)	*/
                // strcpy(szDesc, "1080x1920P@25");
                szDesc = "1080x1920P@25";
                break;
            case MEDIACMD.GS_SIGFORM_1080_24: /*	(274M-1997 Table1 System 10)	*/
                // strcpy(szDesc, "1080x1920P@24");
                szDesc = "1080x1920P@24";
                break;
            case MEDIACMD.GS_SIGFORM_1080_24X: /*	(274M-1997 Table1 System 11)	*/
                // strcpy(szDesc, "1080x1920P@23.98");
                szDesc = "1080x1920P@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_720_60: /*	(296M-1996 Table1 System 1)	*/
                // strcpy(szDesc, "1280x720P@60");
                szDesc = "1280x720P@60";
                break;
            case MEDIACMD.GS_SIGFORM_720_60X: /*	(296M-1996 Table1 System 2)	*/
                // strcpy(szDesc, "1280x720P@59.94");
                szDesc = "1280x720P@59.94";
                break;
            case MEDIACMD.GS_SIGFORM_720_30:
                // strcpy(szDesc, "1280x720P@30");
                szDesc = "1280x720P@30";
                break;
            case MEDIACMD.GS_SIGFORM_720_30X:
                // strcpy(szDesc, "1280x720P@29.97");
                szDesc = "1280x720P@29.97";
                break;
            case MEDIACMD.GS_SIGFORM_DCIN_2048sf_24X: //	(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_23_98)
                // strcpy(szDesc, "2048x1080sf@23.98");
                szDesc = "2048x1080sf@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_DCIN_2048sf_24: //	(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1080sf@24");
                szDesc = "2048x1080sf@24";
                break;
            case MEDIACMD.GS_SIGFORM_DCIN_2048_24X: //	(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
                // strcpy(szDesc, "2048x1080@23.98");
                szDesc = "2048x1080@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_DCIN_2048_24: //		(GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1080@24");
                szDesc = "2048x1080@24";
                break;

            case MEDIACMD.GS_SIGFORM_FILM_2048_15X: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_14_98)
                // strcpy(szDesc, "2048x1556@14.98");
                szDesc = "2048x1556@14.98";
                break;
                // deprecated case GS_SIGFORM_FILM_2048_14://		GS_SIGFORM_FILM_2048_15X	// Deprecated, do not use
            case MEDIACMD.GS_SIGFORM_FILM_2048_15: //		(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_15)
                // strcpy(szDesc, "2048x1556@15");
                szDesc = "2048x1556@15";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_24X: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
                // strcpy(szDesc, "2048x1556@23.98");
                szDesc = "2048x1556@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_24: //		(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1556@24");
                szDesc = "2048x1556@24";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048sf_24: //	(GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1556sf@24");
                szDesc = "2048x1556sf@24";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_15X: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_14_98)
                // strcpy(szDesc, "2048x1536@14.98");
                szDesc = "2048x1536@14.98";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_15: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_15)
                // strcpy(szDesc, "2048x1536@15");
                szDesc = "2048x1536@15";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_24X: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98)
                // strcpy(szDesc, "2048x1536@23.98");
                szDesc = "2048x1536@23.98";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536_24: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1536@24");
                szDesc = "2048x1536@24";
                break;
            case MEDIACMD.GS_SIGFORM_FILM_2048_1536sf_24: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24)
                // strcpy(szDesc, "2048x1536sf@24");
                szDesc = "2048x1536sf@24";
                break;

            case MEDIACMD.GS_SIGFORM_FILM_4096_3112sf_5: //		(GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_5)
                // strcpy(szDesc, "4096x3112@5");
                szDesc = "4096x3112@5";
                break;
            default:
                if((((int) dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_FRAMETYPE) >> MEDIACMD.GS_SIGFORMSHIFT_FRAMETYPE)
                    == MEDIACMD.GS_SIGFORMTYPE_INTERLACED) // (1 << GS_SIGFORMSHIFT_FRAMETYPE)
                {
                }
                else if((((int) dwGsVidStandard &
                    MEDIACMD.GS_SIGFORMMASK_FRAMETYPE) >> MEDIACMD.GS_SIGFORMSHIFT_FRAMETYPE)
                    == MEDIACMD.GS_SIGFORMTYPE_PROGRESSIVE) // (2 << GS_SIGFORMSHIFT_FRAMETYPE)
                {
                }
                else if((((int) dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_FRAMETYPE) >> MEDIACMD.GS_SIGFORMSHIFT_FRAMETYPE)
                    == MEDIACMD.GS_SIGFORMTYPE_SEGMENTEDFRAME) // (4 << GS_SIGFORMSHIFT_FRAMETYPE)
                {
                }
                else
                {
                }

                switch ((int)dwGsVidStandard & MEDIACMD.GS_SIGFORMMASK_FRAMERATE)
                {
                    case MEDIACMD.GS_SIGFORMFRAMERATE_14_98: //	14
                        szDesc = m_strcat(szDesc, "14.98");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_15: //	15
                        szDesc = m_strcat(szDesc, "15");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_23_98: //	23
                        szDesc = m_strcat(szDesc, "23.98");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_24: //	24
                        szDesc = m_strcat(szDesc, "24");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_25: //	25
                        szDesc = m_strcat(szDesc, "25");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_29_97: //	29
                        szDesc = m_strcat(szDesc, "29.97");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_30: //	30
                        szDesc = m_strcat(szDesc, "30");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_47_95: //	47
                        szDesc = m_strcat(szDesc, "47.95");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_48: //	48
                        szDesc = m_strcat(szDesc, "48");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_50: //	50
                        szDesc = m_strcat(szDesc, "50");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_59_94: //	59
                        szDesc = m_strcat(szDesc, "59.94");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_60: //	60
                        szDesc = m_strcat(szDesc, "60");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_71_93: //	71
                        szDesc = m_strcat(szDesc, "71.93");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_72: //	72
                        szDesc = m_strcat(szDesc, "72");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_100: //	100		// 0x64
                        szDesc = m_strcat(szDesc, "100");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_119_88: //119		// 0x77
                        szDesc = m_strcat(szDesc, "119.88");
                        break;
                    case MEDIACMD.GS_SIGFORMFRAMERATE_CUSTOM: //0x100
                        szDesc = m_strcat(szDesc, "Custom");
                        break;
                }
                break;
        }
        return 0;
    }

    /**
     * PORT FROM C++:  For Reference Only - Do Not Use
     * Concatinates
     *
     * @param szDest String
     * @param szValue String
     * @return String
     */
    String m_strcat(String szDest, String szValue)
    {
        if (szDest.length() > 0)
        {
            szDest = szDest.concat(szValue);
        }
        else
        {
            szDest = szValue;
        }
        return szDest;
    }

    /**
     * <p>Title: RateScale</p>
     *
     * <p>Description: Stores the Rate & Scale</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    class RateScale
    {
        long dwRate = 0;
        long dwScale = 0;
    }
}


