package main;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import packetNetworking.*;
import packets.Clip;
import packets.EDLPacket;
import packets.FPSPacket;
import packets.ListPacket;
import packets.PlayPacket;
import packets.TCPacket;

import drastic.mCmdIF.MediaCmdIF;
import drastic.mCmdIF.TCXlat;

public class Main extends Client {
	
	public static MediaCmdIF mci;
	boolean isConnectedToMCR = false;
	
	private int fps = 25;
	
	boolean hasGeneratedList = false;
	String configPath;
	public String edlPath;
	String edlName;
	ListPacket lc;
	TimeCodeTransmitterThread timeCodeTransmitter;
	
	public Main() {
		//super(new byte[] {(byte)129, (byte)241, (byte)3, (byte)138}, 4444);
		super(null, 0);
		
		configPath = "config.txt";
		edlPath = "edl_file.edl";
		edlName = "edl";
		
		new Gui(this);
		
	}
	
	public void disconnectMCR() {
		if(isConnectedToMCR) mci.Disconnect(true);
	}
	
	public void ProcessPacket(Packet pck) {
		
		super.ProcessPacket(pck);
		if(pck instanceof PlayPacket) {
			PlayPacket pc = (PlayPacket)pck;
			//MediaCmdIF.ClipInfo ci = new MediaCmdIF.ClipInfo(pc.text);
			String tc = pc.fromTC;
			int hundreds = TCToFrames(tc);
			tc = framesToTC(hundreds);
			long startFrame = (long)TCToFrames(tc);
			long stopFrame = (long)TCToFrames(pc.toTC);
			mci.PlayClipFromTo(edlName, startFrame, stopFrame+100, false);
			//Runtime.getRuntime().addShutdownHook(new AtShutdownThread());
			return;
		}
		if(pck instanceof EDLPacket) {
			EDLPacket edlCmd = (EDLPacket)pck;
			Talk(new StringPacket("edl received"));
			writeTextFile(edlPath, edlCmd.edlFile);
			mci.Insert(edlName, new File(edlPath).getAbsolutePath(), 0, 0, Integer.MAX_VALUE, 0, 0, 0, false);
			return;
		}
		if(pck instanceof StringPacket) {
			StringPacket spck = (StringPacket)pck;
			spck.text = spck.text.toLowerCase();
			if(spck.text.matches("pause")) {
				mci.Pause();
			} else if(spck.text.matches("stop")) {
				mci.Stop();
			}
		}
		
	}
	
	private String framesToTC(int totalFrames) {

	      int hours = 0;
	      int minutes = 0;
	      int seconds = 0;
	      int frames = 0;

	      hours = totalFrames/(3600*fps);
	      totalFrames -= hours*3600*fps;
	      minutes = totalFrames/(60*fps);
	      totalFrames -= minutes*60*fps;
	      seconds = totalFrames/fps;
	      totalFrames -= seconds*fps;
	      frames = totalFrames;

	      String s = "";
	      if(hours>23) {
	          return "00:00:00:00";
	      }
	      if((hours+"").length()<2) {
	          s += "0"+hours;
	      } else {
	          s += hours;
	      }
	      s += ":";
	      if((minutes+"").length()<2) {
	          s += "0"+minutes;
	      } else {
	          s += minutes;
	      }
	      s += ":";
	      if((seconds+"").length()<2) {
	          s += "0"+seconds;
	      } else {
	          s += seconds;
	      }
	      s += ":";
	      if((frames+"").length()<2) {
	          s += "0"+frames;
	      } else {
	          s += frames;
	      }

	      return s;
	  }

	private int TCToFrames(String tc) {
	    int hours = Integer.parseInt(tc.substring(0, 2));
	    int minutes = Integer.parseInt(tc.substring(3,5));
	    int seconds = Integer.parseInt(tc.substring(6, 8));
	    int frames = Integer.parseInt(tc.substring(9, 11));
	    return hours*3600*fps+minutes*60*fps+seconds*fps+frames;
	}

	protected void hasConnected() {
		super.hasConnected();
		if(!isConnectedToMCR) {
			connectToMCR();
			isConnectedToMCR = true;
		}
		setFPS();
		if(!hasGeneratedList) {
			generateList();
			getClipLengths();
			hasGeneratedList = true;
		}
		sendFPS();
		sendList(lc);
	}
	
	private void setFPS() {
		switch ((int)mci.GetTCTcType() & TCXlat.TC2_TCTYPE_MASK){
			case TCXlat.TC2_TCTYPE_NDF: {
				fps = TCXlat.getTC2_SECOND_NDF();
				break;
			}
			case TCXlat.TC2_TCTYPE_DF: {
				fps = TCXlat.getTC2_SECOND_DROP();
				break;
			}
			case TCXlat.TC2_TCTYPE_PAL: {
				fps = TCXlat.getTC2_SECOND_PAL();
				break;
			}
			case TCXlat.TC2_TCTYPE_NTSCFILM: {
				fps = TCXlat.getTC2_SECOND_FILM();
				break;
			}
			case TCXlat.TC2_TCTYPE_50: {
				fps = TCXlat.getTC2_SECOND_50();
				break;
			}
			case TCXlat.TC2_TCTYPE_5994: {
				fps = TCXlat.getTC2_SECOND_5994();
				break;
			}
			case TCXlat.TC2_TCTYPE_60: {
				fps = TCXlat.getTC2_SECOND_60();
				break;
			}
			case TCXlat.TC2_TCTYPE_2398: {
				fps = TCXlat.getTC2_SECOND_FILM();
				break;
			}
			case TCXlat.TC2_TCTYPE_100: {
				fps = TCXlat.getTC2_SECOND_100();
				break;
			}
		}
	}
	
	private void sendFPS() {
		
		Talk(new FPSPacket(fps));
	}
	
	private void connectToMCR() {
		//Connect to clip recorder
		mci = new MediaCmdIF();
		String address = "localhost";
		int port = 1234;
		mci.Connect(address, port);
		if(timeCodeTransmitter == null) {
			timeCodeTransmitter = new TimeCodeTransmitterThread();
		} else {
			timeCodeTransmitter.stop = true;
			try {
				Thread.sleep(15);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			timeCodeTransmitter = new TimeCodeTransmitterThread();
		}
	}
	
	private void generateList() {
		
		lc = new ListPacket();
		ArrayList<String> configFile = readTextFile(configPath);
		ArrayList<String> files = new ArrayList<String>();
		for(String s : configFile) {
			File f = new File(s);
			files = getAllFilesInDir(f, files);
		}
		for(String s : files) {
			File file = new File(s);
			if(s.endsWith(".mov")) {
				lc.clips.add(new Clip(file.getName()));
				lc.clips.get(lc.clips.size()-1).setPath(s);
			}
			if(file.isDirectory()) {
				String firstImage = imageInDirectory(file);
				if(!firstImage.matches("")) {
					lc.clips.add(new Clip(s+firstImage));
					lc.clips.get(lc.clips.size()-1).setPath(s+firstImage);
				}
			}
		}
		
	}
	
	private String imageInDirectory(File f) {
		
		String[] files = f.list();
		if(files == null || files.length==0) {
			return "";
		}
		for(String s : files) {
			if(s.endsWith(".tif")) {
				return "*.tif";
			}
			if(s.endsWith(".tga")) {
				return "*.tga";
			}
		}
		
		return "";
	}
	
	private ArrayList<String> getAllFilesInDir(File f, ArrayList<String> list) {
		if(f==null) return list;
		if(f.list()==null) return list;
		if(f.list().length == 0) return list;
		for(String s : f.list()) {
			File listFile = new File(f.getPath()+"\\"+s);
			if(!listFile.isDirectory()) {
				list.add(f.getPath()+"\\"+s);
			} else {
				list.add(f.getPath()+"\\"+s+"\\");
				list = getAllFilesInDir(listFile, list);
			}
		}
		return list;
	}
	
	private void getClipLengths() {
		
		for(int i=0;i<lc.clips.size();i++) {
			mci.Insert("temp"+i, lc.clips.get(i).pathname, 0, 0, Integer.MAX_VALUE, 0, 0, 0, false);
			MediaCmdIF.ClipInfo ci = new MediaCmdIF.ClipInfo("temp"+i);
			mci.GetClipInfo(ci);
			long frameCount = ci.lEnd;
			System.out.println(ci.lEnd);
			System.out.println(framesToTC((int)frameCount));
			lc.clips.get(i).length = framesToTC((int)frameCount);
			mci.Blank("temp"+i, 0, Integer.MAX_VALUE, 0, 0, 0, false);
		}
	}
	
	private ArrayList<String> readTextFile(String path) {
		
		ArrayList<String> lines = new ArrayList<String>();
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(new File(path)));
			String line;
			while((line = br.readLine()) != null) {
				lines.add(line);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		
		return lines;
	}
	
	private void writeTextFile(String path, ArrayList<String> content) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(path)));
			
			try{
				for(String line : content) {
					bw.write(line+"\n\r");
				}
			} finally {
				bw.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void sendList(ListPacket list) {
		Talk(list);
	}
	
	public static void main(String[] args) {
		
		new Main();
		
	}
	
	private class TimeCodeTransmitterThread extends Thread {
		
		boolean stop = false;
		
		protected TimeCodeTransmitterThread() {
			start();
		}
		
		public void run() {
			while(!stop) {
				if(connected) Talk(new TCPacket(mci.GetCurTC()));
				try {
					Thread.sleep(30);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

}
