package packetNetworking;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 * Client class. This class communicates with a single server given by the connected address and portnr
 * Class inherits streams from the communicator class, but uses only one output and one inputstream
 * @author Stian
 */
public class Client extends Communicator {

    //How long should the client try to connect to the server
	private static final int timeout = 1000;

    //Socket to connect to
	Socket socket = null;
    //Port number
	int portnr;
    //The internet address to connect to. This class ensures compatibility
    InetSocketAddress address;
    /**
     * Boolean to check whether the client is connected or not
     */
    public boolean connected = false;

    //Object that codes and decodes classes sent over the network
	ObjectSender os;

    //A listener for messages received
	MessageListenerThread messageListener;
	
    /**
     * 
     * @param address
     * @param portnr
     */
    public Client(Object address, int portnr) {
		
		super();
		
		if(address!=null) {
            Connect(address, portnr);
        }
		
	}

    /**
     * Connects to the server
     * @param address
     * @param portnr
     */
    public void Connect(Object address, int portnr) {
        if(address instanceof byte[]) {
            //Address is ip
			try {
				this.address = new InetSocketAddress(InetAddress.getByAddress((byte[])address),portnr);
			} catch (UnknownHostException e) {
				e.printStackTrace();
			}
		}
		if(address instanceof String) {
            //Address is dns name
			this.address = new InetSocketAddress((String)address, portnr);
		}

		this.portnr = portnr;

        //Initilizes streams to stream to and from
        outs = new PrintStream[1];
        ins = new BufferedInputStream[1];

        //Starts a new thread that connects
        (new Thread(new Runnable() { public void run() {Reconnect(); } } ) ).start();
    }
	
    /**
     * Reconnects to the server if the connection breaks down
     */
    public void Reconnect() {
		while(true) {
			try {
                //Socket
				socket = new Socket();
				socket.bind(null);
				socket.connect((InetSocketAddress)address, timeout);

                //Streams
				outs[0] = new PrintStream(socket.getOutputStream(), true);
				ins[0] = new BufferedInputStream(socket.getInputStream());
				
				os = new ObjectSender(outs[0]);
				messageListener = new MessageListenerThread(this, 0);
				Listen();
				hasConnected();
				System.out.println("Client up");
				break;
			} catch (UnknownHostException e1) {
				//e1.printStackTrace();
			} catch (IOException e1) {
				//e1.printStackTrace();
			}
			connected = false;
			System.out.println("Connection lost");
		}
	}

    /**
     * Processes the ping packet only. To handle other packets, this method must be overwritten
     * @param command
     */
    public void ProcessPacket(Packet pck) {
        if(pck==null) return;
        if(pck instanceof StringPacket) {
            StringPacket sp = (StringPacket)pck;
            if(sp.text.substring(0, 4).matches("ping")) {
                Talk(pck);
            }
        }
    }
	
    /**
     * Sends a message to the server
     * @param command
     */
    @Override
    public void Talk(Packet pck) {
		os.SendObject(pck);
	}
	
    /**
     * Listens for messages from the server. Must be called before the clients can receive messages
     */
    public void Listen() {
		messageListener.start();
	}
	
    /**
     * Called after connected to the server. May be overwritten
     */
    protected void hasConnected() {
		connected = true;
	}

}
