package SALT.assessment;

import java.util.ArrayList;
import java.util.List;

/**
 * Contains all settings, which specify a subjective assessment. 
 *
 */
public class Design {
	// additional user questions
	private List<String> additionalSubjectInfos;

	// general test settings
	private int numberOfFaders;
	private int MIDIChannelNumber;
	private List<Integer> MIDIControllerNumberOfFaders;
    private List<Integer> MIDIControllerNumberOfButtons;
	private int MIDIDefaultValue;
	private int MIDIButtonNumberForward;
	private int MIDIButtonNumberBackward;
	private boolean continuousFader;
	private boolean continuousButton;
	private float timeSlice;
	private float totalTime;
	private List<Integer> MIDIButtonNumberContinuous;

    //Commented out - stian
	// IDs and names of trials and items
//	HashMap<Integer, String> trialIDandName;
//	HashMap<Integer, String> itemIDandName;

	// MIDI input, MIDI output, target folder
    private int timeCodeinputdevice;    //Stian
	private int inputdevice;
	private int outputdevice;
	private String targetFolder;	

	/**
	 * Constructs a Design with default values.
	 */
	public Design() {
		additionalSubjectInfos = new ArrayList<String>();
		numberOfFaders = 1;
		MIDIChannelNumber = 1;
		MIDIControllerNumberOfFaders = new ArrayList<Integer>();
		MIDIControllerNumberOfFaders.add(21);
        MIDIControllerNumberOfButtons = new ArrayList<Integer>();
        MIDIControllerNumberOfButtons.add(31);
		MIDIDefaultValue = 50;
		MIDIButtonNumberForward = 73;
		MIDIButtonNumberBackward = 72;
		continuousFader = false;
		continuousButton = false;
		MIDIButtonNumberContinuous = new ArrayList<Integer>();
        
//		trialIDandName = new HashMap<Integer, String>();
//		itemIDandName = new HashMap<Integer, String>();

        timeCodeinputdevice = -1;
		inputdevice = -1;
		outputdevice = -1;
	}
	
	/**
	 * Returns a list of additionalSubjectInfos.
	 * @return
	 */
	public List<String> getAdditionalSubjectInfos() {
		return additionalSubjectInfos;
	}
	/**
	 * Sets a list of additionalSubjectInfos.
	 * @param additionalInfos
	 */
	public void setAdditionalSubjectInfos(List<String> additionalInfos) {
		this.additionalSubjectInfos = additionalInfos;
	}
	/**
	 * Appends the specified additionalInfo element to the end of the additionalSubjectInfos list.
	 * @param additionalInfo
	 */
	public void addAdditionalSubjectInfo(String additionalInfo) {
		this.additionalSubjectInfos.add(additionalInfo);
	}
	/**
	 * Removes the specified additionalInfo element from the additionalSubjectInfos list.
	 * @param additionalInfo
	 */
	public void removeAdditionalSubjectInfo(String additionalInfo) {
		this.additionalSubjectInfos.remove(additionalInfo);
	}
	
	/**
	 * Returns the MIDIButtonNumberBackward. 
	 * @return MIDIButtonNumberBackward
	 */
	public int getMIDIButtonNumberBackward() {
		return MIDIButtonNumberBackward;
	}
	/**
	 * Sets the MIDIButtonNumberBackward. 
	 * @param buttonNumberBackward
	 */
	public void setMIDIButtonNumberBackward(int buttonNumberBackward) {
		MIDIButtonNumberBackward = buttonNumberBackward;
	}
	/**
	 * Returns the MIDIButtonNumberForward. 
	 * @return
	 */
	public int getMIDIButtonNumberForward() {
		return MIDIButtonNumberForward;
	}
	/**
	 * Sets the MIDIButtonNumberForward. 
	 * @param buttonNumberForward
	 */
	public void setMIDIButtonNumberForward(int buttonNumberForward) {
		MIDIButtonNumberForward = buttonNumberForward;
	}
	/**
	 * Returns the MIDIChannelNumber.
	 * @return
	 */
	public int getMIDIChannelNumber() {
		return MIDIChannelNumber;
	}
	/**
	 * Returns the MIDIButtonNumberContinuous.
	 * @return
	 */
	public List<Integer> getMIDIButtonNumberContinuous() {
		return MIDIButtonNumberContinuous;
	}
	/**
	 * Sets the MIDIButtonNumberContinuous. 
	 * @param buttonNumberForward
	 */
	public void setMIDIButtonNumberContinuous(List<Integer> buttonNumberContinuous) {
		MIDIButtonNumberContinuous = buttonNumberContinuous;
	}
	
	/**
	 * Appends the specified buttonNumberContinuous element to the end of the MIDIButtonNumberContinuous List.
	 * @param buttonNumberContinuous
	 */
	public void addMIDIButtonNumberContinuous(Integer buttonNumberContinuous) {
		MIDIButtonNumberContinuous.add(buttonNumberContinuous);
	}
	/**
	 * Removes all of the buttonNumberContinuous elements from the MIDIButtonNumberContinuous List.
	 */
	public void clearMIDIButtonNumberContinuous() {
		MIDIButtonNumberContinuous.clear();
	}
	
	/**
	 * Sets the MIDIChannelNumber.
	 * @param channelNumber
	 */
	public void setMIDIChannelNumber(int channelNumber) {
		MIDIChannelNumber = channelNumber;
	}
	/**
	 * Returns a list of MIDIControllerNumberOfFaders.
	 * @return
	 */
	public List<Integer> getMIDIControllerNumberOfFaders() {
		return MIDIControllerNumberOfFaders;
	}
	/**
	 * Sets a list of MIDIControllerNumberOfFaders.
	 * @param controllerNumberOfFaders
	 */
	public void setMIDIControllerNumberOfFaders(
			List<Integer> controllerNumberOfFaders) {
		MIDIControllerNumberOfFaders = controllerNumberOfFaders;
	}
	/**
	 * Appends the specified controllerNumberOfFaders element to the end of the MIDIControllerNumberOfFaders List.
	 * @param controllerNumberOfFaders
	 */
	public void addMIDIControllerNumberOfFader(Integer controllerNumberOfFaders) {
		MIDIControllerNumberOfFaders.add(controllerNumberOfFaders);
	}
	/**
	 * Removes all of the controllerNumberOfFaders elements from the MIDIControllerNumberOfFaders List.
	 */
	public void clearMIDIControllerNumberOfFaders() {
		MIDIControllerNumberOfFaders.clear();
	}

	/**
	 * Replaces the controllerNumberOfFaders element at the specified position in the MIDIControllerNumberOfFaders list with the specified controllerNumberOfFaders element.
	 * @param index
	 * @param controllerNumberOfFaders
	 */
	public void replaceMIDIControllerNumberOfFader(Integer index,
			Integer controllerNumberOfFaders) {
		MIDIControllerNumberOfFaders.set(index, controllerNumberOfFaders);
	}


    /*
     * Edited in to be able to set midi control buttons for each fader - mostly copy/paste from fader numbers
     * Stian
     * */
    /**
	 * Returns a list of MIDIControllerNumberOfButtons.
	 * @return
	 */
    public List<Integer> getMIDIControllerNumberOfButtons() {
		return MIDIControllerNumberOfButtons;
	}
	/**
	 * Sets a list of MIDIControllerNumberOfButtons.
	 * @param controllerNumberOfButtons
	 */
	public void setMIDIControllerNumberOfButtons(
			List<Integer> controllerNumberOfButtons) {
		MIDIControllerNumberOfButtons = controllerNumberOfButtons;
	}
	/**
	 * Appends the specified controllerNumberOfButtons element to the end of the MIDIControllerNumberOfButtons List.
	 * @param controllerNumberOfButtons
	 */
	public void addMIDIControllerNumberOfButton(Integer controllerNumberOfButtons) {
//        System.out.println(controllerNumberOfButtons);
		MIDIControllerNumberOfButtons.add(controllerNumberOfButtons);
        //System.out.println(MIDIControllerNumberOfButtons.get(MIDIControllerNumberOfButtons.size()-1));
//        System.out.println(getMIDIControllerNumberOfButtons().get(getMIDIControllerNumberOfButtons().size()-1));
	}
	/**
	 * Removes all of the controllerNumberOfButtons elements from the MIDIControllerNumberOfButtons List.
	 */
	public void clearMIDIControllerNumberOfButtons() {
		MIDIControllerNumberOfButtons.clear();
	}
    /**
	 * Replaces the controllerNumberOfButtons element at the specified position in the MIDIControllerNumberOfButtons list with the specified controllerNumberOfButtons element.
	 * @param index
	 * @param controllerNumberOfButtons
	 */
	public void replaceMIDIControllerNumberOfButton(Integer index,
			Integer controllerNumberOfButtons) {
		MIDIControllerNumberOfButtons.set(index, controllerNumberOfButtons);
	}


    /*
     * End of edit in
     * Stian
     * */

	/**
	 * Returns the MIDIDefaultValue of a fader.
	 * @return
	 */
	public int getMIDIDefaultValue() {
		return MIDIDefaultValue;
	}

	/**
	 * Sets the MIDIDefaultValue of a fader.
	 * @param defaultValue
	 */
	public void setMIDIDefaultValue(int defaultValue) {
		MIDIDefaultValue = defaultValue;
	}

	/**
	 * Returns the numberOfFaders.
	 * @return numberOfFaders
	 */
	public int getNumberOfFaders() {
		return numberOfFaders;
	}
	/**
	 * Sets the numberOfFaders. 
	 * @param numberOfFaders
	 */
	public void setNumberOfFaders(int numberOfFaders) {
		this.numberOfFaders = numberOfFaders;
	}

    /**
	 * Returns the MIDI time codeinput device.
     * Stian
	 * @return
	 */
	public int getTimeCodeInputDevice() {
		return timeCodeinputdevice;
	}
	/**
	 * Sets the MIDI time code input device.
     * Stian
	 * @param inputdevice
	 */
	public void setTimeCodeInputDevice(int timeCodeinputdevice) {
		this.timeCodeinputdevice = timeCodeinputdevice;
	}

	/**
	 * Returns the MIDI input device.
	 * @return
	 */
	public int getInputDevice() {
		return inputdevice;
	}
	/**
	 * Sets the MIDI input device.
	 * @param inputdevice
	 */
	public void setInputDevice(int inputdevice) {
		this.inputdevice = inputdevice;
	}

	/**
	 * Returns the outputdevice.
	 * @return the MIDI output device
	 */
	public int getOutputDevice() {
		return outputdevice;
	}
	/**
	 * Sets the MIDI output device.
	 * @param outputdevice
	 */
	public void setOutputDevice(int outputdevice) {
		this.outputdevice = outputdevice;
	}

	/**
	 * Returns the targetFolder.
	 * @return the folder for the sessions
	 */
	public String getTargetFolder() {
		return targetFolder;
	}
	/**
	 * Sets the target folder of the sessions.
	 * @param targetFolder
	 */
	public void setTargetFolder(String targetFolder) {
		this.targetFolder = targetFolder;
	}
	
	/**
	 * Returns a HashMap with ID (key) and identifier (value) for items.
	 * @return
	 */
//	public HashMap<Integer, String> getItemIDandName() {
//		return itemIDandName;
//	}
	/**
	 * Returns the value to which the specified ID (key) is mapped in this identity hash map, or null if the map contains no mapping for this ID (key).
	 * @param ID
	 * @return
	 */
//	public String getItemIDName(Integer ID) {
//		return itemIDandName.get(ID);
//	}
	/**
	 * Sets a HashMap with ID(key) and identifier (value) for items.
	 * @param itemIDandName
	 */
//	public void setItemIDandName(HashMap<Integer, String> itemIDandName) {
//		this.itemIDandName = itemIDandName;
//	}
	/**
	 * Associates the specified name (value) with the specified ID (key) in the map.
	 * @param ID
	 * @param name
	 */
//	public void putItemIDandName(Integer ID, String name) {
//		this.itemIDandName.put(ID, name);
//	}
	/**
	 * Removes the mapping for the ID (key) from the map if present.
	 * @param ID
	 */
//	public void removeItemIDandName(Integer ID) {
//		itemIDandName.remove(ID);
//	}

	/**
	 * Returns a HashMap with ID(key) and identifier (value) for trials.
	 * @return
	 */
//	public HashMap<Integer, String> getTrialIDandName() {
//		return trialIDandName;
//	}
	/**
	 * Returns the value to which the specified ID (key) is mapped in this identity hash map, or null if the map contains no mapping for this ID (key).
	 * @param ID
	 * @return
	 */
//	public String getTrialIDName(Integer ID) {
//		return trialIDandName.get(ID);
//	}
	/**
	 * Sets a HashMap with ID (key) and identifier (value) for trials.
	 * @param trialIDandName
	 */
//	public void setTrialIDandName(HashMap<Integer, String> trialIDandName) {
//		this.trialIDandName = trialIDandName;
//	}
	/**
	 * Associates the specified name (value) with the specified ID (key) in the map.
	 * @param ID
	 * @param name
	 */
//	public void putTrialIDandName(Integer ID, String name) {
//		this.trialIDandName.put(ID, name);
//	}
	/**
	 * Removes the mapping for the ID (key) from the map if present.
	 * @param ID
	 */
//	public void removeTrialIDandName(Integer ID) {
//		trialIDandName.remove(ID);
//	}

	/**
	 * Removes additionalSubjectInfos list, MIDIControllerNumberOfFaders list, HashMap for trials and items from the design.
	 */
	public void clear() {
		additionalSubjectInfos.clear();
		MIDIControllerNumberOfFaders.clear();
        MIDIControllerNumberOfButtons.clear();
//		trialIDandName.clear();
//		itemIDandName.clear();
		MIDIButtonNumberContinuous.clear();
	}

	/**
	 * Returns a boolean.
	 * @return
	 */
	public boolean isContinuousButton() {
		return continuousButton;
	}

	/**
	 * Sets the continuousButton.
	 * @param continuousButton
	 */
	public void setContinuousButton(boolean continuousButton) {
		this.continuousButton = continuousButton;
	}

	/**
	 * Returns a boolean.
	 * @return
	 */
	public boolean isContinuousFader() {
		return continuousFader;
	}

	/**
	 * Sets the continuousFader.
	 * @param timeSlice
	 */
	public void setContinuousFader(boolean continuousFader) {
		this.continuousFader = continuousFader;
	}

	/**
	 * Returns the timeslice.
	 * @return
	 */
	public float getTimeSlice() {
		return timeSlice;
	}

	/**
	 * Sets the timeslice.
	 * @param timeSlice
	 */
	public void setTimeSlice(float timeSlice) {
		this.timeSlice = timeSlice;
	}

	/**
	 * Returns the totalTime.
	 * @return
	 */
	public float getTotalTime() {
		return totalTime;
	}

	/**
	 * Sets the totalTime.
	 * @param totalTime
	 */
	public void setTotalTime(float totalTime) {
		this.totalTime = totalTime;
	}
}
