package SALT.assessment;

import packets.Clip;
import org.jdom.input.*;
import org.jdom.Document;
import org.jdom.Element;

import java.io.File;

import java.util.*;
import mediaplayer.Item;
import mediaplayer.Main;
import mediaplayer.Trial;
import org.jdom.Attribute;

/**
 * To work with a design in SalT, the DesignReader generates a Design object from the data in a .design file.
 * @see org.jdom.input.SAXBuilder
 * @see org.jdom.Document
 */
public class DesignReader {
	
	// Builds a JDOM document from files using a SAX parser. 
	private static SAXBuilder builder = new SAXBuilder();

	/**
	 * Writes the data from a .design file, which has XML style in a Design object.
	 * @param file .design file 
	 * @return Design object
	 */
	public static Design getDesign(File file) {
		Design currentDesign = new Design();
		currentDesign.clear();
		
		// Creates a new empty document.
		Document doc = new Document();
		try {
			// Builds a document from the supplied filename.
			doc = builder.build(file);
		} catch (Exception e) {
			// TODO: handle exception
		}

		// Reads the informations from the XML document and writes in a design.
		Element root = doc.getRootElement();
		Element additionalSubjectInfosChild = root.getChild(
				"additionalSubjectInfos", DesignWriter.tuiNSdesign);

		List<Element> additionalSubjectInfosList = additionalSubjectInfosChild
				.getChildren("additionalSubjectInfo", DesignWriter.tuiNSdesign);
		for (Element additionalSubjectInfo : additionalSubjectInfosList) {
			currentDesign.addAdditionalSubjectInfo(new String(
					additionalSubjectInfo.getText()));
		}

		Element generalTestSettingsChild = root.getChild("generalTestSettings",
				DesignWriter.tuiNSdesign);

		currentDesign.setNumberOfFaders(Integer
				.valueOf(generalTestSettingsChild.getChild("numberOfFaders",
						DesignWriter.tuiNSdesign).getText()));
		currentDesign.setMIDIChannelNumber(Integer
				.valueOf(generalTestSettingsChild.getChild("MIDIChannelNumber",
						DesignWriter.tuiNSdesign).getText()));

		List<Element> MIDIControllerNumberOfFadersList = generalTestSettingsChild
				.getChildren("MIDIControllerNumberOfFaders",
						DesignWriter.tuiNSdesign);
		for (Element MIDIControllerNumberOfFaders : MIDIControllerNumberOfFadersList) {
			currentDesign.addMIDIControllerNumberOfFader(new Integer(
					MIDIControllerNumberOfFaders.getText()));
		}

        List<Element> MIDIControllerNumberOfButtonsList = generalTestSettingsChild
				.getChildren("MIDIControllerNumberOfButtons",
						DesignWriter.tuiNSdesign);
		for (Element MIDIControllerNumberOfButtons : MIDIControllerNumberOfButtonsList) {
			currentDesign.addMIDIControllerNumberOfButton(new Integer(
					MIDIControllerNumberOfButtons.getText()));
		}

		currentDesign.setMIDIDefaultValue(Integer
				.valueOf(generalTestSettingsChild.getChild("MIDIDefaultValue",
						DesignWriter.tuiNSdesign).getText()));
		currentDesign.setMIDIButtonNumberForward(Integer
				.valueOf(generalTestSettingsChild.getChild(
						"MIDIButtonNumberForward", DesignWriter.tuiNSdesign)
						.getText()));
		currentDesign.setMIDIButtonNumberBackward(Integer
				.valueOf(generalTestSettingsChild.getChild(
						"MIDIButtonNumberBackward", DesignWriter.tuiNSdesign)
						.getText()));

        /*
         * edited in - stian
         * */

        Element trialsElement = root.getChild("trials", DesignWriter.tuiNSdesign);
        Attribute a = trialsElement.getAttribute("ItemsPerTrial");
        int itemsPerTrial = Integer.parseInt(a.getValue());
        Main.view.nrOfItemsComboBox.setSelectedIndex(itemsPerTrial-1);
        Main.view.main.changeNrOfItemsPerTrial();
        for(int i=0;i<trialsElement.getChildren().size();i++) {
            Element trialElement = (Element) trialsElement.getChildren().get(i);
            String name = trialElement.getText();
            name = name.trim();
            Trial trial = new Trial(name);
            for(int j=0;j<trialElement.getChildren().size();j++) {
                Element itemElement = (Element) trialElement.getChildren().get(j);
                Element filenameElement = (Element) itemElement.getChildren().get(0);
                String filename = filenameElement.getText();
                Element clipElement = (Element) itemElement.getChildren().get(1);
                String clipName = clipElement.getText();
                String clipPath = ((Element) clipElement.getChildren().get(0)).getText();
                String clipLength = ((Element) clipElement.getChildren().get(1)).getText();
                Clip clip = new Clip(clipName);
                clip.setPath(clipPath);
                clip.setLength(clipLength);
                ArrayList<String> tags = new ArrayList<String>();
                Element tagsElement = (Element) itemElement.getChildren().get(2);
                for(int k=0;k<tagsElement.getChildren().size();k++) {
                    Element tag = (Element) tagsElement.getChildren().get(k);
                    tags.add(tag.getText());
                }
                Item item = new Item(itemElement.getText().trim(), filename, clip);
                item.tags = tags;
                trial.items[j] = item;
            }
            Main.view.main.playlist.addTrial(trial);
        }

        /*
         * end of edit
         * stian
         * */


//		Element trialsChild = root.getChild("trials", DesignWriter.tuiNSdesign);
//		List<Element> trialList = trialsChild.getChildren("trial",
//				DesignWriter.tuiNSdesign);
//		for (Element trial : trialList) {
//			currentDesign.putTrialIDandName(Integer.valueOf(trial.getAttribute(
//					"id").getValue()), trial.getText());
//		}

//		Element itemsChild = root.getChild("items", DesignWriter.tuiNSdesign);
//		List<Element> itemList = itemsChild.getChildren("item",
//				DesignWriter.tuiNSdesign);
//		for (Element item : itemList) {
//			currentDesign.putItemIDandName(Integer.valueOf(item.getAttribute(
//					"id").getValue()), item.getText());
//		}


        currentDesign.setTimeCodeInputDevice((Integer.valueOf(root.getChild(
				"MIDITimeCodeInput", DesignWriter.tuiNSdesign).getText())));
		currentDesign.setInputDevice((Integer.valueOf(root.getChild(
				"MIDIInput", DesignWriter.tuiNSdesign).getText())));
		currentDesign.setOutputDevice((Integer.valueOf(root.getChild(
				"MIDIOutput", DesignWriter.tuiNSdesign).getText())));
		currentDesign.setTargetFolder(root.getChild("SessionTargetFolder",
				DesignWriter.tuiNSdesign).getText());
		
		if ((generalTestSettingsChild.getChild("Rating",
				DesignWriter.tuiNSdesign).getText()).equals("1")){
			currentDesign.setContinuousButton(false);
			currentDesign.setContinuousFader(false);
		}
			
		if ((generalTestSettingsChild.getChild("Rating",
				DesignWriter.tuiNSdesign).getText()).equals("2")){
			currentDesign.setContinuousButton(true);
			currentDesign.setContinuousFader(false);
		
			Element MIDIButtonNumberContinuousChild = generalTestSettingsChild.getChild(
					"MIDIButtonNumberContinuous", DesignWriter.tuiNSdesign);
			List<Element> MIDIButtonNumberContinuousChildList = MIDIButtonNumberContinuousChild.
			getChildren("ButtonNumber", DesignWriter.tuiNSdesign);
			List<Integer> MIDIButtonNumberContinuousList = new ArrayList<Integer>();
			
			for (int MIDIButtonNumberContinuousListElement = 0; MIDIButtonNumberContinuousListElement < MIDIButtonNumberContinuousChildList.size();
			MIDIButtonNumberContinuousListElement++) {
				MIDIButtonNumberContinuousList.add(MIDIButtonNumberContinuousListElement, Integer.valueOf(MIDIButtonNumberContinuousChildList.get(MIDIButtonNumberContinuousListElement).getText()));				
			}
			currentDesign.setMIDIButtonNumberContinuous(MIDIButtonNumberContinuousList);
		}
				
		if ((generalTestSettingsChild.getChild("Rating",
				DesignWriter.tuiNSdesign).getText()).equals("3")){
			currentDesign.setContinuousButton(false);
			currentDesign.setContinuousFader(true);
			currentDesign.setTimeSlice(Integer.valueOf(generalTestSettingsChild.getChild("TimeSlice",
					DesignWriter.tuiNSdesign).getText()));
		}

        Main.view.main.playlist.rearrange();
        
		return currentDesign;
	}
}
