package SALT.assessment;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.File;

import java.util.*;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import SALT.exceptions.WriterException;
import mediaplayer.Item;
import mediaplayer.Main;
import mediaplayer.Trial;

/**
 * The DesignWriter provides a writing method to write the data from a Design object to a .design file. 
 * @see org.jdom.output.XMLOutputter
 */
public class DesignWriter {

	public static final Namespace tuiNSdesign = Namespace
			.getNamespace("urn:tu-ilmenau:de:imt:emt:subjective-assessment:design");

	/**
	 * Writes a Design in a .design file, which has XML style.
	 * @param outputFile the target .design file
	 * @param newDesign the selected design
	 * @throws WriterException
	 */
	public static void writeDesignXML(File outputFile, Design newDesign)
			throws WriterException {


		// Creates a new empty document.
		Document doc = new Document();

		Element rootElement = new Element("testdesign", tuiNSdesign);
		doc.setRootElement(rootElement);

		// Writes additionalSubjectInfos.
		Element additionalSubjectInfosElement = new Element(
				"additionalSubjectInfos", tuiNSdesign);
		rootElement.addContent(additionalSubjectInfosElement);

		List<String> additionalSubjectInfosList = newDesign
				.getAdditionalSubjectInfos();

		for (String additionalSubjectInfo : additionalSubjectInfosList) {

			Element additionalSubjectInfoElement = new Element(
					"additionalSubjectInfo", tuiNSdesign);
			additionalSubjectInfoElement.setText(additionalSubjectInfo);
			additionalSubjectInfosElement
					.addContent(additionalSubjectInfoElement);
		}

		// Writes generalTestSettings. 
		Element generalTestSettingsElement = new Element("generalTestSettings",
				tuiNSdesign);
		rootElement.addContent(generalTestSettingsElement);

		Element numberOfFadersElement = new Element("numberOfFaders",
				tuiNSdesign);
		numberOfFadersElement.setText(String.valueOf(newDesign
				.getNumberOfFaders()));
		generalTestSettingsElement.addContent(numberOfFadersElement);

		Element MIDIChannelNumberElement = new Element("MIDIChannelNumber",
				tuiNSdesign);
		MIDIChannelNumberElement.setText(String.valueOf(newDesign
				.getMIDIChannelNumber()));
		generalTestSettingsElement.addContent(MIDIChannelNumberElement);

		List<Integer> MIDIControllerNumberOfFadersList = newDesign
				.getMIDIControllerNumberOfFaders();

		for (Integer MIDIControllerNumberOfFaders : MIDIControllerNumberOfFadersList) {

			Element MIDIControllerNumberOfFadersElement = new Element(
					"MIDIControllerNumberOfFaders", tuiNSdesign);
			MIDIControllerNumberOfFadersElement.setText(String
					.valueOf(MIDIControllerNumberOfFaders));
			generalTestSettingsElement
					.addContent(MIDIControllerNumberOfFadersElement);
		}

        //Edit in - stian
        List<Integer> MIDIControllerNumberOfButtonsList = newDesign
				.getMIDIControllerNumberOfButtons();

		for (Integer MIDIControllerNumberOfButtons : MIDIControllerNumberOfButtonsList) {

			Element MIDIControllerNumberOfButtonsElement = new Element(
					"MIDIControllerNumberOfButtons", tuiNSdesign);
			MIDIControllerNumberOfButtonsElement.setText(String
					.valueOf(MIDIControllerNumberOfButtons));
			generalTestSettingsElement
					.addContent(MIDIControllerNumberOfButtonsElement);
		}
        //End of edit

		Element MIDIDefaultValueElement = new Element("MIDIDefaultValue",
				tuiNSdesign);
		MIDIDefaultValueElement.setText(String.valueOf(newDesign
				.getMIDIDefaultValue()));
		generalTestSettingsElement.addContent(MIDIDefaultValueElement);

		Element MIDIButtonNumberForwardElement = new Element(
				"MIDIButtonNumberForward", tuiNSdesign);
		MIDIButtonNumberForwardElement.setText(String.valueOf(newDesign
				.getMIDIButtonNumberForward()));
		generalTestSettingsElement.addContent(MIDIButtonNumberForwardElement);

		Element MIDIButtonNumberBackwardElement = new Element(
				"MIDIButtonNumberBackward", tuiNSdesign);
		MIDIButtonNumberBackwardElement.setText(String.valueOf(newDesign
				.getMIDIButtonNumberBackward()));
		generalTestSettingsElement.addContent(MIDIButtonNumberBackwardElement);
		
		Element RatingElement = new Element(
				"Rating", tuiNSdesign);
		String rating = new String();

		if(newDesign.isContinuousButton() == false && newDesign.isContinuousFader() == false){
			rating = "1";
		}		
		if(newDesign.isContinuousButton() == true && newDesign.isContinuousFader() == false){
			rating = "2";
			
			Element MIDIButtonNumbersContinuousElement = new Element(
					"MIDIButtonNumberContinuous", tuiNSdesign);
			
			for (int i = 0; i < newDesign.getMIDIButtonNumberContinuous().size(); i++) {
				Element MIDIButtonNumberContinuousElement = new Element("ButtonNumber", tuiNSdesign);
				MIDIButtonNumberContinuousElement.setText(String.valueOf(newDesign.getMIDIButtonNumberContinuous().get(i)));
				MIDIButtonNumbersContinuousElement.addContent(MIDIButtonNumberContinuousElement);
			}
			generalTestSettingsElement.addContent(MIDIButtonNumbersContinuousElement);
		}
		if(newDesign.isContinuousButton() == false && newDesign.isContinuousFader() == true){
			rating = "3";
			Element TimeSliceElement = new Element(
					"TimeSlice", tuiNSdesign);
			TimeSliceElement.setText(String.valueOf((int)newDesign.getTimeSlice()));
			generalTestSettingsElement.addContent(TimeSliceElement);
		}
		
		RatingElement.setText(rating);
		generalTestSettingsElement.addContent(RatingElement);


		// Writes trials.
		Element trialsElement = new Element("trials", tuiNSdesign);
        int itpt = Trial.getItemsPerTrial();
        
		rootElement.addContent(trialsElement);
        Attribute itemsPerTrial = new Attribute("ItemsPerTrial", itpt+"");
        trialsElement.setAttribute(itemsPerTrial);
        /*
         * edit in stian, change from static items to dynamical items for each trial
         * */
        for(int i=0;i<Main.view.main.playlist.getTrials().size();i++) {
            Trial trial = Main.view.main.playlist.getTrial(i);
            Element trialIDandName = new Element("trial", tuiNSdesign);
            Attribute idAttrib = new Attribute("id", trial.id+"");
            trialIDandName.setAttribute(idAttrib);
            trialIDandName.setText(trial.getName());
            
            //Items
            for(int j=0;j<trial.items.length;j++) {
                Item item = trial.items[j];
                if(item == null) {
                    continue;
                }
                Element itemElement = new Element("item", tuiNSdesign);
                Attribute attrib = new Attribute("id", j+"");
                itemElement.setAttribute(attrib);
                itemElement.setText(item.name);
                Element filenameElement = new Element("filename", tuiNSdesign);
                filenameElement.setText(item.filename);
                itemElement.addContent(filenameElement);
                Element clip = new Element("clip", tuiNSdesign);
                Attribute clipId = new Attribute("id", item.clip.id+"");
                clip.setAttribute(clipId);
                clip.setText(item.clip.name);
                String pn = item.clip.pathname;
                Element clipPath = new Element("path", tuiNSdesign);
                clipPath.setText(pn);
                clip.addContent(clipPath);
                Element clipLength = new Element("length", tuiNSdesign);
                clipLength.setText(item.clip.length);
                clip.addContent(clipLength);
                itemElement.addContent(clip);
                Element itemTags = new Element("tags", tuiNSdesign);
                for(int k=0;k<item.tags.size();k++) {
                    Element itemTag = new Element("tag", tuiNSdesign);
                    itemTag.setText(item.tags.get(k));
                    itemTags.addContent(itemTag);
                }
                itemElement.addContent(itemTags);
                trialIDandName.addContent(itemElement);
            }
            trialsElement.addContent(trialIDandName);
        }
        /*
         * end of edit
         * stian
         * */

//		HashMap<Integer, String> TempHashMap = newDesign.getTrialIDandName();
//		Set<Integer> KeysSet = TempHashMap.keySet();
//		Integer key = new Integer(0);
//
//		for (Iterator<Integer> iterator = KeysSet.iterator(); iterator
//				.hasNext() == true;) {
//
//			key = iterator.next();
//			String trialName = TempHashMap.get((Integer) key);
//			Element trialIDandNameElement = new Element("trial", tuiNSdesign);
//			Attribute idAttribute = new Attribute("id", String
//					.valueOf((Integer) key));
//			trialIDandNameElement.setAttribute(idAttribute);
//			trialIDandNameElement.setText(trialName);
//			trialsElement.addContent(trialIDandNameElement);
//		}
//
//		// Writes items.
//		Element itemsElement = new Element("items", tuiNSdesign);
//		rootElement.addContent(itemsElement);
//
//		TempHashMap = newDesign.getItemIDandName();
//		KeysSet = TempHashMap.keySet();
//
//		for (Iterator<Integer> iterator = KeysSet.iterator(); iterator
//				.hasNext() == true;) {
//
//			key = iterator.next();
//			String itemName = TempHashMap.get((Integer) key);
//			Element itemIDandNameElement = new Element("item", tuiNSdesign);
//			Attribute idAttribute = new Attribute("id", String
//					.valueOf((Integer) key));
//			itemIDandNameElement.setAttribute(idAttribute);
//			itemIDandNameElement.setText(itemName);
//			itemsElement.addContent(itemIDandNameElement);
//		}

        Element MIDITimeCodeInputDeviceElement = new Element("MIDITimeCodeInput", tuiNSdesign);
		MIDITimeCodeInputDeviceElement.setText(String.valueOf(newDesign
				.getTimeCodeInputDevice()));
		rootElement.addContent(MIDITimeCodeInputDeviceElement);

		Element MIDIInputDeviceElement = new Element("MIDIInput", tuiNSdesign);
		MIDIInputDeviceElement.setText(String.valueOf(newDesign
				.getInputDevice()));
		rootElement.addContent(MIDIInputDeviceElement);

		Element MIDIOutputDeviceElement = new Element("MIDIOutput", tuiNSdesign);
		MIDIOutputDeviceElement.setText(String.valueOf(newDesign
				.getOutputDevice()));
		rootElement.addContent(MIDIOutputDeviceElement);

		Element SessionTargetFolderElement = new Element("SessionTargetFolder",
				tuiNSdesign);
		SessionTargetFolderElement.setText(newDesign.getTargetFolder());
		rootElement.addContent(SessionTargetFolderElement);

		// Creates an XMLOutputter with the default Format, which outputs a JDOM document.
		XMLOutputter outputter = new XMLOutputter(Format.getPrettyFormat());

		try {
			FileOutputStream stream = new FileOutputStream(outputFile);
			OutputStreamWriter writer = new OutputStreamWriter(stream, "UTF-8");

			// Prints the Document to the Writer.
			outputter.output(doc, writer);
			writer.close();
			stream.close();

            //Edited in - stian
            Main.view.main.status.AddMessage("Design written");

		} catch (FileNotFoundException e) {
			throw new WriterException(e);
		} catch (UnsupportedEncodingException e) {
			throw new WriterException(e);
		} catch (IOException e) {
			throw new WriterException(e);
		}
	}
}
