package SALT.assessment;

/*
 *	DumpReceiver.java
 *
 *      - f�r Salt-Anwendung optimiert
 *
 */

/*
 * Copyright (c) 1999 - 2001 by Matthias Pfisterer
 * Copyright (c) 2003 by Florian Bomers
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.PrintStream;
import java.util.StringTokenizer;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.SysexMessage;
import mediaplayer.Main;

/**
 * @see javax.sound.midi.Receiver
 */
public class DumpReceiver implements Receiver {

	public long smByteCount = 0;
	public long smCount = 0;

	private Salt saltForReceiver;
	public int y = 0;
	private boolean m_bPrintTimeStampAsTicks;

	// variables for continuous rating
	private long startTime = 0;
	private boolean startTimeFlag = false;

	/**
	 * @param printStream
	 * @param salt
	 */
	public DumpReceiver(PrintStream printStream, Salt salt) {
		this(printStream, false);
		saltForReceiver = salt;
	}

	/**
	 * @param printStream
	 * @param bPrintTimeStampAsTicks
	 */
	public DumpReceiver(PrintStream printStream, boolean bPrintTimeStampAsTicks) {
		m_bPrintTimeStampAsTicks = bPrintTimeStampAsTicks;
	}

	/**
	 * Indicates that the application has finished using the receiver, and that limited resources it requires may be released or made available.
	 * @see javax.sound.midi.Receiver
	 */
	public void close() {
	}

	/**
	 * Sends a MIDI message and time-stamp to the receiver.
	 * @param message the MIDI message to send
	 * @param lTimeStamp the time-stamp for the message, in microseconds
	 * @see javax.sound.midi.Receiver
	 */
	public void send(MidiMessage message, long lTimeStamp) {

        //if(!Main.isRunning) return;
		String strMessage = null;

		// Voice Messages
		if (message instanceof ShortMessage) {
			strMessage = decodeMessage((ShortMessage) message);
		}
		// SysEx Messages
		else if (message instanceof SysexMessage) {
			strMessage = decodeMessage((SysexMessage) message);
            if(strMessage==null) strMessage = "sysex";
		}
		else {
			strMessage = "unbekannte Message";
		}

		String strTimeStamp = null;
		if (m_bPrintTimeStampAsTicks) {
			strTimeStamp = "tick " + lTimeStamp + ": ";
		}
		else {
			if (lTimeStamp == -1L) {
				strTimeStamp = "timestamp [unknown]: ";
			} else {
				strTimeStamp = "timestamp " + lTimeStamp + " us: ";
			}
		}
        
        /*
         * replaced and commented out - stian
         * */
//		saltForReceiver.gui.displayTextAreaMessage("von SALT empfangener String: "
//		+ strTimeStamp + strMessage);

        //Edited in - stian
        Main.view.addMidiMessage(strTimeStamp + strMessage);
	}

	/**
	 * Processes the Voice Message.
	 * @param message
	 * @return 
	 */
	public String decodeMessage(ShortMessage message) {
		String strMessage = null;
		if (message.getCommand() != 0xF0 && saltForReceiver.session != null) {
			int nChannel = message.getChannel() + 1;
			String strChannel = "ch " + nChannel + ": ";

			// default MIDI Channel
			if (nChannel == saltForReceiver.design.getMIDIChannelNumber()) {
				switch (message.getCommand()) {
				case 0x80:
					strMessage = "note off " + message.getData1()
							+ " velocity: " + message.getData2();
                    saltForReceiver.session.addMidiActivity(strMessage);
					break;
				//----------------------- Note On ------------------------------
				case 0x90:

					int keyNum = message.getData1();
					int keyVel = message.getData2();					

					// Analyses a special Note On message from the I3D (SALTControl-ID).
					if ((keyNum == 31) && (keyVel == 1)) {						
						startTime = System.currentTimeMillis();
						startTimeFlag = true;
					}

					// Increments the group.
					if ((keyNum == saltForReceiver.design.getMIDIButtonNumberForward()) && (keyVel > 0)) {
						y = saltForReceiver.currGrp;
						saltForReceiver.incGroup();
						saltForReceiver.sendTrial();
						strMessage = "note on " + keyNum + " velocity: "
								+ keyVel + " <Gruppe "
								+ (saltForReceiver.currGrp + 1) + ">";
                        saltForReceiver.session.addMidiActivity(strMessage);
                        Main.view.main.nextTrial();
					} else
					// Decrements the group.
					if ((keyNum == saltForReceiver.design.getMIDIButtonNumberBackward())
							&& (keyVel > 0)) {
						y = saltForReceiver.currGrp;
						if (y > 0) {
							saltForReceiver.decGroup();
							for (int i = 0; i < saltForReceiver.design
								.getNumberOfFaders(); ++i) {
								saltForReceiver.session.getTrialInfos().get(
										saltForReceiver.currGrp).getItemInfos()
										.get(i).setValue(
												saltForReceiver.design
														.getMIDIDefaultValue());

								// Clears the TimeStampList of the previous trial.
								if (saltForReceiver.design.isContinuousButton())
									saltForReceiver.session.getTrialInfos()
											.get(saltForReceiver.currGrp)
											.getItemInfos().get(i)
											.getTimeStampList().clear();
							}

							saltForReceiver.sendTrial();

							strMessage = "note On " + keyNum + " velocity: "
									+ keyVel + " <Gruppe "
									+ (saltForReceiver.currGrp + 1) + ">";
                            saltForReceiver.session.addMidiActivity(strMessage);
                            Main.view.main.previousTrial();
						}
					}

					// Continuous rating (button): By pressing the ContinuousButton the time 
					// difference to the startTime will be added to the TimeStampList. 	
					if (startTimeFlag == true && saltForReceiver.design.isContinuousButton()
						&& keyNum != 31) {
						for (int i = 0; i < saltForReceiver.design
								.getMIDIButtonNumberContinuous().size(); ++i) {
							if (keyNum == saltForReceiver.design
									.getMIDIButtonNumberContinuous().get(i)) {
								long timeDiff = (System.currentTimeMillis() - startTime);
								float dtimeDiff = (Float.valueOf(timeDiff)
										.floatValue()) / 1000;
								saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
								.getItemInfos().get(i).addTimeStampListElement(saltForReceiver.session
								.getTrialInfos().get(saltForReceiver.currGrp).getItemInfos().get(i)
								.getTimeStampList().size(), dtimeDiff);
							} else {
								strMessage = "note On " + keyNum
										+ " velocity: " + keyVel;
							}
						}
					}

					break;

				// Writes the controller values of the fader (rating of a subject).
				case 0xb0:
					int ctlNum = message.getData1();
					int ctlValue = message.getData2();

                    /*
                     * Edited in - Stian
                     * complies with bcn44
                     * probably temporary
                     * */

                    //Set startTimeFlag
                    if (ctlNum == 50) {
						startTime = System.currentTimeMillis();
						startTimeFlag = true;
                        Main.userHasPressedStart = true;
                        Main.view.main.play();
                        Main.userHasPressedStart = false;
					}

                    // Increments the group.
					if ((ctlNum == saltForReceiver.design.getMIDIButtonNumberForward())) {
						y = saltForReceiver.currGrp;
						saltForReceiver.incGroup();
						saltForReceiver.sendTrial();
						strMessage = "note on " + ctlNum + " velocity: "
								+ ctlValue + " <Gruppe "
								+ (saltForReceiver.currGrp + 1) + ">";
                        saltForReceiver.session.addMidiActivity(strMessage);
                        Main.view.main.nextTrial();
					} else
					// Decrements the group.
					if ((ctlNum == saltForReceiver.design.getMIDIButtonNumberBackward())) {
						y = saltForReceiver.currGrp;
						if (y > 0) {
							saltForReceiver.decGroup();
							for (int i = 0; i < saltForReceiver.design
								.getNumberOfFaders(); ++i) {
								saltForReceiver.session.getTrialInfos().get(
										saltForReceiver.currGrp).getItemInfos()
										.get(i).setValue(
												saltForReceiver.design
														.getMIDIDefaultValue());

								// Clears the TimeStampList of the previous trial.
								if (saltForReceiver.design.isContinuousButton()) {
									saltForReceiver.session.getTrialInfos()
											.get(saltForReceiver.currGrp)
											.getItemInfos().get(i)
											.getTimeStampList().clear();
                                }
							}

							saltForReceiver.sendTrial();

							strMessage = "note On " + ctlNum + " velocity: "
									+ ctlValue + " <Gruppe "
									+ (saltForReceiver.currGrp + 1) + ">";
                            saltForReceiver.session.addMidiActivity(strMessage);
                            Main.view.main.previousTrial();
						}
					}
                    /*
                     * end of edit - stian
                     * */

                    
					// Continuous rating (fader): By moving a fader the value of the fader 
					// and the time difference to the startTime will be added to the TimeStampList. 
					if (saltForReceiver.design.isContinuousFader() && startTimeFlag == true
						&& saltForReceiver.design.isContinuousButton() == false) {
						for (int i = 0; i < saltForReceiver.design.getNumberOfFaders(); ++i) {
							if (ctlNum == saltForReceiver.design.getMIDIControllerNumberOfFaders().get(i)) {
								long timeDiff = (System.currentTimeMillis() - startTime);
								int listIndex = Math.round(timeDiff / saltForReceiver.design.getTimeSlice());
								int newElements = (int) listIndex - saltForReceiver.session
								.getTrialInfos().get(saltForReceiver.currGrp).getItemInfos().get(i)
								.getValueList().size();
								int size = saltForReceiver.session.getTrialInfos().get(saltForReceiver.
								currGrp).getItemInfos().get(i).getValueList().size();

								// Sets fader value and time slice for the previous trial(s).
								for (int index = 0; index < newElements; index++) {

									if (size == 0) {
										saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
										.getItemInfos().get(i).addValueListElement(size + index,
										saltForReceiver.design.getMIDIDefaultValue());
									} else {
										saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
										.getItemInfos().get(i).addValueListElement(size + index,
										saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
										.getItemInfos().get(i).getValueList().get(size+index-1));
									}

									saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
									.getItemInfos().get(i).addTimeStampListElement(size + index,
									(size + index)* (saltForReceiver.design.getTimeSlice()) / 1000);
								}
								// Sets fader value, MIDI controller value of fader and time slice for the actual trial.
								saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
								.getItemInfos().get(i).setValueListElement((int)(listIndex - 1), ctlValue);

								saltForReceiver.session.getTrialInfos().get(saltForReceiver.currGrp)
								.getItemInfos().get(i).setTimeStampListElement((int) (listIndex - 1),
								(float)((saltForReceiver.design.getTimeSlice() * (listIndex - 1)) / 1000));

								strMessage = "cc " + ctlNum + " value: "
										+ ctlValue + "<Fader " + (i + 1) + ">";
                                saltForReceiver.session.addMidiActivity(strMessage);
								break;
							} else {
								strMessage = "cc " + ctlNum + " value: "+ctlValue;
							}

                            /*
                             * edited in to control buttons - stian
                             * continuous rating
                             * */
                            if (ctlNum == saltForReceiver.design.getMIDIControllerNumberOfButtons().get(i)) {
                                Main.view.main.playActiveItem(i);

								strMessage = "cc " + ctlNum + " value: "
										+ ctlValue + "<Fader button " + (i + 1) + ">";
                                saltForReceiver.session.addMidiActivity(strMessage);
								break;
							} else {
								strMessage = "cc " + ctlNum + " value: "+ctlValue;
							}
                            /*
                             * end of edit
                             * */
						}
					}
					// Noncontinuous rating
					if (saltForReceiver.design.isContinuousFader() == false
							&& saltForReceiver.design.isContinuousButton() == false) {
						for (int i = 0; i < saltForReceiver.design
								.getNumberOfFaders(); ++i) {
							if (ctlNum == saltForReceiver.design
									.getMIDIControllerNumberOfFaders().get(i)) {
								saltForReceiver.session.getTrialInfos().get(
										saltForReceiver.currGrp).getItemInfos()
										.get(i).setValue(ctlValue);

								strMessage = "cc " + ctlNum + " value: "
										+ ctlValue + "<Fader " + (i + 1) + ">";
                                saltForReceiver.session.addMidiActivity(strMessage);
								break;
							} else {
								strMessage = "cc " + ctlNum + " value: "+ ctlValue;
							}
                            
                            /*
                             * edited in to control buttons - stian
                             * continuous rating
                             * */
                            if (ctlNum == saltForReceiver.design.getMIDIControllerNumberOfButtons().get(i)) {
                                Main.view.main.playActiveItem(i);

								strMessage = "cc " + ctlNum + " value: "
										+ ctlValue + "<Fader " + (i + 1) + ">";
                                saltForReceiver.session.addMidiActivity(strMessage);
								break;
							} else {
								strMessage = "cc " + ctlNum + " value: "+ctlValue;
							}
                            /*
                             * end of edit
                             * */
						}
						break;
					}

				default:
					strMessage = "unbekannte message: status = "
							+ message.getStatus() + ", byte1 = "
							+ message.getData1() + ", byte2 = "
							+ message.getData2();
                    saltForReceiver.session.addMidiActivity(strMessage);
					break;
				}
			}
			strMessage = strChannel + strMessage;
		}
		smCount++;
		smByteCount += message.getLength();
		return "[" + getHexString(message) + "] " + strMessage;
	}

	/**
	 * Processes the SysEx Message.
	 * @param message
	 * @return
	 */
	public String decodeMessage(SysexMessage message) {
		byte[] abData = message.getData(); // The returned array of bytes does not include the status byte.
		String strMessage = null;
		Integer dataValue;

		if (message.getStatus() == SysexMessage.SYSTEM_EXCLUSIVE
				&& saltForReceiver.session != null) {
			// strMessage = "Sysex message: F0" + getHexString(abData);
			// Analyses the Data-Bytes.
			String itemHex = getHexString(abData);
			StringTokenizer st = new StringTokenizer(itemHex);

			int byteNum = st.countTokens(); // Byte-Anzahl nach F0
//			saltForReceiver.gui.displayTextAreaMessage(itemHex
//					+ " Bytes nach F0: " + byteNum);

			// Data-Byte 1 (7D)
			dataValue = Integer.valueOf(st.nextToken(), 16);
			if ((dataValue.intValue() == 125) && (byteNum == 11)) {
				saltForReceiver.gui
						.displayTextAreaMessage("IAVAS-String received: F0"
								+ getHexString(abData));
				// Data-Byte 2 (Trial-ID)
				dataValue = Integer.valueOf(st.nextToken(), 16);
				saltForReceiver.session.getTrialInfos().get(
						saltForReceiver.currGrp).setId(dataValue.intValue());

				if (saltForReceiver.design.isContinuousButton() == false) {
					// Data-Byte 3.-10. (Item-IDs)
					for (int i = 0; i < saltForReceiver.design
							.getNumberOfFaders(); i++) {
						dataValue = Integer.valueOf(st.nextToken(), 16);
						saltForReceiver.session.getTrialInfos().get(
								saltForReceiver.currGrp).getItemInfos().get(i)
								.setId(dataValue.intValue());
					}
				}
				if (saltForReceiver.design.isContinuousButton() == true) {
					// Data-Byte 3.-10. (Item-IDs)
					for (int i = 0; i < saltForReceiver.design.getMIDIButtonNumberContinuous().size(); i++) {
						dataValue = Integer.valueOf(st.nextToken(), 16);
						saltForReceiver.session.getTrialInfos().get(
								saltForReceiver.currGrp).getItemInfos().get(i)
								.setId(dataValue.intValue());
					}
				}
			}		
		} else if (message.getStatus() == SysexMessage.SPECIAL_SYSTEM_EXCLUSIVE) {
			strMessage = "Continued Sysex message F7" + getHexString(abData);
		}
		return strMessage;
	}

	private static char hexDigits[] = { '0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

	/**
	 * @param aByte
	 * @return
	 */
	public static String getHexString(byte[] aByte) {
		StringBuffer sbuf = new StringBuffer(aByte.length * 3 + 2);
		for (int i = 0; i < aByte.length; i++) {
			sbuf.append(' ');
			sbuf.append(hexDigits[(aByte[i] & 0xF0) >> 4]);
			sbuf.append(hexDigits[aByte[i] & 0x0F]);
		}
		return new String(sbuf);
	}

	/**
	 * @param i
	 * @return
	 */
	private static String intToHex(int i) {
		return "" + hexDigits[(i & 0xF0) >> 4] + hexDigits[i & 0x0F];
	}

	/**
	 * @param sm
	 * @return
	 */
	public static String getHexString(ShortMessage sm) {
		// bug in J2SDK 1.4.1
		// return getHexString(sm.getMessage());
		int status = sm.getStatus();
		String res = intToHex(sm.getStatus());
		// if one-byte message, return
		switch (status) {
		case 0xF6: // Tune Request
		case 0xF7: // EOX
			// System real-time messages
		case 0xF8: // Timing Clock
		case 0xF9: // Undefined
		case 0xFA: // Start
		case 0xFB: // Continue
		case 0xFC: // Stop
		case 0xFD: // Undefined
		case 0xFE: // Active Sensing
		case 0xFF:
			return res;
		}
		res += ' ' + intToHex(sm.getData1());
		// if 2-byte message, return
		switch (status) {
		case 0xF1: // MTC Quarter Frame
		case 0xF3: // Song Select
			return res;
		}
		switch (sm.getCommand()) {
		case 0xC0:
		case 0xD0:
			return res;
		}
		// 3-byte messages left
		res += ' ' + intToHex(sm.getData2());
		return res;
	}
}

/*** DumpReceiver.java ***/

