package SALT.assessment;

import java.util.List;
import java.util.ArrayList;

/**
 * Provides getters and setters for an ItemInfo and a method to compare ItemInfo objects.
 * @see java.lang.Comparable
 * 
 */
public class ItemInfo implements Comparable {

	private int id;
	private int value;
	private List<Integer> valueList;
	private List<Float> timeStampList;
	private List<Integer> buttonList;
	
	/**
	 * Constructs an ItemInfo with the specified ID and rating.
	 * @param id
	 * @param value
	 */
	public ItemInfo(int id, int value) {
		this.id = id;
		this.value = value;
		valueList = new ArrayList<Integer>();
		buttonList = new ArrayList<Integer>();
		timeStampList = new ArrayList<Float>();
	}
	
	/**
	 * Constructs an ItemInfo with the specified ID and rating.
	 * @param id
	 * @param value
	 */
	public ItemInfo(int id) {
		this.id = id;
		value = 0;
		valueList = new ArrayList<Integer>();
		buttonList = new ArrayList<Integer>();
		timeStampList = new ArrayList<Float>();
	}
	
	/**
	 * Returns the ID of the item.
	 * @return
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the ID of the item.
	 * @param id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the rating for the item.
	 * @return
	 */
	public int getValue() {
		return value;
	}

	/**
	 * Sets the rating for the item.
	 * @param value
	 */
	public void setValue(int value) {
		this.value = value;
	}

	/**
	 * Compares item IDs.
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object otherObject) {
		int result = 0;
		ItemInfo myItem = (ItemInfo) otherObject;

		if (myItem.getId() < id) {
			result = 1;
		}

		if (myItem.getId() > id) {
			result = -1;
		}
		return result;
	}

	public List<Integer> getValueList() {
		return valueList;
	}

	public void setValueList(List<Integer> valueList) {
		this.valueList = valueList;
	}
	
	public void addValueToValueList(int value){
		valueList.add(new Integer(value));
	}
	
	
	public List<Integer> getButtonList() {
		return buttonList;
	}

	public void setButtonList(List<Integer> buttonList) {
		this.buttonList = buttonList;
	}
	
	public void addButtonToButtonList(int value){
		buttonList.add(new Integer(value));
	}
	

	public void setValueListElement(int index, int value) {
		valueList.set(index, value);
	}
	
	public void addValueListElement(int index, int value) {
		valueList.add(index, value);
	}

	
	public List<Float> getTimeStampList() {
		return timeStampList;
	}
	
	public void setTimeStampList(List<Float> timeStampList) {
		this.timeStampList = timeStampList;
	}
		
	public void addTimeStampToTimeStampList(float timeStamp){
	timeStampList.add(timeStamp);
	}
	
	public void setTimeStampListElement(int index, float timeStamp) {
		timeStampList.set(index, timeStamp);
	}
	
	public void addTimeStampListElement(int index, float timeStamp) {
		timeStampList.add(index, timeStamp);
	}
}
