package SALT.assessment;

import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Transmitter;
import javax.swing.*;

import SALT.exceptions.WriterException;
import SALT.gui.GUI;
import mediaplayer.TimeCodeListenerThread;

/**
 * Main class for Salt.
 * @see java.awt.event.ActionListener
 * @see java.awt.event.WindowListener;
 */
public class Salt implements ActionListener, WindowListener {
	public GUI gui;

	// actual group number
	public int currGrp = 0;
	
	Receiver receiver = null;

	public Design design;
	public Session session;
    public DumpReceiver dumpReceiver;

	// Contains data about the MidiInputDevices or MidiOutputDevices. 
	private List<MidiDevice.Info> midiInputInfoVector;
	private List<MidiDevice.Info> midiOutputInfoVector;

    private MidiDevice timeCodeInputDevice = null;
	private MidiDevice inputDevice = null;
	private MidiDevice outputDevice = null;
		
	JFrame frame;

	/**
	 * Constructs needful objects for Salt.
	 */
	public Salt() {
		midiInputInfoVector = new ArrayList<MidiDevice.Info>();
		midiOutputInfoVector = new ArrayList<MidiDevice.Info>();

		getMidiInfos();


        /*
         *
         * Commented out. No SALT main gui in the control suite
         * Stian
         *
         * */
//		try {
//			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
//		} catch (Exception e) {
//			// TODO
//		}
//		gui = new GUI(this, this, midiInputInfoVector, midiOutputInfoVector);
//		design = new Design();
	}

	/**
	 * Creates a new session.
	 */
	public void newSession() {
		boolean b = gui.displayNewSessionDialog(session);
		
		if (b){
			sendTrial();

			// Clears the TextArea.
			gui.clearTextArea();
	
			//	Displays the "Test is running" MessageDialog.
			gui.displayWorkingTestMessage();
            
			try {
				File outputFile = new File(design.getTargetFolder() + "/"
					+ session.getFirstName() + "_" + session.getLastName()
					+ "_" + session.getAge() + ".log");
				SessionWriter.writeSessionToLogFile(outputFile, session, design);
			
			} catch (WriterException e) {
				e.printStackTrace();
			}
		
			session.clearTrialInfos();		
			session = null;
		}
	}

	/**
	 * Closes MIDI Devices. 
	 */
	public void closeMidiDevices() {
        if (timeCodeInputDevice != null) {
			timeCodeInputDevice.close();
			gui.displayTextAreaMessage("MIDI Input: "
					+ midiInputInfoVector.get(design.getInputDevice())
							.toString() + " is closed.");
		}
		if (inputDevice != null) {
			inputDevice.close();
			gui.displayTextAreaMessage("MIDI Input: "
					+ midiInputInfoVector.get(design.getInputDevice())
							.toString() + " is closed.");
		}
		if (outputDevice != null) {
			outputDevice.close();
			gui.displayTextAreaMessage("MIDI Output: "
					+ midiOutputInfoVector.get(design.getOutputDevice())
							.toString() + " is closed.");
		}
	}

	/**
	 * Opens MIDI Devices.  
	 */
	public void openMidiDevices() {
        String warningString = new String();
		boolean bcatch = false;
		// Returns if no MIDI time code input, input or output device is chosen.
		if (design.getOutputDevice() == -1 || design.getInputDevice() == -1 || design.getTimeCodeInputDevice() == -1) {
			return;
		}
		
		// Returns if the MIDI input or output device do not exist.
		if (design.getOutputDevice() > midiOutputInfoVector.size()-1 ||
				design.getInputDevice() > midiInputInfoVector.size()-1 ||
                design.getTimeCodeInputDevice() > midiInputInfoVector.size()-1) {
			design.setOutputDevice(-1);
			design.setInputDevice(-1);
            design.setTimeCodeInputDevice(-1);
			return;
		}
        
        try {
			timeCodeInputDevice = MidiSystem.getMidiDevice(midiInputInfoVector
					.get(design.getTimeCodeInputDevice()));
			timeCodeInputDevice.open();
			Transmitter transmitter = timeCodeInputDevice.getTransmitter();
			transmitter.setReceiver(new TimeCodeListenerThread());

            /*
             * Commented out, no gui
             * Stian
             * */
//			gui.displayTextAreaMessage("MIDI Input: "
//					+ midiInputInfoVector.get(design.getInputdevice())
//							.toString() + " is opened.");
		} catch (MidiUnavailableException e) {
			warningString = warningString + "Midi Input Device already used!\n";
			bcatch = true;
		}
		
		try {
			inputDevice = MidiSystem.getMidiDevice(midiInputInfoVector
					.get(design.getInputDevice()));
			inputDevice.open();
			Transmitter transmitter = inputDevice.getTransmitter();
            dumpReceiver = new DumpReceiver(System.out, this);
			transmitter.setReceiver(dumpReceiver);
            /*
             * Commented out, no gui
             * Stian
             * */
//			gui.displayTextAreaMessage("MIDI Input: "
//					+ midiInputInfoVector.get(design.getInputdevice())
//							.toString() + " is opened.");
		} catch (MidiUnavailableException e) {
			warningString = warningString + "Midi Input Device already used!\n";
			bcatch = true;
		}

		try {
			outputDevice = MidiSystem.getMidiDevice(midiOutputInfoVector
					.get(design.getOutputDevice()));
			outputDevice.open();
			receiver = outputDevice.getReceiver();
            /*
             *
             * Commented out, no gui
             * */
//			gui.displayTextAreaMessage("MIDI Output: "
//					+ midiOutputInfoVector.get(design.getOutputdevice())
//							.toString() + " is opened.");
		} catch (MidiUnavailableException e) {
			warningString = warningString + "Midi Output Device already used!";	
			bcatch = true;
		}
		
		if(bcatch){			
			JOptionPane.showMessageDialog(frame,
					"Design couldn�t be opened!\n" +  warningString,
					"Warning!", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Increases the actual group.
	 */
	public void incGroup() {
		++currGrp;
	}

	/**
	 * Decreases the actual group.
	 */
	public void decGroup() {
		--currGrp;
	}

	/**
	 * Recalls MIDI System Informations.
	 */
	private void getMidiInfos() {

		MidiDevice.Info[] infos = MidiSystem.getMidiDeviceInfo();

		for (int i = 0; i < infos.length; i++) {
			try {
				MidiDevice device = MidiSystem.getMidiDevice(infos[i]);

				// MIDIOutputs
				if (device.getMaxReceivers() != 0) {
					midiOutputInfoVector.add(infos[i]);
				}

				// MIDIInputs
				if (device.getMaxTransmitters() != 0) {
					midiInputInfoVector.add(infos[i]);
				}
			} catch (Exception e) {
				//TODO
			}
		}
	}

	/**
	 * Adds a new trial to the session and resets the fader position.
	 */
	public void sendTrial() {

		int scaledValue = 0;

		if (session.getTrialInfos().size() <= currGrp) {
			TrialInfo trialInfo = new TrialInfo(0);
			for (int i = 0; i < design.getNumberOfFaders(); i++) {
				trialInfo.addItemInfo(new ItemInfo(0, design
						.getMIDIDefaultValue()));
			}
			session.addTrialInfo(trialInfo);

            /*
             * edited in - stian
             * needs to tie trial info to trial in main app
             * */
		}

		TrialInfo currentTrial = session.getTrialInfos().get(currGrp);
		List<ItemInfo> currentItemList = currentTrial.getItemInfos();
		ItemInfo currentItem;

		List<Integer> MIDIControllerNumberOfFadersList = design
				.getMIDIControllerNumberOfFaders();
		for (int i = 0; i < design.getNumberOfFaders(); ++i) {

			currentItem = currentItemList.get(i);

            //Commented out. The value should already be in range
			// Scales the values from 0...127.
			//scaledValue = (int) Math.ceil((double) (currentItem.getValue() * 1.27));

			// Sends Controller Messages to fader.
			sendShortMessage(0, MIDIControllerNumberOfFadersList.get(i),
				scaledValue);
		}
	}

	/**
	 * Sends Controller Messages. 
	 * @param ch
	 * @param ccNumb
	 * @param value
	 */
	public void sendShortMessage(int ch, int ccNumb, int value) {
		// Inits the message.
		ShortMessage ctlValueOut = new ShortMessage();

		if (outputDevice.isOpen()) {
			try {
				ctlValueOut.setMessage(ShortMessage.CONTROL_CHANGE, ch, ccNumb,
						value);
				// Sends a MIDI message and time-stamp to receiver.
				receiver.send(ctlValueOut, -1);
			} catch (InvalidMidiDataException e) {
				gui.displayTextAreaMessage("ACHTUNG: MIDI-Controller Message "
						+ "konnte nicht gesendet werden!");
			}
		} else {
			gui.displayTextAreaMessage("ACHTUNG: kein MIDI-Ausgang gew�hlt!");
		}
	}
    
    /*
     * 
     * Added getters and setters to get access to midi info without using a
     * gui instance
     * Stian
     * 
     * */

    public List<Info> getMidiInputInfoVector() {
        return midiInputInfoVector;
    }

    public void setMidiInputInfoVector(List<Info> midiInputInfoVector) {
        this.midiInputInfoVector = midiInputInfoVector;
    }

    public List<Info> getMidiOutputInfoVector() {
        return midiOutputInfoVector;
    }

    public void setMidiOutputInfoVector(List<Info> midiOutputInfoVector) {
        this.midiOutputInfoVector = midiOutputInfoVector;
    }

    /*
     *
     * End of getters and setters
     * Stian
     *
     * */

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {

		String cmd = event.getActionCommand();

		if (cmd.equals("NEWDESIGN")) {
			closeMidiDevices();
			Design tempDesign = gui.displayNewDesignDialog();
			
			if (tempDesign!=null) {
				design = tempDesign;
			}
			
			openMidiDevices();
		}

		if (cmd.equals("OPENDESIGN")) {
			closeMidiDevices();
			Design tempDesign = gui.displayOpenDesignDialog();
			
			if (tempDesign!=null) {
				design = tempDesign;
			}
			
			openMidiDevices();
		}
		
		if (cmd.equals("SAVEDESIGN")) {
			File selectedOutputFile = gui.displaySaveFileChooser();
			File outputFile = new File(selectedOutputFile.getAbsolutePath()+ ".design");
			try {
				DesignWriter.writeDesignXML(outputFile, design);
			} catch (Exception e) {
				//TODO
			}
		}
		if (cmd.equals("NEWSESSION")) {
			currGrp = 0;
			if (session == null) {
				session = new Session();
			}
			session.initWithDesign(design);
			
			// By opening the Session Dialog a Message Dialog is shown,
			// if no MIDI input or output device is chosen.  
			if (design.getOutputDevice() == -1 || design.getInputDevice() == -1) {
				JOptionPane.showMessageDialog(frame,
						"Choose Midi Input and Output Device by using Edit Design! \n",
						"Warning!", JOptionPane.ERROR_MESSAGE);
				return;
			}
			newSession();
		}
		if (cmd.equals("OPENSESSION")) {
			session = gui.openSession(design);
		}
		if (cmd.equals("EXPORT")) {
			gui.displayExportDialog(design);
		}
		if (cmd.equals("EXIT")) {
			gui.setVisible(true);
			gui.dispose();
		}
		if (cmd.equals("EDITDESIGN")) {
			closeMidiDevices();
			gui.displayEditDesignDialog(design);
			openMidiDevices();
		}
		if (cmd.equals("SALTHELP")) {
			gui.displayHelpDialog();
		}
		if (cmd.equals("ABOUTSALT")) {
			gui.displayInformationDialog();
		}
	}

	/**
	 * Main method of Salt.
	 * @param args
	 */
	public static void main(String[] args) {
		Salt salt = new Salt();
	}
	
	/**
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent arg0) {
	}
	/**
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent arg0) {
		closeMidiDevices();
		System.exit(0);
	}
	/**
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent arg0) {
		gui.setVisible(true);
		gui.dispose();
	}
	/**
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent arg0) {
	}
	/**
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent arg0) {
	}
	/**
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent arg0) {
	}
	/**
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent arg0) {
	}
}

/** * Salt.java ** */
