package SALT.assessment;

import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import mediaplayer.Main;

/**
 * Contains all settings, which specify a test subject. 
 *
 */
public class Session {

	private String firstName;
	private String lastName;
	private int Age;

	private List<XtraInfo> XtraInfos;
	private List<TrialInfo> TrialInfos;

    /*
     * extra variables added - Stian
     * */
    private List<String> midiActivity;
    /*
     * end of adding - stian
     * */

	/**
	 * Constructs a XtraInfos and TrialInfos object.
	 */
	public Session() {
		XtraInfos = new ArrayList<XtraInfo>();
		TrialInfos = new ArrayList<TrialInfo>();
        midiActivity = new ArrayList<String>();
	}

	/**
	 * Returns Age.
	 * @return
	 */
	public int getAge() {
		return Age;
	}

	/**
	 * Sets Age.
	 * @param age
	 */
	public void setAge(int age) {
		Age = age;
	}

	/**
	 * Returns firstName.
	 * @return
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * Sets firstName.
	 * @param firstName
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * Returns lastName.
	 * @return
	 */
	public String getLastName() {
		return lastName;
	}

	/**
	 * Sets lastName.
	 * @param lastName
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	/**
	 * Returns a list of XtraInfos.
	 * @return
	 */
	public List<XtraInfo> getXtraInfos() {
		return XtraInfos;
	}

	/**
	 * Sets a list of XtraInfos.
	 * @param xtraInfos
	 */
	public void setXtraInfos(List<XtraInfo> xtraInfos) {
		XtraInfos = xtraInfos;
	}

	/**
	 * Adds a xtraInfo to the list of XtraInfos.
	 * @param xtraInfos
	 */
	public void addXtraInfo(XtraInfo xtraInfos) {
		XtraInfos.add(xtraInfos);
	}
	
	/**
	 * Returns a list of TrialInfos.
	 * @return
	 */
	public List<TrialInfo> getTrialInfos() {
		return TrialInfos;
	}

	/**
	 * Sets a list of TrialInfos.
	 * @param trialInfos
	 */
	public void setTrialInfos(List<TrialInfo> trialInfos) {
		TrialInfos = trialInfos;
	}
	
	/**
	 * Adds a trialInfo to the list of TrialInfos.
	 * @param trialInfo
	 */
	public void addTrialInfo(TrialInfo trialInfo) {
		TrialInfos.add(trialInfo);
	}
	
	/**
	 *  Removes all of the elements from the list of TrialInfos.
	 */
	public void clearTrialInfos() {
		TrialInfos.clear();
	}

    /*
     * Returns the list of midi activities - Stian
     * */
    public List<String> getMidiMessages() {
        return midiActivity;
    }

    /*
     * Adds a string describing midi activity
     * */
    public void addMidiActivity(String s) {
        s = "tc: "+Main.timeCode+"; " + s;
        midiActivity.add(s);
    }

	/**
	 *  Sorts the trials and items into ascending order.
	 */
	public void sortSession() {
		// Sorts the items for each existing trial into ascending order.
		for (TrialInfo currentTrialInfo : TrialInfos) {
			currentTrialInfo.sortItemInfos();
		}

		// Sorts the specified TrialInfos list into ascending order.
		Collections.sort(TrialInfos);
	}

	/**
	 * Adds all XtraInfos from the Design and one TrialInfo to the session. 
	 * @param design
	 */
	public void initWithDesign(Design design) {
			
		if (TrialInfos != null) TrialInfos.clear();

        TrialInfo trialInfo = new TrialInfo(0);
		
		if (design.isContinuousButton() == false){
			for (int j = 0; j < design.getNumberOfFaders(); ++j) {
				trialInfo.addItemInfo(new ItemInfo(0, design.getMIDIDefaultValue()));
			}
		}
		else{
			for (int j = 0; j < design.getMIDIButtonNumberContinuous().size(); ++j) {
				trialInfo.addItemInfo(new ItemInfo(0));
			}	
		}			
		TrialInfos.add(trialInfo);
	}
}
