package SALT.assessment;

import org.jdom.input.*;
import org.jdom.Document;
import org.jdom.Element;

import java.io.File;
import java.util.List;
import java.util.ArrayList;

/**
 * To work with a session in SALT, the SessionReader generates a Session object from the data in a .log file.
 * 
 */
public class SessionReader {
	// Builds a JDOM document from files using a SAX parser. 
	private static SAXBuilder builder = new SAXBuilder();

	/**
	 * Writes the data from a .log-File, which has XML style in a Session object.
	 * @param file .log file
	 * @return Session object
	 */
	public static Session getSortedTrialInfos(File file, Design design) {
		Session currentTestSession = new Session();
		
		// Creates a new empty document.
		Document doc = new Document();
		try {
			// Builds a document from the supplied filename.
			doc = builder.build(file);
		} catch (Exception e) {
			// TODO: handle exception
		}
		
		// Reads the informations from the XML document and writes in a session.
		Element root = doc.getRootElement();
		Element child = root.getChild("subjectInfo", SessionWriter.tuiNSresult);
		currentTestSession.setFirstName(child.getChild("firstName",
				SessionWriter.tuiNSresult).getText());
		currentTestSession.setLastName(child.getChild("lastName",
				SessionWriter.tuiNSresult).getText());
		currentTestSession.setAge(Integer.valueOf(child.getChild("age",
				SessionWriter.tuiNSresult).getText()));
		List<Element> childList = child.getChildren("xtraInfo",
				SessionWriter.tuiNSresult);
		for (Element currentChild : childList) {
			currentTestSession.addXtraInfo(new XtraInfo(currentChild
					.getAttribute("name").getValue(), currentChild.getText()));
		}

		List<Element> childListExtend = new ArrayList<Element>();
		List<Element> childListTimeStamp = new ArrayList<Element>();
		
		childList = root.getChildren("trial", SessionWriter.tuiNSresult);

		for (Element currentChild : childList) {
			TrialInfo currentTrial = new TrialInfo(Integer.valueOf(currentChild
					.getAttribute("id").getValue()));
			childListExtend = currentChild.getChildren("item",
					SessionWriter.tuiNSresult);
			for (Element currentChildExtend : childListExtend) {		
				// noncontinuous rating
				if(design.isContinuousButton() == false && design.isContinuousFader() == false){
				currentTrial.addItemInfo(new ItemInfo(Integer
						.valueOf(currentChildExtend.getAttribute("id")
								.getValue()), Integer
						.valueOf(currentChildExtend.getText())));			
				}
				
				// continuous rating (fader)
				if(design.isContinuousButton() == false && design.isContinuousFader() == true){
					ItemInfo currentItem = new ItemInfo(Integer
						.valueOf(currentChildExtend.getAttribute("id").getValue()));				
					childListTimeStamp = currentChildExtend.getChildren("timestamp",
						SessionWriter.tuiNSresult);
					int index = 0;
					for (Element currentchildListTimeStamp : childListTimeStamp) {
						currentItem.addTimeStampListElement(index, Float
								.valueOf(currentchildListTimeStamp.getAttribute("time").getValue()));
						currentItem.addValueListElement(index, Integer
								.valueOf(currentchildListTimeStamp.getText()));
						index++;
					}
					currentTrial.addItemInfo(currentItem);
				}
				
				// continuous rating (button)
				if(design.isContinuousButton() == true && design.isContinuousFader() == false){
					ItemInfo currentItem = new ItemInfo(Integer
						.valueOf(currentChildExtend.getAttribute("id").getValue()));				
					childListTimeStamp = currentChildExtend.getChildren("timestamp",
						SessionWriter.tuiNSresult);
					int index = 0;
					for (Element currentchildListTimeStamp : childListTimeStamp) {
						currentItem.addTimeStampListElement(index, Float
								.valueOf(currentchildListTimeStamp.getText()));
						index++;
					}
					currentTrial.addItemInfo(currentItem);
				}
			}			
			currentTestSession.addTrialInfo(currentTrial);
		}
		currentTestSession.sortSession();
		return currentTestSession;
	}
}
