package SALT.assessment;

import java.util.*;
import java.io.*;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import SALT.exceptions.WriterException;
import mediaplayer.Main;
import mediaplayer.Trial;

/**
 * The SessionWriter provides two writing methods to write the data from a Session object to a .log file. 
 * 
 */
public class SessionWriter {

	public static final Namespace tuiNSresult = Namespace
			.getNamespace("urn:tu-ilmenau:de:imt:emt:subjective-assessment:result");

	/**
	 * Writes a Session in a .log file, which has XML style.
	 * @param outputFile the target .log file
	 * @param session the selected session
	 * @param design the design to get the informations in the session
	 * @throws WriterException
	 */
	public static void writeSessionToLogFile(File outputFile, Session session,
			Design design) throws WriterException {

		// Creates a new empty document.
		Document doc = new Document();

		Element rootElement = new Element("testResults", tuiNSresult);
		doc.setRootElement(rootElement);

		Element subjectInfoElement = new Element("subjectInfo", tuiNSresult);
		rootElement.addContent(subjectInfoElement);

		Element firstNameElement = new Element("firstName", tuiNSresult);
		firstNameElement.setText(session.getFirstName());
		subjectInfoElement.addContent(firstNameElement);

		Element lastNameElement = new Element("lastName", tuiNSresult);
		lastNameElement.setText(session.getLastName());
		subjectInfoElement.addContent(lastNameElement);

		Element ageElement = new Element("age", tuiNSresult);
		ageElement.setText(String.valueOf(session.getAge()));
		subjectInfoElement.addContent(ageElement);
		
		// Generates xtraInfo elements.
		List<XtraInfo> xtraInfoList = session.getXtraInfos();
		for (XtraInfo xtrainfo : xtraInfoList) {
			Element xtraInfoElement = new Element("xtraInfo", tuiNSresult);
			Attribute nameAttribute = new Attribute("name", xtrainfo.getName());
			xtraInfoElement.setAttribute(nameAttribute);
			xtraInfoElement.setText(xtrainfo.getValue());
			subjectInfoElement.addContent(xtraInfoElement);
		}
		
		// Writes the TrialInfos with their ItemInfos.
		List<TrialInfo> trialInfoList = session.getTrialInfos();
		for (int i=0;i<trialInfoList.size() && i<Main.view.main.playlist.getList().size();i++) {
			TrialInfo trialinfo = trialInfoList.get(i);
//			String trialName = design.getTrialIDName(trialinfo.getId());
//			if (trialName == null) trialName = "No Identifier!";
            String trialName = Main.view.main.playlist.getTrial(i).getName();

					
			Element trialElement = new Element("trial", tuiNSresult);
			trialElement.setAttribute(new Attribute("name", trialName));
//			trialElement.setAttribute(new Attribute("id", String
//					.valueOf(trialinfo.getId())));
            trialElement.setAttribute(new Attribute("id", Main.view.main.playlist.getTrial(i).id+""));
			rootElement.addContent(trialElement);
			for (int j=0;j<trialinfo.getItemInfos().size() && j<Trial.getItemsPerTrial();j++) {
                ItemInfo iteminfo = trialinfo.getItemInfos().get(j);
				//String itemName = design.getItemIDName(iteminfo.getId());
                //Edited in - stian
                String itemName = Main.view.main.playlist.getTrial(i).items[j].name;
				if (itemName == null) itemName = "No Identifier!";
				
					// Writes the TrialInfos with their ItemInfos for the continuous rating (button). 
					if(design.isContinuousButton()){						
							Element itemElement = new Element("item", tuiNSresult);
							itemElement.setAttribute(new Attribute("name", itemName));
//							itemElement.setAttribute(new Attribute("id", String
//									.valueOf(iteminfo.getId())));
                            itemElement.setAttribute(new Attribute("id", j+""));    //Edited in - stian
							for(int index = 0; index < iteminfo.getTimeStampList().size(); index++){
								Element buttonStampElement =  new Element("timestamp", tuiNSresult);
								buttonStampElement.setText("" + iteminfo.getTimeStampList().get(index));							
								itemElement.addContent(buttonStampElement);
							}
							trialElement.addContent(itemElement);
					}
					// Writes the TrialInfos with their ItemInfos for the continuous rating (fader).
					if(design.isContinuousFader()){
                        float remainingTime = 0;
                        float timeStamp = 0;
						if(iteminfo.getTimeStampList().size() != 0) {
                            remainingTime = (design.getTotalTime()/1000) - (iteminfo.getTimeStampList().get
                                ((iteminfo.getTimeStampList().size())-1));
                            timeStamp = (iteminfo.getTimeStampList().get
								((iteminfo.getTimeStampList().size())-1));
                        }
						int newElements = Math.round(remainingTime/(design.getTimeSlice()/1000));
						for(int noNewElements = 0; noNewElements < newElements; noNewElements++){
							timeStamp = timeStamp+(design.getTimeSlice()/1000);
							iteminfo.addTimeStampToTimeStampList(timeStamp);
							iteminfo.addValueToValueList(iteminfo.getValueList().get
									((iteminfo.getValueList().size())-1));
						}
						
						Element itemElement = new Element("item", tuiNSresult);
						itemElement.setAttribute(new Attribute("name", itemName));
//						itemElement.setAttribute(new Attribute("id", String
//									.valueOf(iteminfo.getId())));
                        itemElement.setAttribute(new Attribute("id", j+""));    //Edited in - stian
						for(int index = 0; index < iteminfo.getValueList().size(); index++){	
							Element timeStampElement =  new Element("timestamp", tuiNSresult);
							timeStampElement.setAttribute(new Attribute("time", String
									.valueOf(iteminfo.getTimeStampList().get(index))));
							timeStampElement.setText("" + iteminfo.getValueList().get(index));							
							itemElement.addContent(timeStampElement);
						}
						trialElement.addContent(itemElement);
					}
					// Writes the TrialInfos with their ItemInfos for the noncontinuous rating.
					if(design.isContinuousButton() == false && 
							design.isContinuousFader() == false){
						Element itemElement = new Element("item", tuiNSresult);
						itemElement.setAttribute(new Attribute("name", itemName));
//						itemElement.setAttribute(new Attribute("id", String
//									.valueOf(iteminfo.getId())));
                        itemElement.setAttribute(new Attribute("id", j+""));    //Edited in - stian
						itemElement.setText("" + iteminfo.getValue());
						trialElement.addContent(itemElement);
					}
				}
		}

        /*
         * Edited in - Stian
         * */
        List<String> midiMessages = session.getMidiMessages();
        Element log = new Element("log", tuiNSresult);
        rootElement.addContent(log);
        for (String midiMessage : midiMessages) {
            Element line = new Element("activity", tuiNSresult);
            line.setAttribute("text", midiMessage);
            log.addContent(line);
        }
        /*
         * End of edit - Stian
         * */

		// Creates an XMLOutputter with the default Format, which outputs a JDOM document.
		XMLOutputter outputter = new XMLOutputter(Format.getPrettyFormat());

		try {
			FileOutputStream stream = new FileOutputStream(outputFile);
			OutputStreamWriter writer = new OutputStreamWriter(stream, "UTF-8");

			// Prints the Document to the Writer.
			outputter.output(doc, writer);

			writer.close();
			stream.close();

            Main.view.main.status.AddMessage("Session written");

		} catch (FileNotFoundException e) {
			throw new WriterException(e);
		} catch (UnsupportedEncodingException e) {
			throw new WriterException(e);
		} catch (IOException e) {
			throw new WriterException(e);
		}
	}

	/**
	 * Writes all specified sessions in a .sum-File, which has CSV style and can be imported especially in Microsoft Excel.
	 * @param outputFile the target .sum file
	 * @param subjectInfosList only this subject infos will be write to the .sum file
	 * @param sessions list of selected sessions
     * NO LONGER IN USE!! Stian
	 */
//	public static void writeSessionsToSumFile(File outputFile, List<String> subjectInfosList,
//			List<Session> sessions, Design design, boolean trialItemID) {
//
//		try {
//			FileWriter outWriter = new FileWriter(outputFile);
//
//			// Writes the title line.
//			outWriter.write("SubjectNumber");
//			for (String subjectInfo : subjectInfosList){
//				outWriter.write("|" + subjectInfo);
//			}
//
//			// Creates list of trial IDs to sort them, because the HashMap is unsorted.
//			HashMap<Integer, String> trialMap = design.getTrialIDandName();
//			Set<Integer> KeysSet = trialMap.keySet();
//			Integer trialKey = new Integer(0);
//
//			List<Integer> trialkeylist = new ArrayList<Integer>();
//			for (Iterator<Integer> iterator = KeysSet.iterator(); iterator
//			.hasNext() == true;) {
//				trialKey = iterator.next();
//				trialkeylist.add(trialKey);
//			}
//
//			// Creates lists of item IDs to sort them, because the HashMap is unsorted.
//			HashMap<Integer, String> itemMap = design.getItemIDandName();
//			Set<Integer> itemKeysSet = itemMap.keySet();
//			Integer itemKey = new Integer(0);
//
//			List<Integer> itemkeylist = new ArrayList<Integer>();
//			for (Iterator<Integer> iterator = itemKeysSet.iterator(); iterator
//			.hasNext() == true;) {
//				itemKey = iterator.next();
//				itemkeylist.add(itemKey);
//			}
//
//			// Sorts the trialkeylist and itemkeylist.
//			Collections.sort(trialkeylist);
//			Collections.sort(itemkeylist);
//
//			int trialCounter = 0;
//			int index = 0;
//
//			Integer lastTrialID = 0;
//			List<Integer> trialCounterList = new ArrayList<Integer>();
//
//			for (int listIndex = 0; listIndex < trialkeylist.size(); listIndex++) {
//				trialCounterList.add(listIndex, 0);
//			}
//
//			// Runs through the sessions to create a list with the maximum number
//			// of iterations of each trial (e.g. 1 2 1 0 0...).
//			for (Session session : sessions) {
//				List<TrialInfo> trialInfoList = session.getTrialInfos();
//
//				// erase all items in the list which are not in design
//				for (int i = 0; i < trialInfoList.size();)
//				{
//					if (!trialkeylist.contains(trialInfoList.get(i).getId()))
//					{
//						trialInfoList.remove(i);
//					}
//					else
//					{
//						++i;
//					}
//				}
//
//				lastTrialID = trialInfoList.get(0).getId();
//
//				for (int p = 1; p < trialInfoList.size(); p++) {
//					if (lastTrialID.equals(trialInfoList.get(p).getId())){
//						trialCounter++;
//
//						index = trialkeylist.indexOf(trialInfoList.get(p).getId());
//
//						if(trialCounter >= trialCounterList.get(index)){
//							trialCounterList.set(index, trialCounter);
//						}
//					}
//					else{
//						lastTrialID = trialInfoList.get(p).getId();
//						trialCounter = 0;
//					}
//				}
//
//				trialCounter = 0;
//			}
//
//			int j = 0;
//			int t = 0;
//			List<Integer> trialListArray = new ArrayList<Integer>();
//
//			if(trialItemID == true){
//				for (Integer triallistelement : trialkeylist) {
//					for(int repeat = -1; repeat < trialCounterList.get(j); repeat++){
//						trialListArray.add(t, triallistelement);
//						t++;
//					}
//					j++;
//				}
//			}
//
//
//			int lastTimeStampListSize = 0;
//			int currentTimeStampListSize = 0;
//			List<Integer> itemTimeStampList = new ArrayList<Integer>();
//			List<Integer> lastitemTimeStampList = new ArrayList<Integer>();
//
//			List<List<Integer>> trialTimeStampList = new ArrayList<List<Integer>>();
//
//			for (int itemIndex = 0; itemIndex < itemkeylist.size(); itemIndex++) {
//				itemTimeStampList.add(itemIndex, 0);
//			}
//
//			for (int trialIndex = 0; trialIndex < trialListArray.size(); trialIndex++) {
//				trialTimeStampList.add(trialIndex, itemTimeStampList);
//			}
//
//			for (int trialListArrayElement = 0;  trialListArrayElement < trialListArray.size(); trialListArrayElement++) {
//				for (int itemkeylistElement = 0; itemkeylistElement < itemkeylist.size(); itemkeylistElement++) {
//					for (Session session : sessions) {
//						for (int c = 0; c < session.getTrialInfos().size(); c++) {
//							if((trialListArray.get(trialListArrayElement)).equals(session.getTrialInfos().get(c).getId())){
//								for (int k = 0; k < session.getTrialInfos().get(c).getItemInfos().size(); k++) {
//									if((itemkeylist.get(itemkeylistElement)).equals(session.getTrialInfos().get(c).getItemInfos().get(k).getId())){
//										currentTimeStampListSize = session.getTrialInfos().get(c).getItemInfos().get(k)
//										.getTimeStampList().size();
//										if (currentTimeStampListSize > lastTimeStampListSize) {
//											lastTimeStampListSize = currentTimeStampListSize;
//										}
//									}
//								}
//							}
//						}
//					}
//					itemTimeStampList.set(itemkeylistElement, lastTimeStampListSize);
//					currentTimeStampListSize = 0;
//					lastTimeStampListSize = 0;
//				}
//				for (int i = 0; i < itemTimeStampList.size(); i++) {
//					lastitemTimeStampList.add(i, itemTimeStampList.get(i));
//				}
//				trialTimeStampList.set(trialListArrayElement, lastitemTimeStampList);
//				for (int newItemIndex = 0; newItemIndex < itemkeylist.size(); newItemIndex++) {
//					itemTimeStampList.set(newItemIndex, 0);
//				}
//			}
//
//			//int trialTimeStampListIndex = -1;
//			int n = 0;
//
//			// Creates a list which consists of trial IDs which are repeated
//			// according to the trialCounterList (e.g. 1 1 2 2 2 3 3 4 5...).
//			// Writes trial and item IDs.
//			if(trialItemID == true){
//				for (Integer triallistelement : trialkeylist) {
//					for(int repeat = -1; repeat < trialCounterList.get(n); repeat++){
//						for (Integer itemlistelement : itemkeylist) {
//							if(design.isContinuousButton() == false && design.isContinuousFader() == false){
//								outWriter.write("|" + triallistelement.toString());
//								outWriter.write("," + itemlistelement.toString());
//							}
//
//							if(design.isContinuousButton() == true && design.isContinuousFader() == false){
//								for (int trialTimeStampListIndex = 0; trialTimeStampListIndex < trialTimeStampList.size();
//								trialTimeStampListIndex++) {
//							//	trialTimeStampListIndex++;
//								//itemTimeStampListIndex++;
//									for (int itemTimeStampListIndex = 0; itemTimeStampListIndex < trialTimeStampList.get
//									(trialTimeStampListIndex).size(); itemTimeStampListIndex++) {
//										for (int z = 0; z < trialTimeStampList.get(trialTimeStampListIndex).get(itemTimeStampListIndex); z++) {
//											outWriter.write("|" + (1+trialTimeStampListIndex));
//											outWriter.write("," + (1+itemTimeStampListIndex));
//											outWriter.write("," + (1+z));
//										}
//									}
//								}
//							}
//
//							if(design.isContinuousButton() == false && design.isContinuousFader() == true){
//								for (int timeStampElement = 0; timeStampElement < (design.getTotalTime())/
//								(design.getTimeSlice()); timeStampElement++){
//									outWriter.write("|" + triallistelement.toString());
//									outWriter.write("," + itemlistelement.toString());
//									outWriter.write("," + ((timeStampElement+1)*design.getTimeSlice())/1000);
//								}
//							}
//						}
//						//trialListArray.add(t, triallistelement);
//						//t++;
//					}
//					n++;
//				}
//			}
//			// Writes trial and item names.
//			/*else{
//				for (Integer triallistelement : trialkeylist) {
//					for(int repeat = -1; repeat < trialCounterList.get(n); repeat++){
//						for (Integer itemlistelement : itemkeylist) {
//							if(design.isContinuousButton() == false && design.isContinuousFader() == false){
//								String trialName = trialMap.get(triallistelement);
//								outWriter.write("|" + trialName);
//								String itemName = itemMap.get(itemlistelement);
//								outWriter.write("," + itemName);
//							}
//
//							/*if(design.isContinuousButton() == true && design.isContinuousFader() == false){
//								for (int trialTimeStampListIndex = 0; trialTimeStampListIndex < trialTimeStampList.size();
//								trialTimeStampListIndex++) {
//									for (int itemTimeStampListIndex = 0; itemTimeStampListIndex < trialTimeStampList.get
//									(trialTimeStampListIndex).size(); itemTimeStampListIndex++) {
//										for (int z = 0; z < trialTimeStampList.get(trialTimeStampListIndex).get(itemTimeStampListIndex); z++) {
//											outWriter.write("|" + (1+trialTimeStampListIndex));
//											outWriter.write("," + (1+itemTimeStampListIndex));
//											outWriter.write("," + (1+z));
//										}
//									}
//								}
//							}
//
//							if (design.isContinuousButton() == false && design.isContinuousFader() == true) {
//								for (int timeStampElement = 0; timeStampElement < (design
//								.getTotalTime())/(design.getTimeSlice()); timeStampElement++) {
//									String trialName = trialMap.get(triallistelement);
//									outWriter.write("|" + trialName);
//									String itemName = itemMap.get(itemlistelement);
//									outWriter.write("," + itemName);
//									outWriter.write("," + ((timeStampElement+1)*design.getTimeSlice())/1000);
//								}
//							}
//					//	}
//						//trialListArray.add(t, triallistelement);
//						//t++;
//					//}
//					n++;
//				}
//			//}*/
//
//			Collections.sort(trialListArray);
//
//
//
//			int subjectNumber = 1;
//
//			// Writes the data of all sessions.
//			for (Session session : sessions) {
//				outWriter.write("\n" +subjectNumber);
//				List<XtraInfo> xtraInfoList = session.getXtraInfos();
//
//				for (String subjectInfo : subjectInfosList){
//					if (subjectInfo.equals("First Name"))
//						outWriter.write("|" + session.getFirstName());
//					if (subjectInfo.equals("Last Name"))
//						outWriter.write("|" + session.getLastName());
//					if (subjectInfo.equals("Age"))
//						outWriter.write("|" + session.getAge());
//					for (XtraInfo info : xtraInfoList) {
//						if (info.getName().equals(subjectInfo))
//						outWriter.write("|" + info.getValue());
//					}
//				}
//
//				int in = 0;
//				List<TrialInfo> trialList = session.getTrialInfos();
//				boolean bTrialFound = false;
//				boolean bItemFound = false;
//				boolean bTimeStampFound = false;
//
//				for (Integer trialListArrayElement : trialListArray) {
//					bTrialFound = false;
//
//					if(in < trialList.size()){
//						if (trialListArrayElement.equals(trialList.get(in).getId())){
//							bTrialFound = true;
//							for (Integer itemlistelement : itemkeylist) {
//								if(design.isContinuousButton() == false && design.isContinuousFader() == false){
//									bItemFound = false;
//									for (ItemInfo iteminfo : trialList.get(in).getItemInfos()) {
//										if(itemlistelement.equals(iteminfo.getId())){
//											outWriter.write("|" + iteminfo.getValue());
//											bItemFound=true;
//										}
//									}
//									if (!bItemFound)
//										outWriter.write("| - ");
//								}
//
//								if(design.isContinuousButton() == false && design.isContinuousFader() == true){
//									bItemFound = false;
//									for (ItemInfo iteminfo : trialList.get(in).getItemInfos()) {
//										if(itemlistelement.equals(iteminfo.getId())){
//											List<Integer> valueList = iteminfo.getValueList();
//											for (Integer valueListElement : valueList) {
//												outWriter.write("|" + valueListElement);
//												bItemFound=true;
//											}
//										}
//									}
//									if (!bItemFound)
//										outWriter.write("| - ");
//								}
//
//								if(design.isContinuousButton() == true && design.isContinuousFader() == false){
//									bItemFound = false;
//									for (ItemInfo iteminfo : trialList.get(in).getItemInfos()) {
//										if(itemlistelement.equals(iteminfo.getId())){
//											List<Float> timeStampList = iteminfo.getTimeStampList();
//											for (Float timeStampListElement : timeStampList) {
//												outWriter.write("|" + timeStampListElement);
//												bItemFound=true;
//											}
//										}
//									}
//									if (!bItemFound)
//										outWriter.write("| - ");
//								}
//							}
//							in++;
//						}
//
//						if (!bTrialFound){
//							for (int r=0; r<itemKeysSet.size();r++){
//								outWriter.write("| - ");
//							}
//						}
//					}
//
//					else {
//						for (int s=0; s<itemKeysSet.size();s++){
//							outWriter.write("| - ");
//						}
//					}
//				}
//				subjectNumber++;
//			}
//
//			outWriter.close();
//
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//	}
}