package SALT.assessment;

import java.util.*;

/**
 * Provides getters and setters for a TrialInfo and a methods to compare TrialInfo objects and to sort ItemInfos.
 *
 */
public class TrialInfo implements Comparable {

	private int id;
	private List<ItemInfo> itemInfos;
	
	/**
	 * Constructs a TrialInfo with the specified ID and ItemInfo.
	 * @param id
	 */
	public TrialInfo(int id) {
		this.id = id;
		itemInfos = new ArrayList<ItemInfo>();
	}

	/**
	 * Returns the ID of the trial.
	 * @return
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the ID of the trial.
	 * @param id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Adds an ItemInfo to the trial.
	 * @param itemInfo
	 */
	public void addItemInfo(ItemInfo itemInfo) {
		itemInfos.add(itemInfo);
	}

	/**
	 * Returns a list of ItemInfos.
	 * @return
	 */
	public List<ItemInfo> getItemInfos() {
		return itemInfos;
	}

	/** 
	 * Sorts the TrialInfos list into ascending order of its elements.
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object otherObject) {
		int result = 0;
		TrialInfo compareTrialInfo = (TrialInfo) otherObject;
		if (compareTrialInfo.getId() == id) {
			List<ItemInfo> compareItemInfos = compareTrialInfo.getItemInfos();
			int itemCompareNumber = 0;
			int itemCompareDecision = 0;
			if (compareItemInfos.size() == itemInfos.size()) {
				itemCompareNumber = compareItemInfos.size();
			} else {
				if (compareItemInfos.size() < itemInfos.size()) {
					itemCompareNumber = compareItemInfos.size();
					itemCompareDecision = 1;
				} else {
					itemCompareNumber = itemInfos.size();
					itemCompareDecision = -1;
				}
			}

			for (int i = 0; i < itemCompareNumber; i++) {
				if (compareItemInfos.get(i).getId() != itemInfos.get(i).getId()) {
					if (compareItemInfos.get(i).getId() < itemInfos.get(i)
							.getId()) {
						result = 1;
					} else {
						result = -1;
					}
					break;
				}
			}

			if (result == 0 && itemCompareDecision != 0) {
				if (itemCompareDecision == 1) {
					result = 1;
				} else {
					result = -1;
				}
			}

		} else {
			if (compareTrialInfo.getId() < id) {
				result = 1;
			} else {
				result = -1;
			}
		}

		return result;
	}

	/**
	 *  Sorts the ItemInfos list into ascending order of its elements.
	 */
	public void sortItemInfos() {
		Collections.sort(itemInfos);
	}
}
