/**
 * 
 */
package SALT.gui;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.awt.Dimension;
import java.io.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import SALT.assessment.*;

/**
 * An input dialog to define the export of the sessions.
 *
 */
public class ExportDialog extends Dialog implements ActionListener {

	JPanel sessionPanel;
	JPanel checkBoxPanel;
	JPanel folderPanel;
	
	JFileChooser fileChooser;
	JFileChooser saveFileChooser;
	
	JCheckBox checkBox;
	JCheckBox firstNameCheckBox;
	JCheckBox lastNameCheckBox;
	JCheckBox ageCheckBox;
	
	JRadioButton idRadioButton;
	JRadioButton nameRadioButton;
	
	ButtonGroup radioButtonGroup; 
		
	JButton buttonAdd;
	JButton buttonDelete;
	JButton folderButton;
	
	JList fileList;
	DefaultListModel listModel;
	JScrollPane fileListScroller; 
			
	TitledBorder sessionTitle;
	TitledBorder checkBoxTitle;
	TitledBorder folderTitle;
	
	JFrame frame;
	
	File[] inputFiles;
	File outputFile;
	
	JLabel sessionLabel;
	JLabel subjectInfoLabel;
	JLabel trialItemLabel;
	JLabel folderInfoLabel;
		
	JTextField targetFolderTextField;
	List<JCheckBox> checkBoxList;
		
	/**
	 * Constructs a ExportDialog.
	 * @param additionalSubjectInfosList
	 */
	public ExportDialog(List<String> additionalSubjectInfosList){
		
		checkBoxList = new ArrayList<JCheckBox>();
		
		targetFolderTextField = new JTextField(40);
		targetFolderTextField.setEditable(false);
		targetFolderTextField.setText("Not Selected");
		
		sessionPanel = new JPanel(gridBagLayout);
		checkBoxPanel = new JPanel(gridBagLayout);
		folderPanel = new JPanel(gridBagLayout);
			
		sessionLabel = new JLabel("Add exporting sessions to the list: ");
		subjectInfoLabel = new JLabel("Select exporting subject infos: ");
		trialItemLabel = new JLabel("Select exporting trial/item descriptors: ");
		folderInfoLabel = new JLabel("Select a target folder: ");
						
		fileChooser = new JFileChooser();
		fileChooser.addChoosableFileFilter(new FilterLOG());
		fileChooser.setMultiSelectionEnabled(true);
		
		saveFileChooser = new JFileChooser();
		saveFileChooser.addChoosableFileFilter(new FilterSUM());
			
		listModel = new DefaultListModel();
	    
		sessionTitle = BorderFactory.createTitledBorder("Sessions");
		checkBoxTitle = BorderFactory.createTitledBorder("Subject Infos");
		folderTitle = BorderFactory.createTitledBorder("Folder");
			
		sessionPanel.setBorder(sessionTitle);
		checkBoxPanel.setBorder(checkBoxTitle);
		folderPanel.setBorder(folderTitle);
				
		buttonAdd = makeButton("ADD", "Add", this);
		buttonDelete = makeButton("DELETE", "Delete", this);
		folderButton = makeButton("FOLDER", "...", this);
		
		fileList = new JList(listModel);
					
		fileListScroller = new JScrollPane(fileList);
		fileListScroller.setPreferredSize(new Dimension(180, 135));
		
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(sessionLabel, gridBagConstraints);
		sessionPanel.add(sessionLabel);
		
		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(fileListScroller, gridBagConstraints);
		sessionPanel.add(fileListScroller);
		
		makeGridBagConstraints(0, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(buttonAdd, gridBagConstraints);
		sessionPanel.add(buttonAdd);
		
		makeGridBagConstraints(1, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(buttonDelete, gridBagConstraints);
		sessionPanel.add(buttonDelete);		
				
		firstNameCheckBox = new JCheckBox("First Name");
		firstNameCheckBox.setSelected(true);
		lastNameCheckBox = new JCheckBox("Last Name");
		lastNameCheckBox.setSelected(true);
		ageCheckBox = new JCheckBox("Age");
		ageCheckBox.setSelected(true);
		
		idRadioButton = new JRadioButton("ID");
		idRadioButton.setSelected(true);
		nameRadioButton = new JRadioButton("Name");
		nameRadioButton.setSelected(false);
		
		radioButtonGroup = new ButtonGroup();		
		radioButtonGroup.add(idRadioButton);
		radioButtonGroup.add(nameRadioButton);
		
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(subjectInfoLabel, gridBagConstraints);
		checkBoxPanel.add(subjectInfoLabel);
		
		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(firstNameCheckBox, gridBagConstraints);
		checkBoxPanel.add(firstNameCheckBox);
		
		makeGridBagConstraints(0, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(lastNameCheckBox, gridBagConstraints);
		checkBoxPanel.add(lastNameCheckBox);
		
		makeGridBagConstraints(0, 3, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(ageCheckBox, gridBagConstraints);
		checkBoxPanel.add(ageCheckBox);
		
		int i = 4;
		for (String additionalSubjectInfo : additionalSubjectInfosList) {
			
			checkBox = new JCheckBox(additionalSubjectInfo);
			checkBox.setSelected(true);
			checkBoxList.add(checkBox);
			
			makeGridBagConstraints(0, i, 1, 1, gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagLayout.setConstraints(checkBox, gridBagConstraints);
			checkBoxPanel.add(checkBox);
			i++;
		}
		makeGridBagConstraints(0, i, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(trialItemLabel, gridBagConstraints);
		checkBoxPanel.add(trialItemLabel);
		
		makeGridBagConstraints(0, i+1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(idRadioButton, gridBagConstraints);
		checkBoxPanel.add(idRadioButton);	
		
		makeGridBagConstraints(0, i+2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(nameRadioButton, gridBagConstraints);
		checkBoxPanel.add(nameRadioButton);	
	
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(folderInfoLabel, gridBagConstraints);
		folderPanel.add(folderInfoLabel);
				
		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(targetFolderTextField, gridBagConstraints);
		folderPanel.add(targetFolderTextField);
		
		makeGridBagConstraints(1, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(folderButton, gridBagConstraints);
		folderPanel.add(folderButton);
		
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(sessionPanel, gridBagConstraints);
		dialogPanel.add(sessionPanel);
		
		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(checkBoxPanel, gridBagConstraints);
		dialogPanel.add(checkBoxPanel);
		
		makeGridBagConstraints(0, 1, 2, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(folderPanel, gridBagConstraints);
		dialogPanel.add(folderPanel);
	}
	
	/**
	 * Invokes when an action occurs.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		String cmd = event.getActionCommand();

		if (cmd.equals("ADD")) {
			int choosedOption = fileChooser.showDialog(frame, "Ausw�hlen");
			if (choosedOption == JFileChooser.APPROVE_OPTION) {
				inputFiles = fileChooser.getSelectedFiles();
				for (File file : inputFiles) {
					listModel.addElement(file.getName());
				}
			}
		}

		if (cmd.equals("DELETE")) {
			listModel.removeElement(fileList.getSelectedValue());
		}

		if (cmd.equals("FOLDER")) {
			int choosedOption = saveFileChooser.showSaveDialog(frame);
			if (choosedOption == JFileChooser.APPROVE_OPTION) {
				targetFolderTextField.setText(saveFileChooser.getSelectedFile()
						.getPath());
				File tempOutputFile = saveFileChooser.getSelectedFile();
				outputFile = new File(tempOutputFile.getAbsolutePath() + ".sum");
			}
		}
	}

	/**
	 * Displays the ExportDialog.
	 * 
	 * @param frame
	 * @param design
	 */
	public void display(JFrame frame, Design design) {
		boolean missingInputs = true;
		
		while (missingInputs == true) {
			missingInputs = false;

			int choosedOption = JOptionPane.showOptionDialog(frame,
					dialogPanel, "Export", JOptionPane.DEFAULT_OPTION,
					JOptionPane.PLAIN_MESSAGE, null, options, options[1]);

			if (choosedOption == 0) {

				String warningString = new String();
				if (listModel.size() == 0) {
					warningString = warningString + "Exporting Sessions\n";
					missingInputs = true;
				}
				if (targetFolderTextField.getText().equals("Not Selected") == true) {
					warningString = warningString + "Target Folder\n";
					missingInputs = true;
				}
				if (missingInputs == true)
					JOptionPane.showMessageDialog(frame, "Please select: \n"
							+ warningString, "Warning!",
							JOptionPane.ERROR_MESSAGE);

				if (missingInputs == false) {
					List<String> subjectInfosList = new ArrayList<String>();
					if (firstNameCheckBox.isSelected())
						subjectInfosList.add(firstNameCheckBox.getText());
					if (lastNameCheckBox.isSelected())
						subjectInfosList.add(lastNameCheckBox.getText());
					if (ageCheckBox.isSelected())
						subjectInfosList.add(ageCheckBox.getText());

					for (JCheckBox checkBoxElement : checkBoxList) {
						if (checkBoxElement.isSelected())
							subjectInfosList.add(checkBoxElement.getText());
					}

					boolean trialItemID = true;

					if (idRadioButton.isSelected())
						trialItemID = true;

					if (nameRadioButton.isSelected())
						trialItemID = false;

					List<Session> sessions = new ArrayList<Session>();

					for (int i = 0; i < inputFiles.length; i++) {
						sessions.add(SessionReader
								.getSortedTrialInfos(inputFiles[i], design));
					}
                    //Commented out, not necessary anymore - stian
//					SessionWriter.writeSessionsToSumFile(outputFile,
//							subjectInfosList, sessions, design, trialItemID);
				}
			}
		}

		listModel.clear();

		for (int i = 0; i < checkBoxList.size(); i++) {
			checkBoxList.get(i).setSelected(true);
		}

		firstNameCheckBox.setSelected(true);
		lastNameCheckBox.setSelected(true);
		ageCheckBox.setSelected(true);
		idRadioButton.setSelected(true);
		nameRadioButton.setSelected(false);

		targetFolderTextField.setText("Not Selected");
	}
	
	/**
	 * Updates the ExportDialog.
	 * @param additionalSubjectInfosList
	 */
	public void updateExportDialog(List<String> additionalSubjectInfosList){

		checkBoxPanel.removeAll();
			
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(subjectInfoLabel, gridBagConstraints);
		checkBoxPanel.add(subjectInfoLabel);
		
		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(firstNameCheckBox, gridBagConstraints);
		checkBoxPanel.add(firstNameCheckBox);
		
		makeGridBagConstraints(0, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(lastNameCheckBox, gridBagConstraints);
		checkBoxPanel.add(lastNameCheckBox);
		
		makeGridBagConstraints(0, 3, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(ageCheckBox, gridBagConstraints);
		checkBoxPanel.add(ageCheckBox);
		
		int i = 4;
		for (String additionalSubjectInfo : additionalSubjectInfosList) {
			
			checkBox = new JCheckBox(additionalSubjectInfo);
			checkBox.setSelected(true);
			checkBoxList.add(checkBox);
		
			makeGridBagConstraints(0, i, 1, 1, gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagLayout.setConstraints(checkBox, gridBagConstraints);
			checkBoxPanel.add(checkBox);
			i++;
		}
		
		makeGridBagConstraints(0, i, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(trialItemLabel, gridBagConstraints);
		checkBoxPanel.add(trialItemLabel);
		
		makeGridBagConstraints(0, i+1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(idRadioButton, gridBagConstraints);
		checkBoxPanel.add(idRadioButton);	
		
		makeGridBagConstraints(0, i+2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(nameRadioButton, gridBagConstraints);
		checkBoxPanel.add(nameRadioButton);
	}
}
