/**
 * 
 */
package SALT.gui;

import javax.sound.midi.MidiDevice;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import java.awt.Toolkit;
import java.awt.event.*;

import java.util.List;
import java.util.Vector;
import java.io.*;

import SALT.assessment.*;

/**
 * Manages Layout.java and Dialog.java.
 */
public class GUI extends JFrame {
	
	JFileChooser fileChooser;
	JFileChooser savefileChooser;
	JFileChooser editSessionFileChooser;
	
	Layout layout;
	OpenDesignDialog openDesignDialog;
	DesignDialog designDialog;
	SessionDialog sessionDialog;
	ExportDialog exportDialog;
	InformationDialog informationDialog;
	HelpDialog helpDialog;
			
	/**
	 * @param actionListener
	 * @param windowListener
	 * @param midiInputInfoVector
	 * @param midiOutputInfoVector
	 */
	public GUI(ActionListener actionListener,WindowListener windowListener, List<MidiDevice.Info> midiInputInfoVector,List<MidiDevice.Info> midiOutputInfoVector){
		super("SALT");
		
		fileChooser = new JFileChooser();
		savefileChooser = new JFileChooser();
		editSessionFileChooser = new JFileChooser();
		
		fileChooser.addChoosableFileFilter(new FilterDESIGN());
		savefileChooser.addChoosableFileFilter(new FilterDESIGN());
		editSessionFileChooser.addChoosableFileFilter(new FilterLOG());
		
		fileChooser.setDialogTitle("Open Design"); 
		savefileChooser.setDialogTitle("Save Design"); 
		editSessionFileChooser.setDialogTitle("Edit Session");
				
		layout = new Layout(this,actionListener);
		
		designDialog = new DesignDialog(midiInputInfoVector,midiOutputInfoVector, null);
		informationDialog = new InformationDialog();
		openDesignDialog = new OpenDesignDialog();
		helpDialog = new HelpDialog();
			
		//setIconImage(new ImageIcon(this.getClass().getResource("/icons/about.gif")).getImage());
		
		setSize(720,540);
		setLocation(
		        (Toolkit.getDefaultToolkit().getScreenSize().width-
		         getSize().width) / 2,
		        (Toolkit.getDefaultToolkit().getScreenSize().height-
		         getSize().height) / 2);
		
		addWindowListener(windowListener);		
		setVisible(true);	
	}

	/**
	 * Sets JMenuItems enabled.
	 */
	public void setMenuEnabled(){
		// design
		layout.menuBar.getMenu(0).getItem(2).setEnabled(true);
		layout.menuBar.getMenu(0).getItem(3).setEnabled(true);
		// session
		layout.menuBar.getMenu(0).getItem(5).setEnabled(true);
		layout.toolBar.getComponent(3).setEnabled(true);
		layout.menuBar.getMenu(0).getItem(6).setEnabled(true);
		// export
		layout.menuBar.getMenu(0).getItem(8).setEnabled(true);
		layout.toolBar.getComponent(5).setEnabled(true);		
	}
	
	/**
	 * Calls the DesignDialog to create a new design.
	 * @return
	 */
	public Design displayNewDesignDialog(){
		Design design = new Design();
        boolean b = designDialog.display(this, design, false);
		if (b) {
			if (sessionDialog==null){
				sessionDialog = new SessionDialog(design.getAdditionalSubjectInfos());
				exportDialog = new ExportDialog(design.getAdditionalSubjectInfos());
			}
			else {
				sessionDialog.updateAdditionalSubjectInfos(design.getAdditionalSubjectInfos());
				exportDialog.updateExportDialog(design.getAdditionalSubjectInfos());
			}
			setMenuEnabled();
			return design;
		}
		return null;
	}
	
	/**
	 * Displays a JFileChooser to open a design and an OpenDesignDialog.
	 * @return
	 */
	public Design displayOpenDesignDialog() {

		boolean b = false;
		Design design = new Design();
		int choosedOption = fileChooser.showDialog(this, "Ausw�hlen");

		if (choosedOption == JFileChooser.APPROVE_OPTION) {
			design = DesignReader.getDesign(fileChooser.getSelectedFile());
			b = openDesignDialog.display(this, design);
			if (b) {
				if (sessionDialog == null) {
					sessionDialog = new SessionDialog(design
							.getAdditionalSubjectInfos());
					exportDialog = new ExportDialog(design
							.getAdditionalSubjectInfos());
				} else {
					sessionDialog.updateAdditionalSubjectInfos(design
							.getAdditionalSubjectInfos());
					exportDialog.updateExportDialog(design
							.getAdditionalSubjectInfos());
				}
				setMenuEnabled();
				return design;
			}
		}
		return null;
	}
	
	/**
	 * Calls a SessionDialog to create a new session. 
	 * @param session
	 * @return
	 */
	public boolean displayNewSessionDialog(Session session){
		
        boolean b = sessionDialog.display(this, session, false);    //Edited to comply with new session
		return b; 
	}

	/**
	 * Calls a SessionDialog to edit an existing session.
	 * @param session
	 * @param design
	 */
/*	public void displayEditSessionDialog(Session session, Design design){
		sessionDialog = new SessionDialog(design.getAdditionalSubjectInfos());
		sessionDialog.editSession(this, session);
		
		if (session!=null) {
			try {
				File outputFile = new File(design.getTargetFolder() + "/"
						+ session.getFirstName() + "_" + session.getLastName()
						+ "_" + session.getAge() + ".log");
				SessionWriter.writeSessionToLogFile(outputFile, session, design);
			} catch (WriterException e) {
				e.printStackTrace();
			}
		}
	}	*/
	
	/**
	 * Displays a JFileChooser to open a session.
	 * @return
	 */
	public Session openSession(Design design){
		Session session = new Session();
		int choosedOption = editSessionFileChooser.showDialog(this, "Ausw�hlen");
		if (choosedOption == JFileChooser.APPROVE_OPTION) {
			session = SessionReader.getSortedTrialInfos(editSessionFileChooser.getSelectedFile(),design);
			return session;
		}
		return null;	
	}
	
	/**
	 * Displays a JFileChooser to save a design.
	 * @return
	 */
	public File displaySaveFileChooser(){
				
		int choosedOption = savefileChooser.showSaveDialog(this);
		if (choosedOption == JFileChooser.APPROVE_OPTION) {
			return savefileChooser.getSelectedFile();
		}
		else return null;
	}
		
	/**
	 * Calls the ExportDialog.
	 * @param design
	 */
	public void displayExportDialog(Design design){
		exportDialog.display(this, design);
	}
	
	/**
	 * Calls the InformationDialog.
	 */
	public void displayInformationDialog(){
		informationDialog.display(this);
	}
	
	/**
	 * Calls the HelpDialog.
	 */
	public void displayHelpDialog(){
		helpDialog.display(this);
	}
		
	/**
	 * Calls the EditSessionDialog.
	 * @param design
	 */
	public void displayEditDesignDialog(Design design){
		designDialog.display(this, design, true);
	}
		
	/**
	 * 
	 * @param actionDescription
	 */
	public void displayTextAreaMessage(String actionDescription) {
        layout.textArea.append(actionDescription + "\n");
        layout.textArea.setCaretPosition(layout.textArea.getDocument().getLength());
        
        int excess = layout.textArea.getDocument().getLength() - 1000;
        /*if (excess >= 500) {
           	layout.textArea.replaceRange("", 0, excess);
        }*/
    }
	
	/**
	 * Clears the TextArea.
	 */
	public void clearTextArea() {
		layout.textArea.replaceRange("", 0, layout.textArea.getDocument().getLength());
	}
	
	/**
	 * Returns a Vector of MidiInputNames.
	 * @return
	 */
	public Vector getMidiInputNames() {
		return designDialog.midiInputNames;
	}
	
	/**
	 * Returns a Vector of MidiOutputNames.
	 * @return
	 */
	public Vector getMidiOutputNames() {
		return designDialog.midiOutputNames;
	}
		
	/**
	 * Displays the "Test is running" MessageDialog.
	 */
	public void displayWorkingTestMessage() {
		JOptionPane.showMessageDialog(this,
				"Test is running! Press OK to stop.", "Warning!",
				JOptionPane.WARNING_MESSAGE);
	}
}
