package SALT.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;

/**
 * Creates the SALT GUI.
 *
 */
public class Layout extends Graphics{
	
	private JFrame frame;
	private ActionListener actionListener;
	JTextArea textArea;
	JMenuBar menuBar;
	JToolBar toolBar;
	
	/**
	 * Constructs the Layout.
	 * @param frame
	 * @param actionListener
	 */
	public Layout(JFrame frame, ActionListener actionListener) {
		
		this.actionListener = actionListener;
		this.frame = frame;

		toolBar = new JToolBar("");
		toolBar.setFloatable(false);
		addButtons(toolBar);	
		
		textArea = new JTextArea(20, 50);
		textArea.setEditable(false);
	
		JScrollPane scrollPane = new JScrollPane(textArea);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		menuBar = new JMenuBar();
		frame.setJMenuBar(menuBar);
		addMenu(menuBar);

		frame.getContentPane().add(toolBar, BorderLayout.PAGE_START);
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);		
	}
	
	/**
	 * Creates a JMenu.
	 * @param menuName
	 * @return
	 */
	JMenu createMenu(String menuName) {
		JMenu menu = new JMenu();
		menu.setText(menuName);
		return menu;
	}
	
	/**
	 * Creates JMenuItem for the JMenu.
	 * @param menu
	 * @param SubMenuName
	 * @param actionCommand
	 * @param iconName
	 * @param enabled
	 */
	void createSubMenu(JMenu menu, String SubMenuName, String actionCommand, 
			String iconName, boolean enabled) {
		JMenuItem subMenu = new JMenuItem();
		subMenu = menu.add(SubMenuName);
		subMenu.setActionCommand(actionCommand);
		subMenu.addActionListener(actionListener);
		
		if (enabled==false)
			subMenu.setEnabled(false);
		if ((iconName!=null))
			subMenu.setIcon(new ImageIcon(this.getClass().getResource(iconName)));
	}
	
	/**
	 * Adds a JMenuBar to the JFrame.
	 * @param menuBar
	 */
	void addMenu(JMenuBar menuBar) {
		JMenu menu = null;
				
		menu = createMenu("File");
		createSubMenu(menu, "New Design", "NEWDESIGN", "/icons/new.gif", true);
		createSubMenu(menu, "Open Design...", "OPENDESIGN", "/icons/open.gif", true);
		createSubMenu(menu, "Save Design...", "SAVEDESIGN", "/icons/save.gif", false);
		createSubMenu(menu, "Edit Design...", "EDITDESIGN", "/icons/edit.gif", false);
		menu.addSeparator();
		createSubMenu(menu, "New Session", "NEWSESSION", "/icons/add.gif", false);
		createSubMenu(menu, "Open Session...", "OPENSESSION", "/icons/open.gif", false);
		menu.addSeparator();
		createSubMenu(menu, "Export...", "EXPORT", "/icons/export.gif", false);
		menu.addSeparator();
		createSubMenu(menu, "Exit", "EXIT", "", true);
		menuBar.add(menu);
		
		menu = createMenu("Help");
		createSubMenu(menu, "SALT Help", "SALTHELP", "/icons/help.gif", true);
		menu.addSeparator();
		createSubMenu(menu, "About SALT", "ABOUTSALT", "/icons/about.gif", true);
		menuBar.add(menu);
	}
	
	/**
	 * Add buttons to the toolbar.
	 * @param toolBar
	 */
	void addButtons(JToolBar toolBar) {
		JButton button = null;

		// Creates New Design Button. 
		button = makeNavigationButton("/icons/new.gif", "NEWDESIGN", "New Design", true);
		toolBar.add(button);
		// Creates Open Design Button. 
		button = makeNavigationButton("/icons/open.gif", "OPENDESIGN", "Open Design", true);
		toolBar.add(button);
		toolBar.addSeparator();
		// Creates New Session Button. 
		button = makeNavigationButton("/icons/add.gif", "NEWSESSION", "New Session", false);
		toolBar.add(button);
		toolBar.addSeparator();		
		// Creates Export Button. 
		button = makeNavigationButton("/icons/export.gif", "EXPORT", "Export", false);
		toolBar.add(button);
		toolBar.addSeparator();
		// Creates About SALT Button. 
		button = makeNavigationButton("/icons/about.gif", "ABOUTSALT", "About SALT", true);
		toolBar.add(button);
	}
	
	/**
	 * Creates a new toolbar button.
	 * @param imageName
	 * @param actionCommand
	 * @param toolTipText
	 * @param enabled
	 * @return
	 */
	protected JButton makeNavigationButton(String imageName, String actionCommand,
			String toolTipText, boolean enabled) {

		JButton button = new JButton();
		button.setBorderPainted(false);
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(actionListener); 
		button.setIcon(new ImageIcon(this.getClass().getResource(imageName)));
		
		if (enabled==false)
			button.setEnabled(false);		
		
		return button;
	}
	
	/**
	 * Returns a JFrame.
	 * @return
	 */
	public JFrame getFrame() {
		return frame;
	}
}
