package SALT.gui;

import java.awt.GridBagConstraints;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import SALT.assessment.*;

/**
 * Shows an overview of a selected Design.
 * 
 */
public class OpenDesignDialog extends Dialog {

	JPanel additionalSubjectInfosPanel;
	JPanel trialItemPanel;
	JPanel faderButtonPanel;
	JPanel midiDevicesFolderPanel;

	TitledBorder additionalSubjectInfosTitle;
	TitledBorder trialItemTitle;
	TitledBorder faderButtonTitle;
	TitledBorder midiDevicesFolderTitle;

	JLabel faderButtonLabel;
	JLabel faderButtonContentLabel;
	JLabel additionalSubjectInfosLabel;
	JLabel additionalSubjectInfosContentLabel;
	JLabel midiDevicesFolderLabel;
	JLabel midiDevicesFolderContentLabel;
	JLabel trialItemLabel;
	JLabel trialItemContentLabel;

	/**
	 * Constructs an OpenDesignDialog.
	 */
	public OpenDesignDialog() {
		additionalSubjectInfosPanel = new JPanel(gridBagLayout);
		trialItemPanel = new JPanel(gridBagLayout);
		faderButtonPanel = new JPanel(gridBagLayout);
		midiDevicesFolderPanel = new JPanel(gridBagLayout);

		additionalSubjectInfosTitle = BorderFactory
				.createTitledBorder("Additional Subject Infos");
		trialItemTitle = BorderFactory.createTitledBorder("Rating");
		faderButtonTitle = BorderFactory.createTitledBorder("Fader | Button");
		midiDevicesFolderTitle = BorderFactory
				.createTitledBorder("MIDI Devices | Folder");

		additionalSubjectInfosPanel.setBorder(additionalSubjectInfosTitle);
		trialItemPanel.setBorder(trialItemTitle);
		faderButtonPanel.setBorder(faderButtonTitle);
		midiDevicesFolderPanel.setBorder(midiDevicesFolderTitle);

		faderButtonLabel = new JLabel(
				"<html><p align=right>Number of Faders:<br>"
						+ "MIDI Controller Number of Fader:<br>"
                        + "MIDI Controller Number of Button:<br>"   //Edited in - Stian
						+ "MIDI Channel Number:<br>"
						+ "MIDI Default Value:<br>"
						+ "MIDI Button Number Forward:<br>"
						+ "MIDI Button Number Backward:</p>");

		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(faderButtonLabel, gridBagConstraints);
		faderButtonPanel.add(faderButtonLabel);

		additionalSubjectInfosLabel = new JLabel(
				"<html><p align=right>Additional Subject Infos:</p>");

		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(additionalSubjectInfosLabel,
				gridBagConstraints);
		additionalSubjectInfosPanel.add(additionalSubjectInfosLabel);

		midiDevicesFolderLabel = new JLabel(
				"<html><p align=right>MIDI Time Code Input:<br>" + "MIDI Input:<br>" + "MIDI Output:<br>"
						+ "Session Target Folder:</p>");

		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(midiDevicesFolderLabel, gridBagConstraints);
		midiDevicesFolderPanel.add(midiDevicesFolderLabel);

		trialItemLabel = new JLabel("<html><p align=right>Rating:<br>"
                + "Time Slice:<br>" + "MIDI Button Continuous Number:</p>");

		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(trialItemLabel, gridBagConstraints);
		trialItemPanel.add(trialItemLabel);
	}

	/**
	 * Displays the OpenDesignDialog.
	 * 
	 * @param frame
	 * @param design
	 * @return
	 */
	public boolean display(JFrame frame, Design design) {

		boolean b = false;

		String MIDIControllerNumberOfFadersString = new String();
		List<Integer> MIDIControllerNumberOfFaders = design
				.getMIDIControllerNumberOfFaders();
		for (Integer integer : MIDIControllerNumberOfFaders) {
			MIDIControllerNumberOfFadersString = MIDIControllerNumberOfFadersString
					+ integer.toString() + " | ";
		}
		if (MIDIControllerNumberOfFadersString.length() > 0)
			MIDIControllerNumberOfFadersString = MIDIControllerNumberOfFadersString
					.substring(0,MIDIControllerNumberOfFadersString.length() - 2);

        String MIDIControllerNumberOfButtonsString = new String();
		List<Integer> MIDIControllerNumberOfButtons = design
				.getMIDIControllerNumberOfButtons();
		for (Integer integer : MIDIControllerNumberOfButtons) {
			MIDIControllerNumberOfButtonsString = MIDIControllerNumberOfButtonsString
					+ integer.toString() + " | ";
		}
		if (MIDIControllerNumberOfButtonsString.length() > 0)
			MIDIControllerNumberOfButtonsString = MIDIControllerNumberOfButtonsString
					.substring(0,MIDIControllerNumberOfButtonsString.length() - 2);

		String additionalSubjectInfosString = new String();
		List<String> additionalSubjectInfosList = design
				.getAdditionalSubjectInfos();
		for (String additionalSubjectInfo : additionalSubjectInfosList) {
			additionalSubjectInfosString = additionalSubjectInfosString
			+ additionalSubjectInfo + " | ";
		}
		if (additionalSubjectInfosString.length() > 0)
			additionalSubjectInfosString = additionalSubjectInfosString
					.substring(0, additionalSubjectInfosString.length() - 2);

		additionalSubjectInfosContentLabel = new JLabel("<html><p align=right>"
				+ additionalSubjectInfosString + "</p>");

//        additionalSubjectInfosContentLabel = new JLabel("-");

		faderButtonContentLabel = new JLabel("<html>"
				+ design.getNumberOfFaders() + "<br>"
				+ MIDIControllerNumberOfFadersString + "<br>"
                + MIDIControllerNumberOfButtonsString + "<br>"  //Edited in - Stian
				+ design.getMIDIChannelNumber() + "<br>"
				+ design.getMIDIDefaultValue() + "<br>"
				+ design.getMIDIButtonNumberForward() + "<br>"
				+ design.getMIDIButtonNumberBackward() + "</p>");

		midiDevicesFolderContentLabel = new JLabel("<html>"
				+ design.getTimeCodeInputDevice() + "<br>"
                + design.getInputDevice() + "<br>" + design.getOutputDevice()
				+ "<br>" + design.getTargetFolder() + "</p>");

		String items = new String();
        //Commented out - not necessary - stian
//		HashMap<Integer, String> itemHashMap = design.getItemIDandName();
//		Set<Integer> itemKeysSet = itemHashMap.keySet();
//		Integer itemkey = new Integer(0);
//
//		for (Iterator<Integer> iterator = itemKeysSet.iterator(); iterator
//				.hasNext() == true;) {
//			itemkey = iterator.next();
//			String id = String.valueOf((Integer) itemkey);
//			String name = itemHashMap.get((Integer) itemkey);
//			items = items + id + " - " + name + " | ";
//		}
//		if (items.length() > 0)
//			items = items.substring(0, items.length() - 2);
//
//		String trials = new String();
//		HashMap<Integer, String> trialHashMap = design.getTrialIDandName();
//		Set<Integer> trialKeysSet = trialHashMap.keySet();
//		Integer trialkey = new Integer(0);
//
//		for (Iterator<Integer> iterator = trialKeysSet.iterator(); iterator
//				.hasNext() == true;) {
//			trialkey = iterator.next();
//			String id = String.valueOf((Integer) trialkey);
//			String name = trialHashMap.get((Integer) trialkey);
//			trials = trials + id + " - " + name + " | ";
//		}
//		if (trials.length() > 0)
//			trials = trials.substring(0, trials.length() - 2);

		String ratingString = new String();
		String timeSliceString = new String();
		String MIDIButtonNumberContinuousString = new String();
		
		for (int MIDIButtonNumberContinuousElement = 0; MIDIButtonNumberContinuousElement < design.getMIDIButtonNumberContinuous().size();
		MIDIButtonNumberContinuousElement++) {
			int number = design.getMIDIButtonNumberContinuous().get(MIDIButtonNumberContinuousElement);
			MIDIButtonNumberContinuousString = MIDIButtonNumberContinuousString + number + " | ";
		}		
		if (MIDIButtonNumberContinuousString.length() > 0)
			MIDIButtonNumberContinuousString = MIDIButtonNumberContinuousString.substring(0, MIDIButtonNumberContinuousString.length() - 2);

		if (design.isContinuousButton() == false
				&& design.isContinuousFader() == false) {
			ratingString = "Noncontinuous Rating";
			timeSliceString = "-";
			MIDIButtonNumberContinuousString = "-";
		}
		if (design.isContinuousButton() == true
				&& design.isContinuousFader() == false) {
			ratingString = "Continuous Rating (Button)";
			timeSliceString = "-";
		}
		if (design.isContinuousButton() == false
				&& design.isContinuousFader() == true) {
			ratingString = "Continuous Rating (Fader)";
			timeSliceString = String.valueOf((Float) design.getTimeSlice());
			MIDIButtonNumberContinuousString = "-";
		}

        //Edited to remove trials and items
		trialItemContentLabel = new JLabel("<html>" + ratingString + "<br>"
				+ timeSliceString + "<br>"
				+ MIDIButtonNumberContinuousString + "</p>");

		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(trialItemContentLabel, gridBagConstraints);
		trialItemPanel.add(trialItemContentLabel);

		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(midiDevicesFolderContentLabel,
				gridBagConstraints);
		midiDevicesFolderPanel.add(midiDevicesFolderContentLabel);

		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(additionalSubjectInfosContentLabel,
				gridBagConstraints);
		additionalSubjectInfosPanel.add(additionalSubjectInfosContentLabel);

		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(faderButtonContentLabel,
				gridBagConstraints);
		faderButtonPanel.add(faderButtonContentLabel);

		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(additionalSubjectInfosPanel,
				gridBagConstraints);
		dialogPanel.add(additionalSubjectInfosPanel);

		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(trialItemPanel, gridBagConstraints);
		dialogPanel.add(trialItemPanel);

		makeGridBagConstraints(0, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(faderButtonPanel, gridBagConstraints);
		dialogPanel.add(faderButtonPanel);

		makeGridBagConstraints(0, 3, 1, 1, gridBagConstraints);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagLayout.setConstraints(midiDevicesFolderPanel, gridBagConstraints);
		dialogPanel.add(midiDevicesFolderPanel);

		int choosedOption = JOptionPane.showOptionDialog(frame, dialogPanel,
				"Design", JOptionPane.DEFAULT_OPTION,
				JOptionPane.PLAIN_MESSAGE, null, options, options[1]);

		if (choosedOption == 0)
			b = true;

		additionalSubjectInfosContentLabel.setText("");
		faderButtonContentLabel.setText("");

        //Edited out - stian, not necessary
//		trialItemContentLabel.setText("");
		midiDevicesFolderContentLabel.setText("");

		return b;
	}
}
