package SALT.gui;

import java.awt.GridBagConstraints;
import java.util.List;
import java.util.ArrayList;

import javax.swing.*;

import SALT.assessment.*;
import mediaplayer.Main;

/**
 * An input dialog to define a Session.
 *
 */
public class SessionDialog extends Dialog {

	JLabel firstNameLabel;
	JLabel lastNameLabel;
	JLabel ageLabel;

	JTextField firstNameTextField;
	JTextField lastNameTextField;
	JTextField ageTextField;

	List<JTextField> xtraTextFields;
	List<JLabel> xtraQuestionLabels;

	/**
	 * Constructs a SessionDialog.
	 * @param additionalSubjectInfosList list of additional subject infos (e.g. listener experiences).
	 */
	public SessionDialog(List<String> additionalSubjectInfosList) {

		xtraQuestionLabels = new ArrayList<JLabel>();

		xtraTextFields = new ArrayList<JTextField>();
		firstNameLabel = new JLabel("First Name:");
		lastNameLabel = new JLabel("Last Name:");
		ageLabel = new JLabel("Age:");

		// Adds TextFields for first name, last name and age to the dialogPanel.
		firstNameTextField = new JTextField(15);
		lastNameTextField = new JTextField(15);
		ageTextField = new JTextField(3);

		// Sets the layout of the dialog.
		dialogPanel
				.setBorder(BorderFactory.createTitledBorder("Subject Infos"));
		dialogPanel.setLayout(gridBagLayout);

		// Adds firstNameLabel.
		makeGridBagConstraints(0, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(firstNameLabel, gridBagConstraints);
		dialogPanel.add(firstNameLabel);

		// Adds firstNameTextField.
		makeGridBagConstraints(1, 0, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(firstNameTextField, gridBagConstraints);
		dialogPanel.add(firstNameTextField);

		// Adds lastNameLabel.
		makeGridBagConstraints(0, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(lastNameLabel, gridBagConstraints);
		dialogPanel.add(lastNameLabel);

		// Adds lastNameTextField.
		makeGridBagConstraints(1, 1, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(lastNameTextField, gridBagConstraints);
		dialogPanel.add(lastNameTextField);

		// Adds ageLabel.
		makeGridBagConstraints(0, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.EAST;
		gridBagLayout.setConstraints(ageLabel, gridBagConstraints);
		dialogPanel.add(ageLabel);

		// Adds ageTextField.
		makeGridBagConstraints(1, 2, 1, 1, gridBagConstraints);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagLayout.setConstraints(ageTextField, gridBagConstraints);
		dialogPanel.add(ageTextField);

		// Adds xtraQuestionLabel and xtraQuestionTextField.
		int xtraquestionCounter = 3;

		for (String additionalSubjectInfo : additionalSubjectInfosList) {

			JLabel xtraQuestionLabel = new JLabel(additionalSubjectInfo + ":");
			makeGridBagConstraints(0, xtraquestionCounter, 1, 1,
					gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagLayout.setConstraints(xtraQuestionLabel, gridBagConstraints);
			dialogPanel.add(xtraQuestionLabel);
			xtraQuestionLabels.add(xtraQuestionLabel);

			JTextField xtraQuestionTextField = new JTextField(15);
			makeGridBagConstraints(1, xtraquestionCounter, 1, 1,
					gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagLayout.setConstraints(xtraQuestionTextField,
					gridBagConstraints);
			dialogPanel.add(xtraQuestionTextField);

			xtraTextFields.add(xtraQuestionTextField);
			xtraquestionCounter++;
		}
	}

	/**
	 * Displays the SessionDialog.
	 * @param frame
	 * @param session
	 * @return
	 */
	public boolean display(JFrame frame, Session session, boolean edit) {
		
		boolean b = false; 
		boolean missingInputs = true;

		while (missingInputs == true) {
			missingInputs = false;

            if(edit) {
                firstNameTextField.setText(session.getFirstName());
                lastNameTextField.setText(session.getLastName());
                ageTextField.setText(session.getAge()+"");
                System.out.println(session.getXtraInfos().size());
                for(int i=0;i<session.getXtraInfos().size() && i<Main.salt.design.getAdditionalSubjectInfos().size();i++) {
                    xtraTextFields.get(i).setText(session.getXtraInfos().get(i).getValue());
                }
            }

			int choosedOption = JOptionPane.showOptionDialog(frame,
					dialogPanel, "New Session", JOptionPane.DEFAULT_OPTION,
					JOptionPane.PLAIN_MESSAGE, null, options, options[0]);

            

			if (choosedOption == 0) {

				if (((firstNameTextField.getText().length() < 1) || (lastNameTextField
						.getText().length() < 1))
						|| (ageTextField.getText().length() < 1)) {
					missingInputs = true;
					JOptionPane.showMessageDialog(frame,
							"Inputs not completed!", "Warning!",
							JOptionPane.ERROR_MESSAGE);

				} else {
					
					session.setFirstName(firstNameTextField.getText());				
					session.setLastName(lastNameTextField.getText());
					
					try {
						Integer.decode(ageTextField.getText());
						session.setAge(Integer.valueOf(ageTextField.getText()));
					} catch (NumberFormatException e) {
						// TODO Auto-generated catch block
						//e.printStackTrace();
						JOptionPane.showMessageDialog(frame,
								"Please enter Age numerical!",
								"Warning!", JOptionPane.ERROR_MESSAGE);
						ageTextField.setText("");
						missingInputs = true;
					}

                    //Commented out - stian
//					for(int i = 0; i < session.getXtraInfos().size(); i++ ){
//                        session.getXtraInfos().get(i).setValue(xtraTextFields.get(i).getText());
//                        System.out.println(session.getXtraInfos().get(i).getValue());
//                        xtraTextFields.get(i).setText("");
//					}

                    /*
                     * Edited in - stian
                     * */
                    for(int i=0;i<Main.salt.design.getAdditionalSubjectInfos().size();i++) {
                        session.addXtraInfo(new XtraInfo(
                                Main.salt.design.getAdditionalSubjectInfos().get(i),
                                xtraTextFields.get(i).getText()));
                    }
                    /*
                     * end of edit
                     * */

					b = true;
				}
			}
			else{}
		}
		firstNameTextField.setText("");
		lastNameTextField.setText("");
		ageTextField.setText("");
		
		return b;
	}

	/**
	 * @param additionalSubjectInfosList
	 */
	public void updateAdditionalSubjectInfos(
			List<String> additionalSubjectInfosList) {

		for (JLabel xtraQuestionLabel : xtraQuestionLabels) {
			dialogPanel.remove(xtraQuestionLabel);
		}

		for (JTextField xtraTextField : xtraTextFields) {
			dialogPanel.remove(xtraTextField);
		}

		// Adds xtraQuestionLabel and xtraQuestionTextField.
		int xtraquestionCounter = 3;

		for (String additionalSubjectInfo : additionalSubjectInfosList) {

			JLabel xtraQuestionLabel = new JLabel(additionalSubjectInfo + ":");
			makeGridBagConstraints(0, xtraquestionCounter, 1, 1,
					gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagLayout.setConstraints(xtraQuestionLabel, gridBagConstraints);
			dialogPanel.add(xtraQuestionLabel);
			xtraQuestionLabels.add(xtraQuestionLabel);

			JTextField xtraQuestionTextField = new JTextField(15);
			makeGridBagConstraints(1, xtraquestionCounter, 1, 1,
					gridBagConstraints);
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagLayout.setConstraints(xtraQuestionTextField,
					gridBagConstraints);
			dialogPanel.add(xtraQuestionTextField);

			xtraTextFields.add(xtraQuestionTextField);
			xtraquestionCounter++;
		}
	}
	
	/**
	 * Edits an existing session.
	 * @param frame
	 * @param session
	 * @return
	 */
	public Session editSession(JFrame frame, Session session) {
	
		boolean missingInputs = true;
		
		// Completes the SessionDialog with the data from the session.
		firstNameTextField.setText(session.getFirstName());
		lastNameTextField.setText(session.getLastName());
		ageTextField.setText(String.valueOf(session.getAge()));
		
		int j = 0;
		List<XtraInfo> xtraInfosList= session.getXtraInfos();
		for (XtraInfo info : xtraInfosList) {
			xtraTextFields.get(j).setText(info.getValue());
			j++;
		}
		
		while (missingInputs == true) {
			missingInputs = false;
		
			int choosedOption = JOptionPane.showOptionDialog(frame,
				dialogPanel, "New Session", JOptionPane.DEFAULT_OPTION,
				JOptionPane.PLAIN_MESSAGE, null, options, options[0]);
	
			if (choosedOption == 0) {

				if (((firstNameTextField.getText().length() < 1) || (lastNameTextField
						.getText().length() < 1))
						|| (ageTextField.getText().length() < 1)) {
					missingInputs = true;
					if (missingInputs == true)JOptionPane.showMessageDialog(frame,
						"Inputs not completed!", "Warning!",
						JOptionPane.ERROR_MESSAGE);
				}
				
				if (missingInputs == false){
					session.setFirstName(firstNameTextField.getText());
					session.setLastName(lastNameTextField.getText());
					session.setAge(Integer.valueOf(ageTextField.getText()));

					for(int i = 0; i< session.getXtraInfos().size(); i++){
						session.getXtraInfos().get(i).setValue(xtraTextFields.get(i).getText());
						xtraTextFields.get(i).setText("");
					}
					return session;
				}	
			}
		}
		
	firstNameTextField.setText("");
	lastNameTextField.setText("");
	ageTextField.setText("");
	
	return null;
	}
}
