package mediaplayer;

import packets.Clip;
import SALT.assessment.Design;
import SALT.assessment.Salt;
import SALT.assessment.SessionWriter;
import SALT.exceptions.WriterException;
import packets.PlayPacket;
import packetNetworking.*;
import packets.EDLPacket;
import packets.FPSPacket;
import packets.ListPacket;
import packets.TCPacket;
import gui.ItemDialog;
import gui.PlaylistDialog;
import gui.TrialDialog;
import java.io.File;
import java.util.ArrayList;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import packetNetworking.StringPacket;

/**
 * This class maintains the basic functions of the control suite
 * This includes arranging of items, trials, maintaining a playlist and more.
 * @author Stian
 */
public class Main extends Server {
    
    /**
     * This is the gui of the application. It also gives indirect access to public fields in this class via a main instance
     */
    public static MediaPlayerView view;
    /**
     * An instance of a modified salt version. No salt gui is shown
     */
    public static Salt salt = new Salt();
    /**
     * isRunning is true if a test is running
     */
    public static boolean isRunning = false;
    /**
     * the active item in a trial. used to control item coloring
     */
    public static int activeItem = 0;

    //Misc
    /**
     * The active playlist
     */
    public Playlist playlist;
    /**
     * the list of clip names
     */
    public ArrayList<String> clipList;
    /**
     * 
     */
    public static boolean userHasPressedStart = false;
    /**
     * Packet received from the clip recorder control application
     */
    public ListPacket listPacket;
    /**
     * The status of the control suite
     */
    public Status status;

    /**
     * The current time code
     */
    public static String timeCode;
    /**
     * The stop time for the active clip
     */
    public static String stopTC;

    //Dialogs
    PlaylistDialog playlistDialog;
    ItemDialog itemDialog;
    TrialDialog trialDialog;

    TimeCodeListenerThread tcListener;

    /**
     * Sets up the application. The constructor is called before the gui, but after gui component intializing.
     * @param view
     */
    public Main(MediaPlayerView view) {

        super(new int[]{4444}); //Sets up the server with port 4444.

        Main.view = view;
        playlist = new Playlist("New playlist");

        status = new Status();

        UpdateStatus(-1);
        
        clipList = new ArrayList<String>();
    }

    /**
     * sets a new playlist name
     * @param text
     */
    public void editPlaylistName(String text) {
        playlist = new Playlist(text);
    }

    /**
     * creates a new item dialog
     */
    public void newItem() {
        JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
        itemDialog = new ItemDialog(mainFrame, true, null);
        itemDialog.setLocationRelativeTo(mainFrame);
        MediaPlayerApp.getApplication().show(itemDialog);
    }

    /**
     * edits an existing item
     * @param item
     */
    public void editItem(Item item) {
        JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
        if(item == null) {
            return;
        }
        itemDialog = new ItemDialog(mainFrame, true, item);
        itemDialog.setLocationRelativeTo(mainFrame);
        MediaPlayerApp.getApplication().show(itemDialog);
    }

    void editPlaylistName() {
        JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
        playlistDialog = new PlaylistDialog(mainFrame, true, "edit");
        playlistDialog.setLocationRelativeTo(mainFrame);
        MediaPlayerApp.getApplication().show(playlistDialog);
    }

    /**
     * sets active item in a trial. resets all other active trials
     */
    public void setActiveItem() {
        if(Playlist.getSelectedTrial() == null) return;
        for(Trial t : playlist.getTrials()) {
            t.itemList.setSelectedValue(null, false);
            t.updateDescription();
            //playlist.rearrange();
        }
        Playlist.getSelectedTrial().itemList.setSelectedIndex(activeItem);
    }

    /**
     * sets the active item a the selected trial to index
     * @param index
     */
    public void setActiveItem(int index) {
        activeItem = index;
        for(Trial t : playlist.getTrials()) {
            t.itemList.setSelectedValue(null, false);
            t.updateDescription();
            //playlist.rearrange();
        }
        Trial trial = Playlist.getSelectedTrial();
        view.playlistJList.setSelectedValue(trial, true);
        trial.itemList.setSelectedIndex(index);
//        sendCommand(new PlayCommand(trial.items[index].name, trial.items[index].tcStart, trial.items[index].tcStop));
//        stopTC = trial.items[index].tcStop;
    }

    /**
     * sets the activeitem parameter and plays that item
     * @param index
     */
    public void playActiveItem(int index) {
        activeItem = index;
        for(Trial t : playlist.getTrials()) {
            t.itemList.setSelectedValue(null, false);
            t.updateDescription();
        }
        Trial trial = Playlist.getSelectedTrial();
        view.playlistJList.setSelectedValue(trial, true);
        trial.itemList.setSelectedIndex(index);
        sendPacket(new PlayPacket(trial.items[index].name, trial.items[index].tcStart, trial.items[index].tcStop));
        stopTC = trial.items[index].tcStop;
    }

    private void playNextItem() {
        if(activeItem>=Trial.getItemsPerTrial()-1) {
            salt.dumpReceiver.y = salt.currGrp;
            salt.incGroup();
            salt.sendTrial();
            nextTrial();
        } else {
            playActiveItem(activeItem+1);
        }
    }

    /**
     * Jumps to the next trial and plays the first item in the next trial
     */
    public void nextTrial() {
        if(Playlist.getSelectedTrial()==null) return;
        int index = Playlist.getSelectedTrialIndex() + 1;
        if(index < playlist.trials.size()) {
            Item tmp = playlist.trials.get(index).items[0];
            sendPacket(new PlayPacket(tmp.name, tmp.tcStart, tmp.tcStop));
            stopTC = tmp.tcStop;
            Playlist.setSelectedTrial(index);
            setActiveItem(0);
        } else {
            sendPacket(new StringPacket("Stop"));
            saveSession();
            Playlist.setSelectedTrial(0);
            setActiveItem(0);
        }
    }

    /**
     * jumps to the previous trial and plays the first clip in that trial
     */
    public void previousTrial() {
        if(Playlist.getSelectedTrial()==null) return;
        int index = Playlist.getSelectedTrialIndex();
        if(index > 0) {
            Item tmp = playlist.trials.get(index-1).items[0];
            sendPacket(new PlayPacket(tmp.name,tmp.tcStart,tmp.tcStop));
            stopTC = playlist.trials.get(index-1).items[0].tcStop;
            Playlist.setSelectedTrial(index-1);
            setActiveItem(0);
        } else {
            Item tmp = playlist.trials.get(index-1).items[0];
            sendPacket(new PlayPacket(tmp.name,tmp.tcStart,tmp.tcStop));
            stopTC = playlist.trials.get(index).items[0].tcStop;
            setActiveItem(0);
        }
    }

    /**
     * start the test from the beginning.
     */
    public void play() {

        if(isRunning && !userHasPressedStart) {
            sendPacket(new StringPacket("Pause"));
            isRunning = false;
        } else {
            if(!userHasPressedStart && (salt.design.isContinuousButton() || salt.design.isContinuousFader())) {
                isRunning = true;
                status.AddMessage("The user will start the test");
                return;
            }
            if((Trial)view.playlistJList.getSelectedValue()!=null) {
                salt.sendTrial();
                sendPacket(new PlayPacket(Playlist.getSelectedTrial().items[activeItem].name,
                        Playlist.getSelectedTrial().items[activeItem].tcStart,
                        Playlist.getSelectedTrial().items[activeItem].tcStop));
                stopTC = Playlist.getSelectedTrial().items[activeItem].tcStop;
            } else {
                //AddMessage("You have to select a clip first");
            }
        }
        if((Trial)view.playlistJList.getSelectedValue()==null) {
            status.AddMessage("You have to select a clip first");
        }

    }

    /**
     * Stops playback from the clip recorder
     */
    public void stop() {
        if((Trial)view.playlistJList.getSelectedValue()==null) {
            status.AddMessage("You have to select a clip first");
            return;
        }
        isRunning = false;
        sendPacket(new StringPacket("Stop"));
    }

    /**
     * Sends a packet. Only used to print out the packet before sending
     * @param pck
     */
    public void sendPacket(Packet pck) {
        if(pck==null) return;
        Talk(pck);
    }

    @Override
    public void ProcessPacket(Packet pck) {
        super.ProcessPacket(pck);
        try {
            if(pck instanceof ListPacket) {
                //Process and delegate the list
                ListPacket lc = (ListPacket)pck;
                listPacket = lc;
                for(Clip c : lc.clips) {
                    clipList.add(c.name);
                }
                status.setFilelistStatus(true);
                return;
            }
            if(pck instanceof StringPacket) {
                //Packet is a string, compare and check it
                StringPacket sp = (StringPacket)pck;
                if(sp.text.matches("edl received")) {
                    EDLPacket.hasResponded = true;
                }
            }
            if(pck instanceof TCPacket) {
                //The packet contains timecode
                TCPacket tcCmd = (TCPacket)pck;
                setTimeCode(tcCmd.tc);
                tcCmd = null;
            }
            if(pck instanceof FPSPacket) {
                //Packet contains the used fps
                FPSPacket fpsCmd = (FPSPacket)pck;
                Playlist.fps = fpsCmd.fps;
                fpsCmd = null;
            }
        } catch(NullPointerException e) {
            
        }
        
    }
    
    @Override
    public void UpdateStatus(int status) {
        //Updates connection status and sets necessary status messages
        super.UpdateStatus(status);
        if(isAllSocketsOpen) {
            this.status.setConnectionStatus(true, "Connected");
        } else {
            ArrayList<Integer> badSockets = new ArrayList<Integer>();
            for(int i=0;i<ports.length;i++) {
                if(!clientSocketsOpen[i]) badSockets.add(new Integer(i));
            }
            String errorMessage = "Socket";
            if(badSockets.size()==1) {
                errorMessage += " ";
                errorMessage += badSockets.get(0)+" is not open";
            } else {
                errorMessage += "s ";
                for(int i=0;i<badSockets.size();i++) {
                    if(i==badSockets.size()-1) {
                        errorMessage += " and "+badSockets.get(i).intValue()+" ";
                    } else {
                        errorMessage += badSockets.get(i).intValue()+" ";
                    }
                }
                errorMessage += "are not open";
            }
            
            this.status.setConnectionStatus(false, errorMessage);
            this.status.setRandomizeStatus(false);
        }
    }

    /**
     * sets a design and opens the midi devices
     * @param design
     */
    public void setDesign(Design design) {
        Main.salt.design = design;
        salt.openMidiDevices();
    }

    void editTrial() {
        JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
        if(Playlist.getSelectedTrial() == null) return;
        trialDialog = new TrialDialog(mainFrame, true, true);
        trialDialog.setLocationRelativeTo(mainFrame);
        MediaPlayerApp.getApplication().show(trialDialog);
    }

    void newTrial() {
        JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
        trialDialog = new TrialDialog(mainFrame, true, false);
        trialDialog.setLocationRelativeTo(mainFrame);
        MediaPlayerApp.getApplication().show(trialDialog);
    }

    void deleteTrial() {
        int index = Playlist.getSelectedTrialIndex();
        playlist.removeTrial();
        if(index-1<0) {
            Main.view.playlistJList.setSelectedIndex(0);
        } else {
            Main.view.playlistJList.setSelectedIndex(index-1);
        }
        status.setRandomizeStatus(false);
    }

    /**
     * Called when the user changes the number of items per trial. This updates the list model and the appropriate variables
     * It removes all extra items if the number is lower than the former number. It adds null objects if the number is higher
     */
    public void changeNrOfItemsPerTrial() {
        if(Trial.getItemsPerTrial()==Trial.itemsPerTrial) return;
        int nrOfItems = Trial.getItemsPerTrial();
        DefaultListModel dlm = ((DefaultListModel)view.playlistJList.getModel());
        for(int i=0;i<playlist.trials.size();i++) {
            Item[] newItems = new Item[nrOfItems];
            if(Trial.getItemsPerTrial()>Trial.itemsPerTrial) {
                for(int j=0;j<Trial.itemsPerTrial;j++) {
                    newItems[j] = playlist.trials.get(i).items[j];
                }
                for(int j=0;j<nrOfItems-Trial.itemsPerTrial;j++) {
                    newItems[Trial.itemsPerTrial+j] = null;
                }
            }
            if(Trial.getItemsPerTrial()<Trial.itemsPerTrial) {
                for(int j=0;j<Trial.getItemsPerTrial();j++) {
                    newItems[j] = playlist.trials.get(i).items[j];
                }
            }
            playlist.trials.get(i).items = newItems;
            playlist.trials.get(i).updateDescription();
            dlm.setElementAt(playlist.trials.get(i), i);
        }
        Trial.itemsPerTrial = Trial.getItemsPerTrial();
        this.setActiveItem();
        status.setRandomizeStatus(false);
    }

    /**
     * Saves a session using the sessionwriter
     */
    public void saveSession() {
        try {
            File outputFile = new File(salt.design.getTargetFolder() + "/"
                + salt.session.getFirstName() + "_" + salt.session.getLastName()
                + "_" + salt.session.getAge() + ".log");
            SessionWriter.writeSessionToLogFile(outputFile, salt.session, salt.design);

        } catch (WriterException e) {
            e.printStackTrace();
        }
    }

    /**
     * Sets the current time code. Called each time a new time code is received
     * @param tc
     */
    public void setTimeCode(String tc) {
        timeCode = null;
        timeCode = tc;
        tc = null;
        if(timeCode.substring(0,8).matches(stopTC.substring(0,8))) {
            int curFrame = Integer.parseInt(timeCode.substring(9));
            int stopFrame = Integer.parseInt(stopTC.substring(9));
            if(curFrame>=stopFrame) {
               playNextItem();
            }
        }
    }
}
