/*
 * MediaPlayerView.java
 */

package mediaplayer;

import SALT.assessment.Design;
import SALT.assessment.DesignReader;
import SALT.assessment.DesignWriter;
import SALT.assessment.FilterDESIGN;
import SALT.assessment.FilterLOG;
import SALT.assessment.Session;
import SALT.assessment.SessionReader;
import SALT.gui.DesignDialog;
import SALT.gui.OpenDesignDialog;
import SALT.gui.SessionDialog;
import gui.MediaPlayerAboutBox;
import gui.MidiMessageFrame;
import gui.PanelListCellRenderer;
import gui.StatusFrame;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jdesktop.application.Action;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.io.File;
import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.ScrollPaneConstants;

/**
 * The application's main frame.
 */
public class MediaPlayerView extends FrameView {

    /**
     * A instance of main to get access to its methods. Most actions are passed to main
     */
    public Main main;

    DesignDialog dd;
    OpenDesignDialog odd;
    SessionDialog sd;
    JFileChooser editSessionFileChooser, openDesignFileChooser, saveFileChooser;
    MidiMessageFrame midiMessageFrame;
    StatusFrame statusFrame;

    /**
     * Note that two different ways of viewing frames is used. This frame uses netbeans, with its implications
     * Frames connected to salt, like the design dialog, session dialog and the connected dialogs for saving and more
     * have another way of doing it. They show a dialog with an infinite while loop til det loop is told to stop
     * @param app
     */
    public MediaPlayerView(SingleFrameApplication app) {
        super(app);

        initComponents();

        getFrame().setResizable(false);

        main = new Main(this);
        playlistJList.setCellRenderer(new PanelListCellRenderer());
        jScrollPane1.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        //Ready other gui components for later use
        dd = new DesignDialog(Main.salt.getMidiInputInfoVector(), Main.salt.getMidiOutputInfoVector(), Main.salt.getMidiInputInfoVector());
        editSessionFileChooser = new JFileChooser();
        openDesignFileChooser = new JFileChooser();
        saveFileChooser = new JFileChooser();
        editSessionFileChooser.addChoosableFileFilter(new FilterLOG());
        openDesignFileChooser.addChoosableFileFilter(new FilterDESIGN());
        saveFileChooser.addChoosableFileFilter(new FilterDESIGN());

        //create a status frame and the midi message frame
        midiMessageFrame = new MidiMessageFrame();
        statusFrame = Status.sf;
        new ShowFramesThread();

    }

    //Thread for showing status and midi message window
    private class ShowFramesThread extends Thread {

        protected ShowFramesThread() {
            start();
        }

        @Override
        public void run() {
            while(true) {
                if(getFrame().isShowing()) {
                    showMidiMessageWindow();
                    showStatusMessageWindow();
                    break;
                }
                try {
                    Thread.sleep(30);
                } catch (InterruptedException ex) {
                    Logger.getLogger(MediaPlayerView.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * brings up the midi message window
     */
    public void showMidiMessageWindow() {
        midiMessageFrame.setLocation(getFrame().getLocation().x+getFrame().getWidth()+10, getFrame().getLocation().y);
        MediaPlayerApp.getApplication().show(midiMessageFrame);
    }

    /**
     * brings up the status message window
     */
    public void showStatusMessageWindow() {
        if(midiMessageFrame != null) {
                statusFrame.setLocation(getFrame().getLocation().x+getFrame().getWidth()+10,
                        getFrame().getLocation().y+midiMessageFrame.getHeight()+10);
        } else {
            statusFrame.setLocationRelativeTo(MediaPlayerApp.getApplication().getMainFrame());
        }
        MediaPlayerApp.getApplication().show(statusFrame);
    }

    /**
     * adds midi message to the midi message frame
     * @param msg
     */
    public void addMidiMessage(String msg) {
        midiMessageFrame.addMidiMessage(msg);
    }

    /**
     * 
     */
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = MediaPlayerApp.getApplication().getMainFrame();
            aboutBox = new MediaPlayerAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        MediaPlayerApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        playlistJList = new javax.swing.JList(new DefaultListModel());
        sessionName = new javax.swing.JLabel();
        deleteTrialButton = new javax.swing.JButton();
        editTrial = new javax.swing.JButton();
        newTrialButton = new javax.swing.JButton();
        trialLabel = new javax.swing.JLabel();
        randomizeButton = new javax.swing.JButton();
        nrOfItemsComboBox = new javax.swing.JComboBox();
        shuffleCheckBox = new javax.swing.JCheckBox();
        clearButton = new javax.swing.JButton();
        designLabel = new javax.swing.JLabel();
        newDesignButton = new javax.swing.JButton();
        openDesignButton = new javax.swing.JButton();
        sessionLabel = new javax.swing.JLabel();
        newSessionButton = new javax.swing.JButton();
        openSessionButton = new javax.swing.JButton();
        saveDesignButton = new javax.swing.JButton();
        saveSessionButton = new javax.swing.JButton();
        showStatusButton = new javax.swing.JButton();
        showMidiMessagesButton = new javax.swing.JButton();
        playButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        midiMessageWindowLabel = new javax.swing.JLabel();
        statusMessageLabel = new javax.swing.JLabel();
        editDesignButton = new javax.swing.JButton();
        editSessionButton = new javax.swing.JButton();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenu2 = new javax.swing.JMenu();
        jDesktopPane1 = new javax.swing.JDesktopPane();

        mainPanel.setName("mainPanel"); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        playlistJList.setName("playlistJList"); // NOI18N
        playlistJList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                playlistJListMousePressed(evt);
            }
        });
        jScrollPane1.setViewportView(playlistJList);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(mediaplayer.MediaPlayerApp.class).getContext().getResourceMap(MediaPlayerView.class);
        playlistJList.getAccessibleContext().setAccessibleName(resourceMap.getString("playlist.AccessibleContext.accessibleName")); // NOI18N

        sessionName.setFont(resourceMap.getFont("sessionName.font")); // NOI18N
        sessionName.setText(resourceMap.getString("sessionName.text")); // NOI18N
        sessionName.setName("sessionName"); // NOI18N
        sessionName.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                sessionNameMouseClicked(evt);
            }
        });

        deleteTrialButton.setText(resourceMap.getString("deleteTrialButton.text")); // NOI18N
        deleteTrialButton.setName("deleteTrialButton"); // NOI18N
        deleteTrialButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteTrialButtonActionPerformed(evt);
            }
        });

        editTrial.setText(resourceMap.getString("editTrial.text")); // NOI18N
        editTrial.setName("editTrial"); // NOI18N
        editTrial.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editTrialActionPerformed(evt);
            }
        });

        newTrialButton.setText(resourceMap.getString("newTrialButton.text")); // NOI18N
        newTrialButton.setName("newTrialButton"); // NOI18N
        newTrialButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newTrialButtonActionPerformed(evt);
            }
        });

        trialLabel.setFont(resourceMap.getFont("trialLabel.font")); // NOI18N
        trialLabel.setText(resourceMap.getString("trialLabel.text")); // NOI18N
        trialLabel.setName("trialLabel"); // NOI18N

        randomizeButton.setText(resourceMap.getString("randomizeButton.text")); // NOI18N
        randomizeButton.setName("randomizeButton"); // NOI18N
        randomizeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                randomizeButtonActionPerformed(evt);
            }
        });

        nrOfItemsComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1 item per trial", "2 items per trial", "3 items per trial", "4 items per trial", "5 items per trial", "6 items per trial", "7 items per trial", "8 items per trial" }));
        nrOfItemsComboBox.setName("nrOfItemsComboBox"); // NOI18N
        nrOfItemsComboBox.setPreferredSize(new java.awt.Dimension(56, 23));
        nrOfItemsComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nrOfItemsComboBoxActionPerformed(evt);
            }
        });

        shuffleCheckBox.setText(resourceMap.getString("shuffleCheckBox.text")); // NOI18N
        shuffleCheckBox.setName("shuffleCheckBox"); // NOI18N

        clearButton.setText(resourceMap.getString("clearButton.text")); // NOI18N
        clearButton.setName("clearButton"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 296, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(randomizeButton, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                            .addComponent(clearButton, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                            .addComponent(shuffleCheckBox, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                            .addComponent(nrOfItemsComboBox, 0, 100, Short.MAX_VALUE)
                            .addComponent(trialLabel)
                            .addComponent(deleteTrialButton, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                            .addComponent(editTrial, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)
                            .addComponent(newTrialButton, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE)))
                    .addComponent(sessionName))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(sessionName)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 317, Short.MAX_VALUE)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(trialLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(newTrialButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(editTrial)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(deleteTrialButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(nrOfItemsComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 113, Short.MAX_VALUE)
                        .addComponent(randomizeButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(shuffleCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(clearButton)))
                .addContainerGap())
        );

        designLabel.setText(resourceMap.getString("designLabel.text")); // NOI18N
        designLabel.setName("designLabel"); // NOI18N

        newDesignButton.setText(resourceMap.getString("newDesignButton.text")); // NOI18N
        newDesignButton.setName("newDesignButton"); // NOI18N
        newDesignButton.setPreferredSize(new java.awt.Dimension(59, 23));
        newDesignButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newDesignButtonActionPerformed(evt);
            }
        });

        openDesignButton.setText(resourceMap.getString("openDesignButton.text")); // NOI18N
        openDesignButton.setName("openDesignButton"); // NOI18N
        openDesignButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openDesignButtonActionPerformed(evt);
            }
        });

        sessionLabel.setText(resourceMap.getString("sessionLabel.text")); // NOI18N
        sessionLabel.setName("sessionLabel"); // NOI18N

        newSessionButton.setText(resourceMap.getString("newSessionButton.text")); // NOI18N
        newSessionButton.setName("newSessionButton"); // NOI18N
        newSessionButton.setPreferredSize(new java.awt.Dimension(59, 23));
        newSessionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newSessionButtonActionPerformed(evt);
            }
        });

        openSessionButton.setText(resourceMap.getString("openSessionButton.text")); // NOI18N
        openSessionButton.setName("openSessionButton"); // NOI18N
        openSessionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openSessionButtonActionPerformed(evt);
            }
        });

        saveDesignButton.setText(resourceMap.getString("saveDesignButton.text")); // NOI18N
        saveDesignButton.setName("saveDesignButton"); // NOI18N
        saveDesignButton.setPreferredSize(new java.awt.Dimension(59, 23));
        saveDesignButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveDesignButtonActionPerformed(evt);
            }
        });

        saveSessionButton.setText(resourceMap.getString("saveSessionButton.text")); // NOI18N
        saveSessionButton.setName("saveSessionButton"); // NOI18N
        saveSessionButton.setPreferredSize(new java.awt.Dimension(59, 23));
        saveSessionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveSessionButtonActionPerformed(evt);
            }
        });

        showStatusButton.setText(resourceMap.getString("showStatusButton.text")); // NOI18N
        showStatusButton.setMaximumSize(new java.awt.Dimension(60, 23));
        showStatusButton.setMinimumSize(new java.awt.Dimension(60, 23));
        showStatusButton.setName("showStatusButton"); // NOI18N
        showStatusButton.setPreferredSize(new java.awt.Dimension(60, 23));
        showStatusButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showStatusButtonActionPerformed(evt);
            }
        });

        showMidiMessagesButton.setText(resourceMap.getString("showMidiMessagesButton.text")); // NOI18N
        showMidiMessagesButton.setName("showMidiMessagesButton"); // NOI18N
        showMidiMessagesButton.setPreferredSize(new java.awt.Dimension(60, 23));
        showMidiMessagesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showMidiMessagesButtonActionPerformed(evt);
            }
        });

        playButton.setText(resourceMap.getString("playButton.text")); // NOI18N
        playButton.setName("playButton"); // NOI18N
        playButton.setPreferredSize(new java.awt.Dimension(61, 23));
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playButtonActionPerformed(evt);
            }
        });

        stopButton.setText(resourceMap.getString("stopButton.text")); // NOI18N
        stopButton.setName("stopButton"); // NOI18N
        stopButton.setPreferredSize(new java.awt.Dimension(61, 23));
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        midiMessageWindowLabel.setText(resourceMap.getString("midiMessageWindowLabel.text")); // NOI18N
        midiMessageWindowLabel.setName("midiMessageWindowLabel"); // NOI18N

        statusMessageLabel.setText(resourceMap.getString("statusMessageLabel.text")); // NOI18N
        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        editDesignButton.setText(resourceMap.getString("editDesignButton.text")); // NOI18N
        editDesignButton.setName("editDesignButton"); // NOI18N
        editDesignButton.setPreferredSize(new java.awt.Dimension(59, 23));
        editDesignButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editDesignButtonActionPerformed(evt);
            }
        });

        editSessionButton.setText(resourceMap.getString("editSessionButton.text")); // NOI18N
        editSessionButton.setName("editSessionButton"); // NOI18N
        editSessionButton.setPreferredSize(new java.awt.Dimension(59, 23));
        editSessionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editSessionButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(playButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(stopButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 70, Short.MAX_VALUE)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(statusMessageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(midiMessageWindowLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(showMidiMessagesButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(showStatusButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(20, 20, 20))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(sessionLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(designLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE))
                                .addGap(1, 1, 1)
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(newSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(editSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                        .addComponent(openSessionButton)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(saveSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(newDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(editDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(openDesignButton)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(saveDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addGap(44, 44, 44))
                            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addContainerGap())))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(designLabel)
                    .addComponent(newDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(openDesignButton)
                    .addComponent(saveDesignButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(6, 6, 6)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(sessionLabel)
                    .addComponent(newSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(saveSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(openSessionButton)
                    .addComponent(editSessionButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(showStatusButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(playButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(stopButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(midiMessageWindowLabel)
                    .addComponent(showMidiMessagesButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(30, Short.MAX_VALUE))
        );

        jMenuBar1.setName("jMenuBar1"); // NOI18N

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setName("jMenu1"); // NOI18N
        jMenuBar1.add(jMenu1);

        jMenu2.setText(resourceMap.getString("jMenu2.text")); // NOI18N
        jMenu2.setName("jMenu2"); // NOI18N
        jMenuBar1.add(jMenu2);

        jDesktopPane1.setName("jDesktopPane1"); // NOI18N

        setComponent(mainPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void newTrialButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newTrialButtonActionPerformed
        main.newTrial();
}//GEN-LAST:event_newTrialButtonActionPerformed

    private void editTrialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editTrialActionPerformed
        main.editTrial();
}//GEN-LAST:event_editTrialActionPerformed

    private void deleteTrialButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteTrialButtonActionPerformed
        main.deleteTrial();
}//GEN-LAST:event_deleteTrialButtonActionPerformed

    private void sessionNameMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sessionNameMouseClicked
        main.editPlaylistName();
}//GEN-LAST:event_sessionNameMouseClicked

    private void randomizeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_randomizeButtonActionPerformed
        main.playlist.randomize();
    }//GEN-LAST:event_randomizeButtonActionPerformed

    private void playButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playButtonActionPerformed

        if(!Main.isRunning) {
            if(!main.status.isAllRequirementsFulfilled()) {
                main.status.AddMessage("Can't start without completing randomization");
                return;
            }
            playButton.setText("Pause");
        } else {
            playButton.setText("Play");
        }
        main.play();
    }//GEN-LAST:event_playButtonActionPerformed

    private void newDesignButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newDesignButtonActionPerformed
        Design design = new Design();
        boolean success = dd.display(getFrame(), design, false);
        if(success) {
            //TODO: Update session (probably)
            main.setDesign(design);
            main.status.setDesignStatus(true);
            this.nrOfItemsComboBox.setSelectedIndex(design.getNumberOfFaders()-1);
            main.changeNrOfItemsPerTrial();
        } else {
            if(Main.salt.design == null) main.status.setDesignStatus(false);
        }
    }//GEN-LAST:event_newDesignButtonActionPerformed

    private void openDesignButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openDesignButtonActionPerformed
		Design design = new Design();
		int choosedOption = openDesignFileChooser.showDialog(getFrame(), "Ok");
		if (choosedOption == JFileChooser.APPROVE_OPTION) {
			design = DesignReader.getDesign(openDesignFileChooser.getSelectedFile());
            odd = new OpenDesignDialog();
            boolean success = odd.display(getFrame(), design);
            if(success) {
                //TODO: Update session (probably)
                main.setDesign(design);
                main.status.setDesignStatus(true);
                nrOfItemsComboBox.setSelectedIndex(design.getNumberOfFaders()-1);
                main.changeNrOfItemsPerTrial();
            } else {
                if(Main.salt.design == null) main.status.setDesignStatus(false);
            }
		}
    }//GEN-LAST:event_openDesignButtonActionPerformed

    private void newSessionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newSessionButtonActionPerformed
		if(Main.salt.design!=null) {
            Session sessionTemp = new Session();
            sd = new SessionDialog(Main.salt.design.getAdditionalSubjectInfos());
            boolean success = sd.display(getFrame(), sessionTemp, false);
            if(success) {
                Main.salt.session = sessionTemp;
            } else {
                main.status.setSessionStatus(false);
                sessionTemp = null;
                return;
            }
			Main.salt.session.initWithDesign(Main.salt.design);
            main.status.setSessionStatus(true);
        } else {
            main.status.AddMessage("You have to specify a design before specifying a session");
        }
    }//GEN-LAST:event_newSessionButtonActionPerformed

    private void openSessionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openSessionButtonActionPerformed
        if(Main.salt.design!=null) {
            int choosedOption = editSessionFileChooser.showDialog(getFrame(), "Auswählen");
            if(choosedOption == JFileChooser.APPROVE_OPTION) {
                Main.salt.session = SessionReader.getSortedTrialInfos(editSessionFileChooser.getSelectedFile(), Main.salt.design);
            } else {
                return;
            }
			Main.salt.session.initWithDesign(Main.salt.design);
            main.status.setSessionStatus(true);
            System.out.println(Main.salt.session.getXtraInfos().size());
        } else {
            main.status.AddMessage("You have to specify a design before specifying a session");
        }
    }//GEN-LAST:event_openSessionButtonActionPerformed

    private void nrOfItemsComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nrOfItemsComboBoxActionPerformed
        main.changeNrOfItemsPerTrial();
    }//GEN-LAST:event_nrOfItemsComboBoxActionPerformed

    private void showMidiMessagesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showMidiMessagesButtonActionPerformed
        if(showMidiMessagesButton.getText().matches("Show")) {
            showMidiMessageWindow();
            showMidiMessagesButton.setText("Hide");
        } else {
            if(midiMessageFrame!=null) midiMessageFrame.dispose();
            showMidiMessagesButton.setText("Show");
        }
    }//GEN-LAST:event_showMidiMessagesButtonActionPerformed

    private void saveDesignButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveDesignButtonActionPerformed
        int choosedOption = saveFileChooser.showSaveDialog(getFrame());
		if (choosedOption == JFileChooser.APPROVE_OPTION) {
			//savefileChooser.getSelectedFile();
            File outputFile = new File(saveFileChooser.getSelectedFile().getAbsolutePath()+ ".design");
            try {
                DesignWriter.writeDesignXML(outputFile, Main.salt.design);
            } catch (Exception e) {
                //TODO
            }
		}
		else return;
    }//GEN-LAST:event_saveDesignButtonActionPerformed

    private void saveSessionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveSessionButtonActionPerformed
        main.saveSession();
    }//GEN-LAST:event_saveSessionButtonActionPerformed

    private void showStatusButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showStatusButtonActionPerformed
        if(this.showStatusButton.getText().matches("Show")) {
            showStatusMessageWindow();
            showStatusButton.setText("Hide");
        } else {
            this.statusFrame.dispose();
            showStatusButton.setText("Show");
        }
        
    }//GEN-LAST:event_showStatusButtonActionPerformed

    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        playButton.setText("Play");
        main.stop();
    }//GEN-LAST:event_stopButtonActionPerformed

    private void playlistJListMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_playlistJListMousePressed
        main.setActiveItem();
    }//GEN-LAST:event_playlistJListMousePressed

    private void editDesignButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editDesignButtonActionPerformed
        if(Main.salt.design == null) {
            main.status.AddMessage("There's no design to edit");
            return;
        }
        boolean success = dd.display(getFrame(), Main.salt.design, true);
        if(success) {
            //TODO: Update session (probably)
            main.setDesign(Main.salt.design);
            main.status.setDesignStatus(true);
            this.nrOfItemsComboBox.setSelectedIndex(Main.salt.design.getNumberOfFaders()-1);
            main.changeNrOfItemsPerTrial();
        } else {
            if(Main.salt.design == null) main.status.setDesignStatus(false);
        }
    }//GEN-LAST:event_editDesignButtonActionPerformed

    private void editSessionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editSessionButtonActionPerformed
        if(Main.salt.design!=null) {
            if(Main.salt.session == null) {
                main.status.AddMessage("There's no session to edit");
                return;
            }
            Session sessionTemp = Main.salt.session;
            sd = new SessionDialog(Main.salt.design.getAdditionalSubjectInfos());
            boolean success = sd.display(getFrame(), sessionTemp, true);
            if(success) {
                Main.salt.session = sessionTemp;
            } else {
                sessionTemp = null;
                return;
            }
			Main.salt.session.initWithDesign(Main.salt.design);
            main.status.setSessionStatus(true);
        } else {
            main.status.AddMessage("You have to specify a design before specifying a session");
        }
    }//GEN-LAST:event_editSessionButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton clearButton;
    private javax.swing.JButton deleteTrialButton;
    private javax.swing.JLabel designLabel;
    private javax.swing.JButton editDesignButton;
    private javax.swing.JButton editSessionButton;
    private javax.swing.JButton editTrial;
    private javax.swing.JDesktopPane jDesktopPane1;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JLabel midiMessageWindowLabel;
    private javax.swing.JButton newDesignButton;
    private javax.swing.JButton newSessionButton;
    private javax.swing.JButton newTrialButton;
    public javax.swing.JComboBox nrOfItemsComboBox;
    private javax.swing.JButton openDesignButton;
    private javax.swing.JButton openSessionButton;
    private javax.swing.JButton playButton;
    public javax.swing.JList playlistJList;
    private javax.swing.JButton randomizeButton;
    private javax.swing.JButton saveDesignButton;
    private javax.swing.JButton saveSessionButton;
    private javax.swing.JLabel sessionLabel;
    public javax.swing.JLabel sessionName;
    private javax.swing.JButton showMidiMessagesButton;
    private javax.swing.JButton showStatusButton;
    public javax.swing.JCheckBox shuffleCheckBox;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JButton stopButton;
    private javax.swing.JLabel trialLabel;
    // End of variables declaration//GEN-END:variables

    private JDialog aboutBox;    
}
