package mediaplayer;

import packetNetworking.Server;
import packets.EDLPacket;
import java.util.ArrayList;
import java.util.Collections;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListModel;

/**
 *
 * @author Stian
 */
public class Playlist {

    ArrayList<Trial> trials;

    private static DefaultListModel dlm;

    private static boolean shuffleTimeout = false;

    /**
     * Frames per second used by QuickClipXO. Default value 25.
     */
    public static int fps = 25;

    /**
     * Name of the playlist
     */
    public String name;

    /**
     *
     * @param name
     */
    public Playlist(String name) {

        this.name = name;
        trials = new ArrayList<Trial>();
        dlm = (DefaultListModel)(Main.view.playlistJList.getModel());
        Main.view.sessionName.setText(name);
    }

    /**
     * Randomizes the playlist. This takes four steps.
     * 1. Validate the all conditions, including design, session, connection and file list status.
     * 2. Shuffling of items and trials, if selected
     * 3. Generate start and stop time for each item
     * 4. Generate an edl file, send it to Clip Recorder Cotrol and wait for reply
     */
    public void randomize() {
        //TODO: randomize playlist order

        //Check that everything is ok, including that every item is filled and that the number of faders match the number of items
        boolean conditionsOk = true;
        if(!Main.view.main.status.isConnectionStatus()) {
            conditionsOk = false;
            System.out.println("mornings");
            Main.view.main.status.AddMessage("Check connection to clip recorder");
        }
        if(!Main.view.main.status.isDesignStatus()) {
            conditionsOk = false;
            Main.view.main.status.AddMessage("Make sure a design is loaded");
        }
        if(!Main.view.main.status.isFilelistStatus()) {
            conditionsOk = false;
            Main.view.main.status.AddMessage("Make sure the filelist has been transferred");
        }
        if(!Main.view.main.status.isSessionStatus()) {
            conditionsOk = false;
            Main.view.main.status.AddMessage("Make sure a session is loaded");
        }
        if(!conditionsOk) {
            Main.view.main.status.setRandomizeStatus(false);
            return;
        }

        //Are all items not null
        for(Trial t : trials) {
            for(Item i : t.items) {
                if(i == null) {
                    Main.view.main.status.AddMessage("Make sure every item in each trial is filled");
                    return;
                }
            }
        }

        //Shuffles if selected
        if(Main.view.shuffleCheckBox.isSelected()) {
            ArrayList<Trial> list = new ArrayList<Trial>();
            for(int i=0;i<trials.size();i++) {
                list.add(trials.get(i));
            }
            TimeoutThread timeoutThread = new TimeoutThread(10000);
            timeoutThread.start();
            list = shuffle(list);
            while(!isListOk(list)) {
                list = shuffle(list);
                if(shuffleTimeout) {
                    Main.view.main.status.AddMessage("Shuffling was timed out. Generating a valid list failed.");
                    return;
                }
            }
            timeoutThread.stop = true;
            for(int i=0;i<trials.size();i++) {
                trials.set(i, list.get(i));
            }
        }

        //set time codes and create edl-file and send over. wait for response
        int counter = 0;
        int frames = 0;
        frames += 60*fps;     //Start from a minute
        ArrayList<String> edlFile = startEDLFile();
        for(int i=0;i<trials.size();i++) {
            Trial trial = trials.get(i);
            for(int j=0;j<trial.items.length;j++) {
                Item item = trial.items[j];
                item.tcStart = framesToTC(frames);
                item.tcStop = framesToTC(frames + TCToFrames(item.clip.length));
                frames += 20*fps;       //Add a 20sec gap between the clips
                frames += TCToFrames(item.clip.length);     //Add length of clip
                String edlLine = getEDLLine(item, counter);
                if(edlLine == null) return;
                edlFile.add(edlLine);
                edlFile.add(getEDLFilepath(item));
                counter++;
            }
        }

        //send file
        Main.view.main.Talk(new EDLPacket(edlFile));
        System.out.println("edl sent");
        EDLResponseListenerThread ert = new EDLResponseListenerThread(5000);
        while(!EDLPacket.hasResponded) {
            if(ert.timedOut) {
                Main.view.main.status.AddMessage("Sending EDL file to clip recorder failed. Check connection to clip recorder.");
                ert = null;
                return;
            }
            try {
                Thread.sleep(10);
            } catch (InterruptedException ex) {
                Logger.getLogger(Playlist.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        ert = null;

        //rearranging table
        dlm.clear();
        for(Trial trial : trials) {
            trial.rearrange();
            dlm.addElement(trial);
        }
        
        //do what needs to be done when done
        Main.view.playlistJList.setSelectedIndex(0);    //Sets first trial to be active
        Main.view.main.status.setRandomizeStatus(true); //Sets randomizing to be ok
    }
    
    private String getEDLFilepath(Item item) {
        String s = "; ; ";
        s += item.clip.pathname;
        return s;
    }
    
    private String getEDLLine(Item item, int index) {
        String s = index+"";
        if(s.length()>3) {
            Main.view.main.status.AddMessage("Can't have more than 1000 clips");
            return null;
        }
        String fill = "";
        if(s.length()<3) {
            for(int i=0;i<3-s.length();i++) {
                fill += "0";
            }
        }
        s = fill + s + "  000 V1 C        ";

        s += "00:00:00.00 ";                            //File in
        s += getEDLTCFormat(item.clip.length) + " ";    //File out
        s += getEDLTCFormat(item.tcStart) + " ";        //Run in
        s += getEDLTCFormat(item.tcStop);               //Run out

        return s;
    }

    private String getEDLTCFormat(String tc) {
        
        String s = tc.substring(0,8);
        s += "." + tc.substring(9);

        return s;
    }

    private ArrayList<String> startEDLFile() {
        ArrayList<String> start = new ArrayList<String>();
        start.add("Type: DrasticVVWList");
        start.add("; Example Line Pair(remove semicolon comments at start if copying)");
        start.add("; 000 reel VAA C 00:00:00:00 00:00:30:00 00:01:01:00 01:01:30:00");
        start.add("; This is a comment ; C:\\Share\\ClipNameFile.avi");
        start.add(";");
        start.add(";     Edit Number, Reel ID, Edit Presets, File/Edit In Point, File/Edit Out Point, Record/Run In Point, Record/Run Out Point");
        start.add(";     Comment may be anything but must not contain semi colons, ClipFileName must point to a valid media file.");
        start.add("");
        start.add("TITLE: noname");
        start.add("REM  Format: generic (CMX Like)");
        start.add("FCM: PAL ");
        start.add("REM  Record times are pal ");
        start.add("REM  System timecode flags must be set to pal ");
        start.add("");
        return start;
    }

    private String framesToTC(int totalFrames) {

		int hours = 0;
		int minutes = 0;
		int seconds = 0;
		int frames = 0;

		hours = totalFrames/(3600*fps);
		totalFrames -= hours*3600*fps;
		minutes = totalFrames/(60*fps);
		totalFrames -= minutes*60*fps;
		seconds = totalFrames/fps;
		totalFrames -= seconds*fps;
		frames = totalFrames;

		String s = "";
		if(hours>23) {
			return "00:00:00:00";
		}
		if((hours+"").length()<2) {
			s += "0"+hours;
		} else {
			s += hours;
		}
		s += ":";
		if((minutes+"").length()<2) {
			s += "0"+minutes;
		} else {
			s += minutes;
		}
		s += ":";
		if((seconds+"").length()<2) {
			s += "0"+seconds;
		} else {
			s += seconds;
		}
		s += ":";
		if((frames+"").length()<2) {
			s += "0"+frames;
		} else {
			s += frames;
		}

        return s;
    }

    private int TCToFrames(String tc) {
        int hours = Integer.parseInt(tc.substring(0, 2));
        int minutes = Integer.parseInt(tc.substring(3,5));
        int seconds = Integer.parseInt(tc.substring(6, 8));
        int frames = Integer.parseInt(tc.substring(9, 11));
        return hours*3600*fps+minutes*60*fps+seconds*fps+frames;
    }

    private ArrayList<Trial> shuffle(ArrayList<Trial> list) {

        Collections.shuffle(list);
        for(int i=0;i<list.size();i++) {
            list.get(i).id = i;
        }
        Trial.index = list.size();
        for(int i=0;i<list.size();i++) {
            list.get(i).shuffle();
        }

        return list;
    }

    private boolean isListOk(ArrayList<Trial> list) {
        for(int i=0;i<list.size();i++) {
            Trial trial = list.get(i);
            String lastIDTag = "";
            for(int j=0;j<trial.items.length;j++) {
                Item item = trial.items[j];
                if(item.tags.size()>0) {
                    if(item.tags.get(0).matches(lastIDTag)) {
                        return false;
                    }
                    lastIDTag = item.tags.get(0);
                } else {
                    lastIDTag = "";
                }
                
            }
        }
        return true;
    }

    /**
     * adds a trial to the playlist
     * @param trial
     */
    public void addTrial(Trial trial) {
        trial.updateDescription();
        trials.add(trial);
        dlm.addElement(trial);
        
    }

    /**
     * remove the selected trial
     */
    public void removeTrial() {
        int index = getSelectedTrialIndex();
        dlm.remove(index);
        trials.remove(index);
        
    }

    /**
     * replaces trial at index
     * @param trial
     * @param index
     */
    public void setTrial(Trial trial, int index) {
        trial.updateDescription();
        dlm.setElementAt(trial, index);
        trials.set(index, trial);
    }

    /**
     *
     * @param id
     * @return
     */
    public Trial getTrial(int id) {
        return trials.get(id);
    }

    /**
     *
     * @return
     */
    public ArrayList<Trial> getList() {
        return trials;
    }

    /**
     *
     */
    public void clear() {
        dlm.clear();
    }

    /**
     * return the selected trial
     * @return
     */
    public static Trial getSelectedTrial() {
        int selectedIndex = Main.view.playlistJList.getSelectedIndex();
        if(selectedIndex == -1) return null;
        return (Trial)(dlm.getElementAt((selectedIndex)));
    }

    /**
     * returns the index of the selected trial
     * @return
     */
    public static int getSelectedTrialIndex() {
        return Main.view.playlistJList.getSelectedIndex();
    }

    /**
     *
     * @param index
     */
    public static void setSelectedTrial(int index) {
        Main.view.playlistJList.setSelectedIndex(index);
    }

    /**
     *
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     *
     * @param name
     */
    public void setName(String name) {
        this.name = name;
        Main.view.sessionName.setText(name);
    }

    /**
     *
     * @return
     */
    public ArrayList<Trial> getTrials() {
        return trials;
    }

    /**
     * rearranges the list of trials in the gui. note that this does not do anything to the trial list in memory, only the gui
     */
    public void rearrange() {
        dlm.clear();
        for(Trial trial : trials) {
            dlm.addElement(trial);
        }
    }

    private class EDLResponseListenerThread extends Thread {

        public int timeout;
        public boolean timedOut = false;
        
        public EDLResponseListenerThread(int timeout) {
            this.timeout = timeout;
            start();
        }

        @Override
        public void run() {
            try {
                Thread.sleep(timeout);
            } catch (InterruptedException ex) {
                Logger.getLogger(Playlist.class.getName()).log(Level.SEVERE, null, ex);
            }
            if(!EDLPacket.hasResponded) {
                timedOut = true;
            }
        }

    }

    private class TimeoutThread extends Thread {

        int timeout;
        boolean stop;

        public TimeoutThread(int timeout) {
            this.timeout = timeout;
            stop = false;
        }

        @Override
        public void run() {
            try {
                Thread.sleep(timeout);
                if(!stop) shuffleTimeout = true;
            } catch (InterruptedException ex) {
                Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

    }
}
