package packetNetworking;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This thread represents a thread who's only function is to listen for messages from the server
 * A message is received by first reading the length of the package, then reading that number of bytes from the stream
 * Listening intervals are hardcoded at 30ms.
 * @author Stian
 */
public class MessageListenerThread extends Thread {
	
	Communicator com;   //The communicator associated with the thread. Received messages are passed to this communicator
	int index;          //The index of the communicator. It enables access to the correct client connected to the server
    boolean stop = false;   //A simple boolean to be set if the thread is to be stopped
	
    /**
     * A simple constructor that assigns a communicator to the thread, index is set to zero
     * @param com
     */
    public MessageListenerThread(Communicator com) {

        index = 0;
		this.com = com;
		
	}

    /**
     * Another simple constructor, this also sets a given index
     * @param com
     * @param index
     */
    public MessageListenerThread(Communicator com, int index) {
		
		this.com = com;
		this.index = index;
        
	}

    /**
     * Decode an object and return it
     * @param byteArray
     * @return
     */
    public static Packet DeserializeObject(byte[] byteArray) {

		ByteArrayInputStream bis = new ByteArrayInputStream(byteArray);
		try {
			ObjectInputStream objstream = new ObjectInputStream(bis);
			return (Packet)(objstream.readObject());
		} catch (IOException e) {
			System.out.println("IO-problems");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

    /**
     * Overwrites the run method in Thread. Required to overwrite.
     * This method is called when start() is invoked on the thread.
     * The method mainly consists of a while loop that checks whether the boolean stop is true.
     * If false it will read bytes from the stream and decode it into a command.
     * Loop interval is hardcoded to 30ms.
     * */
    @Override
	public void run() {
		
		int numberOfBytes = 0;
		byte[] data;
		while(!stop) {
			try {
				for(int i=0;i<4;i++) {
                    numberOfBytes += (com.ins[index].read()<<(i*8));    //Read length of package
                }
				if(numberOfBytes!=0) {
					data = new byte[numberOfBytes];     //Initilize holder for bytes
					for(int i=0;i<numberOfBytes;i++) {
						data[i] = (byte)com.ins[index].read();  //Read all bytes into list
					}
                    numberOfBytes = 0;  //Reset number of bytes
                    if(com instanceof Client) {
                        //If the communicator is a client, pass it to the client
                        ((Client)com).ProcessPacket(DeserializeObject(data));
                    }
					if(com instanceof Server) {
                        //If the communicator is a server, pass it to the server
                        ((Server)com).ProcessPacket(DeserializeObject(data));
                    }
                    data = null;
				}
			} catch (IOException e) {
				if(com instanceof Client) {
					((Client)com).Reconnect();  //If disconnected, try to reconnect. This may not be possible
					return;
				}
				
			}
            try {
                Thread.sleep(30);
            } catch (InterruptedException ex) {
                Logger.getLogger(MessageListenerThread.class.getName()).log(Level.SEVERE, null, ex);
            }
		}
	}
}