package drastic.mCmdIF;

class DataBuffer
{
    /**
     * Pointer to mediacmd buffer
     */
    protected byte[] byteDataBuffer; // = new [SIZEOF_MEDIACMD_DATA];

    /**
     * Maximum clip id length in the DEFAULT #MEDIACMD structure.  Larger versions may be allocated and
     * are legal.  Smaller versions should not have less than 10 bytes with zero padding
     * for clip names (8 bytes + NULL) and file name (NULL).
     * Note that the array is actually allocated as CMD_MAX_CLIP_ID_LEN+2+2
     * 260+8+2+2 = 270 bytes (DWORD aligned)
     * CLIP 8 Bytes
     * NULL 1 Byte
     * File 260 Bytes (_MAX_PATH)
     * NULL 1 Byte
     * 2 Alignment padding
     */
    /**
     * Size of the clip data area (not including struct)
     */
    public static final int CMD_MAX_CLIP_ID_LEN = 260 + 8;

    /**
     * Size of the first 3 members (the dnode double linked list pointers)
     */
    public static final int CMD_D_LNODE_LEN = 12;

    /**
     * Size of the mediacmd structure without arbID data area
     */
    public static final int MEDIACMD_BASE_SIZE = 64; // Should match offarbyteID (includes CMD_D_LNODE_LEN)

    /**
     * largest possible single message (align to DWORD)
     */
    public static final int MESSAGE_FULL_SIZE = (MEDIACMD_BASE_SIZE + CMD_MAX_CLIP_ID_LEN + 4);

    /**
     * The maximum size of the arbID member
     */
    public static final int CMD_ABSMAX_CMDID_LEN = CMD_MAX_CLIP_ID_LEN + 2048;

    /**
     * The maximum size of the entire message
     */
    public static final int CMD_ABSMAX_ARRAY_LEN = MESSAGE_FULL_SIZE + CMD_ABSMAX_CMDID_LEN;

  /**
   * On construct, makes a buffer and clears it
   *
   * @param lDataSize int
   */
  public DataBuffer(int lDataSize)
    {
        byteDataBuffer = new byte[lDataSize];

        for (int i = 0; i < lDataSize; i++)
        {
            byteDataBuffer[i] = 0;
        }
    }

    /**
     * On construct, make a buffer and clear it
     */
    public DataBuffer()
    {
        // DataBuffer(CMD_ABSMAX_ARRAY_LEN);
        byteDataBuffer = new byte[CMD_ABSMAX_ARRAY_LEN];

        for (int i = 0; i < CMD_ABSMAX_ARRAY_LEN; i++)
        {
            byteDataBuffer[i] = 0;
        }
        //int test = 0;
    }
}

/**
 * <p>Title: MediaCmd (low level - use MediaCmdIF instead)</p>
 *
 * <p>Description: This class moves structure elements of MediaCmds in and out to the data buffer area
 *<br><br>
 * MediaCmdIF - Main Interface API - uses MediaCmd and VVWNetCmdSender<br>
 * MediaCmd - Structure sent and returned from the VVW<br>
 * VVWNETCMDSENDER - used to send and receive the MediaCmds<br>
 * </p>
 *
 * <p>Copyright: Copyright  1998-2007</p>
 *
 * <p>Company: Drastic Technologies Ltd.<br>
 * 523 The Queensway, Suite 102<br>
 * Toronto, ON, M8Y 1J7<br>
 * CANADA<br>
 * 416 255 5636<br>
 * fax 255 8780<br>
 * http://www.drastictech.com<br>
 * engineering@drastictech.com</p>
 *
 * @see VVWNETCMDSENDER
 * @see MediaCmdIF
 * @author Dean Mitrovich
 * @version 1.0
 */
public class MEDIACMD extends DataBuffer
{
    // Start - Mediacmd version definitions
    // ************************************
    /**
     * Major command versioning for upgrades to the command set. see MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_VERSION_MAJOR = 0x0101;
    /**
     * Minor command versioning for upgrades to the command set. see MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_VERSION_MINOR = 0x0003;
    /**
     * Mask for checking the command set version. see MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_VERSION_MASK = 0xFFFF;
    /**
     * Permanent magic number of command id. see MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_CHECK_VER = 4196728832L;//0xFA250000;
    /**
     * Mask for permanent magic number of command id. see MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_CHECK_MASK = 4294901760L;//0xFFFF0000;
    /**
     * Current version and magic number.  Place in MEDIACMD::dwCmdID
     */
    public static final long MEDIACMD_CURRENT = (MEDIACMD_VERSION_MAJOR | MEDIACMD_VERSION_MINOR | MEDIACMD_CHECK_VER);
    // End - Mediacmd version definitions


    /*       Bad Bad - use bytedataArea[offarbyteID] to bytedataArea[offarbyteID+CMD_ABSMAX_CMDID_LEN]
     *        instead . . . byte[] arbID = n ew b yte[NOT!!!CMD_ABSMAX_ARRAY_LEN+2+2];
     *        was wasteful, confusing copy
     */

    /**
     * Forward play 100% (normal play)
     */
    public static final int SpeedFwdPlay = 65520;
    /**
     * Pause - stop on a field or frame
     */
    public static final int SpeedPause = 0;
    /**
     * Reverse play -100%
     */
    public static final int SpeedRevPlay = -65520;
    /**
     * Maximum speed forward (9000% or 90 times)
     */
    public static final int SpeedFwdMax = 5896800;
    /**
     * Maximum speed reverse (-9000% or -90 times)
     */
    public static final int SpeedRevMax = -SpeedFwdMax;
    /**
     * Alias for 100% play speed
     */
    public static final int SpeedPlay = SpeedFwdPlay;

    /**
     * Forward play 100% (normal play)
     */
    public static final int SPD_FWD_PLAY = 65520;
    /**
     * Pause - stop on a field or frame
     */
    public static final int SPD_PAUSE = 0;
    /**
     * Maximum speed forward (9000% or 90 times)
     */
    public static final int SPD_REV_PLAY = ( -SPD_FWD_PLAY);
    /**
     * Maximum speed forward (9000% or 90 times)
     */
    public static final int SPD_FWD_MAX = 5896800;
    /**
     * Maximum speed reverse (-9000% or -90 times)
     */
    public static final int SPD_REV_MAX = ( -SPD_FWD_MAX);

    /**
     * A 'not possible' speed
     */
    public static final int SPD_ILLEGAL = 2147483647;
    public static final long TC_ILLEGAL = 4294967295L;//0xFFFFFFFFL;
    public static final long CHAN_ILLEGAL = 0xFFFFFFFFL;
    // End - Basic Play speeds


    // Start - The legal commands for a #MEDIACMD struct (ct commands)
    /**
     * cmd Stop - pass through e/e no motion
     */
    public static final int ctStop = 0;
    /**
     * cmd Pause - display still frame or field, also seek with position
     */
    public static final int ctPause = 1;
    /**
     * cmd Play - play at any speed in either direction
     */
    public static final int ctPlay = 2;
    /**
     * cmd Record - record the incoming signal
     */
    public static final int ctRecord = 3;
    /**
     * cmd RecStop - prepare and wait for record
     */
    public static final int ctRecStop = 4;
    /**
     * cmd Eject - eject the current media
     */
    public static final int ctEject = 5;
    /**
     * cmd Transfer - transfer from an internal channel to a vtr or visa versa
     */
    public static final int ctTransfer = 6;
    /**
     * cmd Insert - insert media
     */
    public static final int ctInsert = 7;
    /**
     * cmd Blank - remove but do not destroy media
     */
    public static final int ctBlank = 8;
    /**
     * cmd Delete - remove and delete/destroy media
     */
    public static final int ctDelete = 9;
    /**
     * cmd Trim - trim the current media (virtual)
     */
    public static final int ctTrim = 10;
    /**
     * cmd ChanSelect - select the currrent active channels
     */
    public static final int ctChanSelect = 11;
    /**
     * cmd GetState - get the current state
     */
    public static final int ctGetState = 12;
    /**
     * cmd SetState - do not use
     */
    public static final int ctSetState = 13;
    /**
     * cmd GetValue - get a particular gs(Value)
     */
    public static final int ctGetValue = 14;
    /**
     * cmd SetValue - set a particular gs(Value)
     */
    public static final int ctSetValue = 15;
    /**
     * cmd ValueSupported - see if a gs(Value) exists
     */
    public static final int ctValueSupported = 16;
    /**
     * cmd Error - device/channel is in an error state
     */
    public static final int ctError = 17;
    /**
     * cmd Terminate - device/channel is closing
     */
    public static final int ctTerminate = 18;
    /**
     * cmd Abort - stop all current operations
     */
    public static final int ctAbort = 19;
    // End - The legal commands for a #MEDIACMD struct (ct commands)


    // Start - Flags that modify the actions of commands
    public static final int cfDeferred = 1;
    public static final int cfOverideDeferred = 1 << 30;
    public static final int cfTimeMs = 1 << 1;
    public static final int cfTimeTarget = 1 << 2;
    public static final int cfTimeHouseClock = 1 << 3;
    public static final int cfUseSpeed = 1 << 4;
    public static final int cfUsePresets = 1 << 5;
    public static final int cfUsePosition = 1 << 6;
    public static final int cfUsePositionOffset = 1 << 7;
    public static final int cfUseStart = 1 << 8;
    public static final int cfUseStartOffset = 1 << 9;
    public static final int cfUseEnd = 1 << 10;
    public static final int cfUseEndOffset = 1 << 11;
    public static final int cfUseAllIDs = 1 << 12;
    public static final int cfUseClipID = 1 << 13;
    public static final int cfNoClipFiles = 1 << 14;
    public static final int cfNoTCSpaces = 1 << 15;
    public static final int cfUseCmdAlt = 1 << 16;
    public static final int cfIsShuttle = 1 << 17;
    public static final int cfUsingCurrent = 1 << 18;
    public static final int cfUseFrameCount = 1 << 19;
    public static final int cfFields = 1 << 20;
    public static final int cfRipple = 1 << 21;
    public static final int cfLoop = 1 << 22;
    public static final int cfTrigger = 1 << 23;
    public static final int cfPreview = 1 << 24;
    public static final int cfSecondField = 1 << 27;
    public static final int cfUseNextField = cfSecondField;
    public static final int cfInvert = 1 << 28;
    public static final int cfTest = 1 << 29;
    public static final int cfNoReturn = 1 << 31;
    // End - Flags that modify the actions of commands


    // Start - Video Channels
    public static final int vidChan0 = 1;
    public static final int vidChan1 = 1 << 1;
    public static final int vidChan2 = 1 << 2;
    public static final int vidChan3 = 1 << 3;
    public static final int vidChan4 = 1 << 4;
    public static final int vidChan5 = 1 << 5;
    public static final int vidChan6 = 1 << 6;
    public static final int vidChan7 = 1 << 7;
    public static final int vidChan8 = 1 << 8;
    public static final int vidChan9 = 1 << 9;
    public static final int vidChan10 = 1 << 10;
    public static final int vidChan11 = 1 << 11;
    public static final int vidChan12 = 1 << 12;
    public static final int vidChan13 = 1 << 13;
    public static final int vidChan14 = 1 << 14;
    public static final int vidChan15 = 1 << 15;
    public static final int vidChan16 = 1 << 16;
    public static final int vidChan17 = 1 << 17;
    public static final int vidChan18 = 1 << 18;
    public static final int vidChan19 = 1 << 19;
    public static final int vidChan20 = 1 << 20;
    public static final int vidChan21 = 1 << 21;
    public static final int vidChan22 = 1 << 22;
    public static final int vidChan23 = 1 << 23;
    public static final int vidChan25 = 1 << 25;
    public static final int vidChan26 = 1 << 26;
    public static final int vidChan27 = 1 << 27;
    public static final int vidChan28 = 1 << 28;
    public static final int vidChan29 = 1 << 29;
    public static final int vidChan30 = 1 << 30;
    public static final int vidChan31 = 1 << 31;
    public static final long vidChanAll = 0xFFFFFFFFL;
    // End - Video Channels

    // Start - Audio channels
    public static final int audChan0 = 1;
    public static final int audChan1 = 1 << 1;
    public static final int audChan2 = 1 << 2;
    public static final int audChan3 = 1 << 3;
    public static final int audChan4 = 1 << 4;
    public static final int audChan5 = 1 << 5;
    public static final int audChan6 = 1 << 6;
    public static final int audChan7 = 1 << 7;
    public static final int audChan8 = 1 << 8;
    public static final int audChan9 = 1 << 9;
    public static final int audChan10 = 1 << 10;
    public static final int audChan11 = 1 << 11;
    public static final int audChan12 = 1 << 12;
    public static final int audChan13 = 1 << 13;
    public static final int audChan14 = 1 << 14;
    public static final int audChan15 = 1 << 15;
    public static final int audChan16 = 1 << 16;
    public static final int audChan17 = 1 << 17;
    public static final int audChan18 = 1 << 18;
    public static final int audChan19 = 1 << 19;
    public static final int audChan20 = 1 << 20;
    public static final int audChan21 = 1 << 21;
    public static final int audChan22 = 1 << 22;
    public static final int audChan23 = 1 << 23;
    public static final int audChan24 = 1 << 24;
    public static final int audChan25 = 1 << 25;
    public static final int audChan26 = 1 << 26;
    public static final int audChan27 = 1 << 27;
    public static final int audChan28 = 1 << 28;
    public static final int audChan29 = 1 << 29;
    public static final int audChan30 = 1 << 30;
    public static final int audChan31 = 1 << 31;
    public static final long audChanAll = 0xFFFFFFFFL;
    // End - Audio channels

    // Start - Info channels
    public static final int infLtc = 1;
    public static final int infVitc = 1 << 1;
    public static final int infSrcCtl = 1 << 2;
    public static final int infSrcLtc = 1 << 3;
    public static final int infSrcVitc = 1 << 4;
    public static final int infRecTime = 1 << 5;
    public static final int infRecDate = 1 << 6;
    public static final int infCC = 1 << 7;
    public static final int infAuth = 1 << 8;
    public static final int infCopyright = 1 << 9;
    public static final int infOwner = 1 << 10;
    public static final int infSourceName = 1 << 11;
    public static final int infProxyName = 1 << 12;
    public static final int inf13 = 1 << 13;
    public static final int inf14 = 1 << 14;
    public static final int inf15 = 1 << 15;
    public static final int inf16 = 1 << 16;
    public static final int inf17 = 1 << 17;
    public static final int inf18 = 1 << 18;
    public static final int inf19 = 1 << 19;
    public static final int inf20 = 1 << 20;
    public static final int inf21 = 1 << 21;
    public static final int infVB0 = 1 << 22;
    public static final int infVB1 = 1 << 23;
    public static final int infVB2 = 1 << 24;
    public static final int infVB3 = 1 << 25;
    public static final int infVB4 = 1 << 26;
    public static final int infVB5 = 1 << 27;
    public static final int infVB6 = 1 << 28;
    public static final int infVB7 = 1 << 29;
    public static final int infVB8 = 1 << 30;
    public static final int infVB9 = 1 << 31;
    public static final long infChanAll = 0xFFFFFFFFL;
    // End - Info channels



    // Start - Get set Value Commands
    public static final int gsTc = 1;
    public static final int gsUb = 1 + gsTc;
    public static final int gsLtcTc = 1 + gsUb;
    public static final int gsLtcUb = 1 + gsLtcTc;
    public static final int gsVitcTc = 1 + gsLtcUb;
    public static final int gsVitcUb = 1 + gsVitcTc;
    public static final int gsTcSource = 1 + gsVitcUb;
    public static final int gsTcType = 1 + gsTcSource;
    public static final int gsStart = 1 + gsTcType;
    public static final int gsEnd = 1 + gsStart;
    public static final int gsIn = 1 + gsEnd;
    public static final int gsLastIn = 1 + gsIn;
    public static final int gsOut = 1 + gsLastIn;
    public static final int gsLastOut = 1 + gsOut;
    public static final int gsEditOn = 1 + gsLastOut;
    public static final int gsEditOff = 1 + gsEditOn;
    public static final int gsPreroll = 1 + gsEditOff;
    public static final int gsPostroll = 1 + gsPreroll;
    public static final int gsAutoMode = 1 + gsPostroll;
    public static final int gsPlayDelay = 1 + gsAutoMode;

    public static final int gsLtcTcPreset = 1 + gsPlayDelay;
    public static final int gsLtcUbPreset = 1 + gsLtcTcPreset;
    public static final int gsVitcTcPreset = 1 + gsLtcUbPreset;
    public static final int gsVitcUbPreset = 1 + gsVitcTcPreset;

    /**
     * Returns the block of data for a frame
     * <BR>
     * Used with cmdType::ctSetValue or cmdType::ctGetValue (Not really used at this point)
     * <BR>
     * <BR>  MEDIACMD::arbID - The data block
     * <BR>  MEDIACMD::dwEnd - The size of the data block (0..n)
     * <BR>  MEDIACMD::dwPosition - The expected type of data
     * <HR>
     */
    public static final int gsFrameData = 1 + gsVitcUbPreset;

    /**
     * No data, unknown data
     */
    public static final int GS_FRAMEDATA_UNKNOWN = 0x00000;
    /**
     * Ascii data (all printable)
     */
    public static final int GS_FRAMEDATA_ASCII = 0x00001;
    /**
     * Binary (hex) data
     */
    public static final int GS_FRAMEDATA_HEX = 0x00002;
    /**
     * Telecine RP-215 / DPX Data
     */
    public static final int GS_FRAMEDATA_TELECINE = 0x10001;
    /**
     * Close caption/teletext
     */
    public static final int GS_FRAMEDATA_CC_TTEXT = 0x10002;
    /**
     * Navy telemtry data
     */
    public static final int GS_FRAMEDATA_NAVY = 0x10002;

    public static final int gsKeyCode = gsFrameData + 1;
    public static final int gsInkCode = gsKeyCode + 1;
    public static final int gs215Code = gsInkCode + 1;

    public static final int gsGetNextClip = 90; // Get clip name and info (was first/next - send NULL arbID for first)
    public static final int gsFirstClip = 1 + gsGetNextClip;
    public static final int gsNextClip = 1 + gsFirstClip;
    public static final int gsTCSGetTLClipState = 1 + gsNextClip;
    public static final int gsTCSGetTLClipInfo = 1 + gsTCSGetTLClipState;
    public static final int gsClipInfo = 1 + gsTCSGetTLClipInfo;
    public static final int gsClipCopy = 1 + gsClipInfo;

    public static final int gsAudChan = 100;
    public static final int gsVidChan = 1 + gsAudChan;
    public static final int gsInfChan = 1 + gsVidChan;
    public static final int gsAudSelect = 1 + gsInfChan;
    public static final int gsVidSelect = 1 + gsAudSelect;
    public static final int gsInfSelect = 1 + gsVidSelect;
    public static final int gsAudEdit = 1 + gsInfSelect;
    public static final int gsVidEdit = 1 + gsAudEdit;
    public static final int gsInfEdit = 1 + gsVidEdit;
    public static final int gsEditMode = 1 + gsInfEdit;

    // Start - MetaData set/retieve
    // ************************
    /**
     * Access one meta data element for the current media.
     */
    public static final int gsMetaData = 150;
    /**
     * Access one meta data element for the directory of the current media (.:\..\Default.xml)
     */
    public static final int gsMetaDataDirectory = 1 + gsMetaData;
    /**
     * Access one meta data element for the drive/volume of the current media (\Default.xml).
     */
    public static final int gsMetaDataVolume = 1 + gsMetaDataDirectory;
    /**
     *  Access one meta data element for the defult metadata of the current user (HKEY_CURRENT_USER windows, /home/user/default.xml unix).
     */
    public static final int gsMetaDataCurrentUser = 1 + gsMetaDataVolume;
    /**
     * Access one meta data element for the defult metadata of the current user (HKEY_LOCAL_MACHINE windows, /var/metadata/default.xml unix).
     */
    public static final int gsMetaDataLocalMachine = 1 + gsMetaDataCurrentUser;
    /**
     * Access one meta data element for the defult metadata for the facility (Requires group or facility media proxy and database).
     */
    public static final int gsMetaDataGlobal = 1 + gsMetaDataLocalMachine;
    //
    public static final int gsMetaDataReadWrite = 1 + gsMetaDataGlobal;
    // End - MetaData set/retieve


    // Start - Audio settings use dwAudioChannels (except ltc)
    // ***********************
    /**
     * Audio input select
     */
    public static final int gsAudInSelect = 200;
    /**
     * Audio output select (in general all outputs are active)
     */
    public static final int gsAudOutSelect = 1 + gsAudInSelect;
    /**
     * Audio input level (gain)
     */
    public static final int gsAudInputLevel = 1 + gsAudOutSelect;
    /**
     * Audio output level (master)
     */
    public static final int gsAudOutputLevel = 1 + gsAudInputLevel;
    /**
     * Audio advance level (advanced cue head master) - Not Supported
     */
    public static final int gsAudAdvanceLevel = 1 + gsAudOutputLevel;
    /**
     * Audio output phase
     */
    public static final int gsAudOutPhase = 1 + gsAudAdvanceLevel;
    /**
     * Audio advance phase (advanced cue head master) - Not Supported
     */
    public static final int gsAudOutAdvancePhase = 1 + gsAudOutPhase;
    /**
     * Audio crossfade time (clip effect overlap) - Not Supported
     */
    public static final int gsAudCrossFadeTime = 1 + gsAudOutAdvancePhase;
    /**
     * Enable Ltc on an audio channel
     */
    public static final int gsAudLtcEnable = 1 + gsAudCrossFadeTime;
    /**
     * Set audio channel to use for LTC input if enabled.  Currently will set LTC output to same channel on all VVW drivers.
     */
    public static final int gsAudInLtcChannel = 1 + gsAudLtcEnable;
    /**
     * Set audio channel to use for LTC output if enabled.  Currently will set LTC input to same channel on all VVW drivers.
     */
    public static final int gsAudOutLtcChannel = 1 + gsAudInLtcChannel;
    /**
     * Enable Dtmf on an audio channel
     */
    public static final int gsAudDtmfEnable = 1 + gsAudOutLtcChannel;
    /**
     * Set audio channel to use for DTMF input if enabled.  Currently will set DTMF output to same channel on all VVW drivers.
     */
    public static final int gsAudInDtmfChannel = 1 + gsAudDtmfEnable;
    /**
     * Set audio channel to use for DTMF output if enabled.  Currently will set DTMF input to same channel on all VVW drivers.
     */
    public static final int gsAudOutDtmfChannel = 1 + gsAudInDtmfChannel;
    /**
     * Return the last known RMS and peak value of the audio output.  Max 2 channels returned per call.  2 channels should always be requested
     */
    public static final int gsAudWavePeakRMS = 1 + gsAudOutDtmfChannel;

    /**
     * Get/Set the current bit rate for recording audio per call
     */
    public static final int gsAudInputBitRate = 1 + gsAudWavePeakRMS;
    /**
     * Set the audio input sample rate
     */
    public static final int gsAudInputSampleRate = 1 + gsAudInputBitRate;
    /**
     * Audio input mode
     */
    public static final int gsAudInputMode = 1 + gsAudInputSampleRate;
    /**
     * Audio input head room
     */
    public static final int gsAudInputHeadRoom = 1 + gsAudInputMode;
    /**
     * Audio input original - see sony def
     */
    public static final int gsAudInputOriginal = 1 + gsAudInputHeadRoom;
    /**
     * Enable and disable audio input error protection - GS_TRUE protection enabled, else GS_FALSE
     */
    public static final int gsAudInputErrorProtect = 1 + gsAudInputOriginal;
    /**
     * Does the audio input bit stream contain a copyright flag - GS_TRUE copyright flag is set, else GS_FALSE
     */
    public static final int gsAudInputCopyright = 1 + gsAudInputErrorProtect;
    /**
     * Audio input is in slave mode - GS_TRUE if in slave mode, else GS_FALSE
     */
    public static final int gsAudInputSlave = 1 + gsAudInputCopyright;
    /**
     * Audio bass setting, hardware dependant
     */
    public static final int gsAudInputBass = 1 + gsAudInputSlave;
    /**
     * Audio treble setting, hardware dependant
     */
    public static final int gsAudInputTreble = 1 + gsAudInputBass;
    /**
     * Audio status
     */
    public static final int gsAudInputStatus = 1 + gsAudInputTreble;
    // End - Audio settings

    /**
     * Freeze the video output - 0-un, 1 field, 2 field, 3 both (dwPosition)
     */
    public static final int gsVidFreeze = 300;
    /**
     * Set DDR into pre read (read before write) mode - requires 2 or more channels
     */
    public static final int gsVidPreReadMode = 1 + gsVidFreeze;
    /**
     * First field recorded in edit
     */
    public static final int gsVidEditField = 1 + gsVidPreReadMode;
    /**
     * Record frames or fields
     */
    public static final int gsVidRecFrame = 1 + gsVidEditField;
    /**
     * Play frames or fields
     */
    public static final int gsVidPlayFrame = 1 + gsVidRecFrame;
    /**
     * Disable video edit to edit passthrough
     */
    public static final int gsVidNoEE = 1 + gsVidPlayFrame;
    /**
     * Enable superimposed tc/state/menu output in video
     */
    public static final int gsVidSuperimpose = 1 + gsVidNoEE;

    /**
     * Set the type of up down convert to do
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition - #GS_ANALOGMONITORMETHOD_DIRECT, #GS_ANALOGMONITORMETHOD_SD,
     *   #GS_ANALOGMONITORMETHOD_HD720, #GS_ANALOGMONITORMETHOD_HD1080, #GS_ANALOGMONITORMETHOD_FLIP720,
     *   #GS_ANALOGMONITORMETHOD_FLIP1080
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - #GS_ANALOGMONITORMETHOD_DIRECT, #GS_ANALOGMONITORMETHOD_SD,
     *   #GS_ANALOGMONITORMETHOD_HD720, #GS_ANALOGMONITORMETHOD_HD1080, #GS_ANALOGMONITORMETHOD_FLIP720,
     *   #GS_ANALOGMONITORMETHOD_FLIP1080 or GS_NOT_SUPPORTED
     * <HR>
     */
    public static final int gsVidAnalogMonitorSDType = 1 + gsVidSuperimpose;

    /**
     * Select the output type of the analog SD (Composite, SMPTE, RGB)
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition -
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition -
     * <HR>
     */
    public static final int gsVidAnalogMonitorHDType = 1 + gsVidAnalogMonitorSDType;

    /**
     * Select the output type of the analog HD (RGB, SMPTE, xVidRGB)
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition -
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition -
     * <HR>
     */
    public static final int gsVidAnalogMonitorMethod = 1 + gsVidAnalogMonitorHDType;

    /**
     * Select the method for up converting to HD
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition - #GS_UPCONVERT_ANAMORPHIC, #GS_UPCONVERT_PILLARBOX,
     *    #GS_UPCONVERT_ZOOM14x9, #GS_UPCONVERT_LETTERBOX, #GS_UPCONVERT_ZOOMWIDE
     * Use with:  cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - #GS_UPCONVERT_ANAMORPHIC, #GS_UPCONVERT_PILLARBOX,
     *    #GS_UPCONVERT_ZOOM14x9, #GS_UPCONVERT_LETTERBOX, #GS_UPCONVERT_ZOOMWIDE
     * <HR>
     */
    public static final int gsVidAnalogMonitorUpMode = 1 + gsVidAnalogMonitorMethod;

    /**
     * Select the method for down converting to SD
     * <BR>
     * Use with:  cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition - #GS_DOWNCONVERT_LETTERBOX, #GS_DOWNCONVERT_CROP, #GS_DOWNCONVERT_ANAMORPHIC
     * Use with:  cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - #GS_DOWNCONVERT_LETTERBOX, #GS_DOWNCONVERT_CROP, #GS_DOWNCONVERT_ANAMORPHIC
     * <HR>
     */
    public static final int gsVidAnalogMonitorDownMode = 1 + gsVidAnalogMonitorUpMode;

    /**
     * Set/Get the current pan scan pos and zoom
     * <BR>
     * Use with:  cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition - when to do it (0xFFFFFFFF == immeadiate)
     * <BR>	MEDIACMD::dwStart - X (upper bit 0x80000000 is FLIP)
     * <BR>	MEDIACMD::dwEnd - Y (upper bit 0x80000000 is FLIP)
     * <BR>	MEDIACMD::dwSpeed - Z
     * <BR>	MEDIACMD::dwAudioChannels -  X Aspect
     * <BR>	MEDIACMD::dwVideoChannels -  Y Aspect
     * <BR>	MEDIACMD::dwInfoChannels - Rotate
     * Use with:  cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - when to do it
     * <BR>	MEDIACMD::dwStart - X (upper bit 0x80000000 is FLIP)
     * <BR>	MEDIACMD::dwEnd - Y (upper bit 0x80000000 is FLIP)
     * <BR>	MEDIACMD::dwSpeed - Z
     * <BR>	MEDIACMD::dwAudioChannels -  X Aspect
     * <BR>	MEDIACMD::dwVideoChannels -  Y Aspect
     * <BR>	MEDIACMD::dwInfoChannels - Rotate
     * <HR>
     */
    public static final int gsVidPanScanZoom = 1 + gsVidAnalogMonitorDownMode;

    // Start - Video settings - use dwVideoChannels
    // ****************
    /**
     * Select video input
     */
    public static final int gsVidInSelect = 400;
    /**
     * Select video input genlock type
     */
    public static final int gsVidInLockType = 1 + gsVidInSelect;
    /**
     * Input TBC - Setup (~brightness) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidInSetup = 1 + gsVidInLockType;
    /**
     * Input TBC - Video (~contrast) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidInVideo = 1 + gsVidInSetup;
    /**
     * Input TBC - Hue (~color angle) degrees * 182.  Normal range: 0-65520 (0x0000-0xfff0)
     */
    public static final int gsVidInHue = 1 + gsVidInVideo;
    /**
     * Input TBC - Chroma (~saturation) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidInChroma = 1 + gsVidInHue;
    /**
     * Input TBC - U Chroma or Cb or Y'CrCb Normal range: 0-65535 (0x0000-0xffff). Normally only effects the component video or D1 Serial inputs.
     */
    public static final int gsVidInUChroma = 1 + gsVidInChroma;
    /**
     * Input TBC - V Chroma or Cr or Y'CrCb Normal range: 0-65535 (0x0000-0xffff). Normally only effects the component video or D1 Serial inputs.
     */
    public static final int gsVidInVChroma = 1 + gsVidInUChroma;
    /**
     * Remove color from input signal (black and white luminance data only)
     */
    public static final int gsVidInColorKiller = 1 + gsVidInVChroma;
    /**
     * Automatic gain control
     */
    public static final int gsVidInAGC = 1 + gsVidInColorKiller;
    /**
     * Maximum input bandwidth setting
     */
    public static final int gsVidInBandwidth = 1 + gsVidInAGC;
    /**
     * Black type (NTSC only)
     */
    public static final int gsVidInBlack = 1 + gsVidInBandwidth;
    /**
     * White type (NTSC only)
     */
    public static final int gsVidInWhite = 1 + gsVidInBlack;
    /**
     * Input digital signal coring.  Removal of low order bits to remove DAC aliasing
     */
    public static final int gsVidInCoring = 1 + gsVidInWhite;
    /**
     * Remove (smooth) 100% signal spikes
     */
    public static final int gsVidInPeaking = 1 + gsVidInCoring;
    /**
     * Set video transition sharpness
     */
    public static final int gsVidInSharpness = 1 + gsVidInPeaking;
    /**
     * Set video gamma curve
     */
    public static final int gsVidInGamma = 1 + gsVidInSharpness;
    /**
     * Video input signal format.  May be incorrect depending on some hardware setups.
     */
    public static final int gsVidInSignalFormat = 1 + gsVidInGamma;
    // public static final int gsFreeze = 1 + gsVidInGamma;


    /**
     * Main TBC - Setup (~brightness) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidSetup = 500;
    /**
     * Main TBC - Video (~contrast) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidVideo = 1 + gsVidSetup;
    /**
     * Main TBC - Hue (~color angle) degrees * 182.  Normal range: 0-65520 (0x0000-0xfff0)
     */
    public static final int gsVidHue = 1 + gsVidVideo;
    /**
     * Main TBC - Chroma (~saturation) Normal range: 0-65535 (0x0000-0xffff)
     */
    public static final int gsVidChroma = 1 + gsVidHue;
    /**
     * Main TBC - U Chroma or Cb or Y'CrCb Normal range: 0-65535 (0x0000-0xffff). Normally only effects the component video or D1 Serial paths.
     */
    public static final int gsVidUChroma = 1 + gsVidChroma;
    /**
     * Main TBC - V Chroma or Cr or Y'CrCb Normal range: 0-65535 (0x0000-0xffff). Normally only effects the component video or D1 Serial paths.
     */
    public static final int gsVidVChroma = 1 + gsVidUChroma;
    /**
     * Maximum channel bandwidth setting
     */
    public static final int gsVidBandwidth = 1 + gsVidVChroma;
    /**
     * Black type (NTSC only)
     */
    public static final int gsVidBlackSetup = 1 + gsVidBandwidth;
    /**
     * Remove color from signal path (black and white luminance data only)
     */
    public static final int gsVidColor = 1 + gsVidBlackSetup;

    /**
     * Select video output
     */
    public static final int gsVidOutSelect = 600;
    /**
     * Enable genlock (video black timing signal)
     */
    public static final int gsVidOutGenlock = 1 + gsVidOutSelect;
    /**
     * Select genlock (video black timing signal) source
     */
    public static final int gsVidOutGenlockSource = 1 + gsVidOutGenlock;
    /**
     * Select genlock type (quality)
     */
    public static final int gsVidOutLockType = 1 + gsVidOutGenlockSource;
    /**
     * Horizontal output phase
     */
    public static final int gsVidOutHPhase = 1 + gsVidOutLockType;
    /**
     * Video genlock subcarrier phase timing
     */
    public static final int gsVidOutSubCarrier = 1 + gsVidOutHPhase;
    /**
     * Digital output signal coring.  Removal of low order bits to remove DAC aliasing
     */
    public static final int gsVidOutCoring = 1 + gsVidOutSubCarrier;
    /**
     * Remove (smooth) 100% signal spikes on output
     */
    public static final int gsVidOutPeaking = 1 + gsVidOutCoring;
    /**
     * Generic advanced adjustment 1 (hardware dependant)
     */
    public static final int gsVidOutAdjust1 = 1 + gsVidOutPeaking;
    /**
     * Generic advanced adjustment 2 (hardware dependant)
     */
    public static final int gsVidOutAdjust2 = 1 + gsVidOutAdjust1;
    /**
     * Genlock output delay (not currently used)
     */
    public static final int gsVidOutGenlockDelay = 1 + gsVidOutAdjust2;
    /**
     * Video output genlock input signal format.  May be incorrect depending on some hardware setups.
     */
    public static final int gsVidOutLockSignalFormat = 1 + gsVidOutGenlockDelay;
    // Disable dual link out when in dual link in
    /**
     * Disable dual link out when in dual link in
     */
    public static final int gsVidOutDisableDualLink = 1 + gsVidOutLockSignalFormat;
    public static final int gsVidOutReferenceWipeMix = 1 + gsVidOutDisableDualLink;


    // Compression and internal signal parameters
    // ***************************
    /**
     * Size of picture	Y (Vertical)
     */
    public static final int gsCompChVerticalRes = 700;
    /**
     * Alias for gsCompChVerticalRes
     */
    public static final int gsMpegVerticalRes = gsCompChVerticalRes;

    /**
     * Size of picture	X (Horizontal)
     */
    public static final int gsCompChHorizontalRes = 1 + gsCompChVerticalRes;
    /**
     * Alias for gsCompChHorizontalRes
     */
    public static final int gsMpegHorizontalRes = gsCompChHorizontalRes;

    /**
     * Chroma type 4:0:0, 4:2:0, 4:2:2, 4:4:4
     */
    public static final int gsCompChChromaFormat = 1 + gsCompChHorizontalRes;
    /**
     * Alias for gsCompChChromaFormat
     */
    public static final int gsMpegChromaFormat = gsCompChChromaFormat;

    /**
     * DC Precision (mostly MPEG) 8..12
     */
    public static final int gsCompChDCPrecision = 1 + gsCompChChromaFormat;
    /**
     * Alias for gsCompChDCPrecision
     */
    public static final int gsMpegDCPrecision = gsCompChDCPrecision;

    /**
     * Video signal aspect ratio
     */
    public static final int gsCompChAspectRatio = 1 + gsCompChDCPrecision;
    /**
     * Alias for gsCompChAspectRatio
     */
    public static final int gsMpegAspectRatio = gsCompChAspectRatio;

    /**
     * MPEG file stream standard
     */
    public static final int gsCompChStandard = 1 + gsCompChAspectRatio;
    /**
     * Alias for gsCompChStandard
     */
    public static final int gsMpegStandard = gsCompChStandard;

    /**
     * Video/Audio Language Code
     */
    public static final int gsCompChLanguageCode = 1 + gsCompChStandard;
    /**
     * Alias for gsCompChLanguageCode
     */
    public static final int gsMpegLanguageCode = gsCompChLanguageCode;

    /**
     * Closed Captioning Format
     */
    public static final int gsCompChCCFormat = 1 + gsCompChLanguageCode;
    /**
     * Alias for gsCompChCCFormat
     */
    public static final int gsMpegCCFormat = gsCompChCCFormat;

    /**
     * MPEG Concealment Vector
     */
    public static final int gsCompChConcealmentVector = 1 + gsCompChCCFormat;
    /**
     * Alias for gsCompChConcealmentVector
     */
    public static final int gsMpegConcealmentVector = gsCompChConcealmentVector;

    /**
     * Set encoding to closed gop or open gop
     */
    public static final int gsCompChClosedGop = 1 + gsCompChConcealmentVector;
    /**
     * Alias for gsCompChClosedGop
     */
    public static final int gsMpegClosedGop = gsCompChClosedGop;

    /**
     * Set the next GOP start time code value
     */
    public static final int gsCompChAdjustGopTC = 1 + gsCompChClosedGop;
    /**
     * Alias for gsCompChAdjustGopTC
     */
    public static final int gsMpegAdjustGopTC = gsCompChAdjustGopTC;

    /**
     * Set MPEG encoder to use alternate co-efficent tables
     */
    public static final int gsCompChAltCoEffTable = 1 + gsCompChAdjustGopTC;
    /**
     * Alias for gsCompChAltCoEffTable
     */
    public static final int gsMpegAltCoEffTable = gsCompChAltCoEffTable;

    /**
     * Set encoder to use non linear quantization
     */
    public static final int gsCompChNonLinearQuant = 1 + gsCompChAltCoEffTable;
    /**
     * Alias for gsCompChNonLinearQuant
     */
    public static final int gsMpegNonLinearQuant = gsCompChAltCoEffTable;

    /**
     * Set the multiplexer (overall) bit rate
     */
    public static final int gsCompChMuxRate = 1 + gsCompChNonLinearQuant;
    /**
     * Alias for gsCompChMuxRate
     */
    public static final int gsMpegMuxRate = gsCompChMuxRate;

    /**
     * Audio packet size
     */
    public static final int gsCompChAudPacketSize = 1 + gsCompChMuxRate;
    /**
     * Alias for gsCompChAudPacketSize
     */
    public static final int gsMpegAudPacketSize = gsCompChAudPacketSize;

    /**
     * Video packet size
     */
    public static final int gsCompChVidPacketSize = 1 + gsCompChAudPacketSize;
    /**
     * Alias for gsCompChVidPacketSize
     */
    public static final int gsMpegVidPacketSize = gsCompChVidPacketSize;

    /**
     * Stream ID for AUDIO 0xc0 (0x1c0)
     */
    public static final int gsCompChAudioStreamID = 1 + gsCompChVidPacketSize;
    /**
     * Alias for gsCompChVideoStreamID
     */
    public static final int gsMpegAudioStreamID = gsCompChAudioStreamID;

    /**
     * Stream ID for VIDEO 0xe0 (0x1e0)
     */
    public static final int gsCompChVideoStreamID = 1 + gsCompChAudioStreamID;
    /**
     * Alias for gsCompChVideoStreamID
     */
    public static final int gsMpegVideoStreamID = gsCompChVideoStreamID;

    /**
     * Program ID of the video stream within a transport container
     */
    public static final int gsCompChAudioStreamPID = 1 + gsCompChVideoStreamID;
    /**
     * Alias for gsCompChAudioStreamPID
     */
    public static final int gsMpegAudioStreamPID = gsCompChAudioStreamPID;

    /**
     * Program ID of the audio stream within a transport container
     */
    public static final int gsCompChVideoStreamPID = 1 + gsCompChAudioStreamPID;
    /**
     * Alias for gsCompChVideoStreamPID
     */
    public static final int gsMpegVideoStreamPID = gsCompChVideoStreamPID;

    /**
     * Allow settings to be changed.  Used to determine if settings
     */
    public static final int gsCompChAllowSettings = 1 + gsCompChVideoStreamPID;
    /**
     * Alias for gsCompChAllowSettings
     */
    public static final int gsMpegAllowSettings = gsCompChAllowSettings;

    /**
     * Fourcc code for compression.  Set one video or audio channel set to return fourcc code in dwPosition
     */
    public static final int gsCompChFourCC = 1 + gsCompChAllowSettings;

    /**
     * Bit Count
     */
    public static final int gsCompChBitCount = 1 + gsCompChFourCC;
    /**
     * Image Size
     */
    public static final int gsCompChSizeImage = 1 + gsCompChBitCount;
    /**
     * Rate
     */
    public static final int gsCompChRate = 1 + gsCompChSizeImage;
    /**
     * Scale
     */
    public static final int gsCompChScale = 1 + gsCompChRate;
    /**
     * Pitch
     */
    public static final int gsCompChPitch = 1 + gsCompChScale;

    public static final int gsVideoEncodeFormat = 1 + gsCompChPitch;

    public static final int gsAudioEncodeFormat = 1 + gsVideoEncodeFormat;

    public static final int gsCompChannelChangeMs = 1 + gsAudioEncodeFormat;

    public static final int gsAlphaChromaSource = 1 + gsCompChannelChangeMs;

    public static final int gsCompressionType = 1 + gsAlphaChromaSource;

    //public static final int gsBitDepth = 1 + gsCompressionType;

    public static final int gsVideoStandard = 1 + gsCompressionType;

    public static final int gsResetChannel = 1 + gsVideoStandard;

    public static final int gsEnableHDSDFormat = 1 + gsResetChannel;

    public static final int gsVBlankEnable = 1 + gsEnableHDSDFormat;

    public static final int gsLUTEnable = 1 + gsVBlankEnable;

    public static final int gsAudioFileType = 1 + gsLUTEnable;

    public static final int gsAudioBitSize = 1 + gsAudioFileType;

    public static final int gsAudioFrequency = 1 + gsAudioBitSize;

    /**
     * Channel Compression format
     * NTSC CCIR HD 16x9 etc (dwPosition, supported=dwStart)
     */
    public static final int gsSignalFormat = 900;
    /**
     * MJPG, MPEG2, Uncompressed (dwPosition, supported=dwStart)
     */
    public static final int gsCompType = 1 + gsSignalFormat;
    /**
     * Compression setting by total throughput. Total data throughput - frame size (dwPosition, min=dwStart, max=dwEnd)
     */
    public static final int gsCompRateSize = 1 + gsCompType;
    /**
     * Compression setting by compression ratio. Total data rate - ratio * 100 (eg 2:1 = 200) (dwPosition, min=dwStart, max=dwEnd)
     */
    public static final int gsCompRateRatio = 1 + gsCompRateSize;
    /**
     * Compression setting by compression percentage of original size. Total data rate - precentage * 100 (0-10000) of maximum (dwPosition,
     * min=dwStart, max=dwEnd)
     */
    public static final int gsCompRatePercent = 1 + gsCompRateRatio;
    /**
     * Number of frames per 'group of pictures'.  For MPEG compression as well as defining keyframe interval for Cinepac, Indeo, MPEG-4, etc.
     */
    public static final int gsCompGOPSize = 1 + gsCompRatePercent;
    /**
     * Total storage available on current recording drive in kilobytes
     */
    public static final int gsCompIFactor = 1 + gsCompGOPSize;
    /**
     * Total storage available on current recording drive in kilobytes
     */
    public static final int gsCompBFactor = 1 + gsCompIFactor;
    /**
     * Total storage available on current recording drive in kilobytes
     */
    public static final int gsCompPFactor = 1 + gsCompBFactor;
    /**
     * Total storage available on current recording drive in kilobytes
     */
    public static final int gsCompRefPeriod = 1 + gsCompPFactor;
    /**
     * Total storage available on current recording drive in kilobytes
     */
    public static final int gsTotalStorageAvail = 1 + gsCompRefPeriod;
    /**
     * Total storage free on current recording drive in kilobytes
     */
    public static final int gsTotalStorageFree = 1 + gsTotalStorageAvail;
    /**
     * Total recording time available on current recording drive at current compression level
     */
    public static final int gsTotalTimeAvail = 1 + gsTotalStorageFree;
    /**
     * Total recording time free on current recording drive at current compression level
     */
    public static final int gsTotalTimeFree = 1 + gsTotalTimeAvail;
    /**
     * VTR emulation ID type
     */
    public static final int gsVtrType = 1 + gsTotalTimeFree;

    /**
     * Front panel/GUI Interface Local Mode
     */
    public static final int gsLocal = 1000;
    /**
     * Supported read/write file types
     */
    public static final int gsSupportedFileTypes = 1 + gsLocal;
    /**
     * File types for this channel to ignore
     */
    public static final int gsIgnoreFileTypes = 1 + gsSupportedFileTypes;
    /**
     * Disable recording on this channel or this channel does not support recording.
     */
    public static final int gsRecInhibit = 1 + gsIgnoreFileTypes;
    /**
     * Select recording drive
     */
    public static final int gsRecDrive = 1 + gsRecInhibit;
    /**
     * Select recording file name
     */
    public static final int gsRecFileName = 1 + gsRecDrive;
    /**
     * Recording rate by throughput in kilobytes per second
     */
    public static final int gsRecRate = 1 + gsRecFileName;
    /**
     * Default video/stream record file type
     */
    public static final int gsRecFileFormat = 1 + gsRecRate;
    /**
     * Default audio record file type - Type of audio file, or -1 if embedded
     */
    public static final int gsRecAudFileFormat = 1 + gsRecFileFormat;
    /**
     * Disable file deletion on this channel
     */
    public static final int gsDelInhibit = 1 + gsRecAudFileFormat;
    /**
     * Allows/Inhiibits clips being inserted from Bin or TC Space - Inhibit insertion of clips
     */
    public static final int gsInsInhibit = 1 + gsDelInhibit;
    /**
     * Type of file to convert to.
     */
    public static final int gsConvertFileFormat = 1 + gsInsInhibit;
    /**
     * Default audio conversion file type
     */
    public static final int gsConvertAudFileFormat = 1 + gsConvertFileFormat;
    /**
     * Default length, in frames, for a still graphics file being added as a clip
     */
    public static final int gsDefStillLen = 1 + gsConvertAudFileFormat;
    /**
     * Current reference system time for house VITC or house LTC if available, if not then from system clock interpolated with performanace counter.
     * (System Reference Time)
     */
    public static final int gsSysTime = 1 + gsDefStillLen;
    /**
     * Current hardware port used by channel. Mostly for COMx: port selection of CTL and EXT channels.
     * (DSync MS counter for current channel)
     */
    public static final int gsDSyncMs = 1 + gsSysTime;
    /**
     * Current hardware port used by channel. Mostly for COMx: port selection of CTL and EXT channels.
     * (Currently only for ext drv)
     */
    public static final int gsHwPort = 1 + gsDSyncMs;
    /**
     * Playback only output or allow edit to edit
     */
    public static final int gsPBEE = 1 + gsHwPort;
    /**
     * Video reference for servo select
     */
    public static final int gsServoRefSelect = 1 + gsPBEE;
    /**
     * Head select
     */
    public static final int gsHeadSelect = 1 + gsServoRefSelect;
    /**
     * Colour frame select
     */
    public static final int gsColorFrame = 1 + gsHeadSelect;
    /**
     * Video reference disable
     */
    public static final int gsVidRefDisable = 1 + gsColorFrame;

    /**
     * Get Play count delay for the VTR interp
     */
    public static final int gsPlayCountDelay = 1 + gsVidRefDisable;
    /**
     * Use fake edit mode for MPEG bumping.  Basically, all non play speed commands will be emulated, and once a play (lock) is reached the card will
     * be synced to that time and play.
     * (Dangerous if sync does not happen quickly)
     */
    public static final int gsEmulateEditBumping = 1 + gsPlayCountDelay;
    /**
     * Special command alt value for position requests, goes to key frame nearest to requested frame
     */
    public static final int cmdaltNearestKeyFrame = 1 + gsEmulateEditBumping;
    /**
     * Special command alt value for position requests, goes to key frame after to requested frame
     */
    public static final int cmdaltNextKeyFrame = 1 + cmdaltNearestKeyFrame;
    /**
     * Special command alt value for position requests, goes to key frame before to requested frame
     */
    public static final int cmdaltPrevKeyFrame = 1 + cmdaltNextKeyFrame;
    /**
     * Special command alt value for position requests, goes to first frame of actual (non-black) video after the requested frame
     */
    public static final int cmdaltStartOfMessage = 1 + cmdaltPrevKeyFrame;

    /**
     * Special command is video input valid GS_TRUE / GS_FALSE
     * <BR> MEDIACMD::dwPosition -True / False is input Valid
     * <BR> MEDIACMD::dwStart -signal format of the input <HR>
     */
    public static final int gsVidInputValid = 1 + cmdaltStartOfMessage;
    /**
     * Special command is genlock input valid GS_TRUE / GS_FALSE
     * <BR> MEDIACMD::dwPosition -True / False is genlock Valid
     * <BR> MEDIACMD::dwStart -signal format of the genlock signal <HR>
     */
    public static final int gsVidGenlockValid = 1 + gsVidInputValid;
    /**
     * Special command to set/get edit mode for slow MPEG boards
     * <BR>
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition  - #GS_SERIALEDITMODE_NONE, #GS_SERIALEDITMODE_IGNORE, #GS_SERIALEDITMODE_FAKE
     * Use with: cmdType::ctSetValue
     * <BR>	MEDIACMD::dwPosition  - #GS_SERIALEDITMODE_NONE, #GS_SERIALEDITMODE_IGNORE, #GS_SERIALEDITMODE_FAKE
     * <HR>
     */
    public static final int gsSerialEditMode = 1 + gsVidGenlockValid;

    /**
     * Enable/Disable/Flush error log
     */
    public static final int gsErrorLog = 10000;
    /**
     * Get/Set Error Log Name
     */
    public static final int gsErrorLogName = 1 + gsErrorLog;
    /**
     * Gets the starting ms value in message units
     */
    public static final int gsErrorLogStartMs = 1 + gsErrorLogName;
    /**
     * Gets current ms time per log entries (for relative and date absolute)
     */
    public static final int gsErrorLogCurrentMs = 1 + gsErrorLogStartMs;
    /**
     * Gets the last change value, modified with each new entry.
     */
    public static final int gsErrorLogLastChange = 1 + gsErrorLogCurrentMs;
    /**
     * Gets/Set an error message to/from the log
     */
    public static final int gsErrorLogMessage = 1 + gsErrorLogLastChange;

    /**
     * Buffer levels for playback/record (real time)
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> Not supported
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - Buffers stored in board
     * <BR>	MEDIACMD::dwStart - Buffers in queue to board
     * <BR>	MEDIACMD::dwEnd - Buffers in queue from disk
     * <BR>. MEDIACMD:;dwInfoChannels - total system buffers available in board + memory / system frame size
     * <BR>	MEDIACMD::dwVideoChannels - Selected channel for buffer request
     * <BR>	MEDIACMD::dwAudioChannels - Selected channel for buffer request
     * <HR>
     */
    public static final int gsSysBufferLevel = 20000;
    /**
     * Memory usage
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> Not supported
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - Memory we are using
     * <BR>	MEDIACMD::dwStart - Memory used in system
     * <BR>	MEDIACMD::dwEnd - Total memory in system
     * <HR>
     */
    public static final int gsSysMemoryUsage = 1 + gsSysBufferLevel;
    /**
     * CPU usage
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> Not supported
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - CPU we are using
     * <BR>	MEDIACMD::dwStart - CPU used in system
     * <BR>	MEDIACMD::dwEnd - Portion of CPU in kernel mode
     * <HR>
     */
    public static final int gsSysCPUUsage = 1 + gsSysMemoryUsage;

    /**
     * Sets/Returns auto proxy mode
     */
    public static final int gsProxyMode = 50000;
    /**
     * Returns the status of any proxy generation
     */
    public static final int gsProxyStatus = 1 + gsProxyMode;
    /**
     * Return the next proxy source file name.  If the previous name is set to NULL then return the first clip in the list.
     */
    public static final int gsGetNextProxy = 1 + gsProxyStatus;
    /**
     * Add a new proxy file to the list
     */
    public static final int gsAddProxy = 1 + gsGetNextProxy;
    /**
     * Set a proxy file to next in list
     */
    public static final int gsPromoteProxy = 1 + gsAddProxy;
    /**
     * Remove a proxy file from the list
     */
    public static final int gsRemoveProxy = 1 + gsPromoteProxy;
    /**
     * Get Set the max cpu percentage
     */
    public static final int gsProxyCPUUsage = 1 + gsRemoveProxy;

    /**
     * Get VVW version number
     */
    public static final int gsVVWVersion = 60000;
    /**
     * Get MediaReactor version number
     */
    public static final int gsMEVersion = 1 + gsVVWVersion;
    /**
     * Get VVW type description
     */
    public static final int gsVVWType = 1 + gsMEVersion;
    /**
     * Get VVW channel type
     */
    public static final int gsVVWChannelType = 1 + gsVVWType;
    /**
     * Get/Set VVW channel name
     */
    public static final int gsVVWChannelName = 1 + gsVVWChannelType;
    /**
     * Get license
     */
    public static final int gsVVWLicense = 1 + gsVVWChannelName;

    /**
     * Set/Get info on VGA or Secondary NTSC monitor
     */
    public static final int gsMonitor = 64000;

    /**
     * Handle to Window
     */
    public static final int gsMonitorHwnds = 1 + gsMonitor;
    /**
     * Alias for gsMonitorHwnds for older apps
     */
    public static final int gsHwnds = gsMonitorHwnds;

    /**
     * Turns VGA display on / off without killing to he window. Can use this later to set refresh rates - aspect ratios or what not
     */
    public static final int gsMonitorDisplay = 1 + gsMonitorHwnds;
    /**
     * Get a capture of the current output (input passthrough or current clip output).
     * Use #cmdType::ctGetValue to get a preview.
     */
    public static final int gsMonitorGrab = 1 + gsMonitor;
    public static final int gsDirGetList = 64250;
    public static final int gsDirGetInfo = 1 + gsDirGetList;
    public static final int gsDirGetFileInfo = 1 + gsDirGetInfo;
    public static final int gsDirGetFileGrab = 1 + gsDirGetFileInfo;

    /**
     * Check if channels exist
     */
    public static final int gsChannelsExist = 65536;
    /**
     * Get/Set clip mode state (else time code mode)
     */
    public static final int gsClipMode = 1 + gsChannelsExist;
    /**
     * Get/Set record offset for VVW3x00 replay mode
     */
    public static final int gsRecOffset = 1 + gsClipMode;
    /**
     * Get channel capabilities
     */
    public static final int gsChanCapabilities = 1 + gsRecOffset;
    /**
     * Get last change millisecond time from clip space, tc space or file
     */
    public static final int gsLastChangeMs = 1 + gsChanCapabilities;
    /**
     * Get the state of the GPI ins, reset them with set
     */
    public static final int gsGPIIn = 1 + gsLastChangeMs;
    /**
     * Get last change millisecond time from clip space, tc space or file
     */
    public static final int gsGPIOut = 1 + gsGPIIn;

    /**
     * Get the current millisecond counter on the maching (NOT aligned ot anything).
     * NOTE:  This is handled directly for network channels on server side
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR>	Not Supported
     * Use with: cmdType::ctGetValue
     * <BR>	MEDIACMD::dwPosition - the current millisecond counter
     * <HR>
     */
    public static final int gsCurrentMs = 1 + gsGPIOut; // Current ms (dwPosition)
    /**
    * Get/Set whether we are adding on minute of black to start and end of each clip
    * <BR>
    * Use with: cmdType::ctSetValue
    * <BR>	MEDIACMD::dwPosition - #GS_TRUE, #GS_FALSE
    * Use with: cmdType::ctGetValue
    * <BR>	MEDIACMD::dwPosition - #GS_TRUE, #GS_FALSE
    * <HR>
    */
    public static final int gsClipModePreroll = 1 + gsCurrentMs;	// Are we adding black around the clips

    /**
     * Save the current parameters to the registry
     */
    public static final int gsSaveCurrent = 100000;
    /**
     * Load a new clip space (or new if file does not exist)
     */
    public static final int gsLoadClipSpace = 1 + gsSaveCurrent;
    /**
     * Load a new tc space (or new if file does not exist)
     */
    public static final int gsLoadTCSpace = 1 + gsLoadClipSpace;
    /**
     * Load a new film space (or new if file does not exist)
     */
    public static final int gsLoadFilmSpace = 1 + gsLoadTCSpace;
    // End - Get set value Commands

    /**
     * Log in user (must have rights on the local machine)
     */
    public static final int gsUserLogIn = 900000;
    /**
     * Last change in user status
     */
    public static final int gsUserLastChangeMs = 1 + gsUserLogIn;
    /**
     * Return a list of currently logged in users
     */
    public static final int gsUserList = 1 + gsUserLastChangeMs;
    /**
     * Allow a user access to the unit - ONLY AVAILABLE ON LOCAL MACHINE
     */
    public static final int gsUserAdd = 1 + gsUserList;
    /**
     * Remove a users access to the unit - ONLY AVAILABLE ON LOCAL MACHINE
     */
    public static final int gsUserDel = 1 + gsUserAdd;
    /**
     * Change a users rights - ONLY AVAILABLE ON LOCAL MACHINE
     */
    public static final int gsUserRights = 1 + gsUserDel;
    /**
     * Change current users password (must be logged in)
     */
    public static final int gsUserPasswd = 1 + gsUserRights;

    /**
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> MEDIACMD::dwPosition - frame number
     * <BR> MEDIACMD::arbID - file and directory
     *
     * Use with: cmdType::ctGetValue
     * <BR> MEDIACMD::dwPosition - (GS_TRUE | GS_FALSE) is set picon
     * <BR> MEDIACMD::arbID - file and directory
     */
    public static final int gsPiconFrame = 1000000;

    /**
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> MEDIACMD::dwPosition - frame number
     * <BR> MEDIACMD::arbID - file and directory
     *
     * Use with: cmdType::ctGetValue
     * <BR> MEDIACMD::dwPosition - (GS_TRUE | GS_FALSE) is set jpeg
     * <BR> MEDIACMD::arbID - file and directory
     */
    public static final int gsJpegFrame = 1 + gsPiconFrame;

    /**
     * <BR>
     * Use with: cmdType::ctSetValue
     * <BR> MEDIACMD::arbID - directory to use (NULL = Use Network Directory)
     *
     * Use with: cmdType::ctGetValue
     * <BR> MEDIACMD::arbID - directory using for images
     */
    public static final int gsImageDirectory = 1 + gsJpegFrame;
    public static final int gsFrameInfo = 1 + gsImageDirectory;
    public static final int gsRawFrame = 1 + gsFrameInfo;

    public static final int gsVVWService = 1100000;
    public static final int gsInsertQueue = gsVVWService + 1;
    public static final int gsXlatQueue = gsInsertQueue + 1;
    public static final int gsXMLRateScale = gsXlatQueue + 1;
    public static final int gsXMLFileProperties = gsXMLRateScale + 1;

    // gsEditMode
    public static final int GS_INSERT_EDIT = 0x0001;
    public static final int GS_ASSEMBLE_EDIT = 0x0002;

    public static final int GS_USERRIGHTS_NONE = 0x0000;
    public static final int GS_USERRIGHTS_READ = 0x0001;
    public static final int GS_USERRIGHTS_MODIFY = 0x0002;
    public static final int GS_USERRIGHTS_WRITE = 0x0004;
    public static final int GS_USERRIGHTS_SETUP = 0x0008;
    public static final int GS_USERRIGHTS_PLAY = 0x0010;
    public static final int GS_USERRIGHTS_RECORD = 0x0020;
    public static final int GS_USERRIGHTS_ADD = 0x0100;
    public static final int GS_USERRIGHTS_DELETE = 0x0200;
    public static final int GS_USERRIGHTS_FULL = 0x7FFF;
    public static final int GS_USERRIGHTS_ADMIN = 0x8000;


    public static final long GS_CLIPMODE_ILLEGAL = 0xFFFFFFFFL; //-1; // filetypeillegal
    public static final int GS_CLIPMODE_CLIPSPACE = 0; // filetypeClipspace
    public static final int GS_CLIPMODE_TCSPACE = 1; // filetypeTCSpace
    public static final int GS_CLIPMODE_SINGLE = 2; // filetypeSingle
    public static final int GS_CLIPMODE_FILM = 3; // filetypeFilm

    // Standard values for Get/Set/Supported commands
    public static final int GS_TCSOURCE_LTC = 1;
    public static final int GS_TCSOURCE_VITC = 2;
    public static final int GS_TCSOURCE_CTL = 4;
    public static final int GS_TCSOURCE_TC = 7;
    public static final int GS_TCSOURCE_CLIP = GS_TCSOURCE_TC;

    /**
     * Audio in/out unbalanced (RCA connector) high impedence at -10db
     */
    public static final int GS_AUDSELECT_UNBALANCED_10 = 0x001;
    /**
     * Audio in/out unbalanced (RCA connector) high impedence at -4db
     */
    public static final int GS_AUDSELECT_UNBALANCED_4 = 0x002;
    /**
     * Audio in/out balanced (XLR connector) 600ohm impedence at -10db
     */
    public static final int GS_AUDSELECT_BALANCED_10 = 0x010;
    /**
     * Audio in/out balanced (XLR connector) 600ohm impedence at +4db
     */
    public static final int GS_AUDSELECT_BALANCED_4 = 0x020;
    /**
     * Audio in/out digital single wire
     */
    public static final int GS_AUDSELECT_SPDIF = 0x100;
    /**
     * Audio in/out digital balanced with clock
     */
    public static final int GS_AUDSELECT_AES_EBU = 0x200;
    /**
     * Audio in/out embedded in SDI or HD-SDI video signal
     */
    public static final int GS_AUDSELECT_EMBEDDED = 0x400;
    /**
     * Audio in/out digital balanced with clock
     */
    public static final int GS_AUDSELECT_AES_EBU_PRO = 0x800;
    /**
     * No audio in/out available, or cannot be configured
     */
    public static final int GS_AUDSELECT_NONE = 0;
    /**
     * No Audio Selected leave silent
     */
    public static final int GS_AUDSELECT_SILENT = 0x040;

    // Defines for various audio bit rates
    public static final int GS_AUD_BIT_RATE_32000 = 0x0001;
    public static final int GS_AUD_BIT_RATE_41100 = 0x0002;
    public static final int GS_AUD_BIT_RATE_48000 = 0x0004;
    public static final int GS_AUD_BIT_RATE_56000 = 0x0008;
    public static final int GS_AUD_BIT_RATE_64000 = 0x0010;
    public static final int GS_AUD_BIT_RATE_80000 = 0x0020;
    public static final int GS_AUD_BIT_RATE_96000 = 0x0040;
    public static final int GS_AUD_BIT_RATE_112000 = 0x0080;
    public static final int GS_AUD_BIT_RATE_128000 = 0x0100;
    public static final int GS_AUD_BIT_RATE_160000 = 0x0200;
    public static final int GS_AUD_BIT_RATE_192000 = 0x0400;
    public static final int GS_AUD_BIT_RATE_224000 = 0x0800;
    public static final int GS_AUD_BIT_RATE_256000 = 0x1000;
    public static final int GS_AUD_BIT_RATE_320000 = 0x2000;
    public static final int GS_AUD_BIT_RATE_384000 = 0x4000;

    public static final int GS_AUD_STEREO = 0x001;
    public static final int GS_AUD_JOINT_STEREO = 0x002;
    public static final int GS_AUD_DUAL = 0x004;
    public static final int GS_AUD_SINGLE = 0x008;
    public static final int GS_AUD_MULTIPLE = 0x010;

    public static final int GS_AUD_HEADROOM_18 = 0x01;
    public static final int GS_AUD_HEADROOM_20 = 0x02;

    /**
     * Freeze - no freeze
     */
    public static final int GS_VIDFREEZE_NOT_FROZEN = 0;
    /**
     * Freeze - first (0) field
     */
    public static final int GS_VIDFREEZE_FIELD0 = 1;
    /**
     * Freeze - second (1) field
     */
    public static final int GS_VIDFREEZE_FIELD1 = 2;
    /**
     * Freeze - both fields
     */
    public static final int GS_VIDFREEZE_FRAME = 3;

    /**
     * Standard NTSC or PAL composite video
     */
    public static final int GS_VIDSELECT_COMPOSITE = 0x001;
    /**
     * SVHS/S-Video four wire NTSC or PAL video
     */
    public static final int GS_VIDSELECT_SVIDEO = 0x002;
    /**
     * Secondary NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_COMPOSITE_2 = 0x004;
    /**
     * Third NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_COMPOSITE_3 = 0x008;
    /**
     * BetaCam level YCrCb NTSC or PAL video
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV = 0x010;
    /**
     * Panasonic M2 level YCrCb NTSC or PAL video
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV_M2 = 0x020;
    /**
     * SMPTE standard level YCrCb NTSC or PAL video
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV_SMPTE = 0x040;
    /**
     * RGB at video standard rate
     */
    public static final int GS_VIDSELECT_COMPONENT_RGB = 0x080;
    /**
     * D1 Serial Digital or HDSDI video
     */
    public static final int GS_VIDSELECT_D1_SERIAL = 0x100;
    /**
     * D1 Serial Parallel video
     */
    public static final int GS_VIDSELECT_D1_PARALLEL = 0x200;
    /**
     * SDTI/SDI including high speed transfer video
     */
    public static final int GS_VIDSELECT_SDTI = 0x400;
    /**
     * Secondary NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_COMPOSITE_4 = 0x800;
    /**
     * GS_VIDSELECT_SVIDEO_2Secondary NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_SVIDEO_2 = 0x1000;
    /**
     * Secondary NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV_2 = 0x2000;
    /**
     * Secondary NTSC or PAL video (often monitor selection)
     */
    public static final int GS_VIDSELECT_D1_SERIAL_2 = 0x4000;

    /**
     * Standard NTSC or PAL composite video
     */
    public static final int GS_VIDSELECT_COMPOSITE_JAPAN = 0x8000;
    /**
     * SVHS/S-Video four wire NTSC or PAL video
     */
    public static final int GS_VIDSELECT_SVIDEO_JAPAN = 0x10000;
    /**
     * BetaCam level YCrCb NTSC or PAL video
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV_JAPAN = 0x20000;
    /**
     * SMPTE standard level YCrCb NTSC or PAL video
     */
    public static final int GS_VIDSELECT_COMPONENT_YUV_SMPTE_JAPAN = 0x40000;
    /**
     * xVGA compatible analog RGB
     */
    public static final int GS_VIDSELECT_XVID_RGB = 0x80000;
    /**
     * No video available or no configuratable settings
     */
    public static final int GS_VIDSELECT_NONE = 0;

    /**
     * VTR (unruly hsync) lock
     */
    public static final int GS_VIDLOCKTYPE_VTR = 1;
    /**
     * Perfect lock
     */
    public static final int GS_VIDLOCKTYPE_BROADCAST = 2;

    /**
     * Allow normal bandwidth
     */
    public static final int GS_VIDBAND_STANDARD = 0x01;
    /**
     * Allow medium bandwidth
     */
    public static final int GS_VIDBAND_MEDIUM = 0x02;
    /**
     * Allow high bandwidth
     */
    public static final int GS_VIDBAND_HIGH = 0x04;
    /**
     * Impose notch filter on bandwidth
     */
    public static final int GS_VIDBAND_NOTCH = 0x08;

    /**
     * Black at normal level (7.5 IRE NTSC, 0 IRE PAL)
     */
    public static final int GS_VIDBLACK_SETUP = 0x01;
    /**
     * Crystal black level (0 IRE NTSC, 0 IRE PAL)
     */
    public static final int GS_VIDBLACK_CRYSTAL = 0x02;
    /**
     * Super black level (0 > IRE NTSC/PAL)
     */
    public static final int GS_VIDBLACK_SUPER = 0x04;
    /**
     * Whites are clamped or 100 IRE
     * (100 IRE white max)
     */
    public static final int GS_VIDWHITE_CLAMP = 0x01;
    /**
     * Whites are scaled automatically from black level to 100 IRE
     * (Scale like AGC)
     */
    public static final int GS_VIDWHITE_SCALE = 0x02;
    /**
     * Whites are allowed to be greater then 100 IRE
     * (Any white)
     */
    public static final int GS_VIDWHITE_FREE = 0x04;

    /**
     * No external genlock source (free running on internal clock)
     * No genlock (free run master)
     */
    public static final int GS_LOCKSRC_NONE = 0x0001;
    /**
     * External ref in is genlock source
     * (Lock to external in)
     */
    public static final int GS_LOCKSRC_EXTIN = 0x0002;
    /**
     * Current input is genlock source
     * (Lock to current input)
     */
    public static final int GS_LOCKSRC_INPUT = 0x0004;
    /**
     * Composite (CVBS) input is genlock source
     * (Lock to composite ing)
     */
    public static final int GS_LOCKSRC_CVBS = 0x0008;
    /**
     * S-Video (SVHS) input is genlock source
     * (Lock to S-Video In)
     */
    public static final int GS_LOCKSRC_SVIDEO = 0x0010;
    /**
     * Component Y input is genlock source (Lock to Y of betacam input)
     */
    public static final int GS_LOCKSRC_IN_Y = 0x0020;
    /**
     * SDI serial digital input is genlock source
     * (Lock to Digital Module Ref In.)
     */
    public static final int GS_LOCKSRC_SDI = 0x0040;

    /**
     * Keep analog monitor in line with digital (hd=hd, sd=sd)
     */
    public static final int GS_ANALOGMONITORMETHOD_DIRECT = 0x0000;
    /**
     * Convert everything to the nearest SD type
     */
    public static final int GS_ANALOGMONITORMETHOD_SD =	0x0001;
    /**
     * Convert everything to the nearest 720 HD type
     */
    public static final int GS_ANALOGMONITORMETHOD_HD720 = 0x0002;
    /**
     * Convert everything to the nearest 1080 HD type
     */
    public static final int GS_ANALOGMONITORMETHOD_HD1080 = 0x0003;
    /**
     * SD->HD720 and HD->SD
     */
    public static final int GS_ANALOGMONITORMETHOD_FLIP720 = 0x0004;
    /**
     * SD->HD720 and HD->SD
     */
    public static final int GS_ANALOGMONITORMETHOD_FLIP1080 = 0x0005;

    /**
     * Upconvert to whole screen
     */
    public static final int GS_UPCONVERT_ANAMORPHIC = 0x0000;
    /**
     * Upconvert with bars
     */
    public static final int GS_UPCONVERT_PILLARBOX = 0x0001;
    /**
     * Upconvert with some zoom
     */
    public static final int GS_UPCONVERT_ZOOM14x9 = 0x0002;
    /**
     * Upconvert to letter box
     */
    public static final int GS_UPCONVERT_LETTERBOX = 0x0004;
    /**
     * Upconvert to wide zoom
     */
    public static final int GS_UPCONVERT_ZOOMWIDE = 0x0005;

    /**
     * Down convert with top/bottom black bars
     */
    public static final int GS_DOWNCONVERT_LETTERBOX = 0x0000;
    /**
     * Down convert and crop image
     */
    public static final int GS_DOWNCONVERT_CROP = 0x0001;
    /**
     * Down convert to whole screen
     */
    public static final int GS_DOWNCONVERT_ANAMORPHIC = 0x0002;

    public static final int GS_MPEG_RESOLUTION_120 = 0x001;
    public static final int GS_MPEG_RESOLUTION_240 = 0x002;
    public static final int GS_MPEG_RESOLUTION_288 = 0x004;
    public static final int GS_MPEG_RESOLUTION_352 = 0x008;
    public static final int GS_MPEG_RESOLUTION_480 = 0x010;
    public static final int GS_MPEG_RESOLUTION_512 = 0x020;
    public static final int GS_MPEG_RESOLUTION_544 = 0x040;
    public static final int GS_MPEG_RESOLUTION_576 = 0x080;
    public static final int GS_MPEG_RESOLUTION_608 = 0x100;
    public static final int GS_MPEG_RESOLUTION_704 = 0x200;
    public static final int GS_MPEG_RESOLUTION_720 = 0x400;

    public static final int GS_CHROMA_FORMAT_420 = 0x1;
    public static final int GS_CHROMA_FORMAT_422 = 0x2;
    public static final int GS_CHROMA_FORMAT_444 = 0x4;
    public static final int GS_CHROMA_FORMAT_411 = 0x8;

    // Aliases for backwards compatibility
    public static final int GS_MPEG_CHROMA_FORMAT_420 = GS_CHROMA_FORMAT_420;
    public static final int GS_MPEG_CHROMA_FORMAT_422 = GS_CHROMA_FORMAT_422;
    public static final int GS_MPEG_CHROMA_FORMAT_444 = GS_CHROMA_FORMAT_444;

    public static final int GS_MPEG_DC_PRECISION_8 = 0x1;
    public static final int GS_MPEG_DC_PRECISION_9 = 0x2;
    public static final int GS_MPEG_DC_PRECISION_10 = 0x4;
    public static final int GS_MPEG_DC_PRECISION_11 = 0x8;

    public static final int GS_ASPECT_RATIO_SQUARE = 0x1;
    public static final int GS_ASPECT_RATIO_4x3 = 0x2;
    public static final int GS_ASPECT_RATIO_16x9 = 0x4;
    public static final int GS_ASPECT_RATIO_2_21x1 = 0x8;

    // Aliases for backwards compatibility
    public static final int GS_MPEG_ASPECT_RATIO_SQUARE = GS_ASPECT_RATIO_SQUARE;
    public static final int GS_MPEG_ASPECT_RATIO_4x3 = GS_ASPECT_RATIO_4x3;
    public static final int GS_MPEG_ASPECT_RATIO_16x9 = GS_ASPECT_RATIO_16x9;
    public static final int GS_MPEG_ASPECT_RATIO_2_21x1 = GS_ASPECT_RATIO_2_21x1;

    public static final int GS_MPEG_STANDARD_SYSTEM = 0x1;
    public static final int GS_MPEG_STANDARD_PROGRAM = 0x2;
    public static final int GS_MPEG_STANDARD_TRANSPORT = 0x4;
    public static final int GS_MPEG_STANDARD_ELEMENTARY = 0x8;
    // Alias
    public static final int GS_MPEG_STANDARD_ELEMENTRY = GS_MPEG_STANDARD_ELEMENTARY;

    public static final int GS_MPEG_LANGUAGE_ENGLISH = 0x0001;
    public static final int GS_MPEG_LANGUAGE_SPANISH = 0x0002;
    public static final int GS_MPEG_LANGUAGE_FRENCH = 0x0004;
    public static final int GS_MPEG_LANGUAGE_GERMAN = 0x0008;
    public static final int GS_MPEG_LANGUAGE_JAPANESE = 0x0010;
    public static final int GS_MPEG_LANGUAGE_DUTCH = 0x0020;
    public static final int GS_MPEG_LANGUAGE_DANISH = 0x0040;
    public static final int GS_MPEG_LANGUAGE_FINNISH = 0x0080;
    public static final int GS_MPEG_LANGUAGE_ITALIAN = 0x0100;
    public static final int GS_MPEG_LANGUAGE_GREEK = 0x0200;
    public static final int GS_MPEG_LANGUAGE_PORTUGUESE = 0x0400;
    public static final int GS_MPEG_LANGUAGE_SWEDISH = 0x0800;
    public static final int GS_MPEG_LANGUAGE_RUSSIAN = 0x1000;
    public static final int GS_MPEG_LANGUAGE_CHINESE = 0x2000;

    public static final int GS_MPEG_CC_FORMAT_CCUBE = 0x1;
    public static final int GS_MPEG_CC_FORMAT_ATSC = 0x2;
    public static final int GS_MPEG_CC_FORMAT_CCUBE_REORDER = 0x4;
    public static final int GS_MPEG_CC_FORMAT_ATSC_REORDER = 0x8;

    public static final int GS_MPEG_ONE_FRAMES = 0x0001;
    public static final int GS_MPEG_TWO_FRAMES = 0x0002;
    public static final int GS_MPEG_THREE_FRAMES = 0x0004;
    public static final int GS_MPEG_FOUR_FRAMES = 0x0008;
    public static final int GS_MPEG_FIVE_FRAMES = 0x0010;
    public static final int GS_MPEG_SIX_FRAMES = 0x0020;
    public static final int GS_MPEG_SEVEN_FRAMES = 0x0040;
    public static final int GS_MPEG_EIGHT_FRAMES = 0x0080;
    public static final int GS_MPEG_NINE_FRAMES = 0x0100;
    public static final int GS_MPEG_TEN_FRAMES = 0x0200;
    public static final int GS_MPEG_ELEVEN_FRAMES = 0x0400;
    public static final int GS_MPEG_TWELVE_FRAMES = 0x0800;
    public static final int GS_MPEG_THIRTEEN_FRAMES = 0x1000;
    public static final int GS_MPEG_FOURTEEN_FRAMES = 0x2000;
    public static final int GS_MPEG_FIFTEEN_FRAMES = 0x4000;
    public static final int GS_MPEG_SIXTEEN_FRAMES = 0x8000;

    public static final int VIDEOWRITETYPE_AVI = 0x00000001;
    public static final int VIDEOWRITETYPE_MOV = 0x00000002;
    public static final int VIDEOWRITETYPE_WMV = 0x00000004;
    public static final int VIDEOWRITETYPE_GEN = 0x00000008;
    public static final int VIDEOWRITETYPE_JS = 0x00000010;
    public static final int VIDEOWRITETYPE_DVS = 0x00000020;
    public static final int VIDEOWRITETYPE_IHSS = 0x00000040;
    public static final int VIDEOWRITETYPE_HDR = 0x00000080;
    public static final int VIDEOWRITETYPE_YUV = 0x00000100;
    public static final int VIDEOWRITETYPE_RAW = 0x00000200;
    public static final int VIDEOWRITETYPE_TGA = 0x00000400;
    public static final int VIDEOWRITETYPE_BMP = 0x00000800;
    public static final int VIDEOWRITETYPE_TIFF = 0x00001000;
    public static final int VIDEOWRITETYPE_CIN = 0x00002000;
    public static final int VIDEOWRITETYPE_DPX = 0x00004000;
    public static final int VIDEOWRITETYPE_MPG = 0x00008000;
    public static final int VIDEOWRITETYPE_4224 = 0x00010000;
    public static final int VIDEOWRITETYPE_MXF = 0x00020000;

    /**
     * Turn off monitor
     */
    public static final int GS_MONITORGRAB_NONE = 0x0000;
    /**
     * Type mask (jpg, bmp)
     */
    public static final int GS_MONITORGRAB_TYPE_MASK = 0x00F0;
    /**
     * Use BMP format for image
     */
    public static final int GS_MONITORGRAB_TYPE_BMP = 0x0000;
    /**
     *  Use JPEG format for image
     */
    public static final int GS_MONITORGRAB_TYPE_JPG = 0x0010;
    /**
     * Size mask (full, half, quarter)
     */
    public static final int GS_MONITORGRAB_SIZE_MASK = 0x000F;
    /**
     * Full size image captured
     */
    public static final int GS_MONITORGRAB_SIZE_FULL = 0x0001;
    /**
     * Half size image captured
     */
    public static final int GS_MONITORGRAB_SIZE_HALF = 0x0002;
    /**
     * Quarter size image captured
     */
    public static final int GS_MONITORGRAB_SIZE_QUARTER = 0x0004;
    /**
     * Target/To mask
     */
    public static final int GS_MONITORGRAB_TARGET_MASK = 0x0F00;
    /**
     * Use the arbID area
     */
    public static final int GS_MONITORGRAB_TO_MEMORY = 0x0100;
    /**
     * Save image to a UNC path
     */
    public static final int GS_MONITORGRAB_TO_UNC_PATH = 0x0200;
    /**
     * Save image to web server (use name sent in arbID)
     */
    public static final int GS_MONITORGRAB_TO_HTTP = 0x0400;
    /**
     * Save image through 'to be announced' network transport
     */
    public static final int GS_MONITORGRAB_TO_NETWORK = 0x0800;



    // Masks and shifts

    /**
     * Frame rate mask (portion of return for frame rate)
     */
    public static final int GS_SIGFORMMASK_FRAMERATE = 0x000001ff;
    /**
     * Shift frame rate to 0
     */
    public static final int GS_SIGFORMSHIFT_FRAMERATE = 0;
    /**
     * Horizontal / 8 mask (portion of return for frame rate)
     */
    public static final int GS_SIGFORMMASK_HORIZONTAL = 0x000ffe00;
    /**
     * Horizontal / 8 shift to 0
     */
    public static final int GS_SIGFORMSHIFT_HORIZONTAL = 9;
    /**
     * Vertical / 8 mask (portion of return for frame rate)
     */
    public static final int GS_SIGFORMMASK_VERTICAL = 0x0ff00000;
    /**
     * Vertical / 8 shift to 0
     */
    public static final int GS_SIGFORMSHIFT_VERTICAL = 20;
    /**
     * Frame type mask (portion of return for frame rate)
     */
    public static final long GS_SIGFORMMASK_FRAMETYPE = 0xF0000000L;
    /**
     * Frame type shift to 0
     */
    public static final int GS_SIGFORMSHIFT_FRAMETYPE = 28;

    // Basic frame rate types
    public static final int GS_SIGFORMFRAMERATE_5 = 5;
    public static final int GS_SIGFORMFRAMERATE_6 = 6;
    public static final int GS_SIGFORMFRAMERATE_7_5 = 7;
    public static final int GS_SIGFORMFRAMERATE_10 = 10;

    public static final int GS_SIGFORMFRAMERATE_14_98 = 14;
    public static final int GS_SIGFORMFRAMERATE_15 = 15;
    public static final int GS_SIGFORMFRAMERATE_23_98 = 23;
    public static final int GS_SIGFORMFRAMERATE_24 = 24;
    public static final int GS_SIGFORMFRAMERATE_25 = 25;
    public static final int GS_SIGFORMFRAMERATE_29_97 = 29;
    public static final int GS_SIGFORMFRAMERATE_30 = 30;
    public static final int GS_SIGFORMFRAMERATE_47_95 = 47;
    public static final int GS_SIGFORMFRAMERATE_48 = 48;
    public static final int GS_SIGFORMFRAMERATE_50 = 50;
    public static final int GS_SIGFORMFRAMERATE_59_94 = 59;
    public static final int GS_SIGFORMFRAMERATE_60 = 60;
    public static final int GS_SIGFORMFRAMERATE_71_93 = 71;
    public static final int GS_SIGFORMFRAMERATE_72 = 72;
    public static final int GS_SIGFORMFRAMERATE_100 = 100; // 0x64
    public static final int GS_SIGFORMFRAMERATE_119_88 = 119; // 0x77
    public static final int GS_SIGFORMFRAMERATE_CUSTOM = 0x100;

   // Sizing elements
    public static final int GS_SIGFORMSIZE_240 = 0x01;
    public static final int GS_SIGFORMSIZE_243 = 0x02;
    public static final int GS_SIGFORMSIZE_288 = 0x03; // Gap 4
    public static final int GS_SIGFORMSIZE_320 = 0x08;
    public static final int GS_SIGFORMSIZE_352 = 0x09;
    public static final int GS_SIGFORMSIZE_360 = 0x0a; // Gap 5
    public static final int GS_SIGFORMSIZE_480 = 0x10;
    public static final int GS_SIGFORMSIZE_483 = 0x11;
    public static final int GS_SIGFORMSIZE_486 = 0x12; // Gap 1
    public static final int GS_SIGFORMSIZE_496 = 0x14; // Gap 2
    public static final int GS_SIGFORMSIZE_504 = 0x16;
    public static final int GS_SIGFORMSIZE_512 = 0x17; // Gap 2
    public static final int GS_SIGFORMSIZE_576 = 0x1a;
    public static final int GS_SIGFORMSIZE_600 = 0x1b;
    public static final int GS_SIGFORMSIZE_608 = 0x1c; // Gap 3
    public static final int GS_SIGFORMSIZE_640 = 0x20;
    public static final int GS_SIGFORMSIZE_720 = 0x21;
    public static final int GS_SIGFORMSIZE_768 = 0x22;
    public static final int GS_SIGFORMSIZE_800 = 0x23;
    public static final int GS_SIGFORMSIZE_864 = 0x24;
    public static final int GS_SIGFORMSIZE_988 = 0x25;	// Gap 3
    public static final int GS_SIGFORMSIZE_857 = 0x26;
    public static final int GS_SIGFORMSIZE_960 = 0x28;
    public static final int GS_SIGFORMSIZE_968 = 0x29; // Gap 9
    public static final int GS_SIGFORMSIZE_778 = 0x2A;
    public static final int GS_SIGFORMSIZE_872 = 0x2B; // Gap 4
    public static final int GS_SIGFORMSIZE_1024 = 0x30;
    public static final int GS_SIGFORMSIZE_1035 = 0x31;
    public static final int GS_SIGFORMSIZE_1044 = 0x32;
    public static final int GS_SIGFORMSIZE_1050 = 0x34; // Gap 4
    public static final int GS_SIGFORMSIZE_1052 = 0x33;
    public static final int GS_SIGFORMSIZE_1080 = 0x38;
    public static final int GS_SIGFORMSIZE_1088 = 0x39;
    public static final int GS_SIGFORMSIZE_1096 = 0x3a; // Gap 3
    public static final int GS_SIGFORMSIZE_1102 = 0x3E; // Gap 1
    public static final int GS_SIGFORMSIZE_1152 = 0x40;
    public static final int GS_SIGFORMSIZE_1200 = 0x41; // Gap 1
    public static final int GS_SIGFORMSIZE_1234 = 0x43;	// Gap 6
    public static final int GS_SIGFORMSIZE_1280 = 0x48; // Gap 3
    public static final int GS_SIGFORMSIZE_1332 = 0x49;	// Gap 1
    public static final int GS_SIGFORMSIZE_1400 = 0x4b;
    public static final int GS_SIGFORMSIZE_1440 = 0x4c; // Gap 3
    public static final int GS_SIGFORMSIZE_1536 = 0x50;
    public static final int GS_SIGFORMSIZE_1556 = 0x51; // Gap 6
    public static final int GS_SIGFORMSIZE_1828 = 0x56;
    public static final int GS_SIGFORMSIZE_1714 = 0x57;
    public static final int GS_SIGFORMSIZE_1600 = 0x58;
    public static final int GS_SIGFORMSIZE_1920 = 0x59; // Gap 7
    public static final int GS_SIGFORMSIZE_2048 = 0x60; // Gap 8
    public static final int GS_SIGFORMSIZE_2650 = 0x68; // Gap 4
    public static final int GS_SIGFORMSIZE_3112 = 0x6b;	// Gap 3
    public static final int GS_SIGFORMSIZE_4096 = 0x80;

    // Basic Frame Sizes
    public static final int GS_SIGFORMSIZE_640x480 = ((GS_SIGFORMSIZE_640 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_480 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_640x576 = ((GS_SIGFORMSIZE_640 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_576 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x480 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_480 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x483 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_483 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x486 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_486 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x512 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_512 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x576 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_576 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x608 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_608 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_720x504 = ((GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_504 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_800x600 = ((GS_SIGFORMSIZE_800 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_600 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_960x486 = ((GS_SIGFORMSIZE_960 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_486 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_960x576 = ((GS_SIGFORMSIZE_960 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_576 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_960x504 = ((GS_SIGFORMSIZE_960 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_504 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1024x768 = ((GS_SIGFORMSIZE_1024 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_768 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1024x1024 = ((GS_SIGFORMSIZE_1024 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1024 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1152x864 = ((GS_SIGFORMSIZE_1152 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_864 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1280x1024 = ((GS_SIGFORMSIZE_1280 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1024 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1440x1050 = ((GS_SIGFORMSIZE_1440 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1050 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1600x1200 = ((GS_SIGFORMSIZE_1600 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1200 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1280x720 = ((GS_SIGFORMSIZE_1280 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_720 << GS_SIGFORMSHIFT_VERTICAL));

    public static final int GS_SIGFORMSIZE_1828x778 = ((GS_SIGFORMSIZE_1828 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_778 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1828x988 = ((GS_SIGFORMSIZE_1828 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_988 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1828x1102 = ((GS_SIGFORMSIZE_1828 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1102 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1828x1332 = ((GS_SIGFORMSIZE_1828 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1332 << GS_SIGFORMSHIFT_VERTICAL));


    public static final int GS_SIGFORMSIZE_1920x1035 = ((GS_SIGFORMSIZE_1920 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1035 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1920x1080 = ((GS_SIGFORMSIZE_1920 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1080 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_1920x1088 = ((GS_SIGFORMSIZE_1920 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1088 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2560x1080 = ((GS_SIGFORMSIZE_2650 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1080 << GS_SIGFORMSHIFT_VERTICAL));

    public static final int GS_SIGFORMSIZE_2048x857 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_857 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2048x872 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_872 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2048x1102 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1102 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2048x1234 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1234 << GS_SIGFORMSHIFT_VERTICAL));


    public static final int GS_SIGFORMSIZE_2048x1080 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1080 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2048x1536 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1536 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_2048x1556 = ((GS_SIGFORMSIZE_2048 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1556 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_4096x1714 = ((GS_SIGFORMSIZE_4096 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_1714 << GS_SIGFORMSHIFT_VERTICAL));
    public static final int GS_SIGFORMSIZE_4096x3112 = ((GS_SIGFORMSIZE_4096 << GS_SIGFORMSHIFT_HORIZONTAL) |
        (GS_SIGFORMSIZE_3112 << GS_SIGFORMSHIFT_VERTICAL));

    // Basic frame types
    public static final int GS_SIGFORMTYPE_UNKNOWN = (0);
    public static final int GS_SIGFORMTYPE_INTERLACED = (1 << GS_SIGFORMSHIFT_FRAMETYPE);
    public static final int GS_SIGFORMTYPE_PROGRESSIVE = (2 << GS_SIGFORMSHIFT_FRAMETYPE);
    public static final int GS_SIGFORMTYPE_SEGMENTEDFRAME = (4 << GS_SIGFORMSHIFT_FRAMETYPE);

    /**
     * Signal format NTSC square pixel (320x240 or 640x480) @ 29.97 or 30 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_NTSC = (GS_SIGFORMSIZE_640x480 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);
    /**
     * Signal format PAL square pixel (320x288 or 640x576) @ 25 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_PAL = (GS_SIGFORMSIZE_640x576 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_25);
    /**
     * Signal format NTSC square pixel (360/352x243/240 or 720/704x486/480) @ 29.97 or 30 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_CCIR_NTSC = (GS_SIGFORMSIZE_720x486 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);

    /**
     * Signal format NTSC square pixel (360/352x243/240 or 720/704x486/480) @ 29.97 or 30 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_CCIR_NTSC_P483 = (GS_SIGFORMSIZE_720x483 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_29_97);
    /**
     * Signal format PAL square pixel (360/352x288 or 720/704x576) @ 25 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_CCIR_PAL = (GS_SIGFORMSIZE_720x576 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_25);
    /**
     * Signal format NTSC at 30 hz Progressive_30
     */
    public static final int GS_SIGFORM_CCIR_PNTSC_30 = (GS_SIGFORMSIZE_720x486 | GS_SIGFORMTYPE_PROGRESSIVE |GS_SIGFORMFRAMERATE_30);
    /**
     * Signal format PAL at 25 hz Progressive
     */
    public static final int GS_SIGFORM_CCIR_PPAL_25 = (GS_SIGFORMSIZE_720x576 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_25);
    /**
     * Signal format compressed HD 960x504 29.97
     */
    public static final int GS_SIGFORM_HD360 = (GS_SIGFORMSIZE_960x504 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);
    /**
     * Signal format NTSC High Res (960x486)
     */
    public static final int GS_SIGFORM_ALT_NTSC = (GS_SIGFORMSIZE_960x486 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);
    /**
     * Signal format PAL High Res (960x576)
     */
    public static final int GS_SIGFORM_ALT_PAL = (GS_SIGFORMSIZE_960x576 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_25);

    /**
     * 2200x1125 raster, 1920x1035 production aperture (1888x1017 clean) @ 30 fps gsGetSetValue::gsSignalFormat
     */
    public static final int GS_SIGFORM_1035i_30_260M = (GS_SIGFORMSIZE_1920x1035 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_30);
    /**
     * 2200x1125 raster, 1920x1035 production aperture (1888x1017 clean) @ 29.97 fp gsGetSetValue::gsSignalFormats
     */
    public static final int GS_SIGFORM_1035i_30X_260M = (GS_SIGFORMSIZE_1920x1035 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);
    /**
     * 1920x1080i (274M-1997 Table1 System 4) @ 29.97 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 4)
     */
    public static final int GS_SIGFORM_1080i_30 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_30);
    /**
     * 1920x1080i (274M-1997 Table1 System 4) @ 30 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 5)
     */
    public static final int GS_SIGFORM_1080i_30X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_29_97);
    /**
     * 1920x1080i (274M-1997 Table1 System 4) @ 25 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 6)
     */
    public static final int GS_SIGFORM_1080i_25 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_25);
    /**
     * 1920x1080sf (274M-1997 Table1 System 4) @ 24 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 10)
     */
    public static final int GS_SIGFORM_1080sf_24 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_1080i_24 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_24);
    /**
     * 1920x1080sf (274M-1997 Table1 System 4) @ 23.98 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 11)
     */
    public static final int GS_SIGFORM_1080sf_24X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME | GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_1080i_24X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_23_98);
    /**
     * 1920x1080P (274M-1997 Table1 System 4) @ 30 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 7)
     */
    public static final int GS_SIGFORM_1080_30 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_30);
    /**
     * 1920x1080P (274M-1997 Table1 System 4) @ 29.97 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 8)
     */
    public static final int GS_SIGFORM_1080_30X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_29_97);
    /**
     * 1920x1080P (274M-1997 Table1 System 4) @ 25 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 9)
     */
    public static final int GS_SIGFORM_1080_25 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_25);
    /**
     * 1920x1080P (274M-1997 Table1 System 4) @ 24 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 10)
     */
    public static final int GS_SIGFORM_1080_24 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24);
    /**
     * 1920x1080P (274M-1997 Table1 System 4) @ 23.98 gsGetSetValue::gsSignalFormat
     * (274M-1997 Table1 System 11)
     */
    public static final int GS_SIGFORM_1080_24X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_23_98);
    /**
     * 1920x1080P 60 (Dual P30)
     */
    public static final int GS_SIGFORM_1080_60 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_60);
    /**
     * 1920x1080P 59.94 (Dual P29.97)
     */
    public static final int GS_SIGFORM_1080_60X = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_59_94);
    /**
     * 1920x1080P 50 (Dual 25)
     */
    public static final int GS_SIGFORM_1080_50 = (GS_SIGFORMSIZE_1920x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_50);
    /**
     * 1650x750 raster, 1280x720 production aperture (1248x702 clean): @ 60 gsGetSetValue::gsSignalFormat
     * (296M-1996 Table1 System 1)
     */
    public static final int GS_SIGFORM_720_60 = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_60);
    /**
     * 1650x750 raster, 1280x720 production aperture (1248x702 clean): @ 59.94 gsGetSetValue::gsSignalFormat
     * (296M-1996 Table1 System 2)
     */
    public static final int GS_SIGFORM_720_60X = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_59_94);

    /**
     * 50 Hz DVS, IRT
     * (296M-1996 Table1 System 1)
     */
    public static final int GS_SIGFORM_720_50 = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_50);

    /**
     * Half frame rate 720/60
     */
    public static final int GS_SIGFORM_720_30 = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_30);
    /**
     * Half frame rate 720/59.94
     */
    public static final int GS_SIGFORM_720_30X = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_29_97);
    /**
     * Half 50 Hz DVS, IRT
     * (296M-1996 Table1 System 1)
     */
    public static final int GS_SIGFORM_720_25 = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_25);

    /**
     * (296M-1996 Table1 System 1)
     */
    public static final int GS_SIGFORM_720_24 = (GS_SIGFORMSIZE_1280x720 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24);


    /** VGA res
     */
    public static final int GS_SIGFORM_VESA_640_72 = (GS_SIGFORMSIZE_640x480 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_72);
    public static final int GS_SIGFORM_VESA_800_71X = (GS_SIGFORMSIZE_800x600 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_71_93);
    public static final int GS_SIGFORM_VESA_800_72 = (GS_SIGFORMSIZE_800x600 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_72);
    public static final int GS_SIGFORM_VESA_1024_71X = (GS_SIGFORMSIZE_1024x768 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_71_93);
    public static final int GS_SIGFORM_VESA_1024_72 = (GS_SIGFORMSIZE_1024x768 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_72);
    public static final int GS_SIGFORM_VESA_1280_24 = (GS_SIGFORMSIZE_1280x1024 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_VESA_1280i_30 = (GS_SIGFORMSIZE_1280x1024 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_30);
    public static final int GS_SIGFORM_VESA_1280_71X = (GS_SIGFORMSIZE_1280x1024 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_71_93);
    public static final int GS_SIGFORM_VESA_1280_72 = (GS_SIGFORMSIZE_1280x1024 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_72);
    public static final int GS_SIGFORM_VESA_1600i_30 = (GS_SIGFORMSIZE_1600x1200 | GS_SIGFORMTYPE_INTERLACED | GS_SIGFORMFRAMERATE_30);


    /** Presentation res
    */
    public static final int GS_SIGFORM_DVI_1440_1050_24 = (GS_SIGFORMSIZE_1440x1050 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_DVI_1440_1050_25 = (GS_SIGFORMSIZE_1440x1050 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_DCIN_2048_25 = (GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE | GS_SIGFORMFRAMERATE_25);


    /** Digital cinema 2048x1080
     */
    public static final int GS_SIGFORM_DCIN_2048sf_24X = (GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_DCIN_2048sf_24 = (GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_DCIN_2048_24X = (GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_DCIN_2048_24 = (GS_SIGFORMSIZE_2048x1080 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);


    public static final int GS_SIGFORM_FILM_1828_778_24 = (GS_SIGFORMSIZE_1828x778 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_1828_778_25 = (GS_SIGFORMSIZE_1828x778 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_1828_988_24 = (GS_SIGFORMSIZE_1828x988 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_1828_988_25 = (GS_SIGFORMSIZE_1828x988 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_1828_1102_24 = (GS_SIGFORMSIZE_1828x1102 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_1828_1102_25 = (GS_SIGFORMSIZE_1828x1102 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_1828_1332_24 = (GS_SIGFORMSIZE_1828x1332 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_1828_1332_25 = (GS_SIGFORMSIZE_1828x1332 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);

    public static final int GS_SIGFORM_FILM_2048_857_24 = (GS_SIGFORMSIZE_2048x857 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_857_25 = (GS_SIGFORMSIZE_2048x857 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);

    public static final int GS_SIGFORM_FILM_2048_872_24 = (GS_SIGFORMSIZE_2048x872 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_872_25 = (GS_SIGFORMSIZE_2048x872 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_2048_1102_24 = (GS_SIGFORMSIZE_2048x1102 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_1102_25 = (GS_SIGFORMSIZE_2048x1102 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_2048_1234_24 = (GS_SIGFORMSIZE_2048x1234 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_1234_25 = (GS_SIGFORMSIZE_2048x1234 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);

    /** Film transfer
     */
    public static final int GS_SIGFORM_FILM_2048_15X = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_14_98);
    public static final int GS_SIGFORM_FILM_2048_14 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_14_98);
    public static final int GS_SIGFORM_FILM_2048_15 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_15);
    public static final int GS_SIGFORM_FILM_2048sf_15X = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_14_98);
    public static final int GS_SIGFORM_FILM_2048sf_15 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_15);


    public static final int GS_SIGFORM_FILM_2048_24X = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_FILM_2048_24 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048sf_24X = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_FILM_2048sf_24 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_48 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_48);

    public static final int GS_SIGFORM_FILM_2048_25 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_2048sf_25 = (GS_SIGFORMSIZE_2048x1556 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_2048_1536_25 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_25);
    public static final int GS_SIGFORM_FILM_2048_1536sf_25 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_25);

    public static final int GS_SIGFORM_FILM_2048_1536_15X = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_14_98);
    public static final int GS_SIGFORM_FILM_2048_1536_15 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_15);
    public static final int GS_SIGFORM_FILM_2048_1536sf_15X = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_14_98);
    public static final int GS_SIGFORM_FILM_2048_1536sf_15 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_15);
    public static final int GS_SIGFORM_FILM_2048_1536_24X = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_FILM_2048_1536_24 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_1536sf_24X = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_23_98);
    public static final int GS_SIGFORM_FILM_2048_1536sf_24 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_2048_1536_48X = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_47_95);
    public static final int GS_SIGFORM_FILM_2048_1536_48 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_48);

    public static final int GS_SIGFORM_FILM_4096_1714_24 = (GS_SIGFORMSIZE_4096x1714 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_4096_1714_24X = (GS_SIGFORMSIZE_4096x1714 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_23_98);

    public static final int GS_SIGFORM_FILM_4096_3112sf_5 = (GS_SIGFORMSIZE_2048x1536 | GS_SIGFORMTYPE_SEGMENTEDFRAME |
        GS_SIGFORMFRAMERATE_5);
    public static final int GS_SIGFORM_FILM_4096_3112_24 = (GS_SIGFORMSIZE_4096x3112 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_24);
    public static final int GS_SIGFORM_FILM_4096_3112_24X = (GS_SIGFORMSIZE_4096x3112 | GS_SIGFORMTYPE_PROGRESSIVE |
        GS_SIGFORMFRAMERATE_23_98);

    /**
     * All non video rate types (eg. 15fps, 10fps, 37fps) gsGetSetValue::gsSignalFormat
     */
    public static final long GS_SIGFORM_CUSTOM = 0xF0000000L;
    //
    /**
     * For input and genlock status returns
     */
    public static final long GS_SIGFORM_NOT_PRESENT = 0x00000000L;

    // Supported formats
    public static final int GS_SIGFORM_SUPPORTS_NTSC = 0x00000001;
    public static final int GS_SIGFORM_SUPPORTS_PAL = 0x00000002;
    public static final int GS_SIGFORM_SUPPORTS_HR = 0x00000004;
    public static final int GS_SIGFORM_SUPPORTS_360 = 0x00000008;
    public static final int GS_SIGFORM_SUPPORTS_720 = 0x00000010;
    public static final int GS_SIGFORM_SUPPORTS_1035 = 0x00000020;
    public static final int GS_SIGFORM_SUPPORTS_1080 = 0x00000040;
    public static final int GS_SIGFORM_SUPPORTS_EXTRA8 = 0x00000080; // 728, 1044, 1088
    public static final int GS_SIGFORM_SUPPORTS_1536 = 0x00000100; // 2048x1536
    public static final int GS_SIGFORM_SUPPORTS_1556 = 0x00000200; // 2048x1556
    public static final int GS_SIGFORM_SUPPORTS_DCIN = 0x00000400; // 2048x1080
    public static final int GS_SIGFORM_SUPPORTS_1400 = 0x00000800; // 1440x1050
    public static final int GS_SIGFORM_SUPPORTS_V480 = 0x00010000; // Vesa 640x480
    public static final int GS_SIGFORM_SUPPORTS_V600 = 0x00020000; // Vesa 800x600
    public static final int GS_SIGFORM_SUPPORTS_V768 = 0x00040000; // Vesa 1024x768
    public static final int GS_SIGFORM_SUPPORTS_V1024 = 0x00080000; // Vesa 1280x1024
    public static final int GS_SIGFORM_SUPPORTS_V1200 = 0x00100000; // Vesa 1600x1200
    public static final int GS_SIGFORM_SUPPORTS_V1600 = 0x00200000; // Vesa 1600x1200
    public static final int GS_SIGFORM_SUPPORTS_X2 = 0x20000000; // Vesa 1600x1200
    public static final int GS_SIGFORM_SUPPORTS_X3 = 0x40000000; // Vesa 1600x1200
    public static final int GS_SIGFORM_SUPPORTS_X4 = 0x80000000; // Vesa 1600x1200

    /**
     * Software passed codec on main processor
     */
    public static final int GS_COMPTYPE_SOFTWARE = 0x00000001;
    /**
     * Motion JPEG hardware codec (LSI, Zoran, C-Cube, etc)
     */
    public static final int GS_COMPTYPE_MJPEG = 0x00000002;
    /**
     * MPEG-4 h.264
     */
    public static final int GS_COMPTYPE_H264 = 0x00000004;
    /**
     * JPEG-2000
     */
    public static final int GS_COMPTYPE_JPEG2000 = 0x00000008;
    /**
     * MPEG 1 hardware compatible codec
     */
    public static final int GS_COMPTYPE_MPEG1 = 0x00000010;
    /**
     * HOLE
     * Editable MPEG 2 I Frame Only compatible codec
     */
    public static final int GS_COMPTYPE_MPEG2I = 0x00000040;
    /**
     * MPEG 2 long gop hardware compatible codec
     */
    public static final int GS_COMPTYPE_MPEG2IBP = 0x00000080;
    /**
     * Hardware DV25, DVCPRO. DVCPRO25
     */
    public static final int GS_COMPTYPE_DV25 = 0x00000100; // DV25, DVCPRO. DVCPRO25
    /**
     * Hardware DV50, DVCPRO50
     */
    public static final int GS_COMPTYPE_DV50 = 0x00000200; // DV50, DVCPRO50
    /**
     * Hardware Standard DV Bluebook, DVPRO, DVSD
     */
    public static final int GS_COMPTYPE_DVSD = 0x00000400; // Standard DV Bluebook, DVPRO, DVSD
    /**
     * High Def DV codec
     */
    public static final int GS_COMPTYPE_DV100 = 0x00000800;
    /**
     * 8Bit Y'CrCb uncompressed video
     */
    public static final int GS_COMPTYPE_UN8BIT = 0x00001000;
    /**
     * 10Bit Y'CrCb uncompressed video
     */
    public static final int GS_COMPTYPE_UN10BIT = 0x00002000;
    /**
     * Panasonic HD to SDI codec
     */
    public static final int GS_COMPTYPE_HDPAN = 0x00010000;
    /**
     * Sony HD to SDI codec
     */
    public static final int GS_COMPTYPE_HDSONY = 0x00020000;

    /**
     * Inverted 32 bit TGA
     */
    public static final int GS_COMPTYPE_BGRA_INVERT = 0x00040000;
    /**
     * Uncompressed RGB (DVS)
     */
    public static final int GS_COMPTYPE_ARGB = 0x00100000;
    /**
     * Uncompressed RGBA (DVS)
     */
    public static final int GS_COMPTYPE_RGBA = 0x00200000;
    /**
     * Uncompressed A BGR - TIFF
     */
    public static final int GS_COMPTYPE_ABGR = 0x00400000;
    /**
     * Uncompressed BGR A - BMP/TGA
     */
    public static final int GS_COMPTYPE_BGRA = 0x00800000;
    /**
     * Uncompressed RGB 24 Bit
     */
    public static final int GS_COMPTYPE_RGB = 0x00004000;
    /**
     * Uncompressed BGR 24 Bit
     */
    public static final int GS_COMPTYPE_BGR = 0x20;
    /**
     * Uncompressed Y'CrCb 4:2:2 (DVS, VG)
     */
    public static final int GS_COMPTYPE_YCRCB_422 = 0x01000000;
    /**
     * Uncompressed Y'CrCb 4:2:2A (DVS, Dual VG)
     */
    public static final int GS_COMPTYPE_YCRCB_422A = 0x02000000;
    /**
     * Uncompressed Y'CrCb 4:4:4 (DVS, Dual VG)
     */
    public static final int GS_COMPTYPE_YCRCB_444 = 0x04000000;
    /**
     * Uncompressed Y'CrCb 4:4:4A (DVS, Dual VG)
     */
    public static final int GS_COMPTYPE_YCRCB_444A = 0x08000000;
    /**
     * Uncompressed Y'CrCb 4:4:4A (DVS, Dual VG)
     */
    public static final int GS_COMPTYPE_YCRCB_422_STEREO = 0x10000000;
    /**
     * Uncompressed Y'CrCb 4:2:0
     */
    public static final int GS_COMPTYPE_YCRCB_420 = 0x20000000;
    /**
     * DPX 10 bit rgb
     */
    public static final int GS_COMPTYPE_DPX_RGB10 = 0x40000000;
    /**
     * DPX 10 bit YCbCr
     */
    public static final int GS_COMPTYPE_DPX_YCBCR10 = 0x00080000;

    /**
     * Standard Windows AVI container
     */
    public static final int GS_SUPFILE_AVI = 0x00000001;
    /**
     * OpenDML AVI container
     */
    public static final int GS_SUPFILE_ODML = 0x00000002;
    /**
     * Quicktime Mov/MooV container
     */
    public static final int GS_SUPFILE_QT = 0x00000004;
    /**
     * Avid Open Media Format container
     */
    public static final int GS_SUPFILE_OMFI = 0x00000008;
    /**
     * Drastic Fixed Frame container
     */
    public static final int GS_SUPFILE_FIX = 0x00000100;
    /**
     * Audio only or separate audio formats
     */
    public static final int GS_SUPFILE_AUDONLY = 0x00010000;
    /**
     * Series of still file formats
     */
    public static final int GS_SUPFILE_STILLS = 0x00100000;
    /**
     * Other unspecified formats
     */
    public static final int GS_SUPFILE_UNK = 0x40000000;
    /**
     * Any supported mediareactor format
     */
    public static final int GS_SUPFILE_ANY = 0x80000000;

    /**
     * Allow playback or edit to edit output as necassary
     */
    public static final int GS_PBEE_AUTO = 0x00000000;
    /**
     * Allow playback only output - no passthough
     */
    public static final int GS_PBEE_PB = 0x00000001;
    /**
     * Device dependant output
     */
    public static final int GS_PBEE_DEFAULT = 0x000000FF;

    /**
     * Video servo reference auto
     */
    public static final int GS_SERVOREF_AUTO = 0x00000000;
    /**
     * Video servo reference external only
     */
    public static final int GS_SERVOREF_EXT = 0x00000001;
    /**
     * Video servo reference device default
     */
    public static final int GS_SERVOREF_DEFAULT = 0x000000FF;

    /**
     * Use record/play head
     */
    public static final int GS_HEADSEL_RECPLAY = 0x00000000;
    /**
     * Use play head
     */
    public static final int GS_HEADSEL_PLAY = 0x00000001;
    /**
     * Use device default head
     */
    public static final int GS_HEADSEL_DEFAULT = 0x000000FF;

    /**
     * Edit colour frame 2 field
     */
    public static final int GS_CLRFRM_2FLD = 0x00000000;
    /**
     * Edit colour frame 4 field
     */
    public static final int GS_CLRFRM_4FLD = 0x00000001;
    /**
     * Edit colour frame 8 field
     */
    public static final int GS_CLRFRM_8FLD = 0x00000002;
    /**
     * Edit colour frame device default
     */
    public static final int GS_CLRFRM_DEFAULT = 0x000000FF;

    /**
     * Disable video reference
     */
    public static final int GS_VIDREF_DISABLE = 0x00000000;
    /**
     * Enable video reference
     */
    public static final int GS_VIDREF_ENABLE = 0x00000001;

    /**
     * Channel can play (video or audio or both)
     */
    public static final int GS_CHANCAP_PLAY = 0x00000001;
    /**
     * Channel can reverse play (video or audio or both)
     */
    public static final int GS_CHANCAP_REVPLAY = 0x00000002;
    /**
     * Channel can pause and display frame (video or audio or both)
     */
    public static final int GS_CHANCAP_PAUSE = 0x00000004;
    /**
     * Channel can jog below play speed (video or audio or both)
     */
    public static final int GS_CHANCAP_JOG = 0x00000008;
    /**
     * Channel can shuttle above play speed (video or audio or both)
     */
    public static final int GS_CHANCAP_SHUTTLE = 0x00000010;
    /**
     * Channel can seek to any point (video or audio or both)
     */
    public static final int GS_CHANCAP_SEEK = 0x00000020;
    /**
     * Channel can preview from in to out (video or audio or both)
     */
    public static final int GS_CHANCAP_PREVIEW = 0x00000040;
    /**
     * Channel has a stop mode (video or audio or both)
     */
    public static final int GS_CHANCAP_STOP = 0x00001000;
    /**
     * Channel can pass through video (in stop) (video or audio or both)
     */
    public static final int GS_CHANCAP_ETOE = 0x00002000;
    /**
     * Channel can record (video or audio or both)
     */
    public static final int GS_CHANCAP_RECORD = 0x00004000;
    /**
     * Channel can edit from in to out (video or audio or both)
     */
    public static final int GS_CHANCAP_EDIT = 0x00008000;
    /**
     * Channel can set clip name and prep record (video or audio or both)
     */
    public static final int GS_CHANCAP_RECSTOP = 0x00010000;
    /**
     * Channel can select recording channels (video or audio or both)
     */
    public static final int GS_CHANCAP_SELECTPRESET = 0x00020000;
    /**
     * Channel can eject the media (video or audio or both)
     */
    public static final int GS_CHANCAP_EJECT = 0x00040000;
    /**
     * Channel can play in a loop (video or audio or both)
     */
    public static final int GS_CHANCAP_LOOP = 0x00100000;
    /**
     * Channel can display a vga preview (video or audio or both)
     */
    public static final int GS_CHANCAP_VGAPREVIEW = 0x00200000;
    /**
     * Channel can preview audio on a multi media card (video or audio or both)
     */
    public static final int GS_CHANCAP_AUDPREVIEW = 0x00200000;
    /**
     * Channel can play from a file (video or audio or both)
     */
    public static final int GS_CHANCAP_FILE = 0x01000000;
    /**
     * Channel can play from a network feed (video or audio or both)
     */
    public static final int GS_CHANCAP_NET = 0x02000000;
    /**
     * Channel can act like a clip space (video or audio or both)
     */
    public static final int GS_CHANCAP_CLIPSPACE = 0x10000000;
    /**
     * Channel can act like a vtr time code space (video or audio or both)
     */
    public static final int GS_CHANCAP_TCSPACE = 0x20000000;
    /**
     * Channel can be configures as MPEG -- opens whole bunch of settings on the options (specifically for Argus board right now).
     */
    public static final int GS_CHANCAP_MPEG_ENC = 0x40000000;
    /**
     * Do not use this bit - indicates error
     */
    public static final int GS_CHANCAP_ERROR = 0x80000000;
    /**
     * Channel can do anything except MPEG_ENC (by default this should not be)
     */
    public static final int GS_CHANCAP_ALL = 0x3FFFFFFF;

    public static final int GS_PROXYMODE_NOTHING = 0x000000;
    public static final int GS_PROXYMODE_EVERYTHING = 0x000002;
    public static final int GS_PROXYMODE_RECORD = 0x000001;
    public static final int GS_PROXYMODE_AFTERRECORD = 0x000010;

    /**
     * Command is not supported
     */
    public static final long GS_NOT_SUPPORTED = 0xFFFFFFFFL;
    /**
     * Parameter is bad
     */
    public static final long GS_BAD_PARAM = 0xFFFFFFFEL;
    /**
     * False for boolean
     */
    public static final int GS_FALSE = 0x00;
    /**
     * True for boolean
     */
    public static final int GS_TRUE = 0x01;
    /**
     * Default for cmdType::ctGetValue, cmdType::ctSetValue (usually in relation to VTR setup)
     */
    public static final int GS_DEFAULT = 0xFF;
    /**
     * Use field cmdType::ctGetValue, cmdType::ctSetValue (for pause/freeze as opposed to frame)
     */
    public static final int GS_FIELD = 0x00;
    /**
     * Use field 1 cmdType::ctGetValue, cmdType::ctSetValue (for record/playback starts and edits)
     */
    public static final int GS_FIELD1 = 0x01;
    /**
     * Use field 2 cmdType::ctGetValue, cmdType::ctSetValue (for record/playback starts and edits)
     */
    public static final int GS_FIELD2 = 0x02;
    /**
     * Use frame cmdType::ctGetValue, cmdType::ctSetValue (for pause/freeze as opposed to field)
     */
    public static final int GS_FRAME = 0x03;
    /**
     * Set value to unity (levels, tbc) or default (compression type, amount)
     */
    public static final long GS_UNITY = 0xFFFFFFFFL;

    public static final int GS_ALPHACHROMA_SINGLE = 0x01;
    public static final int GS_ALPHACHROMA_ALPHA = 0x02;
    public static final int GS_ALPHACHROMA_DUAL = 0x04;

    public static final int GS_BITCOUNT_8 = 0x01;
    public static final int GS_BITCOUNT_10 = 0x02;
    public static final int GS_BITCOUNT_24 = 0x04;
    public static final int GS_BITCOUNT_30 = 0x08;
    public static final int GS_BITCOUNT_32 = 0x10;
    public static final int GS_BITCOUNT_12 = 0x20;	/* 4:2:0 */

    // START - This is the MediaCmd Structure
    public static final int offpPrev = 0; // For C this is doubly linked list
    public static final int offpNext = 4; // For C this is doubly linked list
    public static final int offpList = 8; // For C this is doubly linked list
    public static final int offnetTime = 8; // For network, time
    public static final int offCmdID = 12;
    public static final int offStructureSize = 16;
    public static final int offChannel = 20;
    public static final int offCmd = 24;
    public static final int offFlags = 28;
    public static final int offSpeed = 32;
    public static final int offVideoChannels = 36;
    public static final int offAudioChannels = 40;
    public static final int offInfoChannels = 44;
    public static final int offCmdAlt = 48;
    public static final int offPosition = 52;
    public static final int offStart = 56;
    public static final int offEnd = 60;
    public static final int offarbyteID = 64;
    // END - This is the MediaCmd Structure


    // Known file types taken from the vvwBaseClass.h file and used with clip mode
    public static final int filetypeIllegal = -1;
    public static final int filetypeClipSpace = 0;
    public static final int filetypeTCSpace = 1;
    public static final int filetypeSingle = 2;

    public MEDIACMD()
    {
        super();
    }

    public MEDIACMD(int lSizeData)
    {
        super(lSizeData);
    }

    // *********************************************************************************

    /////////////////////////////////////////////////////////////////
    //
    // The info structure provides basic information on the
    //	current information channels available.  For now, this
    //	is mostly LTC in file, SmpteX LTC info, current time
    //	and data, but it leaves from for other VITC readers,
    //	close caption decode and comment.
    //
    // 03/13/98 Used By:
    //		Common\mfFile
    //

    /**
     * The flag indicating the structure is a #VVWINFO structure within a union
     */
    long _VVW_IS_VVWINFO = 0x1000;

    /**
     * <p>Title: vvwInfoMetaTypes</p>
     *
     * <p>Description: Class that contains vvwi metadata types. (similar to a struct)</p>
     *
     * <p>Copyright: Copyright  2003</p>
     *
     * <p>Company: Drastic Technologies Ltd.</p>
     * @author Dean Mitrovich
     * @version 1.0
     */
    public class vvwInfoMetaTypes
    {
        public static final int vvwiFileName = 0;
        public static final int vvwiNativeLocator = vvwiFileName + 1;
        public static final int vvwiUniversalName = vvwiNativeLocator + 1;
        public static final int vvwiIP = vvwiUniversalName + 1;
        public static final int vvwiSourceLocator = vvwiIP + 1;

        public static final int vvwiChannel = vvwiSourceLocator + 1;
        public static final int vvwiChannelName = vvwiChannel + 1;
        public static final int vvwiChannelDescription = vvwiChannelName + 1;
        public static final int vvwiTitle = vvwiChannelDescription + 1;
        public static final int vvwiSubject = vvwiTitle + 1;
        public static final int vvwiCategory = vvwiSubject + 1; // 10
        public static final int vvwiKeywords = vvwiCategory + 1;
        public static final int vvwiRatings = vvwiKeywords + 1;
        public static final int vvwiComments = vvwiRatings + 1;
        public static final int vvwiOwner = vvwiComments + 1;
        public static final int vvwiEditor = vvwiOwner + 1;
        public static final int vvwiSupplier = vvwiEditor + 1;
        public static final int vvwiSource = vvwiSupplier + 1;
        public static final int vvwiProject = vvwiSource + 1;
        public static final int vvwiStatus = vvwiProject + 1;
        public static final int vvwiAuthor = vvwiStatus + 1;  // 20
        public static final int vvwiRevisionNumber = vvwiAuthor + 1;
        public static final int vvwiProduced = vvwiRevisionNumber + 1;
        public static final int vvwiAlbum = vvwiProduced + 1;
        public static final int vvwiArtist = vvwiAlbum + 1;
        public static final int vvwiComposer = vvwiArtist + 1;
        public static final int vvwiCopyright = vvwiComposer + 1;
        public static final int vvwiCreationData = vvwiCopyright + 1;
        public static final int vvwiDescription = vvwiCreationData + 1;
        public static final int vvwiDirector = vvwiDescription + 1;
        public static final int vvwiDisclaimer = vvwiDirector + 1;  // 30
        public static final int vvwiEncodedBy = vvwiDisclaimer + 1;
        public static final int vvwiFullName = vvwiEncodedBy + 1;
        public static final int vvwiGenre = vvwiFullName + 1;
        public static final int vvwiHostComputer = vvwiGenre + 1;
        public static final int vvwiInformation = vvwiHostComputer + 1;
        public static final int vvwiMake = vvwiInformation + 1;
        public static final int vvwiModel = vvwiMake + 1;
        public static final int vvwiOriginalArtist = vvwiModel + 1;
        public static final int vvwiOriginalFormat = vvwiOriginalArtist + 1;
        public static final int vvwiPerformers = vvwiOriginalFormat + 1;  // 40
        public static final int vvwiProducer = vvwiPerformers + 1;
        public static final int vvwiProduct = vvwiProducer + 1;
        public static final int vvwiSoftware = vvwiProduct + 1;
        public static final int vvwiSpecialPlaybackRequirements = vvwiSoftware + 1;
        public static final int vvwiTrack = vvwiSpecialPlaybackRequirements + 1;
        public static final int vvwiWarning = vvwiTrack + 1;
        public static final int vvwiURLLink = vvwiWarning + 1;
        public static final int vvwiEditData1 = vvwiURLLink + 1;
        public static final int vvwiEditData2 = vvwiEditData1 + 1;
        public static final int vvwiEditData3 = vvwiEditData2 + 1;  // 50
        public static final int vvwiEditData4 = vvwiEditData3 + 1;
        public static final int vvwiEditData5 = vvwiEditData4 + 1;
        public static final int vvwiEditData6 = vvwiEditData5 + 1;
        public static final int vvwiEditData7 = vvwiEditData6 + 1;
        public static final int vvwiEditData8 = vvwiEditData7 + 1;
        public static final int vvwiEditData9 = vvwiEditData8 + 1;
        public static final int vvwiVersionString = vvwiEditData9 + 1;
        public static final int vvwiManufacturer = vvwiVersionString + 1;
        public static final int vvwiLanguage = vvwiManufacturer + 1;
        public static final int vvwiFormat = vvwiLanguage + 1;  // 60
        public static final int vvwiInputDevice = vvwiFormat + 1;
        public static final int vvwiDeviceModelNum = vvwiInputDevice + 1;
        public static final int vvwiDeviceSerialNum = vvwiDeviceModelNum + 1;
        public static final int vvwiReel = vvwiDeviceSerialNum + 1;
        public static final int vvwiShot = vvwiReel + 1;
        public static final int vvwiTake = vvwiShot + 1;
        public static final int vvwiSlateInfo = vvwiTake + 1;
        public static final int vvwiFrameAttribute = vvwiSlateInfo + 1;
        public static final int vvwiEpisode = vvwiFrameAttribute + 1;
        public static final int vvwiScene = vvwiEpisode + 1;  // 70
        public static final int vvwiDailyRoll = vvwiScene + 1;
        public static final int vvwiCamRoll = vvwiDailyRoll + 1;
        public static final int vvwiSoundRoll = vvwiCamRoll + 1;
        public static final int vvwiLabRoll = vvwiSoundRoll + 1;
        public static final int vvwiKeyNumberPrefix = vvwiLabRoll + 1;
        public static final int vvwiInkNumberPrefix = vvwiKeyNumberPrefix + 1;
        public static final int vvwiPictureIcon = vvwiInkNumberPrefix + 1;
        public static final int vvwiProxyFile = vvwiPictureIcon + 1;

        public static final int vvwiEND_OF_STRINGS = vvwiInkNumberPrefix + 1;

        public static final int vvwiNumericStart = 0x1000;
        public static final int vvwiTimeCode = vvwiNumericStart + 1;
        public static final int vvwiUserBits = vvwiTimeCode + 1;
        public static final int vvwiVITCTimeCode = vvwiUserBits + 1;
        public static final int vvwiVITCUserBits = vvwiVITCTimeCode + 1;
        public static final int vvwiVITCLine3 = vvwiVITCUserBits + 1;
        public static final int vvwiPosterFrame = vvwiVITCLine3 + 1;
        public static final int vvwiAFrame = vvwiPosterFrame + 1;
        public static final int vvwiAspectRatio = vvwiAFrame + 1;
        public static final int vvwiOriginalRate = vvwiAspectRatio + 1;
        public static final int vvwiOriginalScale = vvwiOriginalRate + 1;
        public static final int vvwiConversions = vvwiOriginalScale + 1;
        public static final int vvwiVersionNumber = vvwiConversions + 1;
        public static final int vvwiFileSize = vvwiVersionNumber + 1;
        public static final int vvwiFileDate = vvwiFileSize + 1;
        public static final int vvwiFileTime = vvwiFileDate + 1;
        public static final int vvwiSequenceNumber = vvwiFileTime + 1;
        public static final int vvwiTotalStreams = vvwiSequenceNumber + 1;
        public static final int vvwiTotalLength = vvwiTotalStreams + 1;
        public static final int vvwiFilmManufacturerCode = vvwiTotalLength + 1;
        public static final int vvwiFilmTypeCode = vvwiFilmManufacturerCode + 1;
        public static final int vvwiWhitePoint = vvwiFilmTypeCode + 1;
        public static final int vvwiBlackPoint = vvwiWhitePoint + 1;
        public static final int vvwiBlackGain = vvwiBlackPoint + 1;
        public static final int vvwiBreakPoint = vvwiBlackGain + 1;
        public static final int vvwiGamma1000 = vvwiBreakPoint + 1;
        public static final int vvwiTagNumber = vvwiGamma1000 + 1;
        public static final int vvwiFlags = vvwiTagNumber + 1;
        public static final int vvwiTimeCodeType = vvwiFlags + 1;
        public static final int vvwiLTCTimeCodeType = vvwiTimeCodeType + 1;
        public static final int vvwiVITCTimeCodeType = vvwiLTCTimeCodeType + 1;

        public static final int vvwiEND_OF_DWORD_V2 = vvwiVITCTimeCodeType + 1;

        // VVVID STRUCT
        public static final int vvwiVideoWidth = 0x10000;
        public static final int vvwiVideoHeight = vvwiVideoWidth + 1;
        public static final int vvwiVideoPlanes = vvwiVideoHeight + 1;
        public static final int vvwiVideoBitCount = vvwiVideoPlanes + 1;
        public static final int vvwiVideoCompression = vvwiVideoBitCount + 1;
        public static final int vvwiVideoSizeImage = vvwiVideoCompression + 1;
        public static final int vvwiVideoXPelsPerMeter = vvwiVideoSizeImage + 1;
        public static final int vvwiVideoYPelsPeMeter = vvwiVideoXPelsPerMeter + 1;
        public static final int vvwiVideoClrUsed = vvwiVideoYPelsPeMeter + 1;
        public static final int vvwiVideoClrImportant = vvwiVideoClrUsed + 1;
        public static final int vvwiVideoReserved = vvwiVideoClrImportant + 1;
        public static final int vvwiVideoFccType = vvwiVideoReserved + 1;
        public static final int vvwiVideoFccHandler = vvwiVideoFccType + 1;
        public static final int vvwiVideoFlags = vvwiVideoFccHandler + 1;
        public static final int vvwiVideoCaps = vvwiVideoFlags + 1;
        public static final int vvwiVideoPriority = vvwiVideoCaps + 1;
        public static final int vvwiVideoLanguage = vvwiVideoPriority + 1;
        public static final int vvwiVideoScale = vvwiVideoLanguage + 1;
        public static final int vvwiVideoRate = vvwiVideoScale + 1;
        public static final int vvwiVideoStart = vvwiVideoRate + 1;
        public static final int vvwiVideoLength = vvwiVideoStart + 1;
        public static final int vvwiVideoInitialFrames = vvwiVideoLength + 1;
        public static final int vvwiVideoSuggestedBufferSize = vvwiVideoInitialFrames + 1;
        public static final int vvwiVideoQuality = vvwiVideoSuggestedBufferSize + 1;
        public static final int vvwiVideoSampleSize = vvwiVideoQuality + 1;
        public static final int vvwiVideoEditCount = vvwiVideoSampleSize + 1;
        public static final int vvwiVideoFormatChangeCount = vvwiVideoEditCount + 1;
        public static final int vvwiVideoPitch = vvwiVideoFormatChangeCount + 1;
        public static final int vvwiVideoDrFlags = vvwiVideoPitch + 1;
        public static final int vvwiVideoFileType = vvwiVideoDrFlags + 1;
        public static final int vvwiVideoResDrastic = vvwiVideoFileType + 1;
        public static final int vvwiAudioType = vvwiVideoResDrastic + 1;
        public static final int vvwiAudioChannels = vvwiAudioType + 1;
        public static final int vvwiAudioFrequency = vvwiAudioChannels + 1;
        public static final int vvwiAudioBits = vvwiAudioFrequency + 1;

        // Add elements here

        public static final int vvwiLastElementPlus1 = vvwiVideoResDrastic + 1;
        // DO NOT ADD ANYTHING BELOW vvwiLastElementPlus1
    }

    /**
     * String array for metadata tag names.
     *
     * NOTE:  There is no way around this currently
     * Field is a mutable array
     * A final static field references an array and can be accessed by malicious code or by accident from another package. This code can freely modify the contents of the array.
     */
    public static final String[] szStringMetaDataTagName =
        {
        "FileName", //"FileName"		// # 1
        "NativeLocator", //"NativeLocator"
        "UniversalName", //"UniversalLocator"
        "IP", //"TCP-IPAddress"
        "SourceLocator", //"SourceLocator"
        "Channel", //"ChannelIdentifier"
        "ChannelName", //"ChannelName"
        "ChannelDescription", //"ChannelDescription"
        "Title", //"Title"
        "Subject", //"Subject"	// # 10
        "Category", //"Category"
        "Keywords", //"Keywords"
        "Ratings", //"Ratings"
        "Comments", //"Comments"
        // "DoNotUse", //"SequenceNumber"
        "Owner", //"Owner"
        "Editor", //"Editor"
        "Supplier", //"Supplier"
        "Source", //"Source"
        "Project", //"Project"	// # 20
        "Status", //"Status"
        "Author", //"Author"
        "RevisionNumber", //"RevisionNumber"
        "Produced", //"Produced"
        "Album", //"Album"
        "Artist", //"Artist"
        "Composer", //"Composer"
        "Copyright", //"Copyright"
        "CreationData", //"CreationData"
        "Description", //"Description"	// # 30
        "Director", //"Director"
        "Disclaimer", //"Disclaimer"
        "EncodedBy", //"EncodedBy"
        "FullName", //"FullName"
        "Genre", //"Genre"
        "HostComputer", //"HostComputer"
        "Information", //"Information"
        "Make", //"Make"
        "Model", //"Model"
        "OriginalArtist", //"OriginalArtist"	// # 40
        "OriginalFormat", //"OriginalFormat"
        "Performers", //"Performers"
        "Producer", //"Producer"
        "Product", //"Product"
        "Software", //"Software"
        "SpecialPlaybackRequirements", //"SpecialPlaybackRequirements"
        "Track", //"Track"
        "Warning", //"Warning"
        "URLLink", //"URL"
        "EditData1", //"EditData1"		// # 50
        "EditData2", //"EditData2"
        "EditData3", //"EditData3"
        "EditData4", //"EditData4"
        "EditData5", //"EditData5"
        "EditData6", //"EditData6"
        "EditData7", //"EditData7"
        "EditData8", //"EditData8"
        "EditData9", //"EditData9"
        "VersionString", //"VersionString"
        "Manufacturer", //"Manufacturer"	// # 60
        "Language", //"Language"
        "Format", //"Format"
        "InputDevice", //"InputDevice"
        "DeviceModelNum", //"DeviceModelNum"
        "DeviceSerialNum", //"DeviceSerialNum"
        "Reel", //"Reel"
        "Shot", //"Shot"
        "Take", //"Take"
        "SlateInfo", //"SlateInfo"
        "FrameAttribute", //"FrameAttribute"	// # 70
        "Episode", //"Episode"
        "Scene", //"Scene"
        "DailyRoll", //"DailyRoll"
        "CamRoll", //"CamRoll"
        "SoundRoll", //"SoundRoll"
        "LabRoll", //"LabRoll"
        "KeyNumberPrefix", //"KeyNumberPrefix"
        "InkNumberPrefix", //"InkNumberPrefix"
        "NU_79", //"NotUsed_79"
        "NU_80", //"NotUsed_80"	// # 80
        "NU_81", //"NotUsed_81"
        "NU_82", //"NotUsed_82"
        "NU_83", //"NotUsed_83"
        "NU_84", //"NotUsed_84"
        "NU_85", //"NotUsed_85"
        "NU_86", //"NotUsed_86"
        "NU_87", //"NotUsed_87"
        "NU_88", //"NotUsed_88"
        "NU_89", //"NotUsed_89"
        "NU_90", //"NotUsed_90"	// # 90
        "NU_91", //"NotUsed_91"
        "NU_92", //"NotUsed_92"
        "NU_93", //"NotUsed_93"
        "NU_94", //"NotUsed_94"
        "NU_95", //"NotUsed_95"
        "NU_96", //"NotUsed_96"
        "NU_97", //"NotUsed_97"
        "NU_98", //"NotUsed_98"
        "NU_99", //"NotUsed_99"	// # 99
    };

    /**
     * String array for DWORD metadata tag names.
     *
     * NOTE:  There is no way around this currently
     * Field is a mutable array
     * A final static field references an array and can be accessed by malicious code or by accident from another package. This code can freely modify the contents of the array.
     */
    public static final String[] szDwordMetaDataTagName =
        {
        "TimeCode", //"TimeCode"
        "UserBits", //"UserBits"
        "VITCTimeCode", //"VITCTimeCode"
        "VITCUserBits", //"VITCUserBits"
        "VITCLine3", //"VITCExtraData"
        "PosterFrame", //"PosterFrame"
        "AFrame", //"A-Frame"
        "AspectRatio", //"AspectRatio"
        "OriginalRate", //"OriginalRate"
        "OriginalScale", //"OriginalScale"
        "Conversions", //"TotalConversions"
        "VersionNumber", //"VersionNumber"
        "FileSize", //"FileSize"
        "FileDate", //"FileDate"
        "FileTime", //"FileTime"
        "SequenceNumber", //"SequenceNumber"
        "TotalStreams", //"TotalStreams"
        "TotalLength", //"TotalLength"
        "FilmManufacturerCode", //"FilmManufacturerCode"
        "FilmTypeCode", //"FilmTypeCode"
        "WhitePoint", //"WhitePoint"
        "BlackPoint", //"BlackPoint"
        "BlackGain", //"BlackGain"
        "BreakPoint", //"BreakPoint"
        "Gamma1000", //"Gamma1000"
        "TagNumber", //"TagNumber"
        "Flags", //"Flags"
        "TimeCodeType",	//"NumericNotUsed_28"
        "LtcTimeCodeType",	//"NumericNotUsed_29"
        "VitcTimeCodeType", //"NumericNotUsed_30"
        // "NU_28", //"NumericNotUsed_28"
        // "NU_29", //"NumericNotUsed_29"
        // "NU_30", //"NumericNotUsed_30"
        "NU_31", //"NumericNotUsed_31"
        "NU_32", //"NumericNotUsed_32"
        "NU_33", //"NumericNotUsed_33"
        "NU_34", //"NumericNotUsed_34"
        "NU_35", //"NumericNotUsed_35"
        "NU_36", //"NumericNotUsed_36"
        "NU_37", //"NumericNotUsed_37"
        "NU_38", //"NumericNotUsed_38"
        "NU_39", //"NumericNotUsed_39"
        "NU_40", //"NumericNotUsed_40"
        "NU_41", //"NumericNotUsed_41"
        "NU_42", //"NumericNotUsed_42"
        "NU_43", //"NumericNotUsed_43"
        "NU_44", //"NumericNotUsed_44"
        "NU_45", //"NumericNotUsed_45"
        "NU_46", //"NumericNotUsed_46"
        "NU_47", //"NumericNotUsed_47"
        "NU_48", //"NumericNotUsed_48"
        "NU_49" //"NumericNotUsed_49"
    };
    //_VVW_TYPES_DEFINE_META_DATA_TAG_NAME_ARRAY_DRASTIC

    // ***********************************************************


    /**
     * Copies an instance of MEDIACMD to the byteDataBuffer array.
     *
     * @param mCmdSrc MEDIACMD
     */
    public void CopyMCMD(MEDIACMD mCmdSrc)
    {
        if (mCmdSrc.StructureSize() < CMD_ABSMAX_ARRAY_LEN)
        {
            for (int i = 0; i < mCmdSrc.StructureSize(); i++)
            {
                byteDataBuffer[i] = mCmdSrc.byteDataBuffer[i];
            }
        }
        else
        {
            for (int i = 0; i < CMD_ABSMAX_ARRAY_LEN; i++)
            {
                byteDataBuffer[i] = mCmdSrc.byteDataBuffer[i];
            }
        }
    }

    /**
     * Set an element of the mediacmd structure for network transport to the vvw
     *
     * @param nDataElem int
     * @param nValueToSet long
     * @return long
     */
    protected long SetDataElem(int nDataElem, long nValueToSet)
    {
        for(int i = 0; i < 4; i++)
        {
            byteDataBuffer[nDataElem + i] = (byte) ((nValueToSet >> (8 * i)) & 0x000000FF);
        }
        return nValueToSet;
    }

    /**
     * Get an element from the current mediacmd or a return from the vvw
     *
     * @param nDataElem int
     * @return long
     */
    protected long GetDataElem(int nDataElem)
    {
        long dwValue = 0, dwTmp;
        for(int i = 0; i < 4; i++)
        {
            dwTmp = (long) byteDataBuffer[nDataElem + i] & 0xff; //(int) (byteDataBuffer[nDataElem + i]) & 0x000000FF;
            dwValue |= (dwTmp << (8 * i));
        }
        return dwValue;
    }

    /**
     * Get an element from the current mediacmd or a return from the vvw as a 'Signed' value
     *
     * @param nDataElem int
     * @return long
     */
    protected long GetDataElemSigned(int nDataElem)
    {
        int dwValue = 0, dwTmp;
        for(int i = 0; i < 4; i++)
        {
            dwTmp = (int) (byteDataBuffer[nDataElem + i]) & 0x000000FF;
            dwValue |= (dwTmp << (8 * i));
        }
        return dwValue;
    }

    /**
     * Clean out a mediacmd to a known state
     */
    public void Init()
    {
        CmdID(MEDIACMD_CURRENT);
        StructureSize(MESSAGE_FULL_SIZE); //MEDIACMD_BASE_SIZE);
        Channel(CHAN_ILLEGAL);
        Speed(SPD_ILLEGAL);
        VideoChannels(vidChanAll);
        AudioChannels(audChanAll);
        InfoChannels(infChanAll);
        CmdAlt(0); //Used to be TC_ILLEGAL
        Position(TC_ILLEGAL);
        Start(TC_ILLEGAL);
        End(TC_ILLEGAL);
        Cmd(ctStop);
        Flags(0);
        for (int i = 0; i < 11; i++)
        {
            byteDataBuffer[offarbyteID + i] = 0;
        }
    }

    /**
     * Called when there is a failure.  Puts the media cmd into a
     * known, failed state.
     */
    public void InitFailure()
    {
        CmdID(MEDIACMD_CURRENT);
        StructureSize(MESSAGE_FULL_SIZE); //MEDIACMD_BASE_SIZE);
        Cmd(-1);
        Flags(0);
        Speed(SPD_ILLEGAL);
        VideoChannels(0);
        AudioChannels(0);
        InfoChannels(0);
        CmdAlt(0); //Used to be TC_ILLEGAL
        Position(TC_ILLEGAL);
        Start(TC_ILLEGAL);
        End(TC_ILLEGAL);
    }

    //=======================================================
    // Main Interface Functions
    //=======================================================

    //=======================================================

    /**
     * Get the netTime element
     *
     * @return long returns the network time (recieve)
     */
    public long netTime()
    {
        return GetDataElem(offnetTime);
    }

    /**
     * Set the netTime element
     *
     * @param netTime long set the network time (send)
     */
    public void netTime(long netTime)
    {
        SetDataElem(offnetTime, netTime);
    }

    /**
     * Get the command ID element
     *
     * @return long
     */
    public long CmdID()
    {
        return GetDataElem(offCmdID);
    }

    /**
     * Set the command ID element
     *
     * @param cmdID long
     */
    public void CmdID(long cmdID)
    {
        SetDataElem(offCmdID, cmdID);
    }

    /**
     * Get the StructSize element
     *
     * @return long
     */
    public long StructureSize()
    {
        return GetDataElem(offStructureSize);
    }

    /**
     * Set the StructSize element
     *
     * @param StructSize long
     */
    public void StructureSize(long StructSize)
    {
        SetDataElem(offStructureSize, StructSize);
    }

    /**
     * Get the Channel element
     *
     * @return long
     */
    public long Channel()
    {
        return GetDataElem(offChannel);
    }

    /**
     * Set the Channel element
     *
     * @param Channel long
     */
    public void Channel(long Channel)
    {
        SetDataElem(offChannel, Channel);
    }

    /**
     * Get the command element
     *
     * @return long
     */
    public long Cmd()
    {
        return GetDataElem(offCmd);
    }

    /**
     * Set the command element
     *
     * @param cmd long
     */
    public void Cmd(long cmd)
    {
        SetDataElem(offCmd, cmd);
    }

    /**
     * Get the Flags element
     *
     * @return long
     */
    public long Flags()
    {
        return GetDataElem(offFlags);
    }

    /**
     * Set the Flags element
     *
     * @param flags long
     */
    public void Flags(long flags)
    {
        SetDataElem(offFlags, flags);
    }

    /**
     * Get the speed element
     *
     * @return long
     */
    public long Speed()
    {
        return GetDataElemSigned(offSpeed);
    }

    /**
     * Set the speed element
     *
     * @param speed long
     */
    public void Speed(long speed)
    {
        SetDataElem(offSpeed, speed);
    }

    /**
     * Get the VideoChannels element
     *
     * @return long
     */
    public long VideoChannels()
    {
        return GetDataElem(offVideoChannels);
    }

    /**
     * Set the VideoChannels element
     *
     * @param channel long
     */
    public void VideoChannels(long channel)
    {
        SetDataElem(offVideoChannels, channel);
    }

    /**
     * Get the AudioChannels element
     *
     * @return long
     */
    public long AudioChannels()
    {
        return GetDataElem(offAudioChannels);
    }

    /**
     * Set the AudioChannels element
     *
     * @param channel long
     */
    public void AudioChannels(long channel)
    {
        SetDataElem(offAudioChannels, channel);
    }

    /**
     * Get the InfoChannels element
     *
     * @return long
     */
    public long InfoChannels()
    {
        return GetDataElem(offInfoChannels);
    }

    /**
     * Set the InfoChannels element
     *
     * @param channel long
     */
    public void InfoChannels(long channel)
    {
        SetDataElem(offInfoChannels, channel);
    }

    /**
     * Get the CmdAlt element
     *
     * @return long
     */
    public long CmdAlt()
    {
        return GetDataElem(offCmdAlt);
    }

    /**
     * Set the CmdAlt element
     *
     * @param cmdAlt long
     */
    public void CmdAlt(long cmdAlt)
    {
        SetDataElem(offCmdAlt, cmdAlt);
    }

    /**
     * Get the offpNext element
     *
     * @return long
     */
    public long GetPnext()
    {
        return GetDataElem(offpNext);
    }

    /**
     * Get the offpPev element
     *
     * @return long
     */
    public long GetPprev()
    {
        return GetDataElem(offpPrev);
    }

    /**
     * Get the Postion element
     *
     * @return long
     */
    public long Position()
    {
        return GetDataElem(offPosition);
    }

    /**
     * Set the Postion element
     *
     * @param position long
     */
    public void Position(long position)
    {
        SetDataElem(offPosition, position);
    }

    /**
     * Get the Start element
     *
     * @return long
     */
    public long Start()
    {
        return GetDataElem(offStart);
    }

    /**
     * Set the Start element
     *
     * @param start long
     */
    public void Start(long start)
    {
        SetDataElem(offStart, start);
    }

    /**
     * Get the End element
     *
     * @return long
     */
    public long End()
    {
        return GetDataElem(offEnd);
    }

    /**
     * Set the End element
     *
     * @param end long
     */
    public void End(long end)
    {
        SetDataElem(offEnd, end);
    }

    /**
     * Set the arbID element
     */
    public void arbIDClear()
    {
        for (int i = 0; i < CMD_ABSMAX_CMDID_LEN; i++)
        {
            byteDataBuffer[offarbyteID + i] = 0;
        }
    }

    /**
     * Get the arbID element
     *
     * @return byte[]
     */
    public byte[] arbID()
    {
        if((StructureSize() - MEDIACMD_BASE_SIZE) < 1)
        {
            // No arbID to return anyway
            return null;
        }
        else
        {
            int intSize = (int) StructureSize() - MEDIACMD_BASE_SIZE;
            byte[] szArbID = new byte[intSize]; // [CMD_ABSMAX_ARRAY_LEN];

            // for (int i = 0; i < CMD_ABSMAX_CMDID_LEN; i++)
            for (int i = 0; i < intSize; i++)
            {
                szArbID[i] = byteDataBuffer[offarbyteID + i];
            }
            return szArbID;
        }
    }

    /**
     * Set the arbID character/byte array member
     *
     * @param arbID byte[] new byte array to set
     */
    public void arbID(byte[] arbID)
    {
        int intSize = (int)StructureSize() - MEDIACMD_BASE_SIZE;

        for (int i = 0; i < intSize; i++) // CMD_ABSMAX_CMDID_LEN; i++)
        {
            byteDataBuffer[offarbyteID + i] = arbID[i];
        }
    }

    /**
     * Check the 'clip' area of 2 arbID arrays
     *
     * @param arbID byte[] first compare array
     * @param arbID2 byte[] second compare array
     * @return boolean true if they match
     */
    public boolean cmpArbID(byte[] arbID, byte[] arbID2)
    {
        if(arbID2[0] == arbID[0] &&
            arbID2[1] == arbID[1] &&
            arbID2[2] == arbID[2] &&
            arbID2[3] == arbID[3] &&
            arbID2[4] == arbID[4] &&
            arbID2[5] == arbID[5] &&
            arbID2[6] == arbID[6] &&
            arbID2[7] == arbID[7]) {
            return true;
        }
        return false;
    }

    /**
     * Get Get the clipname (first 8 char) or full string name as a sting from
     * the character array
     *
     * @return String string from mediacmd return
     */
    public String getArbName()
    {
        int nEnd = 0;

        for (nEnd = 0; nEnd < CMD_ABSMAX_ARRAY_LEN; nEnd++)
        {
            if (byteDataBuffer[offarbyteID + nEnd] == 0)
            {
                break;
            }
        }
        return getArbName(0, nEnd);
    }

    /**
     * Get a string from a specific part of the MediaCmd arbID return
     *
     * @param lFrom int first character to get
     * @param lTo int last character
     * @return String String from that area of the arbID return
     */
    public String getArbName(int lFrom, int lTo)
    {
        if(arbID() != null)
        {
            String name = new String(arbID());
            name = name.substring(lFrom, lTo);
            return name;
        }
        else
        {
            return "";
        }
    }

    /**
     * Set a string into the arbID character array
     *
     * @param name String new string for the cmd array
     * @param lFrom int starting location to place it at
     * @param lTo int end location
     */
    public void setArbName(String name, int lFrom, int lTo)
    {
        int i = 0;
        if (name != null)
        {
            if (name.length() - 1 >= lTo - lFrom)
            {
                for (i = 0; i < name.length() && i < lTo; i++)
                {
                    byteDataBuffer[offarbyteID + i + lFrom] = (byte) name.charAt(i);
                }
                byteDataBuffer[offarbyteID + i + lFrom] = 0;
            }
            else
            {
                for (i = 0; i < name.length(); i++)
                {
                    byteDataBuffer[offarbyteID + i + lFrom] = (byte) name.charAt(i);
                }
                for (i = name.length(); i < lTo; i++)
                {
                    byteDataBuffer[offarbyteID + i + lFrom] = 0;
                }
                byteDataBuffer[offarbyteID + i + lFrom] = 0;
            }
        }
        else
        {
            for (i = lFrom; i < lTo; i++)
            {
                byteDataBuffer[offarbyteID + i] = 0;
            }
        }
    }

    /**
     * Populate Arb from a Byte array specifying From & To.
     *
     * @param name byte[] array of bytes to set
     * @param lFrom int starting location
     * @param lTo int end
     */
    public void setArbFromBytes(byte[] name, int lFrom, int lTo)
   {
       if(byteDataBuffer.length < (name.length + offarbyteID))
       {
           // Increase the Data Buffer to accomodate the large JPEG file
           byteDataBuffer = new byte[name.length + offarbyteID + 2];
           StructureSize(name.length + offarbyteID + 2);
       }

       int i = 0;
       // name cannot be null, but can be empty
       if (name.length == 0)
       {
           if (name.length - 1 >= lTo - lFrom)
           {
               for (i = 0; i < name.length && i < lTo; i++)
               {
                   byteDataBuffer[offarbyteID + i + lFrom] = (byte) name[i];
               }
               byteDataBuffer[offarbyteID + i + lFrom] = 0;
           }
           else
           {
               for (i = 0; i < name.length; i++)
               {
                   byteDataBuffer[offarbyteID + i + lFrom] = (byte) name[i];
               }
               for (i = name.length; i < lTo; i++)
               {
                   byteDataBuffer[offarbyteID + i + lFrom] = 0;
               }
               byteDataBuffer[offarbyteID + i + lFrom] = 0;
           }
       }
       else
       {
           for (i = lFrom; i < lTo; i++)
           {
               byteDataBuffer[offarbyteID + i] = 0;
           }
       }
   }

    /**
     * Takes a tcSource value and returns a string representation of that value.
     *
     * @param dwTcSource long GS_TCSOURCE_xxx
     * @return String Short string value
     */
    public String getTCSource(long dwTcSource)
    {
        switch ( (int) dwTcSource)
        {
            case GS_TCSOURCE_LTC:
                return ("LTC");
            case GS_TCSOURCE_VITC:
                return ("VITC");
            case GS_TCSOURCE_CTL:
                return ("CTL");
            case GS_TCSOURCE_TC:
                return ("TC");
            default:
                return ("ERR");
        }
    }

    /**
     * Copy the 8 character clip name from the supplied arbID to a string
     *
     * @param arbID byte[] character array with the clip name return
     * @return String clip name
     */
    public String copyClipName(byte[] arbID)
    {
        char[] temp = new char[9];
        for (int i = 0; i < 9; i++)
        {
            temp[i] = (char) arbID[i];
        }
        String clipName = new String(temp);
        return (clipName);
    }

    // for debugging only
    public void printDebug(MEDIACMD mCmdSrc)
    {
        System.out.print("DEBUG ");

        System.out.print("  Position: ");
        System.out.print(mCmdSrc.Position());

        System.out.print("  arbID: ");
        System.out.print(mCmdSrc.arbID());

        System.out.print("  Channel: ");
        System.out.print(mCmdSrc.Channel());

        System.out.print("  Info_Channels: ");
        System.out.print(mCmdSrc.InfoChannels());

        System.out.print("  Speed: ");
        System.out.print(mCmdSrc.Speed());

        System.out.print("  Start: ");
        System.out.print(mCmdSrc.Start());

        System.out.print("  End: ");
        System.out.print(mCmdSrc.End());

        System.out.print("  Structure_Size: ");
        System.out.print(mCmdSrc.StructureSize());

        System.out.print("  Video_Channels: ");
        System.out.print(mCmdSrc.VideoChannels());

        System.out.print("  Net_Time: ");
        System.out.print(mCmdSrc.netTime());

        System.out.print("  Audio_Channels: ");
        System.out.print(mCmdSrc.AudioChannels());

        System.out.print("  Cmd: ");
        System.out.print(mCmdSrc.Cmd());

        System.out.print("  Cmd_Alt: ");
        System.out.print(mCmdSrc.CmdAlt());

        System.out.print("  Cmd_ID: ");
        System.out.print(mCmdSrc.CmdID());

        System.out.print("  Flags: ");
        System.out.print(mCmdSrc.Flags());

        System.out.println(" ...End");
    }

    // for debugging only
    public void debugOutCmd(MEDIACMD mCmd)
    {
        // Pre-remove really common commands
        switch((int)mCmd.Cmd()) {
        case ctGetState:
            System.out.print("g");
            return;
        case ctGetValue:
        case ctValueSupported:
            switch((int)mCmd.CmdAlt()) {
            case gsTc:
                System.out.print("gsTc ");
                return;
            case gsUb:
                System.out.print("gsUb ");
                return;
            case gsLtcTc:
                System.out.print("gsLtcTc ");
                return;
            case gsLtcUb:
                System.out.print("gsLtcUb ");
                return;
            case gsVitcTc:
                System.out.print("gsVitcTc ");
                return;
            case gsVitcUb:
                System.out.print("gsVitcUb ");
                return;
            case gsCurrentMs:
                System.out.print("ms ");
                return;
            case gsSysBufferLevel:
                System.out.print("buf ");
                return;
            case gsSysMemoryUsage:
                System.out.print("mem ");
                return;
            case gsSysCPUUsage:
                System.out.print("cpu ");
                return;
            default:
                break;
            }
            break;
        default:
            break;
        }
        System.out.print("\nMCMD");
        // To dump struct sizes
//        System.out.print(mCmd.StructureSize());
        // To dump ms time
        System.out.print(System.currentTimeMillis());
        System.out.print(":");
        switch((int)mCmd.Cmd()) {
        case ctStop:// = 0;
            System.out.print("Stop           ");
            break;
        case ctPause:// = 1;
            System.out.print("Pause          ");
            break;
        case ctPlay:// = 2;
            System.out.print("Play           ");
            break;
        case ctRecord:// = 3;
            System.out.print("Record         ");
            break;
        case ctRecStop:// = 4;
            System.out.print("RecStop        ");
            break;
        case ctEject:// = 5;
            System.out.print("Eject          ");
            break;
        case ctTransfer:// = 6;
            System.out.print("Transfer       ");
            break;
        case ctInsert:// = 7;
            System.out.print("Insert         ");
            break;
        case ctBlank:// = 8;
            System.out.print("Blank          ");
            break;
        case ctDelete:// = 9;
            System.out.print("Delete         ");
            break;
        case ctTrim:// = 10;
            System.out.print("Trim           ");
            break;
        case ctChanSelect:// = 11;
            System.out.print("ChanSelect     ");
            break;
        case ctGetState:// = 12;
            // return;
            System.out.print("GetState       ");
            return;// Meaningless struct until filled

        case ctSetState:// = 13;
            System.out.print("SetState       ");
            break;
        case ctGetValue:// = 14;
            System.out.print("GetValue       ");
            switch((int)mCmd.CmdAlt()) {
            case gsTc:
                System.out.print("gsTc");
                break;
            case gsUb:
                System.out.print("gsUb");
                break;
            case gsLtcTc:
                System.out.print("gsLtcTc");
                break;
            case gsLtcUb:
                System.out.print("gsLtcUb");
                break;
            case gsVitcTc:
                System.out.print("gsVitcTc");
                break;
            case gsVitcUb:
                System.out.print("gsVitcUb");
                break;
            case gsTcSource:
                System.out.print("gsTcSource");
                break;
            case gsTcType:
                System.out.print("gsTcType");
                break;
            case gsStart:
                System.out.print("gsStart");
               break;
           case gsEnd:
               System.out.print("gsEnd");
               break;
            case gsIn:
                System.out.print("gsIn");
                break;
            case gsLastIn:
                System.out.print("gsLastIn");
                break;
            case gsOut:
                System.out.print("gsOut");
                break;
            case gsLastOut:
                System.out.print("gsLastOut");
                break;
            case gsGetNextClip:
                System.out.print("gsGetNextClip");
                break;
            case gsFirstClip:
                System.out.print("gsFirstClip");
                break;
            case gsNextClip:
                System.out.print("gsNextClip");
                break;
            case gsTCSGetTLClipState:
                System.out.print("gsTCSGetTLClipState");
                break;
            case gsTCSGetTLClipInfo:
                System.out.print("gsTCSGetTLClipInfo");
                break;
            case gsClipInfo:
                System.out.print("gsClipInfo");
                break;
            case gsClipCopy:
                System.out.print("gsClipCopy");
                break;


            case gsTotalStorageAvail:
                System.out.print("gsTotalStorageAvail");
                break;
            case gsTotalStorageFree:
                System.out.print("gsTotalStorageFree");
                break;
            case gsTotalTimeAvail:
                System.out.print("gsTotalTimeAvail");
                break;
            case gsTotalTimeFree:
                System.out.print("gsTotalTimeFree");
                break;

            case gsMetaData:
                System.out.print("gsMetaData");
                break;

            case gsVidInSignalFormat:
                System.out.print("gsVidInSignalFormat");
                break;
            case gsVidOutLockSignalFormat:
                System.out.print("gsVidOutLockSignalFormat");
                break;
            case gsSignalFormat:
                System.out.print("gsSignalFormat");
                break;
            case gsAudInputStatus:
                System.out.print("gsAudInputStatus");
                break;

            case gsSysTime:
                System.out.print("gsSysTime");
                break;
            case gsDSyncMs:
                System.out.print("gsDSyncMs");
                break;

            case gsVVWVersion:
                System.out.print("gsVVWVersion");
                break;
            case gsMEVersion:
                System.out.print("gsMEVersion");
                break;
            case gsVVWType:
                System.out.print("gsVVWType");
                break;
            case gsVVWChannelType:
                System.out.print("gsVVWChannelType");
                break;
            case gsVVWChannelName:
                System.out.print("gsVVWChannelName");
                break;

            case gsPiconFrame:
                System.out.print("gsPiconFrame");
                break;
            case gsChannelsExist:
                System.out.print("gsChannelsExist");
                break;
            case gsClipMode:
                System.out.print("gsClipMode");
                break;
            case gsRecOffset:
                System.out.print("gsRecOffset");
                break;
            case gsChanCapabilities:
                System.out.print("gsChanCapabilities");
                break;
            case gsLastChangeMs:
                System.out.print("gsLastChangeMs");
                break;
            case gsCurrentMs:
                System.out.print("gsCurrentMs");
                break;
            case gsErrorLog:
                System.out.print("gsErrorLog");
                break;
            default:
                System.out.print(mCmd.CmdAlt());
                break;
            }
            return;

        case ctSetValue:// = 15;
            System.out.print("SetValue       ");
            break;
        case ctValueSupported:// = 16;
            System.out.print("ValueSupported ");
            break;
        case ctError:// = 17;
            System.out.print("Error          ");
            break;
        case ctTerminate:// = 18;
            System.out.print("Terminate      ");
            break;
        case ctAbort:// = 19;
            System.out.print("Abort          ");
            break;
        default:
            System.out.print("UNKNOWNCMD(" + mCmd.Cmd() + ") ");
            break;
        }
        if((mCmd.Flags() & cfUseSpeed) != 0)
        {
            System.out.print(",Speed=" + mCmd.Speed());
        }
        if((mCmd.Flags() & cfUsePositionOffset) != 0)
        {
            System.out.print(",PosOff=" + mCmd.Position());
        }
        else if((mCmd.Flags() & cfUsePosition) != 0)
        {
            System.out.print(",Pos=" + mCmd.Position());
        }
        if((mCmd.Flags() & cfUseStartOffset) != 0)
        {
            System.out.print(",StartOff=" + mCmd.Start());
        }
        else if((mCmd.Flags() & cfUseStart) != 0)
        {
            System.out.print(",Start=" + mCmd.Start());
        }
        if((mCmd.Flags() & cfUseEndOffset) != 0)
        {
            System.out.print(",EndOff=" + mCmd.End());
        }
        else if((mCmd.Flags() & cfUseEnd) != 0)
        {
            System.out.print(",End=" + mCmd.End());
        }
        if((mCmd.Flags() & cfUsePresets) != 0)
        {
            System.out.print(",Preset=V" + mCmd.VideoChannels() + "A" + mCmd.AudioChannels() + "I" + mCmd.InfoChannels());
        }
        if((mCmd.Flags() & cfUseCmdAlt) != 0)
        {
            if((mCmd.Flags() & cfTimeMs) != 0)
            {
                if((mCmd.Flags() & cfTimeTarget) != 0)
                {
                    System.out.print(",TimeTargetMs=" + mCmd.CmdAlt());
                }
                else if((mCmd.Flags() & cfTimeHouseClock) != 0)
                {
                    System.out.print(",TimeHouseMs=" + mCmd.CmdAlt());
                }
                else
                {
                    System.out.print(",TimeMs=" + mCmd.CmdAlt());
                }
            }
            else
            {
                // Get set value?
                System.out.print(",CmdAlt=" + mCmd.CmdAlt());
            }
        }
        if((mCmd.Flags() & cfUseClipID) != 0)
        {
            System.out.print(",Clip=" + mCmd.CmdID());
        }
    }

    // ***
    /**
     * Standard video descriptor class - from C
     */
    public class VVWVIDEO
    {
        /**
         * Size of the BITMAPINFOHEADER portion of this structure (sizeof(BITMAPINFOHEADER) + any VVWVIDEO::dwReserved used)
         * Size Of BITMAPINO + Used dwReserved
         */
        long biSize;
        /**
         * Width of the video frame
         * Width of bitmap
         */
        long biWidth;
        /**
         * Height of the video frame. CAUTION: For vertically inverted frames this WILL be negative.
         * Height of bitmap (ALWAYS Positive in VVW/MR)
         */
        long biHeight;
        /**
         * Number of RGB groups (like photoshop layers) - Always 1 for our purposes.
         * Number of bitmaps (ALWAYS 1)
         */
        short biPlanes;
        /**
         * Number of bits per pixel (eg. YUV422=16, RGB=24, RGBA=32).
         * Bitcount (24 RGB, 16 YUV) (Alpha seperate).
         */
        short biBitCount;
        /**
         * Compression - a fourcc usually, but not always equal to fccHandler. Denotes compression type of frame - see fccDef.h
         * (Should be same as fccHandler)
         */
        long biCompression;
        /**
         * Size of the image. For uncompressed biWidth*abs(biHeight)*(biBitCount/8) in bytes. For compressed, variable.
         * Size of bitmap (biPitch * biHeight) + Alpha
         */
        long biSizeImage;
        /**
         * Horizontal picture elements per meter - normally 0.
         * (Picture elements per meter)
         */
        long biXPelsPerMeter;
        /**
         * Vertical picture elements per meter - normally 0
         * (Picture elements per meter)
         */
        long biYPelsPerMeter;
        /**
         * For colour tables in dwReserved, the number of RGBQUAD elements used.
         * (Number of pallette entries used in bitmap)
         */
        long biClrUsed;
        /**
         * For colour tables in dwReserved, the number of RGBQUAD elements that are critical to display (for windows pallette wars in < 256
         * colourmode).
         * Required entries to display bitmap.
         */
        long biClrImportant;
        /**
         * The dwReserved may hold many things.  Whatever it holds, the amount used can be determined by
         * subracting sizeof(BITMAPINFOHEADER) from VVWVIDEO::biSize.  Here are some possible uses
         *
          \code
                Table of struct RGBQUAD { BYTE rgbBlue, rgbGreen, rgbRed, rgbReserved; };
          \endcode
          \code
                typedef struct tagJPEGINFOHEADER {
                        // compression-specific fields
                        // these fields are defined for 'JPEG' and 'MJPG'
                        DWORD       JPEGSize;
                        DWORD       JPEGProcess;

                        // Process specific fields
                        DWORD       JPEGColorSpaceID;
                        DWORD       JPEGBitsPerSample;
                        DWORD       JPEGHSubSampling;
                        DWORD       JPEGVSubSampling;
                } JPEGINFOHEADER;
          \endcode
          \code
                typedef struct tagVIDEOINFOHEADER {

                        RECT            rcSource;          // The bit we really want to use
                        RECT            rcTarget;          // Where the video should go
                        DWORD           dwBitRate;         // Approximate bit data rate
                        DWORD           dwBitErrorRate;    // Bit error rate for this stream
                        REFERENCE_TIME  AvgTimePerFrame;   // Average time per frame (100ns units)

                        BITMAPINFOHEADER bmiHeader;

                } VIDEOINFOHEADER;
          \endcode
          \code
                typedef struct tagMPEG1VIDEOINFO {

                        VIDEOINFOHEADER hdr;                    // Compatible with VIDEOINFO
                        DWORD           dwStartTimeCode;        // 25-bit Group of pictures time code
                                                                                                        // at start of data
                        DWORD           cbSequenceHeader;       // Length in bytes of bSequenceHeader
                        BYTE            bSequenceHeader[1];     // Sequence header including
                                                                                                        // quantization matrices if any
                } MPEG1VIDEOINFO;
          \endcode
          \code
                typedef struct tagAnalogVideoInfo {
                        RECT            rcSource;           // Width max is 720, height varies w/ TransmissionStd
                        RECT            rcTarget;           // Where the video should go
                        DWORD           dwActiveWidth;      // Always 720 (CCIR-601 active samples per line)
                        DWORD           dwActiveHeight;     // 483 for NTSC, 575 for PAL/SECAM
                        REFERENCE_TIME  AvgTimePerFrame;    // Normal ActiveMovie units (100 nS)
                } ANALOGVIDEOINFO;
          \endcode
          \code
                typedef struct tagMPEG2VIDEOINFO {
                        VIDEOINFOHEADER2    hdr;
                        DWORD               dwStartTimeCode;        //  ?? not used for DVD ??
                        DWORD               cbSequenceHeader;       // is 0 for DVD (no sequence header)
                        DWORD               dwProfile;              // use enum MPEG2Profile
                        DWORD               dwLevel;                // use enum MPEG2Level
         DWORD               dwFlags;                // use AMMPEG2_* defines.  Reject connection if undefined bits are not 0
                        DWORD               dwSequenceHeader[1];    // DWORD instead of Byte for alignment purposes
                                                                                                                //   For MPEG-2, if a sequence_header is included, the sequence_extension
                                                                                                                //   should also be included
                } MPEG2VIDEOINFO;
          \endcode
          \code
                typedef struct tagVIDEOINFOHEADER2 {
                        RECT                rcSource;
                        RECT                rcTarget;
                        DWORD               dwBitRate;
                        DWORD               dwBitErrorRate;
                        REFERENCE_TIME      AvgTimePerFrame;
         DWORD               dwInterlaceFlags;   // use AMINTERLACE_* defines. Reject connection if undefined bits are not 0
         DWORD               dwCopyProtectFlags; // use AMCOPYPROTECT_* defines. Reject connection if undefined bits are not 0
         DWORD               dwPictAspectRatioX; // X dimension of picture aspect ratio, e.g. 16 for 16x9 display
         DWORD               dwPictAspectRatioY; // Y dimension of picture aspect ratio, e.g.  9 for 16x9 display
                        DWORD               dwReserved1;        // must be 0; reject connection otherwise
                        DWORD               dwReserved2;        // must be 0; reject connection otherwise
                        BITMAPINFOHEADER    bmiHeader;
                } VIDEOINFOHEADER2;
          \endcode
         */
        long[] dwReserved = new long[256]; //_VVWXXX_RESERVED_SIZE];	// Pallette or extended bitmap stucture per 98/NT/2K
        // End BITMAPINFOHEADER

        // Second part is a AVIStreamHeader (AVISTREAMINFO)
        /**
         * For VVWVIDEO structure this is always streamtypeVIDEO == 'vids'
         */
        long fccType; // streamtypeVIDEO, streamtypeAUDIO, streamtypeMIDI, streamtypeTEXT
        /**
         * Codec type, see fccDef.h  Normally the same as VVWVIDEO::biCompression but not always.
         */
        long fccHandler; // Codec - should be same as biCompression
        /*
         * AVISTREAMINFO_ flags such as
                 \code
                 AVISTREAMINFO_DISABLED		0x00000001
                 AVISTREAMINFO_FORMATCHANGES	0x00010000
                 \endcode
         */
        long dwFlags; // Pos.  AVISTREAMINFO_FORMATCHANGES or AVISTREAMINFO_DISABLED
        /**
         * Not sure. See VVWSYSTEM::dwCaps for possible interp if something is set. MS Doc:  currently unused.
         */
        long dwCaps;
        /**
         * Priority of stream (<-MSDoc in relation to other streams in the file I suppose).
         */
        short wPriority;
        /**
         * Language of stream (<-MSDoc but no language id defines)
         */
        short wLanguage;
        /**
         * dwRate/dwScale = frame rate.  See VVSYSTEM::dwScale for more info and table example.
         */
        long dwScale; //  1001  100  1  1  - dwRate / dwScale == frame rate
        /**
         * dwRate/dwScale = frame rate.  See VVSYSTEM::dwRate for more info and table example.
         */
        long dwRate; // 30000 2997 25 24
        /**
         * Delay in units per VVWVIDEO::dwRate/VVWVIDEO::dwScale (for video - frames) for this
         * stream to start in the playback of the file.  NOTE AVI v1.0 and simple avi readers
         * will choke or play incorrectly if this is not 0, so be careful.
         */
        long dwStart; // Starting sample per dwRate/dwScale
        /**
         * Length of the video stream in units per VVWVIDEO::dwRate/VVWVIDEO::dwScale (for video - frames)
         */
        long dwLength; // Length of stream per dwRate/dwScale
        /**
         * Amount of audio in the file before video commences.  For offset files, typically 0.75 sec converted
         * to units per VVWVIDEO::dwRate/VVWVIDEO::dwScale.  For high end files, always zero as audio
         * and video are sent without scew (except premiere, which uses 'rec ' chunks and audio skew)
         */
        long dwInitialFrames; // Audio skew, how much startup audio in stream
        /**
         * Recommended buffer size based on the largest single chunk in the file.  Set by
         * writer, so often incorrect or 0.
         */
        long dwSuggestedBufferSize; // Largest chunk in stream, internally one uncompressed frame
        /**
         * Quality used by the compressor.  Between 0 and 10,000 or -1 if default quality.  For some
         * compressors, the -1 can also mean the quality info is encoded into the frame or in the
         * dwReserved or other private data area.
         */
        long dwQuality; // Codec compression quality
        /**
         * Size, in bytes, of a single data sample. If the value of this member is zero, the samples
         * can vary in size and each data sample (such as a video frame) must be in a separate chunk.
         * A nonzero value indicates that multiple samples of data can be grouped into a single chunk
         * within the file. For video streams, this number is typically zero, although it can be nonzero
         * if all video frames are the same size. For audio streams, this number should be the same
         * as the nBlockAlign member of the WAVEFORMAT or WAVEFORMATEX structure describing the audio.
         */
        long dwSampleSize; // Largest single sample
        /**
         * Dimensions of the video destination rectangle. The values represent the coordinates of
         * upper left corner, the height, and the width of the rectangle.
         */
        java.awt.Rectangle/*16*/ rcFrame; // Frame dimensions
        /**
         * Number of times the stream has been edited. The stream handler maintains this count.
         */
        long dwEditCount; // Number of time stream has been edited
        /**
         * Number of times the stream format has changed. The stream handler maintains this count.
         */
        long dwFormatChangeCount; // Number of time format has been changed
        /**
         * Null-terminated string containing a description of the stream.
         */
        char[] szName = new char[64]; //_VVWXXX_NAME_SIZE];	// Stream identifier
        // End of AVIStreamHeader  (AVISTREAMINFO)

        /**
         * The number of bytes in a row of pixels.  Allows for byte/word/dword alignment of lines as nec for format
         */
        long biPitch; // Normally biWidth * (biBitCount / 8) - Number of bytes in a row

        //
        // Check mask bits for free areas
        // note: fccmod depends on biCompression/fccHandler
        /**
         * Field dominance MASK for VVWVIDEO::dwDrFlags
         */
        static final int MASK___DRFLAGS_FIELD = 0x00000001;
        /**
         * Zero (Second) Field Dominant in VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_ZERO_FIELD_DOMINANT = 0x00000000;
        /**
         * First Field Dominant in VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FIRST_FIELD_DOMINANT = 0x00000001;
        /**
         * KeyFrame MASK for VVWVIDEO::dwDrFlags
         */
        static final int MASK___DRFLAGS_KEYFRAME = 0x00000010;
        /**
         * Stream has key frames, else all key frames VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_HAS_KEYFRAMES = 0x00000010;
        /**
         * Fourcc modifiers MASK for VVWVIDEO::dwDrFlags
         */
        static final int MASK___FCCMODIFIERS = 0x00FF0000;
        /**
         * Stream is DigiSuite MJPG see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_MJPG_DIGISUITE = 0x00000000; // resolve 'mjpg' types
        /**
         * Stream is Miro DC50 MJPG see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_MJPG_DCx0 = 0x00010000;
        /**
         * Stream is DigiSuite Edit MJPG see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_MJPG_DSEDIT = 0x00020000;
        /**
         * Stream is MJPG MS-Dib variant see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_MJPG_JPGDIB = 0x00040000;
        /**
         * Stream is JFIF jpeg see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_MJPG_JFIF = 0x00080000;
        /**
         * Stream should use internal codecs see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_USE_INTERN = 0x00100000; // which codec dup to use
        /**
         * Stream should use quicktime codecs see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_USE_QT = 0x00400000;
        /**
         * Stream should use windows icm/vfw codecs see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_FCC_USE_ICM = 0x00800000;
        /**
         * Private Data MASK see VVWVIDEO::dwDrFlags
         */
        static final int MASK__CODECPRIVATEDATA = 0x0F000000;
        /**
         * Private data format is AVI see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_CODECPRIVATEDATA_AVI = 0x01000000; // Std AVI VVWVIDEO
        /**
         * Private data format is MOV see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_CODECPRIVATEDATA_MOV = 0x02000000; // ImageDesc in VVWVIDEO (as above)
        /**
         * Private data format is OMF see VVWVIDEO::dwDrFlags
         */
        static final int DRFLAGS_CODECPRIVATEDATA_OMF = 0x04000000; // OMF Util
        /**
         * MASK Room for current DTVVW_PREVIEW and more if nec see VVWVIDEO::dwDrFlags
         */
        static final long MASK___PREVIEW = 0xF0000000L;
        /**
         * Our internal flags including:
         * #DRFLAGS_ZERO_FIELD_DOMINANT, #DRFLAGS_FIRST_FIELD_DOMINANT, #DRFLAGS_HAS_KEYFRAMES,
         * #DRFLAGS_FCC_MJPG_DIGISUITE, #DRFLAGS_FCC_MJPG_DCx0, #DRFLAGS_FCC_MJPG_DSEDIT,
         * #DRFLAGS_FCC_MJPG_JPGDIB, #DRFLAGS_FCC_MJPG_JFIF,
         * #DRFLAGS_FCC_USE_INTERN, #DRFLAGS_FCC_USE_QT, #DRFLAGS_FCC_USE_ICM,
         * #DRFLAGS_CODECPRIVATEDATA_AVI, #DRFLAGS_CODECPRIVATEDATA_MOV, #DRFLAGS_CODECPRIVATEDATA_OMF,
         * #DTVVW_PREVIEW
         */
        long dwDrFlags; // Our internal flags
        /**
         * Source File Type
         */
        long dwFileType;
        /**
         * Reserved, init to zero and leave alone
         */
        long dwResDrastic; // Always

    } // VVWVIDEO, * pVVWVIDEO;
}
